#if defined(__DECC) || defined(__DECCXX)
# pragma module driver_nt "X-5"
#endif


/* driver_nt.c */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the Windows NT DriverEntry routine
 *
 * Author:              Bradley M. Waters
 * Creation Date:       21-Jul-1994
 *
 * Modified by:
 *
 * X-5  BMW             Bradley M. Waters        8-Dec-1994
 *      Fixed a variety of problems prior to the 1st internal field test of
 *      IPROBE on Windows NT.
 *
 * X-4  BMW             Bradley M. Waters       25-Oct-1994
 *      Fixed nonpaged pool deallocations to insure that all pool is deallocated.
 *      changed the order of device object delete in Unload routine to insure
 *      that the write queue is completely terminated before the device objects
 *      are deleted.
 *
 * X-3  BMW             Bradley M. Waters       10-Oct-1994
 *      Cleanup of code
 *
 * X-2  BMW             Bradley M. Waters       31-Aug-1994
 *      Integrated Joe Notarangelo's driver that interfaces to the AXP perf.
 *      counters into the IPROBE kernel driver - previously, all counter
 *      manipulation actions were passed onto Joe'e PERFEV4 driver.
 *
 * X-1  BMW             Bradley M. Waters       21-Jul-1994
 *      Original Version.  Based much of the code in this module on
 *      Joe Notarangelo's EV4 counter driver, PERFEV4 - in fact, much
 *      of the code was simply moved from the PERFEV4.C module.
 *
*/
#if 0
#define DBGISR
#define DBGPID
#define DBGPM
#define DBGP
#define DBGPID2
#define DBG10
#define DBGDRV
#define DBGD
#define DBGE
#define DBGDPC
#define DBGCRSH
#define DBGWT

#define TRCE
#define TRCWT
#define TrcPrint DbgPrint
#endif

//#include "stdarg.h"
#include "stdio.h"

#include "ntddk.h"
//#include "dda21064.h"

#include "ipr_kernel_int_defs.h"
#include "ipr_driver_nt.h"
#include "hb_library.h"

#include "trpfrm.h"
#include "perfev4.h"
#include "ipr_isr_nt.h"
#include "ipr_isr_nt_def.h" //wagner

#ifdef   TRCI
#include "TraceDrv.h"
#endif

ULONG HalpGet21064PerformanceVector(ULONG,PKIRQL);

KWAIT_REASON wrtReason = {Executive};
KPROCESSOR_MODE wrtMode = {KernelMode};
BOOLEAN wrtFail = {FALSE};      // Flag indicating whether write thread failed at startup
PKTHREAD wrtThread;             // Write thread

IPR_STATUS closeStatus;
IPR_STATUS openStatus;
IPR_STATUS writeStatus;

int skipHist = 0;
int addHist = 0;

//#ifdef DBGDPC
#ifdef DBGPID
int tmpcnt = 0;
#endif

extern HANDLE sampHandle;
extern UNICODE_STRING *sampFname;
extern UNICODE_STRING *sampDFname;

IPR_WrtStruct *wrtStruct = NULL;       // Pointer to structure containing
                                // spinlock, write queue, and events
                                // for writing sample buffers

// Header for write structure
IPR_BUFHDR write_template = {
	IPR_PROD_IDENT,
        0,                      // 2nd field (size) is filled in by allocation routine
        IPR_WBUFFER_TYPE,
        IPR_WBUFFER_SUBTYPE,
        IPR_WBUFFER_KIND
        };

extern void *GetCurThread(void);
extern void *GetCurTEB(void);

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

NTSTATUS
IprDispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
IprDeviceControl(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

VOID
IprUnload(
    IN PDRIVER_OBJECT DriverObject
    );


#ifdef ALLOC_PRAGMA
#pragma alloc_text(init,DriverEntry)
#pragma alloc_text(init,IprDispatch)
#pragma alloc_text(init,IprDeviceControl)
#pragma alloc_text(init,IprUnload)
#endif

void
WriteThread(DEVICE_EXTENSION *deviceExtension)
{
   IPR_STATUS status;
   IO_STATUS_BLOCK ioStat;
   LONG oldPri;
   IPR_WrtStruct *wrtStruct;
   IPR_WrtList *listEntry;

   HANDLE sampThHandle = 0;
   int64 allocSize = {32768};
   ULONG initAttrb = {OBJ_CASE_INSENSITIVE | OBJ_INHERIT};
   ULONG fileAttrb = {FILE_ATTRIBUTE_NORMAL};
   ULONG shareAcc = {0};
   ULONG createDisp = {FILE_SUPERSEDE};
   ULONG createOpt = {FILE_NON_DIRECTORY_FILE /*| FILE_SEQUENTIAL_ONLY */ /* | FILE_SYNCHRONOUS_IO_ALERT*/};
   ACCESS_MASK accMask = {GENERIC_READ | GENERIC_WRITE /*| SYNCHRONIZE*/};
   int64 bytePos = {0};
   int64 transfSize;

#ifdef TRCE
   ANSI_STRING filenameANSI;
#endif

   OBJECT_ATTRIBUTES sampAttrb;

#ifdef TRC0
   TrcPrint("WriteThread entry, deviceExtension addr %x\n",
      deviceExtension);
#endif

   if (!deviceExtension)
   {
#ifdef TRCE
      TrcPrint("WriteThread Error: ptr. to device extension is null\n");
#endif
      wrtFail = TRUE;
      PsTerminateSystemThread(IPR__NULLPTR);
   }

   wrtStruct = deviceExtension->wrtStruct;
   wrtThread = KeGetCurrentThread();    // Store thread handle in global

   // Boost priority a bit
   oldPri = KeSetBasePriorityThread(wrtThread, +5);
#ifdef TRCWT
   TrcPrint("Bumped thread priority a bit so it will act on queued requests immediately, prev. priority was %d\n",
      oldPri);
#endif

   while (wrtStruct->wrtActive)
   {
      // Wait until a buffer becomes full - alertable wait with no timeout

#ifdef DBGWT
      DbgPrint("Preparing to wait on event %x\n",
         wrtStruct->wrtEvent);
//      DbgBreakPoint();
#endif
      status = KeWaitForSingleObject(&wrtStruct->wrtEvent,
         wrtReason,wrtMode,TRUE,NULL);

#ifdef TRCWT
      TrcPrint("Return from wait in write system thread, status = %x, wrtActive %x\n",
         status,wrtStruct->wrtActive);
#endif
//      if ((status != STATUS_SUCCESS) && (status != STATUS_ALERTED))
      if (status != STATUS_SUCCESS)
      {
#ifdef TRCW
          TrcPrint("WriteThread: Wait on write event returned unexpected status, status=%x\n",status);
#endif
          continue;  // Wait for next event
/*
          wrtStruct->wrtActive = FALSE;
          wrtFail = TRUE;
          if (sampThHandle != 0)
          {
             closeStatus = ZwClose(sampThHandle);
             sampHandle = 0;
          }
          PsTerminateSystemThread(status);
*/
      }
      // Check to see if the thread is active, take appropriate action
      if (wrtStruct->wrtActive)
      {
          // Grab 1st buffer to write from write queue
	  ExInterlockedVerifyList(&wrtStruct->wrtQ);
          (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
             &wrtStruct->wrtQ_SpinLock);
	  ExInterlockedVerifyList(&wrtStruct->wrtQ);
          while (listEntry)
          {
             // bufptr field is either an action (open or close) or
             // the pointer of a buffer to write
             if ((int)listEntry->bufptr == IPR_OPENWRITE)
             {
                // Open sample file

                // If sample file was already open - probably due to an
                // abnormal termination - close it before opening a new file
                if (sampThHandle != 0)
                {
#ifdef TRCE
                   TrcPrint("WriteThread Error, trying to open sample file but file already open\n");
                   TrcPrint("assuming prev. error and closing file, handle %x, to allow open\n",
                      sampThHandle);
#endif
                   closeStatus = ZwClose(sampThHandle);
                   sampThHandle = 0;
                   sampHandle = 0;
                }
#ifdef DBGWT
                DbgPrint("WriteThread calling InitializeObjectAttributes, fnptr %x, dfnptr %x, fileattr %x \n",
                   sampFname,sampDFname,initAttrb);
//                DbgBreakPoint();
#endif

                // Use only the file name for now (I'll implement the default file
                // name later).
                // NOTE: No root directory or security descriptor is passed
                InitializeObjectAttributes(&sampAttrb,sampFname,initAttrb,NULL,NULL);

#ifdef DBGWT
                DbgPrint("Return from InitializeObjectAttributes, sampAttrb len %d, nameptr %x,  rootH %x, secptr %x, attrb %x\n\n",
                   sampAttrb.Length,sampAttrb.ObjectName,sampAttrb.RootDirectory,sampAttrb.SecurityDescriptor,sampAttrb.Attributes);
                DbgPrint("Preparing to open file, mask %x, sampAttrb %x, ioStat %x, allocSize %x,\n",
                   accMask,&sampAttrb,&ioStat,&allocSize);
                DbgPrint("fileAttrb %x, shr %x, disp %x, opt %x\n",
                   fileAttrb,shareAcc,createDisp,createOpt);
//                DbgBreakPoint();
#endif
                status = ZwCreateFile(&sampThHandle,accMask,&sampAttrb,&ioStat,(PLARGE_INTEGER)&allocSize,
                   fileAttrb,shareAcc,createDisp,createOpt,NULL,0);
                openStatus = status;
                // Init. file byte pointers and buffer write counts
                bytePos = 0;
                wrtStruct->buffwrites = 0;      // incremented for each buffer written (including partwrites)
                wrtStruct->partwrites = 0;
                if (!OK_STATUS(status))
                {
#ifdef TRCE
                   TrcPrint("WriteThread Error return from ZwCreateFile, status=%x\n",
                      status);
                   status = RtlUnicodeStringToAnsiString(&filenameANSI,sampFname,TRUE);
                   if (status == STATUS_SUCCESS)
                   {
                      TrcPrint("File that failed to open %s\n",
                         filenameANSI.Buffer);
                      RtlFreeAnsiString(&filenameANSI);
                   }
                   else
                      TrcPrint("Conversion of file name from UNICODE to ANSI failed, status %x\n",
                         status);
#endif
                }
                else if (!OK_STATUS(ioStat.Status))
                {
#ifdef TRCE
                   TrcPrint("WriteThread Error return from ZwCreateFile, ioStat.Status=%x, handle %x\n",
                     ioStat.Status,sampThHandle);
                   status = RtlUnicodeStringToAnsiString(&filenameANSI,sampFname,TRUE);
                   if (status == STATUS_SUCCESS)
                   {
                      TrcPrint("File that failed to open %s\n",
                         filenameANSI.Buffer);
                      RtlFreeAnsiString(&filenameANSI);
                   }
                   else
                      TrcPrint("Conversion of file name from UNICODE to ANSI failed, status %x\n",
                         status);
#endif
                   openStatus = ioStat.Status;
                }
                // Global sampHandle set to sampThHandle for benefit of write routine
                sampHandle = sampThHandle;
                BARRIER();   // Do MB to make sure global vars. are updated
                // Return memory for open entry
                if (listEntry->delEntry)
                   deallocate_memory(listEntry);
#ifdef DBGCRSH
                if (OK_STATUS(openStatus))
                   DbgPrint("File successfully opened, handle %x\n",
                      sampThHandle);
#endif
#ifdef TRCWT
                if (OK_STATUS(openStatus))
                   TrcPrint("File successfully opened, handle %x\n",
                      sampThHandle);
#endif
                // Signal caller that open is complete
                wrtStruct->wrtoState = KeSetEvent(&wrtStruct->wrtoEvent,0,FALSE);
             }
             else // davew
             if ((int)listEntry->bufptr == IPR_CLOSEWRITE)
             {
                if (sampThHandle)
                {
                   status = ZwClose(sampThHandle);
                   if (!OK_STATUS(status))
                   {
#ifdef TRCE
                      TrcPrint("WriteThread Error return from ZwCloseFile, handle %x, status=%x\n",
                         sampThHandle,status);
#endif
                   }
                   sampThHandle = 0;       // Clear out sampThHandle local
                   sampHandle = 0;         // Clear out sampHandle global
                   BARRIER();              // Do MB to make sure globals are updated
                }
                else
                   status = IPR__SUCCESS;
                closeStatus = status;
                // Return memory for close entry
                if (listEntry->delEntry)
                   deallocate_memory(listEntry);
             }
             else // davew
             if (((int)listEntry->bufptr != IPR_OPENWRITE) &&
                ((int)listEntry->bufptr != IPR_CLOSEWRITE))
             {
#ifdef DBGWT
                DbgPrint("WriteThread: Doing WritePreparing for ZwWriteFile, list entry %x, bufptr %x, handle %x, bytePos %Lx\n",
                   listEntry,listEntry->bufptr,sampThHandle,bytePos);
//                DbgBreakPoint();
#endif
                // Make sure pointer is valid
                if (!listEntry->bufptr)
                {
#ifdef TRCWT
                   TrcPrint("WriteThread: Error, bufptr null, listEntry %x\n",
                      listEntry);
#endif
                }
                else
                {
                   // Write entire buffer including the header
#ifdef DBGWT
                   DbgPrint("WriteThread: Preparing for ZwWriteFile, sample buffer at %x, list entry %x, size %d, handle %x, bytePos %Lx, status=%x\n",
                      listEntry->bufptr,listEntry,listEntry->bsize,sampThHandle,bytePos,status);
#endif
                   status = ZwWriteFile(sampThHandle,NULL,NULL,NULL,&ioStat,
                      listEntry->bufptr,listEntry->bsize,
                      (PLARGE_INTEGER)&bytePos,NULL);
                   writeStatus = status;
                   if (!OK_STATUS(status))
                   {
#ifdef TRCE
                      TrcPrint("WriteThread: Error return from ZwWriteFile writing sample buffer at %x, list entry %x, handle %x, size %d, bytePos %Lx, status=%x\n",
                         listEntry->bufptr,listEntry,sampThHandle,listEntry->bsize,bytePos,status);
#endif
                   }
                   else if (!OK_STATUS(ioStat.Status))
                   {
#ifdef TRCE
                      TrcPrint("WriteThread: Error return from ZwWriteFile writing sample buffer at %x, list entry %x, size %d, handle %x, bytePos %Lx, ioStat.Status=%x\n",
                         listEntry->bufptr,listEntry,listEntry->bufptr->header.size,sampThHandle,bytePos,ioStat.Status);
#endif
                      writeStatus = ioStat.Status;
                   }
                   if (OK_STATUS(writeStatus))
                   {
                      // Update bytePos by number of bytes actually written to the file
                      transfSize = ioStat.Information;
                      bytePos += transfSize;
                      if (listEntry->bsize != ioStat.Information)
                      {
#ifdef TRCWT
                         TrcPrint("Only %d bytes written from request of %d bytes\n",
                            ioStat.Information,listEntry->bsize);
#endif
                         writeStatus = IPR__WRITESHORT;
                      }
                      if (listEntry->sampBuff)
                      {
                         wrtStruct->buffwrites++;
                         if ((listEntry->bufptr->areaoffset < listEntry->bufptr->areasize) ||
                          (writeStatus == IPR__WRITESHORT))
                            wrtStruct->partwrites++;
                         status = KIprSbSetFull(listEntry->bufptr,IPR_BUFFER_EMPTY);
                         if (!IPR_SUCCESS(status))
                         {
#ifdef TRCE
                            TrcPrint("WriteThread: Error setting full flag on sample buffer, status=%x\n",status);
#endif
                         }
                      }
                   }
#ifdef TRCWT
                   TrcPrint("WriteThread: Sample Buffer %x written from list head %x, size %d\n",
                      listEntry->bufptr,listEntry,listEntry->bufptr->header.size);
#endif

                   if (listEntry->delBuff)
                   {
                      // Delete buffer after write
                      deallocate_memory(listEntry->bufptr);
                   }
                }
                if (listEntry->delEntry)
                {
                   // Delete queue entry after write
                   deallocate_memory(listEntry);
                }
             }
             // Grab next buffer to write from write queue
#ifdef DBGWT
             DbgPrint("WriteThread: Doing ExInterlockedRemoveHeadList &wrtQ %x wrtQ (%x , %x), &SpinLock %x\n",
                &wrtStruct->wrtQ, wrtStruct->wrtQ.Flink, wrtStruct->wrtQ.Blink, &wrtStruct->wrtQ_SpinLock);
#endif
             ExInterlockedVerifyList(&wrtStruct->wrtQ);
             (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
                &wrtStruct->wrtQ_SpinLock);
	     ExInterlockedVerifyList(&wrtStruct->wrtQ);

          }  // end - while
          // Reset signal to wait for next full buffer
          wrtStruct->wrtState = KeResetEvent(&wrtStruct->wrtEvent);
#ifdef TRCWT
          TrcPrint("WriteThread: Reset write event, prev. setting was %d\n",
             wrtStruct->wrtState);
#endif
      }  // end - if active
   }  // end - while loop on active

   // done with this system thread, bump priority (to finish before caller
   // who may destroy the address space) and signal caller
#ifdef DBGWT
   DbgPrint("Done with write system thread, returning to caller, event %x\n",
      &wrtStruct->wrtdEvent);
//   DbgBreakPoint();
#endif
   if (sampThHandle != 0)
       closeStatus = ZwClose(sampThHandle);
   sampHandle = 0;
#ifdef TRC0
   TrcPrint("WriteThread exit\n");
#endif
   wrtStruct->wrtdState = KeSetEvent(&wrtStruct->wrtdEvent,0,FALSE);
   PsTerminateSystemThread(STATUS_SUCCESS);
}

static IPR_BOOLEAN PCinRange(IPR_PBUFFER *PCrangeBuf,IN uint64 pc)
{
   register IPR_PC_RANGE
      *PCrange;
   register unsigned int i;
   register IPR_BOOLEAN found;

   // If no PC range buffer, than all samples are collected;
   if (!PCrangeBuf)
      return TRUE;

   if (PCrangeBuf->sample_always)
      return TRUE;

   PCrange = (IPR_PC_RANGE *)(PCrangeBuf + 1);

   found = FALSE;
   for (i = 0; i < PCrangeBuf -> rangecnt; i++)
      if (PCrange[i].sample)
      {
         if (pc >= PCrange [i].StartAddress && pc <= PCrange [i].EndAddress)
         {
            found = TRUE;
            break;
         }
      }
   return found;
}

IPR_BOOLEAN SkipInt(IPR_IBUFFER *SkipBufPtr, PDEVICE_EXTENSION DeviceExtension)
{
   register IPR_ISKIP
      *SkipPtr;

   // NOTE: Following code reorged.
   if (!SkipBufPtr)
      return FALSE;
   SkipPtr = (IPR_ISKIP *) (SkipBufPtr + 1);
   // Log a sample only if the skip counter has reached threshold
#ifdef TRCDPC
   if ((DeviceExtension->intCntE % 1000) == 0)
   {
      TrcPrint("SkipPtr %x, counter %d, skip count %d, skip threshold %d\n",
         SkipPtr,DeviceExtension->Counter,SkipPtr [DeviceExtension->Counter].intssincelast,
         SkipPtr [DeviceExtension->Counter].intspersample);
   }
#endif
   if (SkipPtr [DeviceExtension->Counter] . intssincelast
      < SkipPtr [DeviceExtension->Counter] . intspersample)
   {
      SkipPtr [DeviceExtension->Counter] . intssincelast++;
      return TRUE;
   }
   else
   {
      SkipPtr [DeviceExtension->Counter] . intssincelast = 0;
      return FALSE;
   }
}

void DpcISR(IN PKDPC Dpc, IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp, IN IPR_DpcBlock *PassedContext)
{
   uint64
      datum;
   IPR_STATUS status;
   int hIndex;
   register int
      *DataLong;
   register uint64
      *DataQuad;
   register unsigned int dummy;
   PDEVICE_EXTENSION DeviceExtension;
   IPR_BOOLEAN diff_cpu = FALSE;
   IPR_DpcBlock *Context,*prevContext;
   IPR_BUFDSC *BufDscPtr;
//   uint64 curTime;

#ifdef TRC0
   TrcPrint("AXP 21064 - DPC: Enter\n" );
#endif

   if (!DeviceObject)
   {
#ifdef TRCE
      TrcPrint("DpcISR ERROR: device object ptr is null, exiting\n");
      DbgPrint("DpcISR ERROR: device object ptr is null, exiting\n");
#endif
      return;
   }
   DeviceExtension =
     (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   if (!DeviceExtension)
   {
#ifdef TRCE
      TrcPrint("DpcISR ERROR: device extension ptr is null, exiting\n");
      DbgPrint("DpcISR ERROR: device extension ptr is null, exiting\n");
#endif
      return;
   }

#ifdef DBGDPC
   if ((DeviceExtension->intCntE % 1000) == 0)
   {
      DbgPrint("AXP 21064 - DPC: Enter\n" );
   }
#endif


   Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
   Context += DeviceExtension->intCntE % IPR_DPC_BLOCKS;

   BufDscPtr = GETBDPTR(Context->cpu);

   if (BufDscPtr != Context->BufDscPtr)
   {
#ifdef TRCE
      TrcPrint("DPC Error: BD mismatch, Context->BD %x, current %x, Context %x, DevExt %x, Counter %d cpu %d, start %d, end %d, drop %d\n",
         Context->BufDscPtr,BufDscPtr,Context,DeviceExtension,DeviceExtension->Counter,DeviceExtension->cpu,
         DeviceExtension->intCntS,DeviceExtension->intCntE,DeviceExtension->dropInt);
#endif
#ifdef DBGPID
      DbgPrint("DPC Error: BD mismatch, Context->BD %x, current %x, Context %x, DevExt %x, Counter %d cpu %d, start %d, end %d, drop %d\n",
         Context->BufDscPtr,BufDscPtr,Context,DeviceExtension,DeviceExtension->Counter,DeviceExtension->cpu,
         DeviceExtension->intCntS,DeviceExtension->intCntE,DeviceExtension->dropInt);
#endif
   }

   prevContext = NULL;
//   curTime = THIS_TIME();

//   Context = (IPR_DpcBlock *)ExInterlockedRemoveHeadList(&DeviceExtension->sampQ,
//       &DeviceExtension->sampSpin);
   while ((Context != prevContext) && (DeviceExtension->intCntE < DeviceExtension->intCntS))
   {
      // If pid is 0 and ps is a, assume mode is idle
      if ((Context->pid == 0) && (Context->ps == 0xa))
      {
         Context->mode = 2;
#ifdef DBGPID
         if ((DeviceExtension->intCntE % 1000) == 0)
         {
            DbgPrint("***DPC, idle process, pid %x, PAL cur thread %x, Context %x\n",
               Context->pid,GetCurThread(),Context);
         }
#endif
#ifdef TRCDPC
         if ((DeviceExtension->intCntE % 1000) == 0)
         {
            TrcPrint("***DPC, idle process, pid %x, PAL cur thread %x, Context %x\n",
               Context->pid,GetCurThread(),Context);
         }
#endif
      }

#ifdef DBGPID2
      if (((Context->pid & 0xffffff00) != 0) && ((DeviceExtension->intCntE % 1000) == 0))
      {
         DbgPrint("DPC Error, bad pid %x, PAL cur thread %x, Context %x, real proc blk. %x\n",
            Context->pid,GetCurThread(),Context,PsGetCurrentProcess());
         DbgBreakPoint();
      }
#endif

#ifdef DBGPID
      if ((DeviceExtension->intCntE % 1000) == 0)
      {
          DbgPrint("Context %x, intCntS %d, intCntE %d, DevExt %x, Counter %d, cpu %d, BDptr %x, dpcBlk %x\n",
                  Context,DeviceExtension->intCntS,DeviceExtension->intCntE,DeviceExtension,
                     DeviceExtension->Counter,DeviceExtension->cpu,BufDscPtr,DeviceExtension->dpcBlock);
//          DbgBreakPoint();
      }
#endif

      // NOTE: this DPC is associated with an ISR associated with
      // a particular CPU and counter.  The cpu, buffer descriptor
      // and counter fields in the context should never change
      // while draining the dpc queue.
      // However, this DPC can run on a different CPU than the ISR.
      if (!BufDscPtr)
      {
#ifdef TRCE
         TrcPrint("DpcISR ERROR: buff. descriptor ptr in context block nulldevice object ptr is null, skipping sample\n");
#endif
#ifdef DBGE
         DbgPrint("DpcISR ERROR: buff. descriptor ptr in context block nulldevice object ptr is null, skipping sample\n");
         DbgBreakPoint();
#endif
         return;
      }

      // Histogram buffers to fill in?
      if (BufDscPtr -> pHBuf)
      {
         //
         // CREATING HISTOGRAMS
         //

         register IPR_HISTELEM
            *DataElem;
         register IPR_HBUFFER
            *HistoPtr;

         //
         // NOTE: The code below assumes that the buffers are linear, i.e.
         //       that the last one points to null.
         //
         //       Also note that the data array is organized with all the elements
         //       corresponding to a single value are located contiguously.
         //       For example, an element sequence on a chip with three counters
         //       might be val 1 ctr 0, val 1 ctr 1, val 1 ctr 2, val 2 ctr 0,
         //       val 2 ctr 1, val 2 ctr 2, etc.
         //
         //       All subscripting is done manually.
         //
         HistoPtr = BufDscPtr -> pHBuf;
         while (HistoPtr)
         {
#ifdef TRCDPC
            if ((DeviceExtension->intCntE % 10000) == 0)
               TrcPrint("Checking Hist at %x, item %d\n",
                  HistoPtr,HistoPtr->item);
#endif
#ifdef DBGDPC
            if ((DeviceExtension->intCntE % 1000) == 0)
            {
               DbgPrint("Checking Hist at %x, item %d, pc %x\n",
                  HistoPtr,HistoPtr->item,Context->pc);
//               DbgBreakPoint();
            }
#endif

            DataElem = (IPR_HISTELEM *) (HistoPtr + 1);
            switch (HistoPtr -> item.ipr_hmask_l_bits )
            {
               case IPR_HMASK_M_TOTAL :
                  datum = 0;
                  break;
               case IPR_HMASK_M_IPL :
                  datum = Context -> ipl;
                  break;
               case IPR_HMASK_M_MODE :
                  datum = Context -> mode;
#ifdef DBGDPC
                  if ((DeviceExtension->intCntE % 1000) == 0)
                     DbgPrint("Mode %x\n",
                       datum);
#endif
                  break;
               case IPR_HMASK_M_ADDRESS :
                  datum = Context -> pc;
                  break;
               case IPR_HMASK_M_PRIORITY :
                  datum = Context -> priority;
                  break;
               default :
                  datum = datum | (!datum);
                  break;
            }
            if (datum != 0xFFFFFFFF)
            {
               status = IprHbIndex (datum,
                  DeviceExtension->Counter,
                  BufDscPtr -> numcounters,
                  HistoPtr -> minval,
                  HistoPtr -> pcgran,
                  &hIndex);
#ifdef DBGDPC
               if ((DeviceExtension->intCntE % 100) == 0)
               {
                  DbgPrint("Return from IprHbIndex, datum %x, hIndex %d, status %x\n",
                     datum,hIndex,status);
               }
#endif
               if ((IPR_SUCCESS(status)) && (status != IPR__NOTINHIST) &&
                (hIndex < HistoPtr->elemcnt))
               {
#ifdef DBGDPC
                  if ((DeviceExtension->intCntE % 1000) == 0)
                  {
                     DbgPrint("DPC return from IprHbIndex, Histptr %x, datum %x, counter %d, minval %x, gran %d, index %d\n",
                        HistoPtr,
                        datum,
                        DeviceExtension->Counter,
                        HistoPtr -> minval,
                        HistoPtr -> pcgran,
                        hIndex);
                     DbgPrint("intrpt %d, count %d, Mux %d, Event %d\n\n",
                        DeviceExtension->intCntE,
                        DeviceExtension->CounterValue,
                        DeviceExtension->MuxControl,
                        DeviceExtension->EventCount
                        );
                  }
#endif
                  DataElem [hIndex]++;
                  addHist++;
#ifdef DBGDPC
                  if ((addHist % 100) == 0)
                  {
                     DbgPrint("DPC 100 histogram elements added, add count %d, datum %x, pc %x, min %x, gran %x, elemcnt %d, numval %x, index %d, status %x\n",
                        addHist,datum,Context->pc,
                        HistoPtr -> minval,
                        HistoPtr -> pcgran,
                        HistoPtr -> elemcnt,
                        HistoPtr -> numvals,
                        hIndex,
                        status);
                  }
#endif
               }
               else
               {
                  skipHist++;
#ifdef DBGDPC
                  if ((skipHist % 100) == 0)
                  {
                     DbgPrint("DPC Warning: 100 histogram elements skipped, skipped count %d, datum %x, pc %x, min %x, gran %x, elemcnt %d, numval %d, index %d, status %x\n",
                        skipHist,datum,Context->pc,
                        HistoPtr -> minval,
                        HistoPtr -> pcgran,
                        HistoPtr -> elemcnt,
                        HistoPtr -> numvals,
                        hIndex,
                        status);
                  }
#endif
               }
            }
            HistoPtr = HistoPtr -> pNext;
         }
      }

      // Sample buffers to fill in?
      if (BufDscPtr -> pSBuf)
      {
         ////////////////////////
         // COLLECTING SAMPLES //
         ////////////////////////

         register IPR_SBUFFER
            *BufPtr;
         register int
            marked_buffer;
         IPR_WrtStruct *wrtStruct;
         IPR_SBUFFER *wptr;
         IPR_BOOLEAN dropSamp;

#ifdef TRCDPC
         if ((DeviceExtension->intCntE % 500) == 0)
         {
            TrcPrint("ISR taking PC sample, PCrngPtr %x, PC %x, aoffset %x\n",
               BufDscPtr->pPBuf,Context->pc,BufDscPtr->pSBuf->areaoffset);
         }
#endif
         // Check interrupt skip count
         if (!SkipInt(BufDscPtr->pIBuf,DeviceExtension))
         {
            // Ignore the interrupt if the PC is out of the specified ranges and
            // sample always isn't set
            if (PCinRange(BufDscPtr->pPBuf,Context->pc))
            {

               dropSamp = FALSE;
#ifdef TRCDPC
               if ((DeviceExtension->intCntE % 1000) == 0)
               {
                  TrcPrint("Adding PC %x to sample buffer\n",
                  Context->pc);
               }
#endif
               marked_buffer = FALSE;
               BufPtr = BufDscPtr -> pSBuf;
               if (BufPtr -> areaoffset >= BufPtr -> areasize)
               {
#ifdef DBGDPC
                  DbgPrint("Buffer full? BufPtr %x aoff %x asize %x\n",
                  BufPtr,BufPtr->areaoffset,BufPtr->areasize);
//                  DbgBreakPoint();
#endif
                  //
                  // The location is out of range, which means we have finished with
                  // this buffer.  Mark it and try to move on to the next buffer.
                  //
                  if (BufPtr -> areaoffset == BufPtr -> areasize)
                  {
                     BARRIER ();
                     BufPtr -> full = IPR_BUFFER_FULL;
                     BufPtr -> areaoffset++;
                     // Queue buffer for writing
                     do_full_action (BufPtr, BufDscPtr, DeviceExtension);
                     marked_buffer = TRUE;
#ifdef DBGDPC
                     DbgPrint("Done with buffer, BufPtr %x, aoffset %x, Next %x\n",
                        BufPtr,BufPtr->areaoffset,BufPtr->pNext);
#endif
                  }
                  //
                  // The current buffer is full.  Check if the next buffer can
                  // be used.
                  //
                  if (BufPtr -> pNext -> full == IPR_BUFFER_FULL)
                  {
                     // The next buffer is full, so count a missed sample
                     BufPtr -> missedcnt++;
                     // In case buffers somehow didn't get written add buffers to the write
                     // queue

#ifdef DBGDPC
                       DbgPrint("All buffers full/write queue empty, preparing to add buffers to write queue\n",
                          BufPtr);
//                     DbgBreakPoint();
#endif
                     wrtStruct = DeviceExtension->wrtStruct;

                     if IsListEmpty(&wrtStruct->wrtQ)
                     {
                        wptr = BufPtr->pNext;
#ifdef DBGDPC
                        DbgPrint("All buffers full (1), write queue empty, adding %x to write queue %x, wrtStruct %x, wptr %x\n",
                          BufPtr,&wrtStruct->wrtQ,wrtStruct,wptr);
//                      DbgBreakPoint();
#endif
                        BARRIER ();
                        BufPtr -> full = IPR_BUFFER_FULL;
                        BufPtr -> areasize+1;
                        do_full_action (BufPtr, BufDscPtr, DeviceExtension);
                        marked_buffer = TRUE;
                        while (wptr != BufPtr)
                        {
                           if (wptr->full == IPR_BUFFER_FULL)
                           {
#ifdef DBGDPC
                              DbgPrint("All buffers full (2), write queue empty, adding %x to write queue %x\n",
                                 wptr,&wrtStruct->wrtQ);
//                              DbgBreakPoint();
#endif
                              wptr -> full = IPR_BUFFER_FULL;
                              wptr -> areasize+1;
                              do_full_action (wptr, BufDscPtr, DeviceExtension);
                           }
                           wptr = wptr->pNext;
#ifdef DBGDPC
                           DbgPrint("Checking next buffer, wptr %x\n",
                            wptr);
#endif
                        }
#ifdef DBGDPC
                        DbgPrint("Done writing full buffers, wrtStruct %x, wptr %x\n",
                          wrtStruct,wptr);
//                        DbgBreakPoint();
#endif
                     }

#ifdef DBGDPC
                     DbgPrint("Next buffer full, BufPtr %x, Next %x\n",
                        BufPtr,BufPtr->pNext);
//                     DbgBreakPoint();
#endif
                     dropSamp = TRUE;
                  }

                  if (!dropSamp)
                  {

                      // The next buffer is empty, so switch buffers

                      BufPtr = BufPtr -> pNext;
                      BufPtr -> begintime = Context->time;
                      BufPtr -> missedcnt = 0;
                      BufPtr -> areaoffset = 0;
                      BufDscPtr -> pSBuf = BufPtr;
#ifdef DBGDPC
                      DbgPrint("Switching buffers, New BufPtr %x, Int %d, Cntr %d\n",
                         BufPtr,DeviceExtension->intCntE,DeviceExtension->Counter);
//                      DbgBreakPoint();
#endif
                  }
               }   // buffer full check
               if (!dropSamp)
               {
                  //
                  // Save the requested information.  Start with the quadword data
                  // to allow alignment easily.
                  //
                  DataQuad = (uint64 *) ((char *) ((BufPtr) + 1) + BufPtr->areaoffset);

#ifdef TRCDPC
                  if ((DeviceExtension->intCntE % 1000) == 0)
                  {
                     TrcPrint("Saving quad part of sample at %x, BufPtr is %x, Samples start at %x, aoffset %x\n",
                       DataQuad,BufPtr,BufPtr+1,BufPtr->areaoffset);
                  }
#endif
#ifdef DBGDPC
                  if ((DeviceExtension->intCntE % 1000) == 0)
                  {
                     DbgPrint("Dpc %x, DevObj %x, irp %x, context %x, PID %x, ProcBlk %x\n",
                       Dpc,DeviceObject,Irp,Context,Context->pid,THIS_PROCBLK());
//                     DbgBreakPoint();
                  }
#endif
// Check quadword masks

#if (!(IPR_DMASK_M_TIME == 1))
#  error "assertion failed (IPR_DMASK_M_TIME == 1)"
#endif
                  if (BufPtr -> data . ipr_dmask_v_time)
                     if (BufPtr -> dataquad . ipr_dmask_v_time)
                        *DataQuad++ = Context->time;

#if (!(IPR_DMASK_M_PS == (IPR_DMASK_M_TIME << 1)))
#  error "assertion failed (IPR_DMASK_M_PS == (IPR_DMASK_M_TIME << 1))"
#endif
                  if (BufPtr -> data . ipr_dmask_v_ps)
                     if (BufPtr -> dataquad . ipr_dmask_v_ps)
                        *DataQuad++ = Context->ps;

#if (!(IPR_DMASK_M_PC == (IPR_DMASK_M_PS << 1)))
#  error "assertion failed (IPR_DMASK_M_PC == (IPR_DMASK_M_PS << 1))"
#endif
                  if (BufPtr -> data . ipr_dmask_v_pc)
                     if (BufPtr -> dataquad . ipr_dmask_v_pc)
                        *DataQuad++ = Context->pc;

#if (!(IPR_DMASK_M_COUNTER == (IPR_DMASK_M_PC << 1)))
#  error "assertion failed (IPR_DMASK_M_COUNTER == (IPR_DMASK_M_PC << 1))"
#endif
                  if (BufPtr -> data . ipr_dmask_v_counter)
                     if (BufPtr -> dataquad . ipr_dmask_v_counter)
                        *DataQuad++ = Context->counter;

#if (!(IPR_DMASK_M_PID == (IPR_DMASK_M_COUNTER << 1)))
#  error "assertion failed (IPR_DMASK_M_PID == (IPR_DMASK_M_COUNTER << 1))"
#endif
                  if (BufPtr -> data . ipr_dmask_v_pid)
                     if (BufPtr -> dataquad . ipr_dmask_v_pid)
                        *DataQuad++ = (uint64) Context->pid;

#if (!(IPR_DMASK_M_PRIORITY == (IPR_DMASK_M_PID << 1)))
#  error "assertion failed (IPR_DMASK_M_PRIORITY == (IPR_DMASK_M_PID << 1))"
#endif
                  if (BufPtr -> data . ipr_dmask_v_priority)
                     if (BufPtr -> dataquad . ipr_dmask_v_priority)
                        *DataQuad++ = (uint64) Context->priority;
/*
#if (!(IPR_DMASK_M_MAXIMUM == (IPR_DMASK_H_PRIORITY << 1)))
#  error "assertion failed (IPR_DMASK_M_MAXIMUM == (IPR_DMASK_M_PRIORITY << 1))"
#endif
*/
// Check the longword masks

                  DataLong = (int *) DataQuad;

#ifdef TRCDPC
                  if ((DeviceExtension->intCntE % 1000) == 0)
                  {
                     TrcPrint("Saving int part of sample at %x\n",
                      DataLong);
                  }
#endif

#if (!(IPR_DMASK_M_TIME == 1))
#  error "assertion failed (IPR_DMASK_M_TIME == 1)"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_time) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_time)))
                      *DataLong++ = (int) Context->time;

#if (!(IPR_DMASK_M_PS == (IPR_DMASK_M_TIME << 1)))
#  error "assertion failed (IPR_DMASK_M_PS == (IPR_DMASK_M_TIME << 1))"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_ps) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_ps)))
                      *DataLong++ = (int) Context->ps;

#if (!(IPR_DMASK_M_PC == (IPR_DMASK_M_PS << 1)))
#  error "assertion failed (IPR_DMASK_M_PC == (IPR_DMASK_M_PS << 1))"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_pc) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_pc)))
                      *DataLong++ = (int) Context->pc;

#if (!(IPR_DMASK_M_COUNTER == (IPR_DMASK_M_PC << 1)))
#  error "assertion failed (IPR_DMASK_M_COUNTER == (IPR_DMASK_M_PC << 1))"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_counter) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_counter)))
                      *DataLong++ = (int) Context->counter;

#if (!(IPR_DMASK_M_PID == (IPR_DMASK_M_COUNTER << 1)))
#  error "assertion failed (IPR_DMASK_M_PID == (IPR_DMASK_M_COUNTER << 1))"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_pid) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_pid)))
                      *DataLong++ = (int) Context->pid;

#if (!(IPR_DMASK_M_PRIORITY == (IPR_DMASK_M_PID << 1)))
#  error "assertion failed (IPR_DMASK_M_PRIORITY == (IPR_DMASK_M_PID << 1))"
#endif
                  if ((BufPtr -> data . ipr_dmask_v_priority) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_priority)))
                      *DataLong++ = (int) Context->priority;
/*
#if (!(IPR_DMASK_M_MAXIMUM == (IPR_DMASK_H_PRIORITY << 1)))
#  error "assertion failed (IPR_DMASK_M_MAXIMUM == (IPR_DMASK_M_PRIORITY << 1))"
#endif
*/
                  //
                  // Note: In the following section, the casts must be signed in order
                  // for the the "&" operation to work with OpenVMS addresses (the
                  // mask needs to be sign extended).
                  //

                  dummy = ((char *) DataLong + BufPtr->pad
                    - (char *) (BufPtr + 1));
                  // check to see if there is room in the buffer for another sample
                  BufPtr -> areaoffset = dummy;

#ifdef TRCDPC
                  if ((DeviceExtension->intCntE % 250) == 0)
                  {
                     TrcPrint("Done saving sample, bufptr %x, startsmp %x, DataQ %x, DataI %x,new aoff is %x, bmark %d, samp cnt S/E %Lx %Lx\n",
                        BufPtr,BufPtr+1,DataQuad,DataLong,BufPtr->areaoffset,marked_buffer,
                        DeviceExtension->intCntE,DeviceExtension->intCntE);
                  }
#endif
               }  // end - dropSamp check
            }  // end - PCinRange
         }  // end - SkipInt
      }  // end - sample code
      DeviceExtension->intCntE++;
      BARRIER();        // Do a MB to make sure the end count is set
      prevContext = NULL;
      Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
      Context += DeviceExtension->intCntE % IPR_DPC_BLOCKS;
//      Context = (IPR_DpcBlock *)ExInterlockedRemoveHeadList(&DeviceExtension->sampQ,
//          &DeviceExtension->sampSpin);
   }  // end while

#ifdef TRCDPC
   if ((DeviceExtension->intCntE % 1000) == 0)
   {
       TrcPrint("AXP 21064 - DPC exit\n" );
   }
#endif
   return;

}

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    )
{
/*++

Routine Description:

    This routine initializes the IPROBE API kernel mode routines and the
    Axp 21064 performance counter driver.

Arguments:

    DriverObject - Pointer to driver object created by system.

    RegistryPath - Pointer to the Unicode name of the registry path
        for this driver.

Return Value:

    The function value is the final status from the initialization operation.

--*/

    int cnt;
    IPR_BUFHDR *alloc_area;
    POOL_TYPE npp_type = NonPagedPool;

    ULONG wrtAccess = {THREAD_ALL_ACCESS};
    EVENT_TYPE wrtType = {NotificationEvent};
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT deviceObject;
    IPR_STATUS status;
    UNICODE_STRING unicodeString;

    KAFFINITY Affinity,affinISR;
    PDEVICE_OBJECT deviceObject0;
    PDEVICE_OBJECT deviceObject1;
    KIRQL Irql;
    ULONG Vector;

    HANDLE wrtHand;     // Handle of write thread

    WCHAR DeviceNameBuffer[80];
    ULONG ProcessorNumber,MaxProcessors;
    unsigned int ucnt;

#if defined (DBGPID) || defined (DBGD) || defined (TRCE)
    ANSI_STRING handNameANSI;
#endif
#if defined (DBGD) || defined (TRCW)
    KPCR *pcrBlk;
#endif

#ifdef TRCI
    TrcSetup();
    TrcPrint("IprKrnl driver loading...\n");
#endif
#ifdef TRC0
    TrcPrint("IprKrnl Iprobe API kernel: enter\n");
#endif

    // Do basic driver initialization such as finding the allocation
    // granularity and creating the NPP zones and list.

    // Affinity fields of sysdata will be set later after 1st Interrupt is connected...
    sysdata.allocation_granularity = 64;
#ifdef DBGD
    pcrBlk = GetProcessorPCR();
    DbgPrint("PcrBlk %x, SetMem %x, ProcNum %x, Prcb %x\n",
       pcrBlk,pcrBlk->SetMember,(int)pcrBlk->Number,
       pcrBlk->Prcb);
//    DbgBreakPoint();
#endif
#ifdef TRCW
    pcrBlk = GetProcessorPCR();
    TrcPrint("PcrBlk %x, SetMem %x, ProcNum %x, Prcb %x\n",
       pcrBlk,pcrBlk->SetMember,(int)pcrBlk->Number,
       pcrBlk->Prcb);
#endif

    // Initialize nonpaged pool list

    for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
      npp_table.start[cnt] = NULL;

    //
    // Create nonpaged pool zones for nonpaged data structures
    //
    for (cnt=0; cnt < IPR_NUM_ZONES; cnt++)
    {

       alloc_area = ExAllocatePool(npp_type, zinfo[cnt].size);
       if (alloc_area == NULL)
       {
#ifdef TRCE
           TrcPrint("DriverEntry Pool allocation for zone %d failed\n",cnt);
#endif
           return IPR__ALLOCFAIL;
       }
       else
       {
           // Update NPP list to insure that all allocated pool is returned
           status = AddToPoolList(alloc_area);
           if (!IPR_SUCCESS(status))
           {
#ifdef TRCE
               TrcPrint("DriverEntry NPP list update failed, zone= %d, Status=%x\n",
                  cnt,status);
#endif
               return status;
           }
           zinfo[cnt].start = alloc_area;
#ifdef TRCP
           TrcPrint("Pool allocation for zone successful, zone=%d, %d bytes at %x\n",
              cnt,zinfo[cnt].size,alloc_area);
#endif
#ifdef DBGP
           DbgPrint("Pool allocation for zone successful, zone=%d, %d bytes at %x\n",
              cnt,zinfo[cnt].size,alloc_area);
//           DbgBreakPoint();
#endif
           status = ExInitializeZone(&zhdr[cnt],
               zinfo[cnt].granularity,
               alloc_area,
               zinfo[cnt].size);
           if (!OK_STATUS(status))
           {
#ifdef TRCE
               TrcPrint("DriverEntry Zone init failed, zone= %d, Status=%x\n",
                  cnt,status);
#endif
               return IPR__ZONEFAIL;
           }
#ifdef TRCD
           TrcPrint("Zone init. successful, zone= %d, blk size=%d, tot seg size=%d\n",
             cnt,zhdr[cnt].BlockSize,zhdr[cnt].TotalSegmentSize);
#endif
       }
    }

    // Create and initialize spinlocks, queues and events that will be
    // used by this driver.  This is done first because pointers to these
    // structures are included in the device extension so we can just pop
    // in the addresses as the devices are created.  All device extensions
    // point to the write structure (the counter device extensions need
    // this info. to signal buffer full conditions and the write thread
    // is setup based on the IPRKRNL device extension).

    status = allocate_memory(sizeof(IPR_WrtStruct),&write_template,&wrtStruct);
    if (!IPR_SUCCESS(status))
    {
#ifdef TRCE
        TrcPrint("DriverEntry: Failure allocating memory for the write structure, status = %x\n",
                   status );
        return status;
#endif
    }
    // Initialize spinlock for write queue interlocked operations.
    KeInitializeSpinLock(&wrtStruct->wrtQ_SpinLock);

    // Create queue for write buffers.
    InitializeListHead(&wrtStruct->wrtQ);

    // Create event that the write thread will use to wait for
    // writing buffers.  The initial state is FALSE - not signaled
    KeInitializeEvent(&wrtStruct->wrtEvent,wrtType,FALSE);
    KeInitializeEvent(&wrtStruct->wrtdEvent,wrtType,FALSE);
    KeInitializeEvent(&wrtStruct->wrtoEvent,wrtType,FALSE);

    // Write thread is created after 1st device object is created
    // (the device extension is passed to the Write thread)

    //
    // Create exclusive device object for the Iprobe API kernel
    // routines to use.
    //

    RtlInitUnicodeString(&unicodeString, DD_IPROBE_API_DEVICE_NAME_U);

    status = IoCreateDevice(
                DriverObject,           // driver object passed into this routine
                sizeof(DEVICE_EXTENSION), // driver defined extension size
                &unicodeString,         // Device named object
                FILE_DEVICE_IPRKERNEL,  // Driver defined device type
                0,                      // Device Characteristics
                TRUE,                   // Exclusive access
                &deviceObject           // Device object created
                );

    if (status != STATUS_SUCCESS)
    {
#ifdef TRCE
        TrcPrint("DriverEntry: Could not create device object, status=%x\n",status);
#endif
#ifdef DBGD
        DbgPrint("DriverEntry: Could not create device object, status=%x\n",status);
        DbgBreakPoint();
#endif
        return(status);
    }

    deviceObject->Flags |= DO_BUFFERED_IO;

    //
    // Fill in the device extension.
    //

    deviceExtension =
        (PDEVICE_EXTENSION)deviceObject->DeviceExtension;

    deviceExtension -> product_ident = IPR_PROD_IDENT;
    deviceExtension -> majorVersion = IPR_KRNL_MAJOR_VER;
    deviceExtension -> minorVersion = IPR_KRNL_MINOR_VER;
    deviceExtension -> flags = 0;
    deviceExtension -> connected = FALSE;
    deviceExtension->icount = 0;
    deviceExtension->Counter = (AXP21064_PCCOUNTER)0xffffffff;

#ifdef DBGPM
    DbgPrint("IprKrnl device object created, DevObj %x, DevExt %x\n",
       deviceObject,deviceExtension);
#endif

    // API thread (valid cpu range
    // is 0 to IPR_MAX_CPUS-1)

    deviceExtension->wrtStruct = wrtStruct;

    // Create system write thread

    wrtStruct->wrtActive = TRUE;        // Indicate to write system thread that
                                        // it is active.
    status = PsCreateSystemThread(&wrtHand,wrtAccess,NULL,NULL,NULL,
       WriteThread,deviceExtension);
    if( status != STATUS_SUCCESS ){
#ifdef TRCE
        TrcPrint("DriverEntry: Failure creating system thread for writing samples, status = %x\n",
                   status );
#endif
        return status;
    }
#ifdef TRC0
    TrcPrint("System thread for writing samples created using event block at %x\n",
        &wrtStruct->wrtEvent);
#endif

    sysdata.affinity_cpu_mask = 0;
    MaxProcessors = (int)*KeNumberProcessors;
    // Set the system affinity cpu mask based on the number of processors
    for (ucnt = 0; ucnt < MaxProcessors; ucnt++)
       sysdata.affinity_cpu_mask = sysdata.affinity_cpu_mask | (1 << ucnt);
    sysdata.noaffinity_cpu_mask = sysdata.affinity_cpu_mask;

#ifdef DBGD
    DbgPrint("Found %d processors, affinity %x\n",
       MaxProcessors,sysdata.affinity_cpu_mask);
//    DbgBreakPoint();
#endif
#ifdef TRCW
    TrcPrint("DriverEntry creating devices for %d processors, affinity %x\n",
       MaxProcessors,sysdata.affinity_cpu_mask);
#endif

    for (ProcessorNumber = 0; ProcessorNumber < MaxProcessors; ProcessorNumber++)
    {
       //
       // Create exclusive device object for performance counter 0.
       // The device can be opened by only one thread at a time.
       //

       affinISR = (1 << ProcessorNumber);
#ifdef TRC0
       TrcPrint( "DriverEntry Initialize Counter 0\n");
#endif

       swprintf( DeviceNameBuffer, L"%sP%dPC%d",
          DD_AXP21064_PERFORMANCE_DEVICE_NAME_U,
          ProcessorNumber,
          0);

       RtlInitUnicodeString(&unicodeString, DeviceNameBuffer);

       status = IoCreateDevice(
                DriverObject,
                sizeof(DEVICE_EXTENSION),
                &unicodeString,
                FILE_DEVICE_AXP21064,
                0,
                TRUE,
                &deviceObject0
                );

       if (status != STATUS_SUCCESS)
       {
#ifdef TRCE
          TrcPrint(
            "DriverEntry: Could not create device object for counter 0, processor %d, status %x\n",
               ProcessorNumber,status);
          status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
          if (status == STATUS_SUCCESS)
          {
             TrcPrint("Handle that failed to open %s\n",
               handNameANSI.Buffer);
             RtlFreeAnsiString(&handNameANSI);
          }
          else
             TrcPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
               status);
#endif
#ifdef DBGPID
          DbgPrint("DriverEntry: Could not create device object for counter 0, processor %d, status=%x\n",
             ProcessorNumber,status);
          status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
          if (status == STATUS_SUCCESS)
          {
             DbgPrint("Handle that failed to open %s\n",
               handNameANSI.Buffer);
             RtlFreeAnsiString(&handNameANSI);
          }
          else
             DbgPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
               status);
          DbgBreakPoint();
#endif

          return(status);      // may want to provide more robust error handling here - to get rid
                               // devices created for preceding processors (when on an MP)
       }

#ifdef DBGPID
       DbgPrint("DriverEntry: Created device object for counter 0, processor %d, devObj %x, devExt %x, affin %x, status=%x\n",
          ProcessorNumber,deviceObject0,deviceObject0->DeviceExtension,affinISR,status);
       status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
       if (status == STATUS_SUCCESS)
       {
          DbgPrint("Handle open named %s\n",
            handNameANSI.Buffer);
          RtlFreeAnsiString(&handNameANSI);
       }
       else
          DbgPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
            status);
#endif

       deviceObject0->Flags |= DO_BUFFERED_IO;

       //
       // Fill in the device extension.
       //

       deviceExtension =
          (PDEVICE_EXTENSION)deviceObject0->DeviceExtension;
       deviceExtension->DeviceObject = deviceObject0;

       // Initialize spinlock for sample write interlocked operations.
       KeInitializeSpinLock(&deviceExtension->sampSpin);
       // Create queue for DPC buffers.
       InitializeListHead(&deviceExtension->sampQ);

       status = allocate_memory(sizeof(IPR_DpcBlock)*IPR_DPC_BLOCKS,NULL,&deviceExtension->dpcBlock);
       if (!IPR_SUCCESS(status))
       {
#ifdef TRCE
           TrcPrint("AXP 21064: Failure allocating memory for the dpc sample buffer structures, status = %x\n",
                   status );
           return status;
#endif
       }
#ifdef DBGPID
       DbgPrint("Context area allocated at %x of size %d, for cpu %d, counter 0\n",
          deviceExtension->dpcBlock,sizeof(IPR_DpcBlock)*IPR_DPC_BLOCKS,ProcessorNumber);
#endif
       IoInitializeDpcRequest(deviceObject0,DpcISR);

       deviceExtension -> product_ident = IPR_PROD_IDENT;
       deviceExtension -> majorVersion = IPR_KRNL_MAJOR_VER;
       deviceExtension -> minorVersion = IPR_KRNL_MINOR_VER;
       deviceExtension -> flags = 0;
       deviceExtension -> connected = FALSE;
       deviceExtension -> cpu = ProcessorNumber;
       deviceExtension->icount = 0;

       deviceExtension->CounterValue = 0;
       deviceExtension->Counter = PerformanceCounter0;
       deviceExtension->MuxControl = ExternalCounter0;
       deviceExtension->EventCount = CountEvents2xx16;

       deviceExtension->wrtStruct = wrtStruct;
       deviceExtension->intCntS = 0;
       deviceExtension->intCntE = 0;
       deviceExtension->dropInt = 0;
       skipHist = 0;
       addHist = 0;

       // Initialize spinlock for ISR coordination
       status = allocate_memory(sizeof(KSPIN_LOCK),NULL,&deviceExtension->ISRSpin);
       if (!IPR_SUCCESS(status))
       {
#ifdef TRCE
           TrcPrint("DriverEntry: Failure allocating memory for the ISR spinlock, status = %x\n",
                   status );
           return status;
#endif
       }
       KeInitializeSpinLock(deviceExtension->ISRSpin);


#ifdef DBGPM
       DbgPrint("IprKrnl Counter 0 device object created, DevObj %x, DevExt %x\n",
          deviceObject0,deviceExtension);
#endif

       //
       // Capture the necessary interrupt parameters from the HAL and then
       // connect the interrupt for PC0.
       //

#if 0
       Vector = HalGetInterruptVector( Internal,
                                    PROCESSOR_BUS_21064,
                                    PerformanceCounter0,
                                    PerformanceCounter0,
                                    &Irql,
                                    &Affinity );
#else
       Vector = HalGetInterruptVector (ProcessorInternal, 0, 0, 0, &Irql, &Affinity);
#endif

#ifdef TRCW
       TrcPrint("Return from HalGetInterruptVector, counter 0, processor %d, Irql %x, Affinity %x, Vector %x\n",
          ProcessorNumber,Irql,Affinity,Vector);
#endif
#ifdef DBGCRSH
       DbgPrint("Return from HalGetInterruptVector, counter 0, processor %d, Irql %x, Affinity %x, Vector %x, affinISR %x\n",
          ProcessorNumber,Irql,Affinity,Vector,affinISR);
#endif

       status = IoConnectInterrupt( &deviceExtension->InterruptObject,
// davew                         (PKSERVICE_ROUTINE)PerformanceCounterInterrupt,
	                    	 (PKSERVICE_ROUTINE)PCInterruptAddFrame, // davew
                                 (PVOID)deviceExtension,
                                 deviceExtension->ISRSpin,
                                 Vector,      // System vector (from HAL)
                                 Irql,        // Irql (from HAL)
                                 Irql,        // Synchronization Irql (same)
                                 Latched,     // Latched (to allow sharing)
                                 TRUE,        // Sharing = TRUE
                                 affinISR,    // Affinity set to this ProcessorNumber only!
                                 FALSE        // Save floating state = FALSE
                               );

       if( status != STATUS_SUCCESS )
       {
#ifdef DBGCRSH
       DbgPrint("DriverEntry: Failure connecting interrupt, status = %x Vector = %x\n",
          status,Vector );
#endif
#ifdef TRCE
          TrcPrint("DriverEntry: Failure connecting interrupt for counter 0, status = %x\n",
                   status );
#endif
          IoDeleteDevice( deviceObject0 );
          return status;
	}

       //
       // Preserve the interrupt information in the device extension so
       // we can disconnect and reconnect easily.
       //

       deviceExtension->Vector = Vector;
       deviceExtension->Irql = Irql;
       deviceExtension->Affinity = affinISR;

       //
       // Now, disconnect the interrupt!  We connect above to make sure that
       // everything is working.  Due to quirks in the way the interrupts are
       // enabled they will only be enabled when the device is opened.
       // Mark the connection count to zero.
       //

       IoDisconnectInterrupt( deviceExtension->InterruptObject );
       deviceExtension->Started = FALSE;

       //
       // Create exclusive device object for performance counter 1.
       // The device can be opened by only one thread at a time.
       //

#ifdef TRC0
       TrcPrint("AXP 21064 Initialize Counter 1\n");
#endif

       swprintf( DeviceNameBuffer, L"%sP%dPC%d",
          DD_AXP21064_PERFORMANCE_DEVICE_NAME_U,
          ProcessorNumber,
          1);

       RtlInitUnicodeString(&unicodeString, DeviceNameBuffer);

       status = IoCreateDevice(
                DriverObject,
                sizeof(DEVICE_EXTENSION),
                &unicodeString,
                FILE_DEVICE_AXP21064,
                0,
                TRUE,
                &deviceObject1
                );

       if (status != STATUS_SUCCESS)
       {

#ifdef TRCE
          TrcPrint(
            "DriverEntry: Could not create device object for counter 1, processor %d, status %x\n",
               ProcessorNumber,status);
          status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
          if (status == STATUS_SUCCESS)
          {
             TrcPrint("Handle that failed to open %s\n",
               handNameANSI.Buffer);
             RtlFreeAnsiString(&handNameANSI);
          }
          else
             TrcPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
               status);
#endif
#ifdef DBGD
          DbgPrint("DriverEntry: Could not create device object for counter 1, processor %d, status=%x\n",
             ProcessorNumber,status);
          status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
          if (status == STATUS_SUCCESS)
          {
             DbgPrint("Handle that failed to open %s\n",
               handNameANSI.Buffer);
             RtlFreeAnsiString(&handNameANSI);
          }
          else
             DbgPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
               status);
          DbgBreakPoint();
#endif

          return(status);      // may want to provide more robust error handling here - to get rid
                               // devices created for preceding processors (when on an MP)
       }

#ifdef DBGPID
       DbgPrint("DriverEntry: Created device object for counter 1, processor %d, devObj %x, devExt %x, affinISR, status=%x\n",
          ProcessorNumber,deviceObject1,deviceObject1->DeviceExtension,affinISR,status);
       status = RtlUnicodeStringToAnsiString(&handNameANSI,&unicodeString,TRUE);
       if (status == STATUS_SUCCESS)
       {
          DbgPrint("Handle open named %s\n",
            handNameANSI.Buffer);
          RtlFreeAnsiString(&handNameANSI);
       }
       else
          DbgPrint("Conversion of handle name from UNICODE to ANSI failed, status %x\n",
            status);
#endif

       deviceObject1->Flags |= DO_BUFFERED_IO;

       //
       // Fill in the device extension.
       //

       deviceExtension =
        (PDEVICE_EXTENSION)deviceObject1->DeviceExtension;
       deviceExtension->DeviceObject = deviceObject1;

       // Initialize spinlock for sample write interlocked operations.
       KeInitializeSpinLock(&deviceExtension->sampSpin);
       // Create queue for DPC buffers.
       InitializeListHead(&deviceExtension->sampQ);

       status = allocate_memory(sizeof(IPR_DpcBlock)*IPR_DPC_BLOCKS,NULL,&deviceExtension->dpcBlock);
       if (!IPR_SUCCESS(status))
       {
#ifdef TRCE
          TrcPrint("DriverEntry: Failure allocating memory for the dpc sample buffer structures, status = %x\n",
                   status );
          return status;
#endif
       }
#ifdef DBGPID
       DbgPrint("Context area allocated at %x of size %d, for cpu %d, counter 1\n",
          deviceExtension->dpcBlock,sizeof(IPR_DpcBlock)*IPR_DPC_BLOCKS,ProcessorNumber);
#endif
       IoInitializeDpcRequest(deviceObject1,DpcISR);

       deviceExtension -> product_ident = IPR_PROD_IDENT;
       deviceExtension -> majorVersion = IPR_KRNL_MAJOR_VER;
       deviceExtension -> minorVersion = IPR_KRNL_MINOR_VER;
       deviceExtension -> flags = 0;
       deviceExtension -> connected = FALSE;
       deviceExtension -> cpu = ProcessorNumber;
       deviceExtension->icount = 0;

       deviceExtension->CounterValue = 0;
       deviceExtension->Counter = PerformanceCounter1;
       deviceExtension->MuxControl = ExternalCounter1;
       deviceExtension->EventCount = CountEvents2xx12;

       deviceExtension->wrtStruct = wrtStruct;
       deviceExtension->intCntS = 0;
       deviceExtension->intCntE = 0;
       deviceExtension->dropInt = 0;

#ifdef DBGPM
       DbgPrint("IprKrnl device object for counter 1, DevObj %x, DevExt %x\n",
          deviceObject1,deviceExtension);
#endif

       //
       //
       // Capture the necessary interrupt parameters from the HAL and then
       // connect the interrupt for PC1.
       //

#if 0
       Vector = HalGetInterruptVector( Internal,
                                    PROCESSOR_BUS_21064,
                                    PerformanceCounter1,
                                    PerformanceCounter1,
                                    &Irql,
                                    &Affinity );
#else
       Vector = HalGetInterruptVector (ProcessorInternal, 0, 1, 1, &Irql, &Affinity);
#endif

#ifdef TRCW
       TrcPrint("Return from HalGetInterruptVector, counter 1, processor %d, Irql %x, Affinity %x, Vector %x\n",
          ProcessorNumber,Irql,Affinity,Vector);
#endif
#ifdef DBGCRSH
       DbgPrint("Return from HalGetInterruptVector, counter 1, processor %d, Irql %x, Affinity %x, Vector %x, affinISR %x\n",
          ProcessorNumber,Irql,Affinity,Vector,affinISR);
#endif
       status = IoConnectInterrupt( &deviceExtension->InterruptObject,
// davew                         (PKSERVICE_ROUTINE)PerformanceCounterInterrupt,
	                    	 (PKSERVICE_ROUTINE)PCInterruptAddFrame, // davew
                                 (PVOID)deviceExtension,
                                 deviceExtension->ISRSpin,
                                 Vector,      // System vector (from HAL)
                                 Irql,        // Irql (from HAL)
                                 Irql,        // Synchronization Irql (same)
                                 Latched,     // Latched (to allow sharing)
                                 TRUE,        // Sharing = TRUE
                                 affinISR,    // Affinity set to this ProcessorNumber only!
                                 FALSE        // Save floating state = FALSE
                               );

       if( status != STATUS_SUCCESS ){
#ifdef DBGCRSH
       DbgPrint("DriverEntry: Failure connecting interrupt for counter 1, status = %x Vector = %x\n",
          status,Vector );
#endif
#ifdef TRCE
           TrcPrint("AXP 21064: Failure connecting interrupt, status = %x\n",
                status );
#endif
           IoDeleteDevice( deviceObject0 );
           IoDeleteDevice( deviceObject1 );
	   return status;
       }

       //
       // Preserve the interrupt information in the device extension so
       // we can disconnect and reconnect easily.
       //

       deviceExtension->Vector = Vector;
       deviceExtension->Irql = Irql;
       deviceExtension->Affinity = affinISR;

       //
       //
       // Now, disconnect the interrupt!  We connect above to make sure that
       // everything is working.  Due to quirks in the way the interrupts are
       // enabled they will only be enabled when the device is opened.
       // Mark the connection count to zero.
       //

       IoDisconnectInterrupt( deviceExtension->InterruptObject );
       deviceExtension->Started = FALSE;

    }  // end of loop that sets up handles for each Processor

    //
    // Set up the device driver entry points.
    //

    DriverObject->DriverUnload = IprUnload;
    DriverObject->MajorFunction[IRP_MJ_CREATE] = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_CLOSE]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_READ]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_WRITE]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] =
                                             IprDeviceControl;
#ifdef TRC0
    TrcPrint("IprKrnl, Initialize exit...\n");
#endif
#ifdef TRCI
    TrcPrint("IprKrnl driver loaded\n");
#endif

    return(STATUS_SUCCESS);

}

NTSTATUS
IprDeviceControl(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )

/*++

Routine Description:

    This routine is the dispatch routine for device control requests.

    The following device controls are recognized for the performance counter
    part of the driver as defined by Joe Notarangelo:

    IOCTL_AXP21064_RDPALCOUNTERS - Read PALcode Event Counters
    IOCTL_AXP21064_SETCOUNTER - Set events counted per pc interrupt
    IOCTL_AXP21064_SETMUXCONTROL - Set mux control for performance counter
    IOCTL_AXP21064_STARTCOUNTER - Set the counter started (start interrupting)
    IOCTL_AXP21064_STOPCOUNTER - Set the counter stopped (stop interrupting)

    The following device controls are recognized for the IPROBE API part of
    the performance counter driver:

    IOCTL_IPR_GetChipInfo - Return chip info
    IOCTL_IPR_UpdatePERFMON - Change PERFMON settings
    IOCTL_IPR_GetCurrentIpid - Return current internal PID
    IOCTL_IPR_SetAffinity - Update processor affinity
    IOCTL_IPR_SetPme - Update PME bit
    IOCTL_IPR_ResetIpl - Execute PALcode IPL code path (to propagate PME bit)
    IOCTL_IPR_InitSysInfo - Initialize system info. in globals
    IOCTL_IPR_DoneSysInfo - Dealloctes system info. such as nonpaged pool
    IOCTL_IPR_OpenSampFile - Open sample file
    IOCTL_IPR_CloseSampfile - Close sample file
    IOCTL_IPR_WriteFile - Write a buffer to the sample file

    IOCTL_IPR_BdCreate - Buffer descriptor create
    IOCTL_IPR_BdDelete - Buffer descriptor delete
    IOCTL_IPR_BdSetNumCounters - Set number of perf. counters
    IOCTL_IPR_BdSetAction - Setup action to occur at particular time
    IOCTL_IPR_BdSetActiveCounters - Set active counter mask
    IOCTL_IPR_BdAddPcRange - Add a PC range
    IOCTL_IPR_BdSetSkipCount - Set number of interrupts to skip between samples

    IOCTL_IPR_SbCreate - Sample buffer create
    IOCTL_IPR_SbDelete - Sample buffer delete
    IOCTL_IPR_SbSetFull - Set full bit for sample buffer
    IOCTL_IPR_SbGetInfo - Returns info. about the sample buffer
    IOCTL_IPR_SbFind - Returns pointer to sample buffer

    IOCTL_IPR_HbElements - Returns number of elements required for given histogram
       parameters
    IOCTL_IPR_HbIndex - Returns array index for the specified value and counter for
       given histogram parameters
    IOCTL_IPR_HbValue - Return value at given histogram index
    IOCTL_IPR_HbMaxElements - Return maximum number of histogram elements
    IOCTL_IPR_HbCreate - Create histogram buffer
    IOCTL_IPR_HbDelete - Delete histogram buffer
    IOCTL_IPR_HbFind - Return next histogram buffer that matches given criteria
    IOCTL_IPR_HbRead -  Read given elements from histogram buffer
    IOCTL_IPR_HbClear - Clear given elements in histogram buffer

Arguments:

    DeviceObject - Pointer to class device object.

    Irp - Pointer to the request packet.

Return Value:

    Status is returned.

--*/
{
    PDEVICE_EXTENSION deviceExtension;
    PAXP21064_SETCOUNTER EventCountParameters;
    PIO_STACK_LOCATION irpSp;
    PAXP21064_SETMUXCONTROL MuxControlParameters;
    NTSTATUS status;

#ifdef TRC0
    TrcPrint("\nIprKrnl, IprDeviceControl enter...\n");
#endif

    //
    // Get a pointer to the current parameters for this request.  The
    // information is contained in the current stack location.
    //

    irpSp = IoGetCurrentIrpStackLocation(Irp);

#ifdef TRCD
    TrcPrint("Input len= %x, Input ptr= %x\n",
     irpSp->Parameters.DeviceIoControl.InputBufferLength,Irp->AssociatedIrp.SystemBuffer);
    TrcPrint("Output len= %x, Output ptr= %x\n",
     irpSp->Parameters.DeviceIoControl.OutputBufferLength,Irp->UserBuffer);
#endif
    //
    // Case on the device control subfunction that is being performed by the
    // requestor.
    //

    switch (irpSp->Parameters.DeviceIoControl.IoControlCode) {

// NOTE:  For a performance optimization, the data passed in to the
// various dispatched routines could be sent directly to the function.
// Currently the data is moved to variable of types the function expects
// to improve the readability of the code and provide additional
// error checks.

// The following 5 cases were pulled out of the PERFEV4 driver

    //
    // Read the PALcode event counters, if possible.  It is possible that
    // the executing PALcode does not have the counters implemented.
    //

    case IOCTL_AXP21064_RDPALCOUNTERS:

#ifdef TRCD
        TrcPrint("IprKrnl AXP21064 RDPALCOUNTERS ioctl\n");
#endif
        if( irpSp->Parameters.DeviceIoControl.OutputBufferLength <
            sizeof(AXP21064_RDPALCOUNTERS) ){
            status = IPR__INVPARAM;
        } else {
            status = Axp21064ReadPalCounters(
                      (PAXP21064_RDPALCOUNTERS)Irp->AssociatedIrp.SystemBuffer);
        }

        if( status == STATUS_SUCCESS ){
            Irp->IoStatus.Information = sizeof(AXP21064_RDPALCOUNTERS);
        } else {
        Irp->IoStatus.Information = 0;
        }
        break;

    //
    // Set the number of events per interrupt (a.k.a. event count).
    //

    case IOCTL_AXP21064_SETCOUNTER:

#ifdef TRCD
        TrcPrint("IprKrnl AXP21064 SETCOUNTER ioctl\n");
#endif
        deviceExtension = DeviceObject->DeviceExtension;
        EventCountParameters =
                (PAXP21064_SETCOUNTER)Irp->AssociatedIrp.SystemBuffer;

        if( irpSp->Parameters.DeviceIoControl.InputBufferLength !=
            sizeof(AXP21064_SETCOUNTER) )
        {
            status = IPR__INVPARAM;
        } else {
#ifdef TRCD
            TrcPrint("EventCntParm %x, counter %d\n",
                EventCountParameters,deviceExtension->Counter);
#endif
            if( ValidateEventCount( EventCountParameters,
                                    deviceExtension->Counter) == FALSE )
            {
                status = IPR__INVALID_FREQUENCY;
            } else {
                status = SetEventCount( deviceExtension,
                                    EventCountParameters );
            }
        }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Set the mux control for the performance counter, the mux control
    // defines the type of event that is counted
    //

    case IOCTL_AXP21064_SETMUXCONTROL:

#ifdef TRCD
        TrcPrint("IprKrnl AXP21064 SETMUXCONTROL ioctl\n");
#endif
        deviceExtension = DeviceObject->DeviceExtension;
        MuxControlParameters =
                (PAXP21064_SETMUXCONTROL)Irp->AssociatedIrp.SystemBuffer;

        if( irpSp->Parameters.DeviceIoControl.InputBufferLength !=
            sizeof(AXP21064_SETMUXCONTROL) ){
            status = IPR__INVPARAM;
        } else
            if( ValidateMuxControl( MuxControlParameters,
                                    deviceExtension->Counter) == FALSE ) {
                status = IPR__INVALID_EVENT;
        } else {
            status = SetMuxControl( deviceExtension,
                                    MuxControlParameters );
        }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Start the counter, if the counter is already started then
    // consider the command a success.
    //

    case IOCTL_AXP21064_STARTCOUNTER:

#ifdef DBG2
   DbgPrint("STARTCOUNTER action entered\n\n");
   DbgBreakPoint();
#endif

#ifdef TRCD
        TrcPrint("IprKrnl AXP21064 STARTCOUNTER ioctl\n");
#endif
        deviceExtension = DeviceObject->DeviceExtension;
#ifdef DBGDRV
        DbgPrint("DevObj %x, DevExt %x, counter %d, start %d, end %d, drop %d\n",
           DeviceObject,deviceExtension,deviceExtension->Counter,
           deviceExtension->intCntS,deviceExtension->intCntE,
           deviceExtension->dropInt);
#endif
        if( deviceExtension->Started == TRUE ){
            status = STATUS_SUCCESS;
        } else {
            // Reset the interrupt/DPC counts
            deviceExtension->intCntS = 0;
            deviceExtension->intCntE = 0;
            deviceExtension->dropInt = 0;
            status = StartCounter( deviceExtension );
        }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Stop the counter, if the counter is already stopped then
    // consider the command a success.
    //

    case IOCTL_AXP21064_STOPCOUNTER:
    {

#ifdef TRCD
        TrcPrint("IprKrnl AXP21064 STOPCOUNTER ioctl\n");
#endif
        deviceExtension = DeviceObject->DeviceExtension;
        if( deviceExtension->Started != TRUE ){
            status = STATUS_SUCCESS;
        } else {
            status = StopCounter( deviceExtension );
        }

#ifdef TRCD
/* Temporarily commented out (Rajan Narayanaswamy 11-Jan-96)
        TrcPrint("Temps: cpu %d, stackLoc %x, icount %d, IntObj %x, BDptr %x\n",
           deviceExtension->cpu,deviceExtension->stackLoc,
           deviceExtension->icount,
           deviceExtension->InterruptObj,
           deviceExtension->bdptr);
        TrcPrint("Fir %Lx Psr %x, oldIrql %x pMode %x\n",
           deviceExtension->Fir,
           deviceExtension->Psr,
           deviceExtension->OldIrql,
           deviceExtension->PrevMode);
*/
#endif
        Irp->IoStatus.Information = 0;
        break;
     }
//  All the rest of the cases are defined for the IPROBE API

//
// General routine shells
//

    case IOCTL_IPR_GetChipInfo:
    {
        int alen,olen;
        IPR_CHIPINFO *retptr;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API GetChipInfo ioctl\n");
#endif
        // OutputBuffer and length is structure to put chip info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_CHIPINFO)) ||
                ( olen != sizeof(IPR_CHIPINFO)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            retptr = Irp->AssociatedIrp.SystemBuffer;   // Input buffer
#ifdef TRCD
            TrcPrint("retptr=%x \n",
                retptr);
#endif
            status = KIprGetChipInfo(retptr);
            Irp->IoStatus.Information = alen;
#ifdef TRCD
            TrcPrint("retptr=%x, majorIdent=%x, minorIdent=%x\n",
                retptr,retptr->majorIdent,retptr->minorIdent);
#endif
        }
        break;
    }
    case IOCTL_IPR_UpdatePerfmon:
    {
        int alen,olen;
        IPR_ARGUPerfmon *palist;
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API UpdatePerfmon ioctl\n");
        TrcPrint("\n\n*** E R R O R  This should not be called ***\n\n");
#endif
        // OutputBuffer is used to contain Ipid
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGUPerfmon)) ||
                (olen != sizeof(IPR_ARGUPerfmon)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
#ifdef TRCD
            TrcPrint("cpu= %d, numctrsupd= %d, *counterblock= %x\n",
               palist->cpu,palist->numctrsupd,palist->counterblock);
#endif
            status = KIprUpdatePerfmon(palist->cpu,palist->numctrsupd,
                palist->counterblock);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_GetCurrentIpid:
    {
        int *ipid,alen,olen;

    // NOTE:  Currently, this dispatch should not occur - the ipid is
    // obtained via a user mode call in the shell
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API GetCurrentIpid ioctl\n");
#endif
        // OutputBuffer is used to contain Ipid
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(int)) ||
                (olen != sizeof(int)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            ipid = (int *) Irp->AssociatedIrp.SystemBuffer;
            status = KIprGetCurrentIpid(ipid);
#ifdef TRCD
            TrcPrint("ipid= %x, *ipid= %x\n",
               ipid,*ipid);
#endif
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SetAffinity:
    {
        int alen,olen;
        IPR_ARGAffinity *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SetAffinity ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGAffinity)) ||
           ( olen != sizeof(IPR_ARGAffinity)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
#ifdef TRCD
            TrcPrint("cpu= %x, setting= %x\n",
               palist->cpu,palist->setting);
#endif
            status = KIprSetAffinity(palist->cpu,palist->setting);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SetPme:
    {
        int alen,olen;
        IPR_ARGPme *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SetPme ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGPme)) ||
           ( olen != sizeof(IPR_ARGPme)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
#ifdef TRCD
            TrcPrint("pid= %x, state= %x, prev_state= %x\n",
               palist->pid,palist->state,palist->prev_state);
#endif
            status = KIprSetPme(palist->pid,
               palist->state,
               palist->children,
               palist->prev_state);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_ResetIpl:
    {
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API ResetIpl ioctl\n");
#endif
        status = KIprResetIpl();

        Irp->IoStatus.Information = 0;
        break;

    case IOCTL_IPR_InitSysInfo:
    {
        int alen,olen;
        IPR_SYSTEM_DATA *newsysdata;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API InitSysInfo ioctl\n");
#endif
        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_SYSTEM_DATA)) ||
           ( olen != sizeof(IPR_SYSTEM_DATA)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           newsysdata = Irp->AssociatedIrp.SystemBuffer;
#ifdef TRCD
           TrcPrint("newsysdata=%x \n",
                newsysdata);
#endif
           status = KIprInitSysInfo(newsysdata);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_SYSTEM_DATA);
#ifdef TRCD
              TrcPrint("newsysdata= %x, cpu mask= %x, alloc. gran= %x\n",
                newsysdata,newsysdata->affinity_cpu_mask,
                newsysdata->allocation_granularity);
#endif
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }

    case IOCTL_IPR_DoneSysInfo:
    {

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API DoneSysInfo ioctl\n");
#endif
        status = KIprDoneSysInfo();

        Irp->IoStatus.Information = 0;
        break;
    }

    case IOCTL_IPR_OpenSampFile:
    {
        int alen,olen;
        IPR_ARGOpenSamp *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API OpenSampFile ioctl\n");
#endif
        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGOpenSamp)) ||
           ( olen != sizeof(IPR_ARGOpenSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;
           status = KIprOpenSampFile(palist->fname,
                palist->defname);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_ARGOpenSamp);
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }
    case IOCTL_IPR_CloseSampFile:
    {
        int alen,olen;
        IPR_ARGCloseSamp *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API CloseSampFile ioctl\n");
#endif
        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGCloseSamp)) ||
           ( olen != sizeof(IPR_ARGCloseSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;
           status = KIprCloseSampFile(palist->buffwrites,palist->partwrites);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = alen;
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }
    case IOCTL_IPR_WriteFile:
    {
        int alen,olen;
        IPR_ARGWriteSamp *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API WriteFile ioctl\n");
#endif
        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGWriteSamp)) ||
           ( olen != sizeof(IPR_ARGWriteSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;

           status = KIprWriteFile(palist->bptr,
                palist->bsize);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_ARGWriteSamp);
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }

//
// Buffer descriptor Shells
//
    }
    case IOCTL_IPR_BdCreate:
    {
        int alen,olen;
        IPR_ARGBDCreate *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdCreate ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDCreate)) ||
           ( olen != sizeof(IPR_ARGBDCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
#ifdef TRCD
            TrcPrint("IprKrnl BdCreate cpu=%d, retptr=%x\n",
                palist->cpu,
                palist->retptr);
#endif
            status = KIprBdCreate(palist->cpu,palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdDelete:
    {
        int *cpu,clen,olen;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdDelete ioctl\n");
#endif
        // InputBuffer and Length is cpu

        cpu = Irp->AssociatedIrp.SystemBuffer;
        clen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if( (clen != sizeof(int)) || (olen != sizeof(int)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdDelete(*cpu);
            Irp->IoStatus.Information = clen;
        }
        break;
    }
    case IOCTL_IPR_BdSetNumCounters:
    {
        int alen,olen;
        IPR_ARGBDNumctrs *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdSetNumCounters ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDNumctrs)) ||
           ( olen != sizeof(IPR_ARGBDNumctrs)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
#ifdef TRCD
            TrcPrint("cpu=%d, numctrs=%d\n",
               palist->cpu,palist->numctrs);
#endif
            status = KIprBdSetNumCounters(palist->cpu,palist->numctrs);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdSetAction:
    {
        int alen,olen;
        IPR_ARGBDAction *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdSetAction ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDAction)) ||
           ( olen != sizeof(IPR_ARGBDAction)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdSetAction(palist->cpu,
               palist->action_type,
               palist->action,
               palist->targetroutine,
               palist->targetpid);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_BdSetActiveCounters:
    {
        int alen,olen;
        IPR_ARGBDActctrs *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdSetActiveCounters ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDActctrs)) ||
           ( olen != sizeof(IPR_ARGBDActctrs)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdSetActiveCounters(palist->cpu,palist->ctrmask);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdAddPcRange:
    {
        int alen,olen;
        IPR_ARGBDPcrng *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdAddPcRange ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDPcrng)) ||
           ( olen != sizeof(IPR_ARGBDPcrng)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdAddPcRange(palist->cpu,palist->pcrange);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdSetSkipCount:
    {
        int alen,olen;
        IPR_ARGBDSkip *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API BdSetSkipCount ioctl\n");
#endif
       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDSkip)) ||
           ( olen != sizeof(IPR_ARGBDSkip)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = KIprBdSetSkipCount(palist->cpu,
               palist->ctr,
               palist->skipvalue);
            Irp->IoStatus.Information = alen;
        }
        break;

//
// Sample Buffer shells
//
    }
    case IOCTL_IPR_SbCreate:
    {
        int alen,olen;
        IPR_ARGSBCreate *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SbCreate ioctl\n");
#endif
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBCreate)) ||
           (olen != sizeof(IPR_ARGSBCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbCreate(palist->cpu,
               palist->size,
               palist->numbuffs,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SbDelete:
    {
        int alen,olen;
        IPR_ARGSBDelete *palist;
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SbDelete ioctl\n");
#endif
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBDelete)) ||
           (olen != sizeof(IPR_ARGSBDelete)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbDelete(palist->cpu,palist->bufptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbSetFull:
    {
        int alen,olen;
        IPR_ARGSBSetFull *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SbSetFull ioctl\n");
#endif
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBSetFull)) ||
           (olen != sizeof(IPR_ARGSBSetFull)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbSetFull(palist->bufptr,palist->full_val);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbGetInfo:
    {
        int alen,olen;
        IPR_ARGSBGetInfo *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SbGetInfo ioctl\n");
#endif
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBGetInfo)) ||
           (olen != sizeof(IPR_ARGSBGetInfo)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbGetInfo(palist->bufptr,palist->descrip);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbFind:
    {
        int alen,olen;
        IPR_ARGSBFind *palist;
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API SbFind ioctl\n");
#endif
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBFind)) ||
           (olen != sizeof(IPR_ARGSBFind)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbFind(palist->cpu,
               palist->descrip,
               palist->previous_buffer,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
//
// Histogram Buffer Shells
//

    case IOCTL_IPR_HbElements:
    {
        int alen,olen;
        IPR_ARGHBElements *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbElements ioctl\n");
#endif
       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBElements)) ||
            (olen != sizeof(IPR_ARGHBElements)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = IprHbElements(palist->numctrs,
              palist->numval,
              palist->gran,
              palist->numelem);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbIndex:
    {
        int alen,olen;
        IPR_ARGHBIndex *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbIndex ioctl\n");
#endif
       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBIndex *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBIndex)) ||
           (olen != sizeof(IPR_ARGHBIndex)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = IprHbIndex(palist->value,
               palist->ctr,
               palist->numctrs,
               palist->minval,
               palist->gran,
               palist->index);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbValue:
    {
        int alen,olen;
        IPR_ARGHBValue *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbValue ioctl\n");
#endif
        // InputBuffer and Length is argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBValue *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBValue)) ||
           (olen != sizeof(IPR_ARGHBValue)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = IprHbValue(palist->index,
               palist->numctrs,
               palist->minval,
               palist->gran,
               palist->value);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbCreate:
    {
        int alen,olen;
        IPR_ARGHBCreate *palist;
#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbCreate ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBCreate *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBCreate)) ||
           (olen != sizeof(IPR_ARGHBCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbCreate(palist->cpu,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbDelete:
    {
        int alen,olen;
        IPR_ARGHBDelete *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbDelete ioctl\n");
#endif

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBDelete *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBDelete)) ||
           (olen != sizeof(IPR_ARGHBDelete)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbDelete(palist->cpu, palist->bufptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbFind:
    {
        int alen,olen;
        IPR_ARGHBFind *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbFind ioctl\n");
#endif
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBFind *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBFind)) ||
           (olen != sizeof(IPR_ARGHBFind)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbFind(palist->cpu,
               palist->previous_buffer,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbRead:
    {
        int alen,olen;
        IPR_ARGHBRead *palist;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbRead ioctl\n");
#endif
        // InputBuffer and Length is argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBRead *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBRead)) ||
           (olen != sizeof(IPR_ARGHBRead)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbRead(palist->cpu,
               palist->bufptr,
               palist->start,
               palist->cnt,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbClear:
    {
        IPR_ARGHBClear *palist;
        int alen,olen;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbClear ioctl\n");
#endif
       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBClear)) ||
           (olen != sizeof(IPR_ARGHBClear)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = KIprHbClear(palist->cpu,
               palist->bufptr,
               palist->start,
               palist->cnt);
            Irp->IoStatus.Information = 0;
        }

        break;
    }
    case IOCTL_IPR_HbMaxElements:
    {
        IPR_ARGHBMaxElem *palist;
        int alen,olen;

#ifdef TRCD
        TrcPrint("IprKrnl IPROBE API HbMaxElements ioctl\n");
#endif
       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBMaxElem)) ||
           (olen != sizeof(IPR_ARGHBMaxElem)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbMaxElements(palist->elem);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    //
    // Unrecognized device control request.
    //

    default:

        status = IPR__INVPARAM;
        break;

    }

    //
    // Mark the status and complete the request.
    //

    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#ifdef TRC0
    TrcPrint("IprKrnl IPROBE API DeviceControl: exit, status=%x\n\n",status);
#endif
    return(status);

}

NTSTATUS
IprDispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )
/*++

Routine Description:

    This routine is the dispatch for the IPROBE API kernel routines.
    The routine will field open/close requests as well as reads
    and writes.

Arguments:

    DeviceObject - Pointer to class device object.

    Irp - Pointer to the the I/O request packet.

Return Value:

    The status of the caller's requested operation is returned, generally
    for this driver that status will be success if the request is an
    expected one for the device.

--*/
{
    PDEVICE_EXTENSION deviceExtension;
    ULONG information;
    PIO_STACK_LOCATION irpSp;
    NTSTATUS status;

#ifdef TRC0
    TrcPrint("IprKrnl, IprDispatch enter...\n");
#endif

    //
    // Get the current Irp stack location to retrieve the function codes
    // and parameters.
    //

    irpSp = IoGetCurrentIrpStackLocation( Irp );

    //
    // Perform the requested operation based on the major function.
    //

    switch( irpSp->MajorFunction ){

    //
    // Create/Open the device, start the counter.
    //

    case IRP_MJ_CREATE:

#ifdef TRC0
        TrcPrint( "IprKrnl IPROBE API kernel Dispatch: Create\n" );
#endif
        // Create function here
        deviceExtension = DeviceObject->DeviceExtension;
        deviceExtension->connected = TRUE;
        deviceExtension->CounterValue = 0;
        deviceExtension->icount = 0;
//        StartCounter( deviceExtension );
        status = STATUS_SUCCESS;
        information = 0;
        break;

    //
    // Close the device by disabling the counter.
    //

    case IRP_MJ_CLOSE:

#ifdef TRC0
        TrcPrint( "IprKrnl IPROBE API kernel Dispatch: Close\n" );
#endif
        // Close function here
        deviceExtension = DeviceObject->DeviceExtension;
        deviceExtension->connected = FALSE;
//        StopCounter( DeviceObject->DeviceExtension );
        status = STATUS_SUCCESS;
        information = 0;
        break;

    //
    // Read the counter.
    //

    case IRP_MJ_READ:

#ifdef TRC0
        TrcPrint( "IprKrnl IPROBE API kernel Dispatch: Read\n" );
        TrcPrint( "Read.Length = %d\n", irpSp->Parameters.Read.Length );
#endif
        if( irpSp->Parameters.Read.Length != sizeof(AXPCOUNTER) ){
            status = IPR__INVPARAM;
            information = 0;
        } else {
            deviceExtension = DeviceObject->DeviceExtension;
            *(AXPCOUNTER *)(Irp->AssociatedIrp.SystemBuffer) =
                    deviceExtension->CounterValue;
            status = STATUS_SUCCESS;
            information = sizeof(AXPCOUNTER);
        }
        break;

    //
    // Write the counter.
    //

    case IRP_MJ_WRITE:

#ifdef TRC0
        TrcPrint( "IprKrnl IPROBE API kernel Dispatch: Write\n" );
        TrcPrint( "Write.Length = %d\n", irpSp->Parameters.Write.Length );
#endif
        if( irpSp->Parameters.Write.Length != sizeof(AXPCOUNTER) ){
            status = IPR__INVPARAM;
            information = 0;
        } else {
            deviceExtension = DeviceObject->DeviceExtension;
            deviceExtension->CounterValue =
                    *(AXPCOUNTER *)(Irp->AssociatedIrp.SystemBuffer);
            status = STATUS_SUCCESS;
            information = sizeof(AXPCOUNTER);
        }
        break;

    } //end switch( irpSp->MajorFunction )

    Irp->IoStatus.Status = status;
    Irp->IoStatus.Information = information;
    IoCompleteRequest( Irp, IO_NO_INCREMENT );

#ifdef TRC0
    TrcPrint("IprKrnl IPROBE API kernel Dispatch: exit, status=%x\n",status);
#endif

    return status;

}


VOID
IprUnload(
    IN PDRIVER_OBJECT DriverObject
    )
/*++

Routine Description:

    This routine unloads the Alpha IPROBE API kernel performance counter driver.

Arguments:

    DriverObject - Pointer to class driver object.

Return Value:

    None.

--*/
{
    PDEVICE_OBJECT deviceObject;
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT nextDeviceObject;
    IPR_STATUS status;

    IPR_WrtStruct *wrtStruct;
    int cnt;
    KEVENT unloadWait;
    int64 waitTime;

#ifdef TRC0
    TrcPrint("IprKrnl, IprUnload enter...\n");
#endif

    // Get the device objects and extensions - device objects
    // are chained in the driver object in reverse order from
    // that in which they were created.

    deviceObject = DriverObject->DeviceObject;
    deviceExtension = deviceObject->DeviceExtension;

    // Save the address of the write structure so the spinlock, queues,
    // and events associated with writing can be deleted.

    wrtStruct = deviceExtension->wrtStruct;

    // Disconnect counter interrupts

    while (deviceObject)
    {
       deviceExtension =  deviceObject->DeviceExtension;
       if (deviceExtension->Counter != 0xffffffff)
       {

          //
          // Disconnect the counter interrupt objects for the driver.
          //

          if( deviceExtension->Started == TRUE )
          {
             IoDisconnectInterrupt( deviceExtension->InterruptObject );
          }
       }
       deviceObject = deviceObject->NextDevice;
    }
    // Stop the write thread.  No priority increment and wait
    // after the event is set to allow the system thread to terminate.
    // NOTE: wrtStruct was grabbed above before we got rid of the
    // device extensions.

    if ((!wrtFail) && (wrtStruct))
    {
       if (wrtStruct->wrtActive)
       {
          wrtStruct->wrtActive = FALSE; // Indicate to write thread that
                                        // it is to terminate.
#ifdef DBG10
          DbgPrint("wrtStruct %x, wrtEvent %x\n",
             wrtStruct,&wrtStruct->wrtEvent);
          DbgBreakPoint();
#endif
          wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,TRUE);

          // Wait while system thread runs down - alertable wait with no timeout,
          // The write thread bumps its priority to allow it to run down completely
          // before control is returned here and the address space is trashed.
/*
#if TRCD
          TrcPrint("Waiting for event, timeout is %Lx\n",
             timeOut);
#endif
*/
          status = KeWaitForSingleObject(&wrtStruct->wrtdEvent,
             wrtReason,wrtMode,TRUE,NULL);
//             wrtReason,wrtMode,TRUE,&timeOut);

#ifdef DBG10
          DbgPrint("Return from wait in IprUnload, status = %x\n",
             status);
          DbgBreakPoint();
#endif
          if ((status != STATUS_SUCCESS) && (status != STATUS_ALERTED)
             && (status != STATUS_TIMEOUT))
          {
#ifdef TRCE
             TrcPrint("IprUnload: Wait on write event failed, status=%x\n",status);
#endif
          }
// Wait for 1 sec. to make sure that write thread is completely done
          KeInitializeEvent(&unloadWait,SynchronizationEvent,FALSE);
          waitTime = -1000000;  // wait for 1/10 second
          status = KeWaitForSingleObject(&unloadWait,Executive,KernelMode,TRUE,(PLARGE_INTEGER)&waitTime);
          if (status != STATUS_TIMEOUT)
          {
#ifdef TRCW
             TrcPrint("Unload Warning: KeWaitForSingleObject status %x not timeout status as expected.\n",
                status);
#endif
          }
       }
    }

    //
    // Delete the write structure
    //

    if (wrtStruct)
    {
       deallocate_memory(wrtStruct);
       wrtStruct = NULL;
       deviceExtension->wrtStruct = NULL;
    }
    if (writeWait)
    {
       deallocate_memory(writeWait);
       writeWait = NULL;
    }

    // Delete the device objects and DPC queue blocks

    deviceObject = DriverObject->DeviceObject;

    while (deviceObject)
    {
       deviceExtension =  deviceObject->DeviceExtension;
       if (deviceExtension->Counter == 0xffffffff)
       {
          //
          // Delete the IPRKRNL device object.
          //
          nextDeviceObject = deviceObject->NextDevice;
          IoDeleteDevice(deviceObject);
          deviceObject = nextDeviceObject;
       }
       else
       {
          // Counter device, delete DPC queue blocks and object

          //
          // Delete the DPC queue blocks
          //

          if (deviceExtension->dpcBlock)
          {
             deallocate_memory(deviceExtension->dpcBlock);
             deviceExtension->dpcBlock = NULL;
          }

          //
          // Delete the counter device objects.
          //

          nextDeviceObject = deviceObject->NextDevice;
          IoDeleteDevice(deviceObject);
          deviceObject = nextDeviceObject;
       }
    }
    //
    // Clean up any NPP that was inadvertantly left undeleted.
    //

    //
    // Delete nonpaged pool zones for nonpaged data structures
    //

    for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
    {
       if (npp_table.start[cnt] != NULL)
       {
#ifdef TRCP
          TrcPrint("Deleting pool (list location %d), loc=%x\n",
             cnt,npp_table.start[cnt]);
#endif
#ifdef DBGP
          DbgPrint("Deleting pool (list location %d), loc=%x\n",
             cnt,npp_table.start[cnt]);
#endif
          ExFreePool(npp_table.start[cnt]);
          npp_table.start[cnt] = NULL;
       }
    }

#ifdef TRC0
    TrcPrint("IprKrnl IPROBE API kernel Unload: exit\n");
#endif
    return;
}

BOOLEAN
PerformanceCounterInterrupt(
    PKINTERRUPT InterruptObject,
    PDEVICE_EXTENSION DeviceExtension,
    PKTRAP_FRAME ktrap_stackf
    )
/*++

Routine Description:

    This routine is the interrupt handler for performance counter
    interrupts.  The interrupt is handled by incrementing the count
    value for the counter by the current event count.

    N.B. - The interrupt has already been acknowledged by the PALcode.

Arguments:

    InterruptObject - Supplies a pointer to the interrupt object for this
                          interrupt.

    DeviceExtension - Supplies a pointer to the device extension for the
                          performance counter that is interrupting.

Return Value:

    TRUE is returned.

--*/
{
/*
   KTRAP_FRAME *ktrap_stackf;
   CPUXX
      *cpuDB;
   IPR_BUFDSC
      *BufDscPtr;
   VOID *stackptr;
   IPR_DpcBlock *Context;
//   LIST_ENTRY *prevTail;
   PKTHREAD curThrd;
   int *curProc;

//#ifdef DBGISR
#ifdef DBGPID
//   KPRCB *pPRCB;
   int *curTEB;
#endif
*/

#ifdef TRCISR0
   TrcPrint("AXP 21064 - Interrupt: Enter\n" );
#endif

   if (!DeviceExtension)
   {
#ifdef TRCE
       TrcPrint("ISR: Error, DeviceExtension is null\n");
#endif
       return TRUE;
   }
   DisablePerformanceCounter(DeviceExtension); // davew

   DeviceExtension->icount++;   // Count number of interrupts

   DeviceExtension->stackptr = GetCurrentStackLoc();
   // Get ktrap frame on stack pointer: add 32 + stack_used_on ISR entry
   // bytes for stack (call fram) usage in this ISR.
   // NOTE: care must be taken when modifying this ISR (or even using a
   // new ver. of the compiler to check the stack usage on entry.
   // If stack usage changes it is necessary to update the following computation
   // to get to the trap frame.

   // add stack used by isr, plus size of kernel trap frame plus pagesize-1
//   DeviceExtension->ktrap_stackf = (KTRAP_FRAME *)((( (uint)(DeviceExtension->stackptr) + 32 + sizeof(KTRAP_FRAME) + 0xfff) & 0xfffff000) - sizeof(KTRAP_FRAME));
   DeviceExtension->ktrap_stackf = ktrap_stackf;

   if (((DeviceExtension->icount % 10) == 0) && (DeviceExtension->cpu != THIS_CPU_NUM()))
   {
#ifdef DBGPID
      DbgPrint("ISR Error: DEcpu %d, actual cpu %d, Counter %d, DevExt %x, DPCblk %x\n",
         DeviceExtension->cpu,THIS_CPU_NUM(),DeviceExtension->Counter,DeviceExtension,DeviceExtension->dpcBlock);
#endif
   }
   if ((DeviceExtension->icount % 1000) == 0)
   {
#ifdef DBGPID
      DbgPrint("ISR enter, DevExt %x, dpcBlk %x, cpu %d, counter %d, icount %d. intCntS %d\n",
         DeviceExtension,DeviceExtension->dpcBlock,DeviceExtension->cpu,DeviceExtension->Counter,
         DeviceExtension->icount,DeviceExtension->intCntS);
#endif
   }
#ifdef TRCISR
   if ((DeviceExtension->icount % 10000) == 0)
      TrcPrint("\n\nIprobe kernel ISR entered, interrupt %d, counter %d\n",
         DeviceExtension->icount,DeviceExtension->Counter);
#endif

   if (!DeviceExtension->connected)     // If not yet opened, exit...
      return TRUE;

#ifdef DBGISR
    if ((DeviceExtension->icount % 1000) == 0)
       DbgPrint("ISR Device connected, icount %d, DevExt %x\n",
          DeviceExtension->icount,DeviceExtension);
#endif

   DeviceExtension->cpuDB = THIS_CPU ();         // Get Processor PCR (call_pal rdpcr)
   DeviceExtension->curThrd = DeviceExtension->cpuDB->Prcb->CurrentThread;
   DeviceExtension->curProc = (int *)IoGetCurrentProcess();

#ifdef DBGISR
   if ((DeviceExtension->icount % 1000) == 0)
   {
      DbgPrint("\nPCR %x, PRCB %x, PalRsrvAddr %x, counter %d, cpu %d\n",
         DeviceExtension->cpuDB,DeviceExtension->cpuDB->Prcb,DeviceExtension->cpuDB->PalReserved,DeviceExtension->Counter,DeviceExtension->cpu);
      DbgPrint("curTh %x, rstrtblk %x, curProc %x\n",
         DeviceExtension->cpuDB->Prcb->CurrentThread,DeviceExtension->cpuDB->Prcb->RestartBlock,DeviceExtension->curProc);
      DbgPrint("IntObj %x, DevExt %x, icount %d, stack %x, ktrapfr %x\n",
         InterruptObject,DeviceExtension,DeviceExtension->icount,DeviceExtension->stackptr,DeviceExtension->ktrap_stackf);
//      DbgBreakPoint();
   }
#endif

#ifdef TRCISR
   if ((DeviceExtension->icount % 10000) == 0)
      TrcPrint("Clock cycle %d, 2nd level cache %d fill %d, 3rd level cache %d fill %d, 4th level cache %d fill %d\n",
         DeviceExtension->cpuDB->CycleClockPeriod,
         DeviceExtension->cpuDB->SecondLevelCacheSize,DeviceExtension->cpuDB->SecondLevelCacheFillSize,
         DeviceExtension->cpuDB->ThirdLevelCacheSize,DeviceExtension->cpuDB->ThirdLevelCacheFillSize,
         DeviceExtension->cpuDB->FourthLevelCacheSize,DeviceExtension->cpuDB->FourthLevelCacheFillSize);
#endif

#ifdef TRCISR
   if ((DeviceExtension->icount % 10000) == 0)
      TrcPrint("cpuDB %x, ktrap_stackf\n",DeviceExtension->cpuDB,DeviceExtension->ktrap_stackf);
#endif

   DeviceExtension->BufDscPtr = GETBDPTR(DeviceExtension->cpuDB->Number);
   if (!DeviceExtension->BufDscPtr)
   {
#ifdef TRCISR
      if ((DeviceExtension->icount % 1000) == 0)
      {
         TrcPrint("ISR exiting, no buffer descriptor\n");
      }
#endif
#ifdef DBGPID
      if ((DeviceExtension->icount % 1000) == 0)
      {
         DbgPrint("ISR exiting, no buffer descriptor\n");
      }
#endif
      return TRUE;              // If no buffer descriptor, only compatibility action (above) is done
   }
#ifdef TRCISR
   if ((DeviceExtension->icount % 100) == 0)
      TrcPrint("BufDscPtr %x, interrupt %d, cpu %d\n",DeviceExtension->BufDscPtr,DeviceExtension->icount, DeviceExtension->cpuDB->Number );
#endif

   if (!(DeviceExtension->BufDscPtr -> validcounters & (1 << DeviceExtension->Counter)))
   {
#ifdef TRCISR
      if ((DeviceExtension->icount % 1000) == 0)
      {
         TrcPrint("ISR exiting, %x not active counter %x\n",
            DeviceExtension->Counter,DeviceExtension->BufDscPtr -> validcounters);
      }
#endif
#ifdef DBGPID
      if ((DeviceExtension->icount % 1000) == 0)
      {
         DbgPrint("DevExt %x, ISR exiting, %x not active counter %x\n",
            DeviceExtension,DeviceExtension->Counter,DeviceExtension->BufDscPtr -> validcounters);
      }
#endif
      return TRUE;
   }

    // Action from old PERFEV4 driver, retain this action for compatibility
   DeviceExtension->CounterValue += DeviceExtension->EventCount;
#ifdef TRCISR
   if ((DeviceExtension->icount % 10000) == 0)
      TrcPrint("Bumped counter %d (MUX %x) by %d to %d\n",
        DeviceExtension->Counter,DeviceExtension->Counter,
        DeviceExtension->EventCount,DeviceExtension->CounterValue);
#endif

#ifdef DBGISR
   if ((DeviceExtension->icount % 1000) == 0)
   {
      DbgPrint("ISR %x is active counter %x\n",
         DeviceExtension->Counter,DeviceExtension->BufDscPtr -> validcounters);
   }
#endif
   // Fill in data structure passed to Dpc - this info. will allow the Dpc
   // to provide the necessary info. for processing the histogram or
   // sample for this interrupt.

   DeviceExtension->Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
   DeviceExtension->Context += (DeviceExtension->intCntS % IPR_DPC_BLOCKS);

   if (((DeviceExtension->intCntS - DeviceExtension->intCntE) > IPR_DPC_BLOCKS) &&
    (DeviceExtension->intCntE < DeviceExtension->intCntS))
   {
      // More than IPR_DPC_BLOCKS queued to DPC for writing, drop info. from this interrupt
      DeviceExtension->dropInt++;
      if ((DeviceExtension->dropInt % 10) == 0)
      {
         // Make sure the DPC is triggered to drain the queue
         // of samples
         IoRequestDpc(DeviceExtension->DeviceObject,DeviceExtension->DeviceObject->CurrentIrp,NULL);
      }
#ifdef TRCW
      if ((DeviceExtension->dropInt % 1000) == 0)
      {
         TrcPrint("ISR Warning: 1000 interrupts dropped, DevExt %x, start %d, end %d, dropped %d, counter %d, cpu %d\n",
            DeviceExtension,DeviceExtension->intCntS,DeviceExtension->intCntE,DeviceExtension->dropInt,
            DeviceExtension->Counter,DeviceExtension->cpu);
      }
#endif
#ifdef DBGPID
      if ((DeviceExtension->dropInt % 1000) == 0)
      {
         DbgPrint("ISR Warning: 1000 interrupts dropped, DevExt %x. start %d, end %d, dropped %d, counter %d, cpu %d\n",
            DeviceExtension,DeviceExtension->intCntS,DeviceExtension->intCntE,DeviceExtension->dropInt,
            DeviceExtension->Counter,DeviceExtension->cpu);
      }
#endif
   }
   else
   {
      DeviceExtension->intCntS++;
      DeviceExtension->Context->cpu = THIS_CPU_NUM();
      DeviceExtension->Context->BufDscPtr = DeviceExtension->BufDscPtr;
      DeviceExtension->Context->time = THIS_TIME();
      DeviceExtension->Context->ps = THIS_PS();
      DeviceExtension->Context->pc = THIS_PC();
      DeviceExtension->Context->counter = DeviceExtension->Counter;
      DeviceExtension->Context->pid = THIS_PID();
#if 0
      if ((DeviceExtension->Context->pc < (ULONG)0x80000000) &&
	  (DeviceExtension->Context->pc != 0))
	{
	  DbgPrint("DE's pc=0x%x, ps=0x%x, pid=%d\n",
		   DeviceExtension->Context->pc,
		   DeviceExtension->Context->ps,
		   DeviceExtension->Context->pid);
	}
#endif
#ifdef DBGPID2
      if (((DeviceExtension->Context->pid & 0xffffff00) != 0) && ((DeviceExtension->intCntS % 1000) == 0))
      {
         DbgPrint("ISR Error, bad pid %x, curProc %x, curThrd %x, PAL cur thread %x, Context %x\n",
            DeviceExtension->Context->pid,DeviceExtension->curProc,
            DeviceExtension->curThrd,GetCurThread(),DeviceExtension->Context);
      }
#endif
      DeviceExtension->Context->priority = THIS_PRIORITY();
      DeviceExtension->Context->ipl = THIS_IPL();
      DeviceExtension->Context->mode = THIS_MODE();
      BARRIER();        // Do an MB here to make sure that the count and context
                        // area is updated

/*
      prevTail = ExInterlockedInsertTailList(&DeviceExtension->sampQ,
         (LIST_ENTRY *)Context,
         &DeviceExtension->sampSpin);
*/
      // Pass context to DpcForIsr if necessary
      if (/*(!prevTail) || */((DeviceExtension->icount % IPR_DPC_FREQ) == 0))
      {
         IoRequestDpc(DeviceExtension->DeviceObject,DeviceExtension->DeviceObject->CurrentIrp,DeviceExtension->Context);
#ifdef DBGPID
         if ((DeviceExtension->icount % 100) == 0)
         {

            DbgPrint("dpc disp, Context %x, ps %x, pc %x, counter %x, ipl %x, mode %x, PID %x, pri %d, cpu %d, BDptr %x, intCntS %d, dpcB %x, stack %x, kt %x, DevExt %x, CurCpu %d\n",
               DeviceExtension->Context,DeviceExtension->Context->ps,DeviceExtension->Context->pc,
               DeviceExtension->Context->counter,DeviceExtension->Context->ipl,DeviceExtension->Context->mode,
               DeviceExtension->Context->pid,DeviceExtension->Context->priority,DeviceExtension->Context->cpu,
               DeviceExtension->Context->BufDscPtr,DeviceExtension->intCntS,DeviceExtension->dpcBlock,
               DeviceExtension->stackptr,DeviceExtension->ktrap_stackf,DeviceExtension,THIS_CPU_NUM());
//            DbgBreakPoint();
         }
//         if (Context->ipl > 32)
         if ((DeviceExtension->Context->ps > 0xf) && ((DeviceExtension->icount % 100) == 0))
         {
            DbgPrint("dpc disp,, Context %x, ps %x, pc %x, counter %x, ipl %x, mode %x, PID %x, pri %d, cpu %d, BDptr %x, intCntS %d, dpcB %x, stack %x, kt %x, DevExt %x, CurCpu %d\n",
               DeviceExtension->Context,DeviceExtension->Context->ps,DeviceExtension->Context->pc,
               DeviceExtension->Context->counter,DeviceExtension->Context->ipl,DeviceExtension->Context->mode,
               DeviceExtension->Context->pid,DeviceExtension->Context->priority,DeviceExtension->Context->cpu,
               DeviceExtension->Context->BufDscPtr,DeviceExtension->intCntS,DeviceExtension->dpcBlock,
               DeviceExtension->stackptr,DeviceExtension->ktrap_stackf,DeviceExtension,THIS_CPU_NUM());
//            DbgBreakPoint();
         }
#endif
#ifdef TRCISR
         if ((DeviceExtension->icount % 1000) == 0)
         {

            TrcPrint("dpc disp, Context %x, ps %x, pc %x, counter %x, ipl %x, mode %x, PID %x, pri %d, cpu %d, BDptr %x, intCntS %d, dpcB %x, stack %x, kt %x, DevExt %x, CurCpu %d\n",
               DeviceExtension->Context,DeviceExtension->Context->ps,DeviceExtension->Context->pc,
               DeviceExtension->Context->counter,DeviceExtension->Context->ipl,DeviceExtension->Context->mode,
               DeviceExtension->Context->pid,DeviceExtension->Context->priority,DeviceExtension->Context->cpu,
               DeviceExtension->Context->BufDscPtr,DeviceExtension->intCntS,DeviceExtension->dpcBlock,
               DeviceExtension->stackptr,DeviceExtension->ktrap_stackf,DeviceExtension,THIS_CPU_NUM());
         }
#endif
      }
   }
   EnablePerformanceCounter(DeviceExtension); // davew
#ifdef DBGISR
   if ((DeviceExtension->icount % 1000) == 0)
   {
       DbgPrint("AXP 21064 - Interrupt: Exit\n" );
   }
#endif
   return TRUE;
}

