#ifndef __BIT_LIBRARY_LOADED
#define __BIT_LIBRARY_LOADED 1
//
//
// Module:	BIT_LIBRARY.H
//
// Version:	X-1
//
/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1995 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//
// Abstract:	This header file contains bit manipulation macros and
//		function prototypes.
//
// Author:		Brian F. Wibecan
// Creation Date:	20-Jun-1995
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan	20-Jun-1995
//	Original version.
//

#include "ints.h"

//
// Modify the argument to set all bits in the argument
//
#define BIprAll1(b) ((b) | ~(b))

//
// Create a bit mask with only the specified bit set
//
extern uint64 BIprB2M (unsigned int b);

//
// Return TRUE if bit is set in the mask, FALSE otherwise
//
#define BIprIsSet(m,b) (((m) & BIprB2M(b)) != 0)

//
// Return a mask with bits set that are set in either mask
//
#define BIprSet(m,c) ((m) | (c))

//
// Return a mask with bits set that are set in both masks
//
#define BIprBoth(m,c) ((m) & (c))

//
// Return a mask with bits set that are set only in the first mask, and not
// in the second
//
#define BIprClr(m,c) ((m) & ~(c))

//
// Create a bit mask with all bits set from the specified one down
//
#define BIprSetLo(b) (BIprB2M((b) + 1) - 1)

//
// Create a bit mask with all bits from the specified one up set
//
#define BIprSetHi(b) (~(BIprB2M(b) - 1))

//
// Set the specified bit in the mask
//
#define BIprSet1(m,b) BIprSet(m,BIprB2M(b))

//
// Clear the specified bit in the mask
//
#define BIprClr1(m,b) BIprClr(m,BIprB2M(b))

//
// Return a mask consisting only the lowest set bit in the passed mask
//
// *** NOTE *** This macro uses the passed argument more than once
//
#define BIprLowest(m) ((m) & (~(m) + 1))

//
// Return TRUE if exactly one bit is set in the passed mask
//
// *** NOTE *** This macro uses the passed argument more than once
//
#define BIprIs1(m) ((m) && BIprLowest(m) == (m))

extern int BIprCnt64 (uint64 x);

#define BIprCnt(x) BIprCnt64(x)

//
// Determine the bit number of the lowest set bit
//
extern int BIprWhich (uint64 m);

//
// Clear all the bits in the mask below the specified bit
//
#define BIprClrLo(m,b) BIprClr (m, BIprSetLo ((b) - 1))

//
// Clear all the bits in the mask above the specified bit
//
#define BIprClrHi(m,b) (BIprClr (m, BIprSetHi ((b) + 1)))

//
// Find the relative position of the specified bit within the set mask bits
//
extern int BIprRelPos (uint64 m, unsigned int b);

#endif // #ifndef __BIT_LIBRARY_LOADED
