#ifndef __IPR_HDR_LOADED
#define __IPR_HDR_LOADED
//
//
// Module:	IPR_HDR.H
//
// Version:	X-1
//
/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1995 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//
// Abstract:	This contains the structure used for storing the state of
//		iprobe. Some fields in this structure are filled up by OS
//		specific routines and they are available in ipr_os_vms.c.
//

/*
---------------
Modification History
~~~~~~~~~~~~~~~~~~~~
02    Aug-1994, Ravindra Kulkarni
      Included the file ipr_struct.h

01    Apr-1994, Ravindra Kulkarni
      Wrote original code.
*/
# include "ipr_struct.h"

/* function prototypes */
extern int read_user_options(IPR_EVENT *events,       /* events to sample */
            IPR_FREQ *freqs,      /* frequency of sampling */
            IPR_COUNT *skips,     /* skip value vector */
            int NumEvents,      /* length of prev 3 vectors */
            IPR_MMASK modemask,  /* modes to measure in */
            IPR_DMASK datamask,  /* data mask for collecting samples */
            IPR_DMASK quaddatamask,  /* data mask for collecting samples */
            IPR_COUNT buffer_count,   /* # of bufs specified */
            IPR_COUNT buffer_size,    /* size of buffers specified */
            time_t delay,       /* start delay */
            time_t duration,    /* run duration */
            IPR_COUNT interval,   /* output interval */
            struct pc_ranges *pc_range, /*  PC range information    */
            char *outfile,      /* outfile name */
            IPR_COUNT method);    /* measuring method */

extern int display_user_options(FILE *fp);
extern void display_system_header(FILE *fp);
extern int write_header_info();

extern struct keywords sample_data_table[];
extern struct keywords sample_quad_data_table[];
struct system_info   {

    /* Time stamps */
    uint64 timestamp;       /* High-accuracy, OS-specific format time stamp */
    uint64 cpu_type;
    time_t start_time;

    /* Range descriptions (keep rangeptr aligned on 64-bit boundary) */
    uint64 *rangeptr;       /* range pointer  */
    IPR_COUNT rangecnt;       /* range information */

    /*  Machine specific information */

    char *nodename;         /* node name of the current system */
    char *osname;           /* OS name and version of current system */
    char *modelname;        /* The name of current model */
    int NumCpus;            /* Number of CPU's present on current system */
    int memorysize;         /* The physical number of pages available */
    int pagelength;         /* The size of each page in bytes */
    int num_ctrs;           /* The number of counters present on the system */
    uint64 cpu_speed;       /* cpu speed of current system */

    /*  Counter specific information */

    IPR_EVENT *sel_events;    /* events selected by user */
    IPR_FREQ *sel_freqs;      /* frequencies selected by the user */
    int NumEvents;          /* Number of events selected */
    IPR_EVENT *act_events;    /* actual selected events on the current system
                               These are as returned by pfmEventCounterMap()
                             */
    IPR_FREQ *act_freqs;      /* actual selected freqs on the current system
                               These are as returned by pfmFreqCounterMap() */

    IPR_CTRMASK counter_mask; /* counter mask on current system */
    int null_proc_id;       /* Null process id for current system */
    int avail_cpu_mask;     /* cpu mask for current system */

    /*  command line arguments */

    time_t delay;           /* Amount of delay in seconds */
    time_t duration;        /* Duration of intended operation */
    IPR_COUNT buffer_count;   /* Number of buffers used */
    IPR_COUNT buffer_size;    /*  size of sample buffers used */
    IPR_COUNT interval;       /* The interval for counting mode */
    char *outfile;          /* Output file name */
    IPR_COUNT method;         /* Method of measurement */
    time_t start;           /* Start time of measurement */
    time_t cur_time;        /* Present time */
    IPR_COUNT *skips;         /* Sample skip info */
    IPR_MMASK modemask;      /* modes to measure in */
    IPR_DMASK datamask;      /* data mask for collecting samples. */
    IPR_DMASK quaddatamask;      /* data mask for collecting samples. */
    struct pc_ranges *pc_range; /*  PC ranges entered by user */

    /*   OS  related information */

     long OS_type;           /* OS_VMS, OS_NT, OS_OSF1 etc */
     long OS_ver_major;      /* OS major version number */
     long OS_ver_minor;      /* OS minor version number */

    /* Signature for sample file */
    char signature[8];
}sys_info;
#endif // ifndef __IPR_HDR_LOADED
