#ifndef __IPR_KEYLIB_LOADED
#define __IPR_KEYLIB_LOADED
//
//
// Module:	IPR_KEYLIB.H
//
// Version:	X-1
//
/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1995 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
    Structures and prototypes for the keyword table manipulation
    routines.
Modification History
~~~~~~~~~~~~~~~~~~~~
02  Jun-94, Ravindra Kulkarni
    Added keywords T_COLLECT, T_NOCOLLECT, T_ADD

01  May-94, Greg Tarsa
    Author.
*/
# include "ipr_api_ext_defs.h"
# include "ints.h"

/* This should probably be "void *" */
#ifndef ADDR
#define ADDR uint64
#endif

/* option types */
#define T_NONE      0       /* no associated value */
#define T_REQ_NUM   1       /* has required numeric value */
#define T_REQ_FILE  2       /* has required file name */
#define T_REQ_METH  3       /* has required method keyword */
#define T_REQ_MODE  4       /* has required mode keyword */
#define T_REQ_RSPEC 5       /* has required range spec */

/* option attributes */
#define A_NOATTR    0       /* no special attrs */
#define A_INVISIBLE (1<<1)      /* is invisible */
#define A_REPEATING (1<<2)      /* can be repeated */

struct keywords
    {
    char *name;     /* text name of option */
    long num;       /* internal code for option */
    unsigned long type;      /* arg type of option */
    long attr;      /* attributes (i.e., visible, repeating) */
    char *def_val;  /* default value of argument, if applicable */
    char *desc;     /* short description of option */
    };

/*
    pc_ranges are stored in a singly linked list.
*/
struct pc_ranges
    {
    ADDR start;
    ADDR end;
    unsigned long granularity;
    struct pc_ranges *next;
    };

/* define the acceptable keyword tags */
#define T_BUFFER_COUNT   1
#define T_BEGIN_TIME     2
#define T_END_TIME   3
#define T_INTERVAL   4
#define T_OUTPUT     5
#define T_METHOD     6
#define T_MODE       7
#define T_HELP       8
#define T_QUIET      9
#define T_DEBUG     10
#define T_PERCPU    11
#define T_PEREVENT  12
#define T_RANGE     13
#define T_BUFSIZ    14
#define T_INPUT     15
#define T_COLLECT   16
#define T_NOCOLLECT 17
#define T_ADD       18

/* define the acceptable methods */
#define METH_SAMPLE (1<<1)
#define METH_COUNT  (1<<2)
#define METH_TOTAL  (1<<3)
#define METH_IPL    (1<<4)
#define METH_MODE   (1<<5)
#define METH_ADDR   (1<<6)
#define METH_PRIORITY (1<<7)


#define METH_ALL ( METH_TOTAL | METH_IPL | METH_MODE | METH_ADDR | \
    METH_PRIORITY )
/* define the acceptable modes */
#define MODE_DEFAULT    0
#define MODE_K      IPR_MMASK_M_KERNEL
#define MODE_E      IPR_MMASK_M_EXECUTIVE
#define MODE_S      IPR_MMASK_M_SUPERVISOR
#define MODE_U      IPR_MMASK_M_USER
#define MODE_PAL    IPR_MMASK_M_PAL
#define MODE_SEL    IPR_MMASK_M_SELECTED

/* function prototypes */
struct keywords *find_by_name(struct keywords *table, char *ostr);
struct keywords *find_by_num(struct keywords *table, long num);
struct keywords *find_opt_by_name(char *ostr);
struct keywords *find_opt_by_num(long num);
struct keywords *find_meth_by_name(char *ostr);
struct keywords *find_meth_by_num(long num);
struct keywords *find_mode_by_name(char *ostr);
struct keywords *find_mode_by_num(long num);
int bstreql(char *s1, char *s2);
int bstrneql(char *s1, char *s2, long n);
int parse_event(char *src_espec, IPR_EVENT *evtptr, IPR_FREQ *freqptr, IPR_COUNT *skipptr);
int parse_pc_range(char *src_pspec, ADDR *begaddr, ADDR *endaddr, unsigned long *granularity);

#endif // ifndef __IPR_KEYLIB_LOADED
