#ifndef __IPR_STRUCT_LOADED
#define __IPR_STRUCT_LOADED
//
//
// Module:	IPR_STRUCT.H
//
// Version:	X-1
//
/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1995 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
    ipr_struct.h

    Data structures and definitions for the IPROBE program.
-----
Modification History
~~~~~~~~~~~~~~~~~
09  Sep-94, Ravindra Kulkarni
    Changed default granularity to 2 from 1

08  Aug-94, Ravindra Kulkarni
    Added conditional inclusion of ipr_struct.h itself.
    Included time.h in place of sys/time.h in case of NT.
    Added macros for printf formats.
    Defined CVTINPADDR as strtoq in case of vms and strtoul in case of
    osf and nt.
    Redefined access as _access in case of nt.

07  Jun-94, Ravindra Kulkarni
    Added buffer_size argument in do_sampling.

06  May-94, Ravindra Kulkarni
    Added min_value in the declaration of do_histogram.

05  May-94, Greg Tarsa
    Added declarations for multiple method support and mode support.

04  Apr-94, Ravindra Kulkarni
    Moved the struct sys_info to ipr_hdr_vms.h
    Included the file ipr_hdr_vms.h

03  Mar-94, Greg Tarsa
    Added ipr__dbg flag.
    Removed EVTARRSIZE and friends.
    Added prototype for set_default_events.

02  Feb-94  Ravindra Kulkarni
    Fixed prototype defs; Added to eventdesc structure.

01  Dec-93, Greg Tarsa
    Wrote original code.

*/
# define OS_VMS 1
# define OS_OSF1 2
# define OS_NT 3

# if  defined(__VMS) || defined(__osf__)
#include <sys/time.h>
# endif

# if defined(NT) || defined(__NT)
#include <time.h>
#include <windows.h>
#include <wchar.h>
# endif

#include <stdio.h>
#include <stdlib.h> /* RTL routine prototypes */
# include "ints.h"

#include "ipr_keylib.h"

# include "ipr_api_ext_defs.h"
# include "ipr_events.h"
# if defined(__NT)
# include "ipr_api_macros_nt.h"
# endif

# define MAX_ELEMENTS 2560

typedef int INT32;  /* should be right for NT, OSF, and VMS */
typedef volatile unsigned FLAG;
typedef unsigned long FLAGS;

#if defined(unix) || defined(NT) || defined(__vms)
#define FLAG_CHAR1 '-'
/*  #define FLAG_CHAR2 '/'  */
#endif

#define EVT_SEP '.'     /* eventspec field separator */

#ifndef TRUE
#define TRUE (1==1)
#define FALSE (1==2)
#endif

/* limit values */
#define MAX_EVENTSPECS IPR_MAX_COUNTERS
#define MIN_BUFFER_COUNT 1
#define MAX_BUFFER_COUNT 128    /* arbitrary */
#define MIN_INTERVAL 1
#define MAX_INTERVAL 32767
#define MIN_TIME 0
#define MAX_TIME ((unsigned) -1)

/* default values for Command line options */
#define DEF_EVENT       IPR_EVT_CYCLES
#define DEF_BUFFER_COUNT    3
#define DEF_BUFFER_SIZE     8192
#define DEF_BEGIN_TIME      0
#define DEF_END_TIME        0
#define DEF_INTERVAL        1
#define DEF_OUTPUT      "pcsample.dat"
#define DEF_INPUT       "iprobe.cmd"
#define DEF_METHOD      METH_COUNT
#define DEF_MODE        0 /* this may not be valid */
#define DEF_IPR_DMASK    (0XFF)
#define DEF_GRANULARITY     2

/* Define the command line (CL) keyword bit flags */
#define CL_BUFFER_COUNT (1<<1)
#define CL_BEGIN_TIME   (1<<2)
#define CL_END_TIME (1<<3)
#define CL_INTERVAL (1<<4)
#define CL_OUTFILE  (1<<5)
#define CL_METHOD   (1<<6)
#define CL_MODE     (1<<6)
#define CL_HELP     (1<<7)
#define CL_QUIET    (1<<8)
#define CL_PEREVENT (1<<9)
#define CL_RANGE    (1<<10)
#define CL_BUFSIZ (1<<11)
#define CL_INPUT  (1<<12)
#define CL_COLLECT (1<<13)
#define CL_NOCOLLECT (1<<14)
#define CL_ADD (1<<15)

/*  define the bit flags for data mask. */
# define CL_TIME    (1<<1)


/* define the special frequencies */
# if  defined(__VMS) || defined (__NT)
# define FREQ_DEF    0XFFFFFFFF
# endif
# if defined (__osf__)
# define FREQ_DEF 0XFFFFFFFFFFFFFFFF
# endif

# define DEF_END_ADDRESS FREQ_DEF

#define FREQ_OFF    0
#define FREQ_LOW    IPR_FREQUENCY_LOWEST
#define FREQ_HIGH   IPR_FREQUENCY_HIGHEST
#define FREQ_MIN    (1<<31)
#define FREQ_MAX    (1<<1)

/*  This code takes care of different formats on vms osf1 and nt */

# if  defined(__VMS) || defined(NT) || defined (__NT)
# define PRINT_FORMAT1 " %12Ld %9Ld"
# define ADDRFORMAT "%x"
# endif

# if defined(__osf__)
# define PRINT_FORMAT1 " %14lu %9lu"
# define ADDRFORMAT "%lx"
# endif

# if  defined(__VMS) || defined(NT) || defined(__NT)
# define PRINT_FORMAT2 " %12Ld"
# endif

# if defined(__osf__)
# define PRINT_FORMAT2 " %14lu"
# endif

# if  defined(__VMS) || defined(NT) || defined(__NT)
# define PRINT_FORMAT3 "%x"
# endif

# if defined(__osf__)
# define PRINT_FORMAT3 "%lx"
# endif

# if  defined(__VMS)
# define CVTINPADDR strtoq
# endif

# if defined(__osf__) || defined(NT) || defined(__NT)
# define CVTINPADDR strtoul
# endif

#if  defined(__VMS)
#define DEFAULT_DATA (IPR_DMASK_M_PS | IPR_DMASK_M_PC | IPR_DMASK_M_COUNTER |\
                      IPR_DMASK_M_PID)
#endif

#if defined(__osf__)
#define DEFAULT_DATA ( IPR_DMASK_M_COUNTER | IPR_DMASK_M_PID )
#endif

#if defined(NT) || defined(__NT)
#define DEFAULT_DATA ( IPR_DMASK_M_PS | IPR_DMASK_M_PC | IPR_DMASK_M_COUNTER |\
     IPR_DMASK_M_PID )
#endif

#if  defined(__VMS)
#define DEFAULT_QUADDATA (0)
#endif

#if defined(__osf__)
#define DEFAULT_QUADDATA (IPR_DMASK_M_PS | IPR_DMASK_M_PC)
#endif

#if defined(NT) || defined(__NT)
#define DEFAULT_QUADDATA (0)
#endif

#if defined(NT) || defined(__NT)
# define access _access
#endif


/* Following definitions are to be deleted after they have been defined in API */


struct eventspec    {
    char *name;     /* string name of event */
    IPR_EVENT num;    /* numeric name of event */
    IPR_FREQ freq;    /* frequency to measure event */
    };

struct eventdesc    {
    char *name;     /* string name of event */
    IPR_EVENT num;    /* numeric name of event */
    };

/* global debug flag */
extern long ipr__dbg;

/* function prototypes */
IPR_EVENT find_event_number_by_name (char *ename, IPR_BOOLEAN print_message);
//struct eventdesc *find_event_by_number(IPR_EVENT num);
//struct eventdesc *find_event_by_name(char *name);
void set_default_events(struct eventspec *events);

IPR_STATUS do_sampling (
         IPR_EVENT *selected_evts,    /* array of evts to measure */
         IPR_FREQ *selected_freqs,    /* freqs to measure at */
         IPR_COUNT *selected_skips,   /* which interrupts to measure */
         IPR_COUNT selection_count,   /* size of the prev 2 arrays */
         IPR_MMASK modes2use,    /* prog modes to measure in */
         IPR_DMASK datamask,     /*  data mask to collect samples */
         IPR_DMASK quaddatamask,     /*  data mask to collect quad samples */
         time_t start_time,     /* # secs till start of meas */
         time_t end_time,       /* # secs to measure */
         IPR_COUNT buffer_count,  /* number of buffers to use */
         IPR_COUNT buffer_size,   /* size of buffers to use */
         IPR_COUNT range_count,        /* Number of pc ranges */
         struct pc_ranges *pc_range, /* Pointer to addr range */
         char *sfile_name);     /* sampling file name */

IPR_STATUS do_counting (
        IPR_EVENT *selected_evts, /* array of evts to measure */
        IPR_FREQ *selected_freqs, /* freqs to measure at */
        IPR_COUNT selection_count, /* size of the prev 2 arrays */
        IPR_MMASK modes2use, /* prog modes to measure in */
        time_t start_time,  /* # secs till start of meas */
        time_t end_time,    /* # secs to measure */
        time_t interval    /* output interval */
        );

int process_args(
         int argc, char **argv, /* command line info */
         FLAGS *flags,      /* ret flags for which options set */
         IPR_COUNT *buffer_count, /* ret # of buffers specified */
         IPR_COUNT *buffer_size,  /* ret size of buffers specified */
         time_t *start,     /* ret # of secs till start */
         time_t *finish,        /* ret # of secs to measure */
         IPR_COUNT *interval,     /* ret length of interval */
         char **outfile,        /* ret name of sampling file */
         FLAGS *method,     /* ret method of measuring */
         IPR_MMASK *modemask,    /* ret prog modes to measure */
         IPR_DMASK *datamask,    /* data mask for collecting samples */
         IPR_DMASK *quaddatamask,    /* data mask for collecting samples */
         struct pc_ranges *pc_range,/* ptr to list of pc_ranges */
         IPR_EVENT *ret_evtarr,   /* ret array of events to measure */
         IPR_FREQ *ret_freqarr,   /* ret array of freqs to measure at */
         IPR_COUNT *ret_skiparr,  /* ret array of skip values */
         IPR_COUNT *ret_arrsize); /* ret length of prev 2 arrays */

 void wait_for_time(time_t wait_time) ; /* sleep until wait_time */

/*
 Signal handling routines that have OS-dependent implementation.
 See ipr_os_osf1.c, ipr_os_vms.c, and ipr_os_nt.c for details.
*/
/*
IPR_STATUS OS_declare_wakeup_handler(FLAG *flag);
IPR_STATUS OS_declare_ctrlc_handler(FLAG *flag);
IPR_STATUS OS_declare_pfm_handler(FLAG *flag);
void OS_cancel_wakeup_handler();
void OS_cancel_ctrlc_handler();
void OS_cancel_pfm_handler();
IPR_STATUS OS_schedule_wakeup_time(time_t wait_time);
void OS_wait_for_signal();
IPR_STATUS OS_clear_repeating_wakeup ();
IPR_STATUS OS_schedule_repeating_wakeup (time_t delta_time);
*/
IPR_STATUS do_histogram (IPR_HMASK hist_type,  /* histogram type */
    IPR_EVENT *selected_evts,     /* array of evts to measure */
    IPR_FREQ *selected_freqs,     /* freqs to measure at */
    IPR_COUNT selection_count,    /* size of the prev 2 arrays */
    IPR_MMASK modes2use,     /* prog modes to measure in */
    IPR_COUNT start_time,     /* # secs till start of meas */
    IPR_COUNT end_time,       /* # secs to measure */
    IPR_COUNT interval,       /* Output interval */
    unsigned range_count,   /*  Number of pc ranges */
    struct pc_ranges *pc_range,/*  Pointer to the actual ranges */
    int histogram_count);      /* number of histograms entered */

int bstreql(char *s1, char *s2);
void print_event_table(FILE *stream);
int find_low_freq_by_event(IPR_EVENT event);
int find_high_freq_by_event(IPR_EVENT event);

#endif // ifndef __IPR_STRUCT_LOADED
