#ifndef __IPREDUCE_LOADED
#define __IPREDUCE_LOADED
//
//
// Module:	IPREDUCE.H
//
// Version:	X-1
//
/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1995 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//
//
// Abstract:	This header file contains definitions used in the sampling data
//		reduction program IPREDUCE.
//
// Author:		Brian F. Wibecan
// Creation Date:	20-Jun-1995
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan	20-Jun-1995
//	Original version.
//

#ifdef WIN32
#  include <windows.h>
#endif
#include <time.h>
#include <stdio.h>
#include "ipr_api_ext_defs.h"
#include "ipr_events.h"
#include "ipr_sample.h"
#include "getopt.h"

#define FILE_SIGNATURE "IPROBE"

#define OS_VMS  1
#define OS_OSF  2
#define OS_NT   3
#define INSSIZE 4
#define ELEMCHUNK 1024
#define STATISTICS_BARSIZE 100
#define PROFILE_BARSIZE 50
#define EOF_STATUS IPR__INVALID
#define OVERFLOW_STATUS IPR__UNAVAILABLE
#define BLOCK_SIZE 512
#define PAGE_MASK 8191
#define CACHE_SHIFT 5
#define GLOBAL_INDENT 2
#define NAME_SIZE 40
#define HEXBASE 16
#define BUFSIZE 200
#define NUMSIZE 32
#define IMGCHUNK 16
#define GBLCHUNK 128
#define EVT_NAMES (sizeof (eventargs) / sizeof (EVNT_ALIAS))
#define ERRMSG(x) fprintf(stderr, x)

#define SEPCHAR ':'

#define STRNEQ(s1,s2) (strcmp (s1, s2))
#define STREQ(s1,s2) (!strcmp (s1, s2))
#define ARGS_DONE -2
#define ARGS_INVALID -1
#define ARGS_HELP 2
#define ARGS_ADDRESS_NEEDED 1
#define ARGS_NOADDRESS 0
#define MAX_CCTRS 8
#ifdef __osf__
#  define ADDRESS_DIGITS 16
#else
#  define ADDRESS_DIGITS 8
#endif

#ifdef __VMS
#  define MODE_LIST_INITIALIZER {"Kernel", "Exec", "Super", "User", "Int","Null", "\?\?\?\?"}
#endif

#ifdef __osf__
#  define MODE_LIST_INITIALIZER {"Kernel", "User", "Idle", "\?\?\?\?"}
#endif

#ifdef WIN32
#  define MODE_LIST_INITIALIZER {"Kernel", "User", "Idle", "\?\?\?\?"}
#endif

typedef struct _imagetype
{
   uint64 base;                     /* Base address of the image */
   char *name;                      /* Name of the image */
   int new_index;                   /* Used in permutation inversion */
   IPR_COUNT *counter;              /* Number of samples per HW counter */
} IMAGETYPE;

typedef struct _globaltype
{
   uint64 base;                     /* Base address of this routine */
   char *name;                      /* Name of the routine */
   int ref;                         /* Index in image array */
   IPR_COUNT *counter;              /* Number of samples per HW counter */
} GLOBALTYPE;

typedef struct _valelem
{
   uint64 value;                    /* Histogram element value */
   unsigned int count;              /* Number of samples in this bar */
} VALELEM;

typedef struct _valarray
{
   VALELEM *pElem;                  /* Bars */
   unsigned int ECount;             /* Number of bars */
   unsigned int MaxECount;          /* Size of the bar array */
} VALARRAY;

typedef struct _drec
{
  uint64 time;
  uint64 PC;
  uint64 PSval;
  unsigned int IPL;
  unsigned int mode;
  unsigned int PID;
  unsigned int counter;
  unsigned int CPU;
  unsigned int cache;
  unsigned int priority;
} DREC;

typedef struct _reporttype
{
   int profile;
   int statistics;
} REPORTTYPE;

#define make_ftype(fname,fdtype) \
typedef struct _ ## fname \
{ \
   fdtype total; \
   fdtype time; \
   fdtype PC; \
   fdtype IPL; \
   fdtype mode; \
   fdtype PID; \
   fdtype counter; \
   fdtype CPU; \
   fdtype cache; \
   fdtype filter; \
   fdtype entries; \
   fdtype events; \
   fdtype priority; \
} fname

make_ftype(FILTERTYPE,IPR_COUNT);
make_ftype(FILTERFLAGS,int);

//typedef struct _filtertype
//{
//   IPR_COUNT total;
//   IPR_COUNT time;
//   IPR_COUNT PC;
//   IPR_COUNT IPL;
//   IPR_COUNT mode;
//   IPR_COUNT PID;
//   IPR_COUNT counter;
//   IPR_COUNT CPU;
//   IPR_COUNT cache;
//   IPR_COUNT filter;
//   IPR_COUNT entries;
//   IPR_COUNT events;
//   IPR_COUNT priority;
//} FILTERTYPE;

typedef struct _fhdr
{
	 char Signature[8];      /* file header signature identifying iprobe sample files */
	 uint64 cpu_speed;       /* cpu speed of current system */
	 uint64 timestamp;       /* starting time in vms  format */

	 uint64 cpu_type;        /* Alpha chip type */
	 int num_ctrs;           /* The number of counters present on the system */
	 IPR_CTRMASK counter_mask; /* counter mask on current system */
	 int avail_cpu_mask;     /* cpu mask for current system */

	 long OS_type;           /* OS_VMS, OS_NT, OS_OSF1 etc */
	 long OS_ver_major;      /* OS major version number */
	 long OS_ver_minor;      /* OS minor version number */

	 time_t start;       /* starting time in unix format */

	 int data_type;          /* sampling data , histogram data etc */
	 IPR_COUNT rangecnt;       /* number of ranges */
	 int null_proc_id;       /* Null process id for current system */

	 IPR_DMASK data;
	 IPR_DMASK dataquad;
}FHDR;

/* Variable part of the sample file header */

typedef IPR_EVENT FHact_event;
typedef IPR_FREQ FHact_freq;
typedef int FHskip;
typedef uint64  FHrange;

typedef char *evtname;

typedef struct _aliases
{
	char       *alias;
	IPR_EVENT        eventcode;
} EVNT_ALIAS;

#ifdef __VMS
#include "psldef.h"
#pragma member_alignment save

#pragma member_alignment
typedef union psldef PSL;
typedef union _psl_quad
{
   PSL xpsl;
   uint64 xquad;
} PSL_QUAD;

#pragma member_alignment restore

#define DATE_TIME_FORMAT "|!DB-!MAAU-!Y4|!H04:!M0:!S0.!C4|"
#endif

extern int
   buffer_count,
   missed_sample_count,
   events_requested;

extern int
   img_arr_size,
   gbl_arr_size,
   max_img_arr_size,
   max_gbl_arr_size,
   MaxRecordCount,
   MinRecordCount;

extern uint64
string_to_date (char *s);

extern char
*date_to_string (uint64 v);

extern void
parse_PS (DREC *pRec, FHDR *pFH);

extern unsigned int
GetModeSelection(char *p);

extern char *parse_data_area (IPR_SBUFFER *pBH, DREC *pRec, char *pBuf);

#ifdef WIN32
HANDLE
open_data_file (char *FileName, FHDR *pHdr);

int
get_hdr_var(HANDLE sFile, void *ptr, int size);

IPR_STATUS
get_record (HANDLE sfile, DREC *pRecord, FHDR *pHdr);
#else
FILE *
open_data_file (char *FileName, FHDR *pHdr);

int
get_hdr_var(FILE *sFile, void *ptr, int size);

IPR_STATUS
get_record (FILE *sfile, DREC *pRecord, FHDR *pHdr);
#endif
#endif // ifndef __IPREDUCE_LOADED
