/*++

Copyright (c) 1993  Digital Equipment Corporation

Module Name:

    perfev4.h

Abstract:

    Private include file for the Alpha AXP 21064 performance counter driver.

Author:

    Joe Notarangelo  26-Jun-1993

Environment:

    Kernel mode only.

Notes:


Revision History:

   Brad Waters          25-Oct-1994
   Added more info. and removed obsolete fields in device extension.

   Brad Waters          1-Sep-1994
   Added info. to the device extension for the IPROBE API.

--*/

#ifndef _PERFEV4_
#define _PERFEV4_

//
// Include the DECchip 21064 specific include file.
//

#include "axp21064.h"


//
// Define the device extension for the performance counter driver.
//

typedef struct _DEVICE_EXTENSION{

    //
    // The current counter value for the performance counter.
    //

    AXPCOUNTER CounterValue;

    //
    // The performance counter number.
    //

    AXP21064_PCCOUNTER Counter;

    //
    // The indicator that defines if the counter is currently started.
    //

    BOOLEAN Started;

    //
    // The current mux control for the performance counter.
    //

    AXP21064_PCMUXCONTROL MuxControl;

    //
    // The current number of events counted per interrupt.
    //

    AXP21064_PCEVENTCOUNT EventCount;

    //
    // Pointer to the interrupt object for the performance counter.
    //

    PKINTERRUPT InterruptObject;

    //
    // The system-specific interrupt information used to connect and
    // disconnect the interrupt.
    //

    ULONG Vector;
    KIRQL Irql;
    KAFFINITY Affinity;
    KSPIN_LOCK *ISRSpin;            // spin lock for ISR

    uint64 product_ident;              // Product identification
    int majorVersion;
    int minorVersion;

    void *wrtStruct;
    uint64 intCntS;
    uint64 intCntE;
    LIST_ENTRY sampQ;           // queue of buffers for DPC
    KSPIN_LOCK sampSpin;        // spin lock for DPC
    void *dpcBlock;
    unsigned int dropInt;

    int flags;
    int connected;

// Temporary vars.

    unsigned int cpu;

//    void *bdptr;

    unsigned int icount;
    DEVICE_OBJECT *DeviceObject;        // Device Object for DpcForIsr

    KTRAP_FRAME *ktrap_stackf;
    KPCR
      *cpuDB;
   IPR_BUFDSC
      *BufDscPtr;
   VOID *stackptr;
   IPR_DpcBlock *Context;
   PKTHREAD curThrd;
   int *curProc;

} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

//
// Private types.
//

//
// Enumerated type of symbolic values for the event counts, these are
// the actual values written to the ICCSR.
//

typedef enum _EVENT_COUNT{
    EventCountLow=0,
    EventCountHigh=1
} EVENT_COUNT, *PEVENT_COUNT;

//
// Enumerate type of the symbolic values for enable/disable of the
// performance counter as specified by the PALcode.
//

typedef enum _COUNTER_CONTROL{
    CounterDisable = 0,
    CounterEnable = 1
} COUNTER_CONTROL, *PCOUNTER_CONTROL;

//
// Prototypes for exported driver routines.
//

BOOLEAN
PerformanceCounterInterrupt(
    PKINTERRUPT InterruptObject,
    PDEVICE_EXTENSION DeviceExtension,
    PKTRAP_FRAME ktrap_stackf
    );

BOOLEAN
PCInterruptAddFrame(
    PKINTERRUPT InterruptObject, 
    PDEVICE_EXTENSION DeviceExtension
    );

#if DBGPSUP

VOID
Axp21064DebugPrint(
    ULONG DebugPrintLevel,
    PCCHAR DebugMessage,
    ...
    );

#define AxpPrint(x) Axp21064DebugPrint x

#else

#define AxpPrint(x)

#endif //DBG


//
// Prototypes for support routines.
//

BOOLEAN
ValidateEventCount(
    PAXP21064_SETCOUNTER EventCountParameters,
    AXP21064_PCCOUNTER PerformanceCounter
    );

NTSTATUS
SetEventCount(
    PDEVICE_EXTENSION DeviceExtension,
    PAXP21064_SETCOUNTER EventCountParameters
    );

BOOLEAN
ValidateMuxControl(
    PAXP21064_SETMUXCONTROL MuxControlParameters,
    AXP21064_PCCOUNTER PerformanceCounter
    );

NTSTATUS
SetMuxControl(
    PDEVICE_EXTENSION DeviceExtension,
    PAXP21064_SETMUXCONTROL MuxControlParameters
    );

NTSTATUS
StartCounter(
    PDEVICE_EXTENSION DeviceExtension
    );

NTSTATUS
StopCounter(
    PDEVICE_EXTENSION DeviceExtension
    );

NTSTATUS
Axp21064ReadPalCounters(
    PAXP21064_RDPALCOUNTERS PalCounters
    );

ULONG
AxpReadPalCounters(
    PVOID CountersBuffer,
    ULONG BufferLength
    );

VOID
AxpWritePerformanceCounter(
    ULONG PerformanceCounter,
    ULONG Enable,
    ULONG MuxControl OPTIONAL,
    ULONG EventCount OPTIONAL
    );

PVOID
GetProcessorPCR(void
    );

PVOID
GetCurrentStackLoc(void
    );


#endif //_PERFEV4_
