/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the Windows NT DriverEntry routine
 *
 * Author:              Bradley M. Waters
 * Creation Date:       21-Jul-1994
 *
 * Modified by:
 *
 * X-5  BMW             Bradley M. Waters        8-Dec-1994
 *      Fixed a variety of problems prior to the 1st internal field test of
 *      IPROBE on Windows NT.
 *
 * X-4  BMW             Bradley M. Waters       25-Oct-1994
 *      Fixed nonpaged pool deallocations to insure that all pool is deallocated.
 *      changed the order of device object delete in Unload routine to insure
 *      that the write queue is completely terminated before the device objects
 *      are deleted.
 *
 * X-3  BMW             Bradley M. Waters       10-Oct-1994
 *      Cleanup of code
 *
 * X-2  BMW             Bradley M. Waters       31-Aug-1994
 *      Integrated Joe Notarangelo's driver that interfaces to the AXP perf.
 *      counters into the IPROBE kernel driver - previously, all counter
 *      manipulation actions were passed onto Joe'e PERFEV4 driver.
 *
 * X-1  BMW             Bradley M. Waters       21-Jul-1994
 *      Original Version.  Based much of the code in this module on
 *      Joe Notarangelo's EV4 counter driver, PERFEV4 - in fact, much
 *      of the code was simply moved from the PERFEV4.C module.
 *
*/

#include "stdio.h"

#include "ntddk.h"

#include "ipr_kernel_int_defs.h"
#include "ipr_driver_nt.h"
#include "hb_library.h"

#include "trpfrm.h"
#include "perfev4.h"
#include "ipr_isr_nt.h"
#include "ipr_isr_nt_def.h" //wagner

#include "TraceDrv.h"

typedef enum {StoreDeviceObject, RetrieveDeviceObject} DeviceObjectOperation;

ULONG HalpGet21064PerformanceVector(ULONG,PKIRQL);

extern void PCInterruptAddFrame(void);

KWAIT_REASON wrtReason = {Executive};
KPROCESSOR_MODE wrtMode = {KernelMode};
BOOLEAN wrtFail = {FALSE};      // Flag indicating whether write thread failed at startup
PKTHREAD wrtThread;             // Write thread

IPR_STATUS closeStatus;
IPR_STATUS openStatus;
IPR_STATUS writeStatus;

int skipHist = 0;
int addHist = 0;

extern HANDLE sampHandle;
extern UNICODE_STRING *sampFname;
extern UNICODE_STRING *sampDFname;

IPR_WrtStruct *wrtStruct = NULL;       // Pointer to structure containing
                                // spinlock, write queue, and events
                                // for writing sample buffers

// Header for write structure
IPR_BUFHDR write_template = {
	IPR_PROD_IDENT,
        0,                      // 2nd field (size) is filled in by allocation routine
        IPR_WBUFFER_TYPE,
        IPR_WBUFFER_SUBTYPE,
        IPR_WBUFFER_KIND
        };

extern void *GetCurThread(void);
extern void *GetCurTEB(void);

DEVICE_OBJECT *GetSetDeviceObject(DeviceObjectOperation op,
		                  unsigned long int ProcessorNumber,
                                  unsigned long int CounterNumber,
                                  DEVICE_OBJECT *DeviceObject);
NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

NTSTATUS
IprDispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
IprDeviceControl(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

VOID
IprUnload(
    IN PDRIVER_OBJECT DriverObject
    );


#ifdef ALLOC_PRAGMA
#pragma alloc_text(init,DriverEntry)
#pragma alloc_text(init,IprDispatch)
#pragma alloc_text(init,IprDeviceControl)
#pragma alloc_text(init,IprUnload)
#endif

void
WriteThread(DEVICE_EXTENSION *deviceExtension)
{
   IPR_STATUS status;
   IO_STATUS_BLOCK ioStat;
   LONG oldPri;
   IPR_WrtStruct *wrtStruct;
   IPR_WrtList *listEntry;

   HANDLE sampThHandle = 0;
   int64 allocSize = {32768};
   ULONG initAttrb = {OBJ_CASE_INSENSITIVE | OBJ_INHERIT};
   ULONG fileAttrb = {FILE_ATTRIBUTE_NORMAL};
   ULONG shareAcc = {0};
   ULONG createDisp = {FILE_SUPERSEDE};
//   ULONG createOpt = {FILE_NON_DIRECTORY_FILE /*| FILE_SEQUENTIAL_ONLY */ /* | FILE_SYNCHRONOUS_IO_ALERT*/};
//   ACCESS_MASK accMask = {GENERIC_READ | GENERIC_WRITE /*| SYNCHRONIZE*/};
   ULONG createOpt = {FILE_NON_DIRECTORY_FILE};
   ACCESS_MASK accMask = {GENERIC_READ | GENERIC_WRITE};
   int64 bytePos = {0};
   int64 transfSize;
   OBJECT_ATTRIBUTES sampAttrb;

   if (!deviceExtension)
   {
      wrtFail = TRUE;
      PsTerminateSystemThread(IPR__NULLPTR);
   }

   wrtStruct = deviceExtension->wrtStruct;
   wrtThread = KeGetCurrentThread();    // Store thread handle in global

   // Boost priority a bit
   oldPri = KeSetBasePriorityThread(wrtThread, +5);

   while (wrtStruct->wrtActive)
   {
      // Wait until a buffer becomes full - alertable wait with no timeout

      status = KeWaitForSingleObject(&wrtStruct->wrtEvent,
         wrtReason,wrtMode,TRUE,NULL);

      if (status != STATUS_SUCCESS)
      {
          continue;  // Wait for next event
      }
      // Check to see if the thread is active, take appropriate action
      if (wrtStruct->wrtActive)
      {
          // Grab 1st buffer to write from write queue
	  ExInterlockedVerifyList(&wrtStruct->wrtQ);
          (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
             &wrtStruct->wrtQ_SpinLock);
	  ExInterlockedVerifyList(&wrtStruct->wrtQ);
          while (listEntry)
          {
             // bufptr field is either an action (open or close) or
             // the pointer of a buffer to write
             if ((int)listEntry->bufptr == IPR_OPENWRITE)
             {
                // Open sample file

                // If sample file was already open - probably due to an
                // abnormal termination - close it before opening a new file
                if (sampThHandle != 0)
                {
                   closeStatus = ZwClose(sampThHandle);
                   sampThHandle = 0;
                   sampHandle = 0;
                }

                // Use only the file name for now (I'll implement the default file
                // name later).
                // NOTE: No root directory or security descriptor is passed
                InitializeObjectAttributes(&sampAttrb,sampFname,initAttrb,NULL,NULL);

                status = ZwCreateFile(&sampThHandle,accMask,&sampAttrb,&ioStat,(PLARGE_INTEGER)&allocSize,
                   fileAttrb,shareAcc,createDisp,createOpt,NULL,0);
                openStatus = status;
                // Init. file byte pointers and buffer write counts
                bytePos = 0;
                wrtStruct->buffwrites = 0;      // incremented for each buffer written (including partwrites)
                wrtStruct->partwrites = 0;
                if (!OK_STATUS(status))
                {
                }
                else if (!OK_STATUS(ioStat.Status))
                {
                   openStatus = ioStat.Status;
                }
                // Global sampHandle set to sampThHandle for benefit of write routine
                sampHandle = sampThHandle;
                BARRIER();   // Do MB to make sure global vars. are updated
                // Return memory for open entry
                if (listEntry->delEntry)
                   deallocate_memory(listEntry);
                // Signal caller that open is complete
                wrtStruct->wrtoState = KeSetEvent(&wrtStruct->wrtoEvent,0,FALSE);
             }
             else // davew
             if ((int)listEntry->bufptr == IPR_CLOSEWRITE)
             {
                if (sampThHandle)
                {
                   status = ZwClose(sampThHandle);
                   if (!OK_STATUS(status))
                   {
                   }
                   sampThHandle = 0;       // Clear out sampThHandle local
                   sampHandle = 0;         // Clear out sampHandle global
                   BARRIER();              // Do MB to make sure globals are updated
                }
                else
                   status = IPR__SUCCESS;
                closeStatus = status;
                // Return memory for close entry
                if (listEntry->delEntry)
                   deallocate_memory(listEntry);
             }
             else // davew
             if (((int)listEntry->bufptr != IPR_OPENWRITE) &&
                ((int)listEntry->bufptr != IPR_CLOSEWRITE))
             {
                // Make sure pointer is valid
                if (!listEntry->bufptr)
                {
                }
                else
                {
                   // Write entire buffer including the header
                   status = ZwWriteFile(sampThHandle,NULL,NULL,NULL,&ioStat,
                      listEntry->bufptr,listEntry->bsize,
                      (PLARGE_INTEGER)&bytePos,NULL);
                   writeStatus = status;
                   if (!OK_STATUS(status))
                   {
                   }
                   else if (!OK_STATUS(ioStat.Status))
                   {
                      writeStatus = ioStat.Status;
                   }
                   if (OK_STATUS(writeStatus))
                   {
                      // Update bytePos by number of bytes actually written to the file
                      transfSize = ioStat.Information;
                      bytePos += transfSize;
                      if (listEntry->bsize != ioStat.Information)
                      {
                         writeStatus = IPR__WRITESHORT;
                      }
                      if (listEntry->sampBuff)
                      {
                         wrtStruct->buffwrites++;
                         if ((listEntry->bufptr->areaoffset < listEntry->bufptr->areasize) ||
                          (writeStatus == IPR__WRITESHORT))
                            wrtStruct->partwrites++;
                         status = KIprSbSetFull(listEntry->bufptr,IPR_BUFFER_EMPTY);
                         if (!IPR_SUCCESS(status))
                         {
                         }
                      }
                   }

                   if (listEntry->delBuff)
                   {
                      // Delete buffer after write
                      deallocate_memory(listEntry->bufptr);
                   }
                }
                if (listEntry->delEntry)
                {
                   // Delete queue entry after write
                   deallocate_memory(listEntry);
                }
             }
             // Grab next buffer to write from write queue
             ExInterlockedVerifyList(&wrtStruct->wrtQ);
             (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
                &wrtStruct->wrtQ_SpinLock);
	     ExInterlockedVerifyList(&wrtStruct->wrtQ);

          }  // end - while
          // Reset signal to wait for next full buffer
          wrtStruct->wrtState = KeResetEvent(&wrtStruct->wrtEvent);
      }  // end - if active
   }  // end - while loop on active

   // done with this system thread, bump priority (to finish before caller
   // who may destroy the address space) and signal caller
   if (sampThHandle != 0)
       closeStatus = ZwClose(sampThHandle);
   sampHandle = 0;
   wrtStruct->wrtdState = KeSetEvent(&wrtStruct->wrtdEvent,0,FALSE);
   PsTerminateSystemThread(STATUS_SUCCESS);
}

//void
//WriteThread(DEVICE_EXTENSION *deviceExtension)
//{
//   IPR_STATUS status;
//   IO_STATUS_BLOCK ioStat;
//   LONG oldPri;
//   IPR_WrtStruct *wrtStruct;
//   IPR_WrtList *listEntry;
//
//   HANDLE sampThHandle = 0;
//   int64 allocSize = {32768};
//   ULONG initAttrb = {OBJ_CASE_INSENSITIVE | OBJ_INHERIT};
//   ULONG fileAttrb = {FILE_ATTRIBUTE_NORMAL};
//   ULONG shareAcc = {0};
//   ULONG createDisp = {FILE_SUPERSEDE};
//   ULONG createOpt = {FILE_NON_DIRECTORY_FILE /*| FILE_SEQUENTIAL_ONLY */ /* | FILE_SYNCHRONOUS_IO_ALERT*/};
//   ACCESS_MASK accMask = {GENERIC_READ | GENERIC_WRITE /*| SYNCHRONIZE*/};
//   int64 bytePos = {0};
//   int64 transfSize;
//   OBJECT_ATTRIBUTES sampAttrb;
//
//   if (!deviceExtension)
//   {
//      wrtFail = TRUE;
//      PsTerminateSystemThread(IPR__NULLPTR);
//   }
//
//   wrtStruct = deviceExtension->wrtStruct;
//   wrtThread = KeGetCurrentThread();    // Store thread handle in global
//
//   // Boost priority a bit
//   oldPri = KeSetBasePriorityThread(wrtThread, +5);
//
//   while (wrtStruct->wrtActive)
//   {
//      // Wait until a buffer becomes full - alertable wait with no timeout
//
//      status = KeWaitForSingleObject(&wrtStruct->wrtEvent,
//         wrtReason,wrtMode,TRUE,NULL);
//
//      if (status != STATUS_SUCCESS)
//      {
//          continue;  // Wait for next event
//      }
//      // Check to see if the thread is active, take appropriate action
//      if (wrtStruct->wrtActive)
//      {
//          // Grab 1st buffer to write from write queue
//          (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
//             &wrtStruct->wrtQ_SpinLock);
//          while (listEntry)
//          {
//             // bufptr field is either an action (open or close) or
//             // the pointer of a buffer to write
//             if ((int)listEntry->bufptr == IPR_OPENWRITE)
//             {
//                // Open sample file
//
//                // If sample file was already open - probably due to an
//                // abnormal termination - close it before opening a new file
//                if (sampThHandle != 0)
//                {
//                   closeStatus = ZwClose(sampThHandle);
//                   sampThHandle = 0;
//                   sampHandle = 0;
//                }
//
//                // Use only the file name for now (I'll implement the default file
//                // name later).
//                // NOTE: No root directory or security descriptor is passed
//                InitializeObjectAttributes(&sampAttrb,sampFname,initAttrb,NULL,NULL);
//
//                status = ZwCreateFile(&sampThHandle,accMask,&sampAttrb,&ioStat,(PLARGE_INTEGER)&allocSize,
//                   fileAttrb,shareAcc,createDisp,createOpt,NULL,0);
//                openStatus = status;
//                // Init. file byte pointers and buffer write counts
//                bytePos = 0;
//                wrtStruct->buffwrites = 0;      // incremented for each buffer written (including partwrites)
//                wrtStruct->partwrites = 0;
//                if (!OK_STATUS(status))
//                {
//                }
//                else if (!OK_STATUS(ioStat.Status))
//                {
//                   openStatus = ioStat.Status;
//                }
//                // Global sampHandle set to sampThHandle for benefit of write routine
//                sampHandle = sampThHandle;
//                BARRIER();   // Do MB to make sure global vars. are updated
//                // Return memory for open entry
//                if (listEntry->delEntry)
//                   deallocate_memory(listEntry);
//                // Signal caller that open is complete
//                wrtStruct->wrtoState = KeSetEvent(&wrtStruct->wrtoEvent,0,FALSE);
//             }
//             if ((int)listEntry->bufptr == IPR_CLOSEWRITE)
//             {
//                if (sampThHandle)
//                {
//                   status = ZwClose(sampThHandle);
//                   if (!OK_STATUS(status))
//                   {
//                   }
//                   sampThHandle = 0;       // Clear out sampThHandle local
//                   sampHandle = 0;         // Clear out sampHandle global
//                   BARRIER();              // Do MB to make sure globals are updated
//                }
//                else
//                   status = IPR__SUCCESS;
//                closeStatus = status;
//                // Return memory for close entry
//                if (listEntry->delEntry)
//                   deallocate_memory(listEntry);
//             }
//             if (((int)listEntry->bufptr != IPR_OPENWRITE) &&
//                ((int)listEntry->bufptr != IPR_CLOSEWRITE))
//             {
//                // Make sure pointer is valid
//                if (!listEntry->bufptr)
//                {
//                }
//                else
//                {
//                   // Write entire buffer including the header
//                   status = ZwWriteFile(sampThHandle,NULL,NULL,NULL,&ioStat,
//                      listEntry->bufptr,listEntry->bsize,
//                      (PLARGE_INTEGER)&bytePos,NULL);
//                   writeStatus = status;
//                   if (!OK_STATUS(status))
//                   {
//                   }
//                   else if (!OK_STATUS(ioStat.Status))
//                   {
//                      writeStatus = ioStat.Status;
//                   }
//                   if (OK_STATUS(writeStatus))
//                   {
//                      // Update bytePos by number of bytes actually written to the file
//                      transfSize = ioStat.Information;
//                      bytePos += transfSize;
//                      if (listEntry->bsize != ioStat.Information)
//                      {
//                         writeStatus = IPR__WRITESHORT;
//                      }
//                      if (listEntry->sampBuff)
//                      {
//                         wrtStruct->buffwrites++;
//                         if ((listEntry->bufptr->areaoffset < listEntry->bufptr->areasize) ||
//                          (writeStatus == IPR__WRITESHORT))
//                            wrtStruct->partwrites++;
//                         status = KIprSbSetFull(listEntry->bufptr,IPR_BUFFER_EMPTY);
//                         if (!IPR_SUCCESS(status))
//                         {
//                         }
//                      }
//                   }
//
//                   if (listEntry->delBuff)
//                   {
//                      // Delete buffer after write
//                      deallocate_memory(listEntry->bufptr);
//                   }
//                }
//                if (listEntry->delEntry)
//                {
//                   // Delete queue entry after write
//                   deallocate_memory(listEntry);
//                }
//             }
//             // Grab next buffer to write from write queue
//             (LIST_ENTRY *)listEntry = ExInterlockedRemoveHeadList(&wrtStruct->wrtQ,
//                &wrtStruct->wrtQ_SpinLock);
//          }  // end - while
//          // Reset signal to wait for next full buffer
//          wrtStruct->wrtState = KeResetEvent(&wrtStruct->wrtEvent);
//      }  // end - if active
//   }  // end - while loop on active
//
//   // done with this system thread, bump priority (to finish before caller
//   // who may destroy the address space) and signal caller
//   if (sampThHandle != 0)
//       closeStatus = ZwClose(sampThHandle);
//   sampHandle = 0;
//   wrtStruct->wrtdState = KeSetEvent(&wrtStruct->wrtdEvent,0,FALSE);
//   PsTerminateSystemThread(STATUS_SUCCESS);
//}

static IPR_BOOLEAN PCinRange(IPR_PBUFFER *PCrangeBuf,IN uint64 pc)
{
   register IPR_PC_RANGE
      *PCrange;
   register unsigned int i;
   register IPR_BOOLEAN found;

   // If no PC range buffer, than all samples are collected;
   if (!PCrangeBuf)
      return TRUE;

   if (PCrangeBuf->sample_always)
      return TRUE;

   PCrange = (IPR_PC_RANGE *)(PCrangeBuf + 1);

   found = FALSE;
   for (i = 0; i < PCrangeBuf -> rangecnt; i++)
      if (PCrange[i].sample)
      {
         if (pc >= PCrange [i].StartAddress && pc <= PCrange [i].EndAddress)
         {
            found = TRUE;
            break;
         }
      }
   return found;
}

IPR_BOOLEAN SkipInt(IPR_IBUFFER *SkipBufPtr, PDEVICE_EXTENSION DeviceExtension)
{
   register IPR_ISKIP
      *SkipPtr;

   // NOTE: Following code reorged.
   if (!SkipBufPtr)
      return FALSE;
   SkipPtr = (IPR_ISKIP *) (SkipBufPtr + 1);
   // Log a sample only if the skip counter has reached threshold
   if (SkipPtr [DeviceExtension->Counter] . intssincelast
      < SkipPtr [DeviceExtension->Counter] . intspersample)
   {
      SkipPtr [DeviceExtension->Counter] . intssincelast++;
      return TRUE;
   }
   else
   {
      SkipPtr [DeviceExtension->Counter] . intssincelast = 0;
      return FALSE;
   }
}

void DpcISR(IN PKDPC Dpc, IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp, IN IPR_DpcBlock *PassedContext)
{
   uint64
      datum;
   IPR_STATUS status;
   int hIndex;
   register int
      *DataLong;
   register uint64
      *DataQuad;
   register unsigned int dummy;
   PDEVICE_EXTENSION DeviceExtension;
   IPR_BOOLEAN diff_cpu = FALSE;
   IPR_DpcBlock *Context,*prevContext;
   IPR_BUFDSC *BufDscPtr;
//   uint64 curTime;


   if (!DeviceObject)
   {
      return;
   }
   DeviceExtension =
     (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;
   if (!DeviceExtension)
   {
      return;
   }



   Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
   Context += DeviceExtension->intCntE % IPR_DPC_BLOCKS;

   BufDscPtr = GETBDPTR(Context->cpu);

   if (BufDscPtr != Context->BufDscPtr)
   {
   }

   prevContext = NULL;
   while ((Context != prevContext) && (DeviceExtension->intCntE < DeviceExtension->intCntS))
   {
      // If pid is 0 and ps is a, assume mode is idle
      if ((Context->pid == 0) && (Context->ps == 0xa))
      {
         Context->mode = 2;
      }



      // NOTE: this DPC is associated with an ISR associated with
      // a particular CPU and counter.  The cpu, buffer descriptor
      // and counter fields in the context should never change
      // while draining the dpc queue.
      // However, this DPC can run on a different CPU than the ISR.
      if (!BufDscPtr)
      {
         return;
      }

      // Histogram buffers to fill in?
      if (BufDscPtr -> pHBuf)
      {
         //
         // CREATING HISTOGRAMS
         //

         register IPR_HISTELEM
            *DataElem;
         register IPR_HBUFFER
            *HistoPtr;

         //
         // NOTE: The code below assumes that the buffers are linear, i.e.
         //       that the last one points to null.
         //
         //       Also note that the data array is organized with all the elements
         //       corresponding to a single value are located contiguously.
         //       For example, an element sequence on a chip with three counters
         //       might be val 1 ctr 0, val 1 ctr 1, val 1 ctr 2, val 2 ctr 0,
         //       val 2 ctr 1, val 2 ctr 2, etc.
         //
         //       All subscripting is done manually.
         //
         HistoPtr = BufDscPtr -> pHBuf;
         while (HistoPtr)
         {

            DataElem = (IPR_HISTELEM *) (HistoPtr + 1);
            switch (HistoPtr -> item.ipr_hmask_l_bits )
            {
               case IPR_HMASK_M_TOTAL :
                  datum = 0;
                  break;
               case IPR_HMASK_M_IPL :
                  datum = Context -> ipl;
                  break;
               case IPR_HMASK_M_MODE :
                  datum = Context -> mode;
                  break;
               case IPR_HMASK_M_ADDRESS :
                  datum = Context -> pc;
                  break;
               case IPR_HMASK_M_PRIORITY :
                  datum = Context -> priority;
                  break;
               default :
                  datum = datum | (!datum);
                  break;
            }
            if (datum != 0xFFFFFFFF)
            {
               status = IprHbIndex (datum,
                  DeviceExtension->Counter,
                  BufDscPtr -> numcounters,
                  HistoPtr -> minval,
                  HistoPtr -> pcgran,
                  &hIndex);
               if ((IPR_SUCCESS(status)) && (status != IPR__NOTINHIST) &&
                (hIndex < HistoPtr->elemcnt))
               {
                  DataElem [hIndex]++;
                  addHist++;
               }
               else
               {
                  skipHist++;
               }
            }
            HistoPtr = HistoPtr -> pNext;
         }
      }

      // Sample buffers to fill in?
      if (BufDscPtr -> pSBuf)
      {
         ////////////////////////
         // COLLECTING SAMPLES //
         ////////////////////////

         register IPR_SBUFFER
            *BufPtr;
         register int
            marked_buffer;
         IPR_WrtStruct *wrtStruct;
         IPR_SBUFFER *wptr;
         IPR_BOOLEAN dropSamp;

         // Check interrupt skip count
         if (!SkipInt(BufDscPtr->pIBuf,DeviceExtension))
         {
            // Ignore the interrupt if the PC is out of the specified ranges and
            // sample always isn't set
            if (PCinRange(BufDscPtr->pPBuf,Context->pc))
            {

               dropSamp = FALSE;
               marked_buffer = FALSE;
               BufPtr = BufDscPtr -> pSBuf;
               if (BufPtr -> areaoffset >= BufPtr -> areasize)
               {
                  //
                  // The location is out of range, which means we have finished with
                  // this buffer.  Mark it and try to move on to the next buffer.
                  //
                  if (BufPtr -> areaoffset == BufPtr -> areasize)
                  {
                     BARRIER ();
                     BufPtr -> full = IPR_BUFFER_FULL;
                     BufPtr -> areaoffset++;
                     // Queue buffer for writing
                     do_full_action (BufPtr, BufDscPtr, DeviceExtension);
                     marked_buffer = TRUE;
                  }
                  //
                  // The current buffer is full.  Check if the next buffer can
                  // be used.
                  //
                  if (BufPtr -> pNext -> full == IPR_BUFFER_FULL)
                  {
                     // The next buffer is full, so count a missed sample
                     BufPtr -> missedcnt++;
                     // In case buffers somehow didn't get written add buffers to the write
                     // queue

                     wrtStruct = DeviceExtension->wrtStruct;

                     if IsListEmpty(&wrtStruct->wrtQ)
                     {
                        wptr = BufPtr->pNext;
                        BARRIER ();
                        BufPtr -> full = IPR_BUFFER_FULL;
                        BufPtr -> areasize+1;
                        do_full_action (BufPtr, BufDscPtr, DeviceExtension);
                        marked_buffer = TRUE;
                        while (wptr != BufPtr)
                        {
                           if (wptr->full == IPR_BUFFER_FULL)
                           {
                              wptr -> full = IPR_BUFFER_FULL;
                              wptr -> areasize+1;
                              do_full_action (wptr, BufDscPtr, DeviceExtension);
                           }
                           wptr = wptr->pNext;
                        }
                     }

                     dropSamp = TRUE;
                  }

                  if (!dropSamp)
                  {

                      // The next buffer is empty, so switch buffers

                      BufPtr = BufPtr -> pNext;
                      BufPtr -> begintime = Context->time;
                      BufPtr -> missedcnt = 0;
                      BufPtr -> areaoffset = 0;
                      BufDscPtr -> pSBuf = BufPtr;
                  }
               }   // buffer full check
               if (!dropSamp)
               {
                  //
                  // Save the requested information.  Start with the quadword data
                  // to allow alignment easily.
                  //
                  DataQuad = (uint64 *) ((char *) ((BufPtr) + 1) + BufPtr->areaoffset);

// Check quadword masks

                  if (BufPtr -> data . ipr_dmask_v_time)
                     if (BufPtr -> dataquad . ipr_dmask_v_time)
                        *DataQuad++ = Context->time;

                  if (BufPtr -> data . ipr_dmask_v_ps)
                     if (BufPtr -> dataquad . ipr_dmask_v_ps)
                        *DataQuad++ = Context->ps;

                  if (BufPtr -> data . ipr_dmask_v_pc)
                     if (BufPtr -> dataquad . ipr_dmask_v_pc)
                        *DataQuad++ = Context->pc;

                  if (BufPtr -> data . ipr_dmask_v_counter)
                     if (BufPtr -> dataquad . ipr_dmask_v_counter)
                        *DataQuad++ = Context->counter;

                  if (BufPtr -> data . ipr_dmask_v_pid)
                     if (BufPtr -> dataquad . ipr_dmask_v_pid)
                        *DataQuad++ = (uint64) Context->pid;

                  if (BufPtr -> data . ipr_dmask_v_priority)
                     if (BufPtr -> dataquad . ipr_dmask_v_priority)
                        *DataQuad++ = (uint64) Context->priority;
// Check the longword masks

                  DataLong = (int *) DataQuad;


                  if ((BufPtr -> data . ipr_dmask_v_time) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_time)))
                      *DataLong++ = (int) Context->time;

                  if ((BufPtr -> data . ipr_dmask_v_ps) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_ps)))
                      *DataLong++ = (int) Context->ps;

                  if ((BufPtr -> data . ipr_dmask_v_pc) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_pc)))
                      *DataLong++ = (int) Context->pc;

                  if ((BufPtr -> data . ipr_dmask_v_counter) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_counter)))
                      *DataLong++ = (int) Context->counter;

                  if ((BufPtr -> data . ipr_dmask_v_pid) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_pid)))
                      *DataLong++ = (int) Context->pid;

                  if ((BufPtr -> data . ipr_dmask_v_priority) &&
                    (!(BufPtr -> dataquad . ipr_dmask_v_priority)))
                      *DataLong++ = (int) Context->priority;
                  //
                  // Note: In the following section, the casts must be signed in order
                  // for the the "&" operation to work with OpenVMS addresses (the
                  // mask needs to be sign extended).
                  //

                  dummy = ((char *) DataLong + BufPtr->pad
                    - (char *) (BufPtr + 1));
                  // check to see if there is room in the buffer for another sample
                  BufPtr -> areaoffset = dummy;

               }  // end - dropSamp check
            }  // end - PCinRange
         }  // end - SkipInt
      }  // end - sample code
      DeviceExtension->intCntE++;
      BARRIER();        // Do a MB to make sure the end count is set
      prevContext = NULL;
      Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
      Context += DeviceExtension->intCntE % IPR_DPC_BLOCKS;
//      Context = (IPR_DpcBlock *)ExInterlockedRemoveHeadList(&DeviceExtension->sampQ,
//          &DeviceExtension->sampSpin);
   }  // end while

   return;

}

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    )
{
/*++

Routine Description:

    This routine initializes the IPROBE API kernel mode routines and the
    Axp 21064 performance counter driver.

Arguments:

    DriverObject - Pointer to driver object created by system.

    RegistryPath - Pointer to the Unicode name of the registry path
        for this driver.

Return Value:

    The function value is the final status from the initialization operation.

--*/

    IPR_BUFHDR *alloc_area;
    POOL_TYPE npp_type = NonPagedPool;

    EVENT_TYPE wrtType = {NotificationEvent};
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT deviceObject, deviceObject0;
    IPR_STATUS status;
    UNICODE_STRING unicodeString;

    KAFFINITY Affinity;
    KIRQL Irql;

    HANDLE wrtHand;

    WCHAR DeviceNameBuffer[80];
    ULONG ProcessorNumber,
          CounterNumber,
          MaxProcessors,
          ucnt,
          Vector,
          wrtAccess = {THREAD_ALL_ACCESS};

//TrcSetup();
//TrcPrint("Driver entry\n");
    // Do basic driver initialization such as finding the allocation
    // granularity and creating the NPP zones and list.

    // Affinity fields of sysdata will be set later after 1st Interrupt is connected...
    sysdata.allocation_granularity = 64;

    // Initialize nonpaged pool list

    for (ucnt=0; ucnt < IPR_MAX_NPP_BLOCKS; ucnt++)
      {
	npp_table.start[ucnt] = NULL;
      }

    //
    // Create nonpaged pool zones for nonpaged data structures
    //
    for (ucnt=0; ucnt < IPR_NUM_ZONES; ucnt++)
      {

	alloc_area = ExAllocatePool(npp_type, zinfo[ucnt].size);
	if (alloc_area == NULL)
	  {
		return IPR__ALLOCFAIL;
	  }
	else
	  {
		// Update NPP list to insure that all allocated pool is returned
		status = AddToPoolList(alloc_area);

		if (!IPR_SUCCESS(status))
		  {
			return status;
		  }

		zinfo[ucnt].start = alloc_area;
		status = ExInitializeZone(&zhdr[ucnt],
		zinfo[ucnt].granularity,
		alloc_area,
		zinfo[ucnt].size);

		if (!OK_STATUS(status))
		  {
			return IPR__ZONEFAIL;
		  }
	  }
      }

    // Create and initialize spinlocks, queues and events that will be
    // used by this driver.  This is done first because pointers to these
    // structures are included in the device extension so we can just pop
    // in the addresses as the devices are created.  All device extensions
    // point to the write structure (the counter device extensions need
    // this info. to signal buffer full conditions and the write thread
    // is setup based on the IPRKRNL device extension).

    status = allocate_memory(sizeof(IPR_WrtStruct),&write_template,&wrtStruct);

    if (!IPR_SUCCESS(status))
    {
	return status;
    }

    // Initialize spinlock for write queue interlocked operations.
    KeInitializeSpinLock(&wrtStruct->wrtQ_SpinLock);

    // Create queue for write buffers.
    InitializeListHead(&wrtStruct->wrtQ);

    // Create event that the write thread will use to wait for
    // writing buffers.  The initial state is FALSE - not signaled
    KeInitializeEvent(&wrtStruct->wrtEvent,wrtType,FALSE);
    KeInitializeEvent(&wrtStruct->wrtdEvent,wrtType,FALSE);
    KeInitializeEvent(&wrtStruct->wrtoEvent,wrtType,FALSE);

    // Write thread is created after 1st device object is created
    // (the device extension is passed to the Write thread)

    //
    // Create exclusive device object for the Iprobe API kernel
    // routines to use.
    //

    RtlInitUnicodeString(&unicodeString, DD_IPROBE_API_DEVICE_NAME_U);

    status = IoCreateDevice(
                DriverObject,           // driver object passed into this routine
                sizeof(DEVICE_EXTENSION), // driver defined extension size
                &unicodeString,         // Device named object
                FILE_DEVICE_IPRKERNEL,  // Driver defined device type
                0,                      // Device Characteristics
                TRUE,                   // Exclusive access
                &deviceObject           // Device object created
                );

    if (status != STATUS_SUCCESS)
    {
        return(status);
    }

    deviceObject->Flags |= DO_BUFFERED_IO;

    //
    // Fill in the device extension.
    //

    deviceExtension =
        (PDEVICE_EXTENSION)deviceObject->DeviceExtension;

    deviceExtension -> product_ident = IPR_PROD_IDENT;
    deviceExtension -> majorVersion = IPR_KRNL_MAJOR_VER;
    deviceExtension -> minorVersion = IPR_KRNL_MINOR_VER;
    deviceExtension -> flags = 0;
    deviceExtension -> connected = FALSE;
    deviceExtension->icount = 0;
    deviceExtension->Counter = (AXP21064_PCCOUNTER)0xffffffff;


    // API thread (valid cpu range
    // is 0 to IPR_MAX_CPUS-1)

    deviceExtension->wrtStruct = wrtStruct;

    // Create system write thread

    wrtStruct->wrtActive = TRUE;        // Indicate to write system thread that
                                        // it is active.
    status = PsCreateSystemThread(&wrtHand,wrtAccess,NULL,NULL,NULL,
       WriteThread,deviceExtension);
    if( status != STATUS_SUCCESS ){
        return status;
    }

    sysdata.affinity_cpu_mask = 0;

for (CounterNumber = 0; CounterNumber < 3; CounterNumber++)
  {
	Vector = HalGetInterruptVector (ProcessorInternal, 0, CounterNumber, CounterNumber, &Irql, &Affinity);
	sysdata.affinity_cpu_mask |= Affinity;

	ProcessorNumber = 0;

	while (Affinity != 0)
	  {
		if (Affinity & (1 << ProcessorNumber) == 0)
	  	  {
			ProcessorNumber++;
			continue;
		  }

		//
		// Create exclusive device object for performance counter.
		// The device can be opened by only one thread at a time.
		//
		swprintf( DeviceNameBuffer,
                  L"%sP%dPC%d",
                  DD_AXP21064_PERFORMANCE_DEVICE_NAME_U,
                  ProcessorNumber,
		  CounterNumber);

		RtlInitUnicodeString(&unicodeString, DeviceNameBuffer);

		status = IoCreateDevice(
                  DriverObject,
                  sizeof(DEVICE_EXTENSION),
                  &unicodeString,
                  FILE_DEVICE_AXP21064,
                  0,
                  TRUE,
                  &deviceObject0);

		if (status != STATUS_SUCCESS)
		  {
			return(status);      // may want to provide more robust error handling here - to get rid
					     // devices created for preceding processors (when on an MP)
		  }

		GetSetDeviceObject(StoreDeviceObject,
                                   ProcessorNumber,
				   CounterNumber,
				   deviceObject0);

		deviceObject0->Flags |= DO_BUFFERED_IO;

		//
		// Fill in the device extension.
		//

		deviceExtension = (PDEVICE_EXTENSION)deviceObject0->DeviceExtension;
		deviceExtension->DeviceObject = deviceObject0;

		// Initialize spinlock for sample write interlocked operations.
		KeInitializeSpinLock(&deviceExtension->sampSpin);

		// Create queue for DPC buffers.
		InitializeListHead(&deviceExtension->sampQ);

		status = allocate_memory(sizeof(IPR_DpcBlock)*IPR_DPC_BLOCKS,
                  NULL,
                  &deviceExtension->dpcBlock);

		if (!IPR_SUCCESS(status))
		  {
			return(status);
		  }

		IoInitializeDpcRequest(deviceObject0,DpcISR);

		deviceExtension -> product_ident = IPR_PROD_IDENT;
		deviceExtension -> majorVersion = IPR_KRNL_MAJOR_VER;
		deviceExtension -> minorVersion = IPR_KRNL_MINOR_VER;
		deviceExtension -> flags = 0;
		deviceExtension -> connected = FALSE;
		deviceExtension -> cpu = ProcessorNumber;
		deviceExtension->icount = 0;
		deviceExtension->CounterValue = 0;
		deviceExtension->Counter = CounterNumber;
		deviceExtension->MuxControl = ExternalCounter0;
		deviceExtension->EventCount = CountEvents2xx16;
		deviceExtension->wrtStruct = wrtStruct;
		deviceExtension->intCntS = 0;
		deviceExtension->intCntE = 0;
		deviceExtension->dropInt = 0;
		deviceExtension->Vector = Vector;
		deviceExtension->Irql = Irql;
		deviceExtension->Affinity = 1 << ProcessorNumber;

		skipHist = 0;
		addHist = 0;

		// Initialize spinlock for ISR coordination
		status = allocate_memory(sizeof(KSPIN_LOCK),
                  NULL,
                  &deviceExtension->ISRSpin);
		if (!IPR_SUCCESS(status))
		  {
			return(status);
		  }

		KeInitializeSpinLock(deviceExtension->ISRSpin);

		//
		// Capture the necessary interrupt parameters from the HAL and then
		// connect the interrupt for PC0.
		//
		status = IoConnectInterrupt( &deviceExtension->InterruptObject,
                (PKSERVICE_ROUTINE)PCInterruptAddFrame,
                (PVOID)deviceExtension,
                deviceExtension->ISRSpin,
                Vector,
                Irql,
                Irql,
                Latched,
                TRUE,
                (KAFFINITY) (1 << ProcessorNumber),
                FALSE);

		if( status != STATUS_SUCCESS )
		  {
			IoDeleteDevice(deviceObject0);
			return status;
		  }

		//
		// Now, disconnect the interrupt!  We connect above to make sure that
		// everything is working.  Due to quirks in the way the interrupts are
		// enabled they will only be enabled when the device is opened.
		// Mark the connection count to zero.
		//
		IoDisconnectInterrupt( deviceExtension->InterruptObject );

		deviceExtension->Started = FALSE;

		Affinity &= ~(1 << ProcessorNumber);
		ProcessorNumber++;

	  } //End of while (affinity != 0)

  }//End of for (CounterNumber = 0; CounterNumber < 3; CounterNumber++)

    sysdata.noaffinity_cpu_mask = sysdata.affinity_cpu_mask;

    //
    // Set up the device driver entry points.
    //

    DriverObject->DriverUnload = IprUnload;
    DriverObject->MajorFunction[IRP_MJ_CREATE] = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_CLOSE]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_READ]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_WRITE]  = IprDispatch;
    DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] =
                                             IprDeviceControl;

    return(STATUS_SUCCESS);

}

NTSTATUS
IprDeviceControl(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )

/*++

Routine Description:

    This routine is the dispatch routine for device control requests.

    The following device controls are recognized for the performance counter
    part of the driver as defined by Joe Notarangelo:

    IOCTL_AXP21064_RDPALCOUNTERS - Read PALcode Event Counters
    IOCTL_AXP21064_SETCOUNTER - Set events counted per pc interrupt
    IOCTL_AXP21064_SETMUXCONTROL - Set mux control for performance counter
    IOCTL_AXP21064_STARTCOUNTER - Set the counter started (start interrupting)
    IOCTL_AXP21064_STOPCOUNTER - Set the counter stopped (stop interrupting)

    The following device controls are recognized for the IPROBE API part of
    the performance counter driver:

    IOCTL_IPR_GetChipInfo - Return chip info
    IOCTL_IPR_UpdatePERFMON - Change PERFMON settings
    IOCTL_IPR_GetCurrentIpid - Return current internal PID
    IOCTL_IPR_SetAffinity - Update processor affinity
    IOCTL_IPR_SetPme - Update PME bit
    IOCTL_IPR_ResetIpl - Execute PALcode IPL code path (to propagate PME bit)
    IOCTL_IPR_InitSysInfo - Initialize system info. in globals
    IOCTL_IPR_DoneSysInfo - Dealloctes system info. such as nonpaged pool
    IOCTL_IPR_OpenSampFile - Open sample file
    IOCTL_IPR_CloseSampfile - Close sample file
    IOCTL_IPR_WriteFile - Write a buffer to the sample file

    IOCTL_IPR_BdCreate - Buffer descriptor create
    IOCTL_IPR_BdDelete - Buffer descriptor delete
    IOCTL_IPR_BdSetNumCounters - Set number of perf. counters
    IOCTL_IPR_BdSetAction - Setup action to occur at particular time
    IOCTL_IPR_BdSetActiveCounters - Set active counter mask
    IOCTL_IPR_BdAddPcRange - Add a PC range
    IOCTL_IPR_BdSetSkipCount - Set number of interrupts to skip between samples

    IOCTL_IPR_SbCreate - Sample buffer create
    IOCTL_IPR_SbDelete - Sample buffer delete
    IOCTL_IPR_SbSetFull - Set full bit for sample buffer
    IOCTL_IPR_SbGetInfo - Returns info. about the sample buffer
    IOCTL_IPR_SbFind - Returns pointer to sample buffer

    IOCTL_IPR_HbElements - Returns number of elements required for given histogram
       parameters
    IOCTL_IPR_HbIndex - Returns array index for the specified value and counter for
       given histogram parameters
    IOCTL_IPR_HbValue - Return value at given histogram index
    IOCTL_IPR_HbMaxElements - Return maximum number of histogram elements
    IOCTL_IPR_HbCreate - Create histogram buffer
    IOCTL_IPR_HbDelete - Delete histogram buffer
    IOCTL_IPR_HbFind - Return next histogram buffer that matches given criteria
    IOCTL_IPR_HbRead -  Read given elements from histogram buffer
    IOCTL_IPR_HbClear - Clear given elements in histogram buffer

Arguments:

    DeviceObject - Pointer to class device object.

    Irp - Pointer to the request packet.

Return Value:

    Status is returned.

--*/
{
    PDEVICE_EXTENSION deviceExtension;
    uint64 *EventCountParameters, *MuxControlParameters;
    PIO_STACK_LOCATION irpSp;
    NTSTATUS status;


    //
    // Get a pointer to the current parameters for this request.  The
    // information is contained in the current stack location.
    //

    irpSp = IoGetCurrentIrpStackLocation(Irp);

    //
    // Case on the device control subfunction that is being performed by the
    // requestor.
    //

    switch (irpSp->Parameters.DeviceIoControl.IoControlCode) {

// NOTE:  For a performance optimization, the data passed in to the
// various dispatched routines could be sent directly to the function.
// Currently the data is moved to variable of types the function expects
// to improve the readability of the code and provide additional
// error checks.

// The following 5 cases were pulled out of the PERFEV4 driver

    //
    // Read the PALcode event counters, if possible.  It is possible that
    // the executing PALcode does not have the counters implemented.
    //

    case IOCTL_AXP21064_RDPALCOUNTERS:

        if( irpSp->Parameters.DeviceIoControl.OutputBufferLength <
            sizeof(AXP21064_RDPALCOUNTERS) ){
            status = IPR__INVPARAM;
        } else {
            status = Axp21064ReadPalCounters(
                      (PAXP21064_RDPALCOUNTERS)Irp->AssociatedIrp.SystemBuffer);
        }

        if( status == STATUS_SUCCESS ){
            Irp->IoStatus.Information = sizeof(AXP21064_RDPALCOUNTERS);
        } else {
        Irp->IoStatus.Information = 0;
        }
        break;

    //
    // Set the number of events per interrupt (a.k.a. event count).
    //

    case IOCTL_AXP21064_SETCOUNTER:

	deviceExtension = DeviceObject->DeviceExtension;
        EventCountParameters = (uint64 *)Irp->AssociatedIrp.SystemBuffer;

        if(irpSp->Parameters.DeviceIoControl.InputBufferLength != sizeof(uint64 *))
          {
		status = IPR__INVPARAM;
	  } 
	else
	  {
                status = SetEventCount(deviceExtension, EventCountParameters);
          }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Set the mux control for the performance counter, the mux control
    // defines the type of event that is counted
    //

    case IOCTL_AXP21064_SETMUXCONTROL:

        deviceExtension = DeviceObject->DeviceExtension;
        MuxControlParameters = (uint64 *)Irp->AssociatedIrp.SystemBuffer;


        if(irpSp->Parameters.DeviceIoControl.InputBufferLength != sizeof(uint64 *))
	  {
            status = IPR__INVPARAM;
          } 
	else
	  {
            status = SetMuxControl(deviceExtension, MuxControlParameters);
        }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Start the counter, if the counter is already started then
    // consider the command a success.
    //

    case IOCTL_AXP21064_STARTCOUNTER:

        deviceExtension = DeviceObject->DeviceExtension;
        if( deviceExtension->Started == TRUE ){
            status = STATUS_SUCCESS;
        } else {
            // Reset the interrupt/DPC counts
            deviceExtension->intCntS = 0;
            deviceExtension->intCntE = 0;
            deviceExtension->dropInt = 0;
            status = StartCounter( deviceExtension );
        }

        Irp->IoStatus.Information = 0;
        break;

    //
    // Stop the counter, if the counter is already stopped then
    // consider the command a success.
    //

    case IOCTL_AXP21064_STOPCOUNTER:
    {

        deviceExtension = DeviceObject->DeviceExtension;
//        if( deviceExtension->Started != TRUE ){
//            status = STATUS_SUCCESS;
//        } else {
            status = StopCounter( deviceExtension );
//        }

       Irp->IoStatus.Information = 0;
         break;
     }
//  All the rest of the cases are defined for the IPROBE API

//
// General routine shells
//

    case IOCTL_IPR_GetChipInfo:
    {
        int alen,olen;
        IPR_CHIPINFO *retptr;

        // OutputBuffer and length is structure to put chip info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_CHIPINFO)) ||
                ( olen != sizeof(IPR_CHIPINFO)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            retptr = Irp->AssociatedIrp.SystemBuffer;   // Input buffer
            status = KIprGetChipInfo(retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_UpdatePerfmon:
    {
        int alen,olen;
        IPR_ARGUPerfmon *palist;
        // OutputBuffer is used to contain Ipid
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGUPerfmon)) ||
                (olen != sizeof(IPR_ARGUPerfmon)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprUpdatePerfmon(palist->cpu,palist->numctrsupd,
                palist->counterblock);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_GetCurrentIpid:
    {
        int *ipid,alen,olen;

    // NOTE:  Currently, this dispatch should not occur - the ipid is
    // obtained via a user mode call in the shell
        // OutputBuffer is used to contain Ipid
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(int)) ||
                (olen != sizeof(int)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            ipid = (int *) Irp->AssociatedIrp.SystemBuffer;
            status = KIprGetCurrentIpid(ipid);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SetAffinity:
    {
        int alen,olen;
        IPR_ARGAffinity *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGAffinity)) ||
           ( olen != sizeof(IPR_ARGAffinity)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSetAffinity(palist->cpu,palist->setting);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SetPme:
    {
        int alen,olen;
        IPR_ARGPme *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGPme)) ||
           ( olen != sizeof(IPR_ARGPme)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSetPme(palist->pid,
               palist->state,
               palist->children,
               palist->prev_state);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_ResetIpl:
    {
        status = KIprResetIpl();

        Irp->IoStatus.Information = 0;
        break;

    case IOCTL_IPR_InitSysInfo:
    {
        int alen,olen;
        IPR_SYSTEM_DATA *newsysdata;

        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_SYSTEM_DATA)) ||
           ( olen != sizeof(IPR_SYSTEM_DATA)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           newsysdata = Irp->AssociatedIrp.SystemBuffer;
           status = KIprInitSysInfo(newsysdata);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_SYSTEM_DATA);
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }

    case IOCTL_IPR_DoneSysInfo:
    {

        status = KIprDoneSysInfo();

        Irp->IoStatus.Information = 0;
        break;
    }

    case IOCTL_IPR_OpenSampFile:
    {
        int alen,olen;
        IPR_ARGOpenSamp *palist;

        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGOpenSamp)) ||
           ( olen != sizeof(IPR_ARGOpenSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;
           status = KIprOpenSampFile(palist->fname,
                palist->defname);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_ARGOpenSamp);
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }
    case IOCTL_IPR_CloseSampFile:
    {
        int alen,olen;
        IPR_ARGCloseSamp *palist;

        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGCloseSamp)) ||
           ( olen != sizeof(IPR_ARGCloseSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;
           status = KIprCloseSampFile(palist->buffwrites,palist->partwrites);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = alen;
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }
    case IOCTL_IPR_WriteFile:
    {
        int alen,olen;
        IPR_ARGWriteSamp *palist;

        // OutputBuffer and length is structure to put init. info. into
        alen = irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if(( alen != sizeof(IPR_ARGWriteSamp)) ||
           ( olen != sizeof(IPR_ARGWriteSamp)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        }
        else
        {
           palist = Irp->AssociatedIrp.SystemBuffer;

           status = KIprWriteFile(palist->bptr,
                palist->bsize);
           if (IPR_SUCCESS(status))
           {
              Irp->IoStatus.Information = sizeof(IPR_ARGWriteSamp);
           }
           else
              Irp->IoStatus.Information = 0;
        }
        break;
    }

//
// Buffer descriptor Shells
//
    }
    case IOCTL_IPR_BdCreate:
    {
        int alen,olen;
        IPR_ARGBDCreate *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDCreate)) ||
           ( olen != sizeof(IPR_ARGBDCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdCreate(palist->cpu,palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdDelete:
    {
        int *cpu,clen,olen;

        // InputBuffer and Length is cpu

        cpu = Irp->AssociatedIrp.SystemBuffer;
        clen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        if( (clen != sizeof(int)) || (olen != sizeof(int)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdDelete(*cpu);
            Irp->IoStatus.Information = clen;
        }
        break;
    }
    case IOCTL_IPR_BdSetNumCounters:
    {
        int alen,olen;
        IPR_ARGBDNumctrs *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDNumctrs)) ||
           ( olen != sizeof(IPR_ARGBDNumctrs)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdSetNumCounters(palist->cpu,palist->numctrs);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdSetAction:
    {
        int alen,olen;
        IPR_ARGBDAction *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDAction)) ||
           ( olen != sizeof(IPR_ARGBDAction)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdSetAction(palist->cpu,
               palist->action_type,
               palist->action,
               palist->targetroutine,
               palist->targetpid);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_BdSetActiveCounters:
    {
        int alen,olen;
        IPR_ARGBDActctrs *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDActctrs)) ||
           ( olen != sizeof(IPR_ARGBDActctrs)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdSetActiveCounters(palist->cpu,palist->ctrmask);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdAddPcRange:
    {
        int alen,olen;
        IPR_ARGBDPcrng *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDPcrng)) ||
           ( olen != sizeof(IPR_ARGBDPcrng)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprBdAddPcRange(palist->cpu,palist->pcrange);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_BdSetSkipCount:
    {
        int alen,olen;
        IPR_ARGBDSkip *palist;

       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if(( alen != sizeof(IPR_ARGBDSkip)) ||
           ( olen != sizeof(IPR_ARGBDSkip)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = KIprBdSetSkipCount(palist->cpu,
               palist->ctr,
               palist->skipvalue);
            Irp->IoStatus.Information = alen;
        }
        break;

//
// Sample Buffer shells
//
    }
    case IOCTL_IPR_SbCreate:
    {
        int alen,olen;
        IPR_ARGSBCreate *palist;

        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBCreate)) ||
           (olen != sizeof(IPR_ARGSBCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbCreate(palist->cpu,
               palist->size,
               palist->numbuffs,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_SbDelete:
    {
        int alen,olen;
        IPR_ARGSBDelete *palist;
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBDelete)) ||
           (olen != sizeof(IPR_ARGSBDelete)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbDelete(palist->cpu,palist->bufptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbSetFull:
    {
        int alen,olen;
        IPR_ARGSBSetFull *palist;

        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBSetFull)) ||
           (olen != sizeof(IPR_ARGSBSetFull)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbSetFull(palist->bufptr,palist->full_val);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbGetInfo:
    {
        int alen,olen;
        IPR_ARGSBGetInfo *palist;

        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBGetInfo)) ||
           (olen != sizeof(IPR_ARGSBGetInfo)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbGetInfo(palist->bufptr,palist->descrip);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_SbFind:
    {
        int alen,olen;
        IPR_ARGSBFind *palist;
        // InputBufferLength is argument list length
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGSBFind)) ||
           (olen != sizeof(IPR_ARGSBFind)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprSbFind(palist->cpu,
               palist->descrip,
               palist->previous_buffer,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
//
// Histogram Buffer Shells
//

    case IOCTL_IPR_HbElements:
    {
        int alen,olen;
        IPR_ARGHBElements *palist;

       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBElements)) ||
            (olen != sizeof(IPR_ARGHBElements)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = IprHbElements(palist->numctrs,
              palist->numval,
              palist->gran,
              palist->numelem);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbIndex:
    {
        int alen,olen;
        IPR_ARGHBIndex *palist;

       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBIndex *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBIndex)) ||
           (olen != sizeof(IPR_ARGHBIndex)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = IprHbIndex(palist->value,
               palist->ctr,
               palist->numctrs,
               palist->minval,
               palist->gran,
               palist->index);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbValue:
    {
        int alen,olen;
        IPR_ARGHBValue *palist;

        // InputBuffer and Length is argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBValue *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBValue)) ||
           (olen != sizeof(IPR_ARGHBValue)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = IprHbValue(palist->index,
               palist->numctrs,
               palist->minval,
               palist->gran,
               palist->value);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbCreate:
    {
        int alen,olen;
        IPR_ARGHBCreate *palist;
        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBCreate *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBCreate)) ||
           (olen != sizeof(IPR_ARGHBCreate)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbCreate(palist->cpu,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbDelete:
    {
        int alen,olen;
        IPR_ARGHBDelete *palist;


        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBDelete *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBDelete)) ||
           (olen != sizeof(IPR_ARGHBDelete)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbDelete(palist->cpu, palist->bufptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbFind:
    {
        int alen,olen;
        IPR_ARGHBFind *palist;

        // InputBufferLength is length of the argument list
        // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBFind *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBFind)) ||
           (olen != sizeof(IPR_ARGHBFind)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbFind(palist->cpu,
               palist->previous_buffer,
               palist->descrip,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }
        break;
    }
    case IOCTL_IPR_HbRead:
    {
        int alen,olen;
        IPR_ARGHBRead *palist;

        // InputBuffer and Length is argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = (IPR_ARGHBRead *) Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBRead)) ||
           (olen != sizeof(IPR_ARGHBRead)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbRead(palist->cpu,
               palist->bufptr,
               palist->start,
               palist->cnt,
               palist->retptr);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    case IOCTL_IPR_HbClear:
    {
        IPR_ARGHBClear *palist;
        int alen,olen;

       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBClear)) ||
           (olen != sizeof(IPR_ARGHBClear)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {

            status = KIprHbClear(palist->cpu,
               palist->bufptr,
               palist->start,
               palist->cnt);
            Irp->IoStatus.Information = 0;
        }

        break;
    }
    case IOCTL_IPR_HbMaxElements:
    {
        IPR_ARGHBMaxElem *palist;
        int alen,olen;

       // InputBufferLength is length of the argument list
       // InputBuffer is the argument list

        alen = (int) irpSp->Parameters.DeviceIoControl.InputBufferLength;
        olen = irpSp->Parameters.DeviceIoControl.OutputBufferLength;
        palist = Irp->AssociatedIrp.SystemBuffer;

        if( (alen != sizeof(IPR_ARGHBMaxElem)) ||
           (olen != sizeof(IPR_ARGHBMaxElem)))
        {
            status = IPR__INVPARAM;
            Irp->IoStatus.Information = 0;
        } else {
            status = KIprHbMaxElements(palist->elem);
            Irp->IoStatus.Information = alen;
        }

        break;
    }
    //
    // Unrecognized device control request.
    //

    default:
        status = IPR__INVPARAM;
        break;

    }

    //
    // Mark the status and complete the request.
    //

    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    return(status);

}

NTSTATUS
IprDispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )
/*++

Routine Description:

    This routine is the dispatch for the IPROBE API kernel routines.
    The routine will field open/close requests as well as reads
    and writes.

Arguments:

    DeviceObject - Pointer to class device object.

    Irp - Pointer to the the I/O request packet.

Return Value:

    The status of the caller's requested operation is returned, generally
    for this driver that status will be success if the request is an
    expected one for the device.

--*/
{
    PDEVICE_EXTENSION deviceExtension;
    ULONG information;
    PIO_STACK_LOCATION irpSp;
    NTSTATUS status;


    //
    // Get the current Irp stack location to retrieve the function codes
    // and parameters.
    //

    irpSp = IoGetCurrentIrpStackLocation( Irp );

    //
    // Perform the requested operation based on the major function.
    //

    switch( irpSp->MajorFunction ){

    //
    // Create/Open the device, start the counter.
    //

    case IRP_MJ_CREATE:

        // Create function here
        deviceExtension = DeviceObject->DeviceExtension;
        deviceExtension->connected = TRUE;
        deviceExtension->CounterValue = 0;
        deviceExtension->icount = 0;
//        StartCounter( deviceExtension );
        status = STATUS_SUCCESS;
        information = 0;
        break;

    //
    // Close the device by disabling the counter.
    //

    case IRP_MJ_CLOSE:

        // Close function here
        deviceExtension = DeviceObject->DeviceExtension;
        deviceExtension->connected = FALSE;
//        StopCounter( DeviceObject->DeviceExtension );
        status = STATUS_SUCCESS;
        information = 0;
        break;

    //
    // Read the counter.
    //

    case IRP_MJ_READ:

        if( irpSp->Parameters.Read.Length != sizeof(AXPCOUNTER) ){
            status = IPR__INVPARAM;
            information = 0;
        } else {
            deviceExtension = DeviceObject->DeviceExtension;
            *(AXPCOUNTER *)(Irp->AssociatedIrp.SystemBuffer) =
                    deviceExtension->CounterValue;
            status = STATUS_SUCCESS;
            information = sizeof(AXPCOUNTER);
        }
        break;

    //
    // Write the counter.
    //

    case IRP_MJ_WRITE:

        if( irpSp->Parameters.Write.Length != sizeof(AXPCOUNTER) ){
            status = IPR__INVPARAM;
            information = 0;
        } else {
            deviceExtension = DeviceObject->DeviceExtension;
            deviceExtension->CounterValue =
                    *(AXPCOUNTER *)(Irp->AssociatedIrp.SystemBuffer);
            status = STATUS_SUCCESS;
            information = sizeof(AXPCOUNTER);
        }
        break;

    } //end switch( irpSp->MajorFunction )

    Irp->IoStatus.Status = status;
    Irp->IoStatus.Information = information;
    IoCompleteRequest( Irp, IO_NO_INCREMENT );


    return status;

}

VOID
IprUnload(
    IN PDRIVER_OBJECT DriverObject
    )
/*++

Routine Description:

    This routine unloads the Alpha IPROBE API kernel performance counter driver.

Arguments:

    DriverObject - Pointer to class driver object.

Return Value:

    None.

--*/
{
    PDEVICE_OBJECT deviceObject;
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT nextDeviceObject;
    IPR_STATUS status;

    IPR_WrtStruct *wrtStruct;
    int cnt;
    KEVENT unloadWait;
    int64 waitTime;


    // Get the device objects and extensions - device objects
    // are chained in the driver object in reverse order from
    // that in which they were created.

    deviceObject = DriverObject->DeviceObject;
    deviceExtension = deviceObject->DeviceExtension;

    // Save the address of the write structure so the spinlock, queues,
    // and events associated with writing can be deleted.

    wrtStruct = deviceExtension->wrtStruct;

    // Disconnect counter interrupts

    while (deviceObject)
    {
       deviceExtension =  deviceObject->DeviceExtension;
       if (deviceExtension->Counter != 0xffffffff)
       {

          //
          // Disconnect the counter interrupt objects for the driver.
          //

          if( deviceExtension->Started == TRUE )
          {
             IoDisconnectInterrupt( deviceExtension->InterruptObject );
          }
       }
       deviceObject = deviceObject->NextDevice;
    }
    // Stop the write thread.  No priority increment and wait
    // after the event is set to allow the system thread to terminate.
    // NOTE: wrtStruct was grabbed above before we got rid of the
    // device extensions.

    if ((!wrtFail) && (wrtStruct))
    {
       if (wrtStruct->wrtActive)
       {
          wrtStruct->wrtActive = FALSE; // Indicate to write thread that
                                        // it is to terminate.
          wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,TRUE);

          // Wait while system thread runs down - alertable wait with no timeout,
          // The write thread bumps its priority to allow it to run down completely
          // before control is returned here and the address space is trashed.
          status = KeWaitForSingleObject(&wrtStruct->wrtdEvent,
             wrtReason,wrtMode,TRUE,NULL);
//             wrtReason,wrtMode,TRUE,&timeOut);

          if ((status != STATUS_SUCCESS) && (status != STATUS_ALERTED)
             && (status != STATUS_TIMEOUT))
          {
          }
// Wait for 1 sec. to make sure that write thread is completely done
          KeInitializeEvent(&unloadWait,SynchronizationEvent,FALSE);
          waitTime = -1000000;  // wait for 1/10 second
          status = KeWaitForSingleObject(&unloadWait,Executive,KernelMode,TRUE,(PLARGE_INTEGER)&waitTime);
          if (status != STATUS_TIMEOUT)
          {
          }
       }
    }

    //
    // Delete the write structure
    //

    if (wrtStruct)
    {
       deallocate_memory(wrtStruct);
       wrtStruct = NULL;
       deviceExtension->wrtStruct = NULL;
    }
    if (writeWait)
    {
       deallocate_memory(writeWait);
       writeWait = NULL;
    }

    // Delete the device objects and DPC queue blocks

    deviceObject = DriverObject->DeviceObject;

    while (deviceObject)
    {
       deviceExtension =  deviceObject->DeviceExtension;
       if (deviceExtension->Counter == 0xffffffff)
       {
          //
          // Delete the IPRKRNL device object.
          //
          nextDeviceObject = deviceObject->NextDevice;
          IoDeleteDevice(deviceObject);
          deviceObject = nextDeviceObject;
       }
       else
       {
          // Counter device, delete DPC queue blocks and object

          //
          // Delete the DPC queue blocks
          //

          if (deviceExtension->dpcBlock)
          {
             deallocate_memory(deviceExtension->dpcBlock);
             deviceExtension->dpcBlock = NULL;
          }

          //
          // Delete the counter device objects.
          //

          nextDeviceObject = deviceObject->NextDevice;
          IoDeleteDevice(deviceObject);
          deviceObject = nextDeviceObject;
       }
    }
    //
    // Clean up any NPP that was inadvertantly left undeleted.
    //

    //
    // Delete nonpaged pool zones for nonpaged data structures
    //

    for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
    {
       if (npp_table.start[cnt] != NULL)
       {
          ExFreePool(npp_table.start[cnt]);
          npp_table.start[cnt] = NULL;
       }
    }

    return;
}

BOOLEAN PerformanceCounterInterrupt(PKINTERRUPT InterruptObject,
  PDEVICE_EXTENSION DeviceExtension,
  PKTRAP_FRAME ktrap_stackf)
//
//
//Routine Description:
//
//    This routine is the interrupt handler for performance counter
//    interrupts.  The interrupt is handled by incrementing the count
//    value for the counter by the current event count.
//
//    N.B. - The interrupt has already been acknowledged by the PALcode.
//
//Arguments:
//
//    InterruptObject - Supplies a pointer to the interrupt object for this
//                          interrupt.
//
//    DeviceExtension - Supplies a pointer to the device extension for the
//                          performance counter that is interrupting.
//
//Return Value:
//
//    TRUE is returned.
//
//
  {

	if (!DeviceExtension)
	  {
		return TRUE;
	  }

	DeviceExtension->icount++;   // Count number of interrupts

	DeviceExtension->stackptr = GetCurrentStackLoc();

	DeviceExtension->ktrap_stackf = ktrap_stackf;

	if (!DeviceExtension->connected)
	  {
		return TRUE;
	  }


	DeviceExtension->cpuDB = THIS_CPU ();         // Get Processor PCR (call_pal rdpcr)
	DeviceExtension->curThrd = DeviceExtension->cpuDB->Prcb->CurrentThread;
	DeviceExtension->curProc = (int *)IoGetCurrentProcess();

	DeviceExtension->BufDscPtr = GETBDPTR(DeviceExtension->cpuDB->Number);

	if (!DeviceExtension->BufDscPtr)
	  {
		return TRUE;              // If no buffer descriptor, only compatibility action (above) is done
	  }

	if (!(DeviceExtension->BufDscPtr -> validcounters & (1 << DeviceExtension->Counter)))
	  {
		return TRUE;
	  }

	// Action from old PERFEV4 driver, retain this action for compatibility
	DeviceExtension->CounterValue += DeviceExtension->EventCount;

	// Fill in data structure passed to Dpc - this info. will allow the Dpc
	// to provide the necessary info. for processing the histogram or
	// sample for this interrupt.

	DeviceExtension->Context = (IPR_DpcBlock *)DeviceExtension->dpcBlock;
	DeviceExtension->Context += (DeviceExtension->intCntS % IPR_DPC_BLOCKS);

	if (((DeviceExtension->intCntS - DeviceExtension->intCntE) > IPR_DPC_BLOCKS) &&
	  (DeviceExtension->intCntE < DeviceExtension->intCntS))
	  {
		// More than IPR_DPC_BLOCKS queued to DPC for writing, drop info. from this interrupt
		DeviceExtension->dropInt++;
		if ((DeviceExtension->dropInt % 10) == 0)
		  {
			// Make sure the DPC is triggered to drain the queue of samples
			IoRequestDpc(DeviceExtension->DeviceObject,
			  DeviceExtension->DeviceObject->CurrentIrp,
			  NULL);
		  }
	  }
	else
	  {
		DeviceExtension->intCntS++;
		DeviceExtension->Context->cpu = THIS_CPU_NUM();
		DeviceExtension->Context->BufDscPtr = DeviceExtension->BufDscPtr;
		DeviceExtension->Context->time = THIS_TIME();
		DeviceExtension->Context->ps = THIS_PS();
		DeviceExtension->Context->pc = THIS_PC();
		DeviceExtension->Context->counter = DeviceExtension->Counter;
		DeviceExtension->Context->pid = THIS_PID();
		DeviceExtension->Context->priority = THIS_PRIORITY();
		DeviceExtension->Context->ipl = THIS_IPL();
		DeviceExtension->Context->mode = THIS_MODE();
		BARRIER();        // Do an MB here to make sure that the count and context area is updated

		// Pass context to DpcForIsr if necessary
		if (((DeviceExtension->icount % IPR_DPC_FREQ) == 0))
		  {
			IoRequestDpc(DeviceExtension->DeviceObject,
			  DeviceExtension->DeviceObject->CurrentIrp,
			  DeviceExtension->Context);
		  }
	  }
	return TRUE;
  }

DEVICE_OBJECT *GetSetDeviceObject(DeviceObjectOperation op,
		                 unsigned long int ProcessorNumber,
                                 unsigned long int CounterNumber,
                                 DEVICE_OBJECT *DeviceObject)
  {
	static DEVICE_OBJECT *Devs[32][3] = {{NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL}, 
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}, {NULL, NULL, NULL},
                                             {NULL, NULL, NULL}, {NULL, NULL, NULL}};
	DEVICE_OBJECT *pDO = DeviceObject;

	switch (op)
	  {
		case StoreDeviceObject:
			Devs[ProcessorNumber][CounterNumber] = pDO;
			break;

		case RetrieveDeviceObject:
			pDO = Devs[ProcessorNumber][CounterNumber];
			break;

		default:
			pDO = NULL;
			break;
	  }

	return pDO;
  }
