/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1997 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
/*
 * Abstract: This file contains the supporting definitions for source module
 *           NtReadPdb.c
 *
 *
 * Author: Jim Goddard
 *
 * Creation Date: 3-Jul-1997
 *
 * Modified by:
 *
 * 	Jim Goddard		3-Jul-1994
 *		Original Version. 
 *
*/

#define WIN32_LEAN_AND_MEAN 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <imagehlp.h>
#include "oemdbi.h"

#ifdef TRUE
#undef TRUE
#endif
#ifdef FALSE
#undef FALSE
#endif

typedef enum 
{
	TRUE = (1 == 1),
	FALSE = (1 == 2)
} boolean;

#define NUL '\0'

/****************************************************************************************************************/
/*														*/
/* Type: debug_record												*/
/*														*/
/* Purpose: Defines the layout of the debug record found in an executable image.				*/
/*														*/
/* Notes:													*/
/*	o This is used in lieu of an 'official' struct from MS that defines the layout of the debug record.	*/
/*	  When (if?) MS publishes such a layout then use it instead of this.					*/
/*														*/
/****************************************************************************************************************/
typedef struct
{
	char DebugType[4];		// either 'NB10' or 'NB11'
	unsigned long int Offset;	// always zero
	SIG Signature;			// PDB's signature'
	AGE Age;			// PDB's age
	char PdbPath[_MAX_PATH];	// PDB path string...length is indeterminate
} debug_record;

/****************************************************************************************************************/
/*														*/
/* Type: primary_reason												*/
/*														*/
/* Purpose: Defines the primary reason codes used by the PDB processing functions (IprNtPdb.c) to describe	*/
/*          final status conditions.										*/
/*														*/
/* Notes:													*/
/*	o What the primary codes mean and associated secondary codes:						*/
/*														*/
/*		PRC_SUCCESS - success, no errors, no secondary reason						*/
/*														*/
/*		PRC_OPEN - failed to open the executable image, secondary reason is the errno			*/
/*														*/
/*		PRC_MAP - failed to map the image's debug information, secondary reason is GetLastError()	*/
/*														*/
/*		PRC_SEEK - failed to seek to a position within the image, secondary reason is errno		*/
/*														*/
/*		PRC_READ - failed to read from the image file, secondary reason is errno			*/
/*														*/
/*		PRC_NO_PDB - this image has no associated PDB, no secondary reason				*/
/*														*/
/*		PRC_ALLOCATION - failed to dynamically allocate a buffer, no secondary reason			*/
/*														*/
/*		PRC_DBI_INTERFACE - failed to open the PDB/DBI interfaces or access the image's section	table.	*/
/*				    If the former then the secondary reason is PdbQueryLastError(); otherwise,	*/
/*				    its GetLastError().								*/
/*														*/
/*		PRC_QUERY_SYMBOLS - querying a modules symbols failed, secondary reason is PdbQueryLastError()	*/
/*														*/
/****************************************************************************************************************/
typedef enum
{
	PRC_SUCCESS,			// this is an NB10 or later image and the pdb_info block
					// contains valid information
	PRC_OPEN,			// can't open image
	PRC_MAP,			// can't map debug information protion of image
	PRC_SEEK,			// seeking a debug record failed
	PRC_READ,			// failed to read a debug record
	PRC_NO_PDB,			// this is not an NB10 or later image
	PRC_ALLOCATION,			// failed to allocate a temporary buffer
	PRC_DBI_INTERFACE,		// can't open the DBI API interface or load the image
	PRC_QUERY_SYMBOLS,		// can't query symbols
	PRC_1ST_CALLBACK_REASON	= 1000	// first reason available for callback functions
} primary_reason;

/****************************************************************************************************************/
/*														*/
/* Type: reason_code												*/
/*														*/
/* Purpose: Defines the layout of the reason code								*/
/*														*/
/* Notes: none													*/
/*														*/
/****************************************************************************************************************/
typedef struct
{
	primary_reason Primary;
	unsigned long int Secondary;
} reason_code;

/****************************************************************************************************************/
/*														*/
/* Type: pdb_info												*/
/*														*/
/* Purpose: Defines the layout of the information returned by GetPdbInformation()				*/
/*														*/
/* Notes: none													*/
/*														*/
/****************************************************************************************************************/
typedef struct
{
	boolean Valid;			// TRUE if the information in this block is valid
	SIG Signature;			// PDB's signature
	AGE Age;			// PDB's age
	char PdbPath[_MAX_PATH];	// PDB path string
	char *BaseAddress;		// image base address
	reason_code Reason;		// final condition of operation
} pdb_info;

boolean GetPdbInformation(char *pImageName, pdb_info *pPdbInformation);
void DumpMemory(void *pMemory, unsigned long int SizeToDump);
char *DecodePdbErrorCode(EC ErrorCode, char *ErrorString);
char * RecordTypeToString(SYM_ENUM_e RecordType, char *TypeBuffer);
void PrintSymbol(PROCSYM32 *pProcessRecord, void *VirtualAddress);
void *MapSymbol(PROCSYM32 *pProcessRecord, char *ImageBaseAddress, LOADED_IMAGE *pImageHeader);
boolean GetSymbolsFromPdb(char *pImageName, boolean (*pSymbolCallBack)(char *pModuleName, char *pSymbolName, char *SymbolVirtualAddress, unsigned long int SymbolLength, reason_code *Reason), reason_code *Reason);
boolean PdbSymbolCallBack(char *pModuleName, char *pSymbolName, char *SymbolVirtualAddress, unsigned long int SymbolLength, reason_code *Reason);
