/*++

Copyright (c) 1993  Digital Equipment Corporation

Module Name:

    dda21064.h

Abstract:

    This is the include file that defines all constants and types for
    the Alpha AXP 21064 performance counter device.

Author:

    Joe Notarangelo  24-Jun-1993

Revision History:

    Brad Waters      30-Aug-1994
        Temporarily restored DD_AXP21064_PCx_DEVICE_NAME definitions from
        earlier version of this file using new formats for MP devices.

    Joe Notarangelo  05-Feb-1994
        Add support for controlling the external counter selects for the
        21066.

N.B. This driver and include file are useful on Alpha systems only!

--*/

#ifndef _DDA21064_
#define _DDA21064_

//
// Base Device Name - this string serves as the basis for each I/O device
// name created for the performance counters.
//
// The actual device names are of the following format:
//
// \\Device\Axp21064PxPCy
//
// where x = processor number
//       y = performance counter number
//
// eg. processor 0, performance counter 0
//
// \\Device\Axp21064P0PC0
//

#define DD_AXP21064_PERFORMANCE_DEVICE_NAME "\\Device\\Axp21064"
#define DD_AXP21164_PERFORMANCE_DEVICE_NAME "\\Device\\Axp21164"
#define DD_AXP21064_PERFORMANCE_DEVICE_NAME_U L"\\Device\\Axp21064"

//
// Device Name - this string is the name of the device.  It is the name
// that should be passed to NtOpenFile when accessing the device.
//

//#define DD_AXP21064_PC0_DEVICE_NAME    "\\Device\\Axp21064P0PC0"
//#define DD_AXP21064_PC0_DEVICE_NAME_U L"\\Device\\Axp21064P0PC0"

//#define DD_AXP21064_PC1_DEVICE_NAME    "\\Device\\Axp21064P0PC1"
//#define DD_AXP21064_PC1_DEVICE_NAME_U L"\\Device\\Axp21064P0PC1"


//
// Define processor constants.
//

#define MAXIMUM_PROCESSORS (32)
#define PROCESSOR_0 (0)

//
// Define the file device type for the Axp 21064 driver.
//
// N.B. - Number is selected to be above Microsoft reserved range.
//

#define FILE_DEVICE_AXP21064  (21064+20000)

//
// NtDeviceIoControlFile IoControlCode values for this device.
//
// Warning:  Remember that the low two bits of the code specify how the
//           buffers are passed to the driver!
//

#define IOCTL_AXP21064_RDPALCOUNTERS \
        CTL_CODE(FILE_DEVICE_AXP21064, 0, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_AXP21064_SETCOUNTER \
        CTL_CODE(FILE_DEVICE_AXP21064, 1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_AXP21064_SETMUXCONTROL \
        CTL_CODE(FILE_DEVICE_AXP21064, 2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_AXP21064_STARTCOUNTER \
        CTL_CODE(FILE_DEVICE_AXP21064, 3, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_AXP21064_STOPCOUNTER \
        CTL_CODE(FILE_DEVICE_AXP21064, 4, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_AXP21066_EXTERNALSELECT \
        CTL_CODE(FILE_DEVICE_AXP21064, 5, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Types for the device.
//

//
// Type of the counter value.
//

typedef ULONGLONG AXPCOUNTER;
typedef AXPCOUNTER *PAXPCOUNTER;

//
// Types of performance counters.
//

typedef enum _AXP21064_PCCOUNTER{
        PerformanceCounter0,
        PerformanceCounter1,
        PerformanceCounter2
} AXP21064_PCCOUNTER, *PAXP21064_PCCOUNTER;

//
// Mux control values
//

typedef enum _AXP21064_PCMUXCONTROL{
        TotalIssues = 0x0,
        PipelineDry = 0x2,
        LoadInstruction = 0x4,
        PipelineFrozen = 0x6,
        BranchInstructions = 0x8,
        PalMode = 0xb,
        TotalCycles = 0xa,
        TotalNonIssues = 0xc,
        ExternalCounter0 = 0xe,
        DcacheMiss = 0x0,
        IcacheMiss = 0x1,
        DualIssues = 0x2,
        BranchMispredicts = 0x3,
        FPInstructions = 0x4,
        IntegerOperate = 0x5,
        StoreInstructions = 0x6,
        ExternalCounter1 = 0x7
} AXP21064_PCMUXCONTROL, *PAXP21064_PCMUXCONTROL;

//
// Counter control values.
//

typedef enum _AXP21064_PCEVENTCOUNT{
        CountEvents2xx8 = 0x100,
        CountEvents2xx12 = 0x1000,
        CountEvents2xx16 = 0x10000
} AXP21064_PCEVENTCOUNT, *PAXP21064_PCEVENTCOUNT;

#if 0 //jnfix - delete
//
// Define the type for the 21066 external counters.
//

typedef struct _AXP21066_EXTERNALCOUNTER{
        ExternalCounter0 = 0,
        ExternalCounter1 = 1
} AXP21066_EXTERNALCOUNTER, *PAXP21066_EXTERNALCOUNTER;

#endif //0

//
// Define counter select values for the external counters of
// the 21066.
//

typedef enum _AXP21066_EXTERNALSELECT{
        BCacheReads = 0x0,
        BCacheWrites = 0x1,
        DRAMReads = 0x2,
        DRAMWrites = 0x3,
        DRAMPageModes = 0x4,
        DRAMPageModeMisses = 0x5,
        GraphicsReads = 0x6,
        GraphicsWrites = 0x7,
        BCacheHits = 0x0,
        BCacheCleanMisses = 0x1,
        BCacheDirtyMisses = 0x2,
        BCacheHitsAndDMA = 0x3,
        BCacheMissesAndDMA = 0x4,
        CPUMaskedWrites = 0x5,
        DMAMaskedWrites = 0x6,
        MemoryControllerIdle = 0x7
} AXP21066_EXTERNALSELECT, *PAXP21066_EXTERNALSELECT;

//
// Structure passed for ioctl = IOCTL_AXP21064_RDPALCOUNTERS,
// data is returned in structure.
//

typedef struct _AXP21064_RDPALCOUNTERS{
        AXPCOUNTER MachineCheckCount;
        AXPCOUNTER ArithmeticExceptionCount;
        AXPCOUNTER InterruptCount;
        AXPCOUNTER ItbMissCount;
        AXPCOUNTER NativeDtbMissCount;
        AXPCOUNTER PalDtbMissCount;
        AXPCOUNTER ItbAccessViolationCount;
        AXPCOUNTER DtbAccessViolationCount;
        AXPCOUNTER UnalignedAccessCount;
        AXPCOUNTER IllegalInstructionCount;
        AXPCOUNTER FloatingEnableExceptionCount;
        AXPCOUNTER ItbTranslationNotValidCount;
        AXPCOUNTER DtbTranslationNotValidCount;
        AXPCOUNTER PteMissCount;
        AXPCOUNTER PalKernelStackMissCount;
        AXPCOUNTER PdeTranslationNotValidCount;
        AXPCOUNTER HaltCount;
        AXPCOUNTER RestartCount;
        AXPCOUNTER DrainAbortCount;
        AXPCOUNTER InitPalCount;
        AXPCOUNTER WrentryCount;
        AXPCOUNTER SwapIrqlCount;
        AXPCOUNTER ReadIrqlCount;
        AXPCOUNTER DisableInterruptCount;
        AXPCOUNTER EnableInterruptCount;
        AXPCOUNTER SwapPalcodeCount;
        AXPCOUNTER SetSoftwareInterruptCount;
        AXPCOUNTER ClearSoftwareInterruptCount;
        AXPCOUNTER ReturnFromExceptionCount;
        AXPCOUNTER ReturnFromSystemCallCount;
        AXPCOUNTER SwapContextCount;
        AXPCOUNTER SwapProcessCount;
        AXPCOUNTER ReadMcesCount;
        AXPCOUNTER WriteMcesCount;
        AXPCOUNTER TbInvalidateAllCount;
        AXPCOUNTER TbInvalidateSingleCount;
        AXPCOUNTER DtbInvalidateSingleCount;
        AXPCOUNTER ReadKernelStackCount;
        AXPCOUNTER SwapKernelStackCount;
        AXPCOUNTER ReadProcessorStatusCount;
        AXPCOUNTER ReadPcrCount;
        AXPCOUNTER ReadThreadCount;
        AXPCOUNTER ReadCountersCount;
        AXPCOUNTER ReadStateCount;
        AXPCOUNTER WritePerfmonCount;
        AXPCOUNTER InitPcrCount;
        AXPCOUNTER UserBreakpointCount;
        AXPCOUNTER SystemCallCount;
        AXPCOUNTER ImbCount;
        AXPCOUNTER GenerateTrapCount;
        AXPCOUNTER ReadTebCount;
        AXPCOUNTER KernelBreakpointCount;
        AXPCOUNTER CallKernelDebuggerCount;
        AXPCOUNTER TbisAsnCount;
        AXPCOUNTER Misc1Count;
        AXPCOUNTER Misc2Count;
        AXPCOUNTER Misc3Count;
} AXP21064_RDPALCOUNTERS, *PAXP21064_RDPALCOUNTERS;

//
// Structure passed for ioctl = IOCTL_AXP21064_SETCOUNTER,
// data is input for control.
//

typedef struct _AXP21064_SETCOUNTER{
        AXP21064_PCEVENTCOUNT EventCount;
} AXP21064_SETCOUNTER, *PAXP21064_SETCOUNTER;

//
// Structure passed for ioctl =  IOCTL_AXP21064_SETMUXCONTROL,
// data is input for control.
//

typedef struct _AXP21064_SETMUXCONTROL{
        AXP21064_PCMUXCONTROL MuxControl;
} AXP21064_SETMUXCONTROL, *PAXP21064_SETMUXCONTROL;

//
// Structure passed for ioctl = IOCTL_AXP21066_EXTERNALSELECT
//

typedef struct _AXP21066_EXTERNALSELECTION{
        AXP21066_EXTERNALSELECT ExternalSelect;
} AXP21066_EXTERNALSELECTION, *PAXP21066_EXTERNALSELECTION;

#endif // _DDA21064_
