#ifndef __EV4_ROUTINES_LOADED
#define __EV4_ROUTINES_LOADED 1
/*
 * Module:	EV4_ROUTINES.H
 *
 * Version:	X-6
 */

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1993, 1994 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
 * Abstract:	This header file contains EV4-specific definitions used
 * 		by the EV4 performance counter support code.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-6	BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-5	BFW1051		Brian F. Wibecan	13-Mar-1994
 *	Fixed what I left out last time.
 *
 * X-4	BFW1051		Brian F. Wibecan	13-Mar-1994
 *	Moved terminator key for MUX lists to PERFMONDEF.
 *
 * X-3	BFW1035		Brian F. Wibecan	08-Mar-1994
 *	Modified EV4SHARED to include array of events for each counter.
 *
 * X-2	???		Brian F. Wibecan	???
 *	Added EV$SHARED typedef.
 *
 * X-1	BFW1000		Brian F. Wibecan	16-Dec-1993
 *	Original version.
 *
*/

#include "ctr_int_defs.h"

typedef enum
{
   //
   // MUX selections for counter 0
   //
   EV4_MUX0_ISSUES,              // Total issues divided by 2
   EV4_MUX0_UNUSED_1,            // Unused value
   EV4_MUX0_PIPELINE_DRY,        // Nothing issued, no valid Istream data
   EV4_MUX0_UNUSED_3,            // Unused value
   EV4_MUX0_LOAD_INSTRUCTIONS,   // All load instructions
   EV4_MUX0_UNUSED_5,            // Unused value
   EV4_MUX0_PIPELINE_FROZEN,     // Nothing issued, resource conflict
   EV4_MUX0_UNUSED_7,            // Unused value
   EV4_MUX0_BRANCH_INSTRUCTIONS, // All brances (cond, uncond, JSR, HW_REI)
   EV4_MUX0_UNUSED_9,            // Unused value
   EV4_MUX0_CYCLES,              // Total cycles
   EV4_MUX0_PAL_MODE,            // Cycles while in PAL mode
   EV4_MUX0_NON_ISSUES,          // Total non-issues divided by 2
   EV4_MUX0_UNUSED_13,           // Unused value
   EV4_MUX0_EXTERNAL,            // External event supplied by pin
   EV4_MUX0_UNUSED_15,           // Unused value
   EV4_MUX0_TOTAL
} EV4_MUX0;

typedef enum
{
   //
   // MUX selections for counter 1
   //
   EV4_MUX1_DCACHE_MISSES,       // Total Dcache misses
   EV4_MUX1_ICACHE_MISSES,       // Total Icache misses
   EV4_MUX1_DUAL_ISSUE_CYCLES,   // Cycles of dual issue
   EV4_MUX1_BRANCH_MISPREDICTS,  // Branch mispredicts (cond, JSR, HW_REI)
   EV4_MUX1_FP_INSTRUCTIONS,     // FP operate instr (not br, load, store)
   EV4_MUX1_INTEGER_OPERATE,     // Integer op (incl lda, ldah into R0-R30)
   EV4_MUX1_STORE_INSTRUCTIONS,  // Total store instructions
   EV4_MUX1_EXTERNAL,            // External event supplied by pin
   EV4_MUX1_TOTAL
} EV4_MUX1;

//
// Counter 0 frequency selections
//
#define EV4_FREQ0_LOW 16
#define EV4_FREQ0_HIGH 12

//
// Counter 1 frequency selections
//
#define EV4_FREQ1_LOW 12
#define EV4_FREQ1_HIGH 8

#define EV4_COUNTER_COUNT 2
#define EV4_COUNTER_MASK 3
#define EV4_ADJUSTED_COUNT 1

typedef struct _ev4_context_block
   {
      IPR_MMASK modes;
      int all_specified;
      uint64 muxvalue;
      CTRCTXDATA counter [EV4_COUNTER_COUNT];
   } ev4_context_block;

typedef struct {
   ev4_context_block
      context [CONTEXT_AREA_COUNT];
   IPR_EVENTSET
      ctr_events [EV4_COUNTER_COUNT],
      adjusted_events [EV4_ADJUSTED_COUNT];
   IPR_FMASK
      funcs;
   IPR_MMASK
      modes;
   int
      adjusted_shifts [EV4_ADJUSTED_COUNT];
   IPR_FREQ
      ctr_freqs [EV4_COUNTER_COUNT],
      ctr_freq_hi [EV4_COUNTER_COUNT],
      ctr_freq_lo [EV4_COUNTER_COUNT];
   IPR_EVENT
      events_0 [EV4_MUX0_TOTAL],
      events_1 [EV4_MUX1_TOTAL];
} EV4SHARED;

// OS specific routines
extern uint64
   ev4_get_mux (CTRCTXDATA *cctx);
extern IPR_STATUS
   Ev4SetModes (int ctx, IPR_MMASK modes),
   Ev4SetEvents (int ctx, uint64 *MUXvalue),
   Ev4Enable (int ctx, IPR_CTRMASK ctrs),
   Ev4Disable (int ctx, IPR_CTRMASK ctrs);
extern void
   Ev4OSInit (void);

#ifdef __VMS
#  include "ev4_routines_vms.h"
#endif
#ifdef __osf__
#  include "ev4_routines_osf1.h"
#endif
#ifdef __NT
#  include "ev4_routines_nt.h"
#endif

#endif /* ifndef __EV4_ROUTINES_LOADED */
