#ifndef __IPR_ISR_NT_DEF_LOADED
#define __IPR_ISR_NT_DEF_LOADED 1
#if defined(__NT) || defined (NT)
/* IPR_ISR_NT_DEF.C
/*
 * Module:      IPR_ISR_NT.DEF
 *
 * Version:     X-1
 */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993, 1994 BY                                             */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
 * Abstract:    This file contains the Windows NT support routines for the
 *              interrupt service routines for the performance counters.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       1-Sep-1994
 *
 * Modified By:
 */

// Windows NT specific routine definitions

void do_full_action (IPR_SBUFFER *bufptr, IPR_BUFDSC *bD, DEVICE_EXTENSION *DeviceExtension)
{
    LIST_ENTRY *prevTail;
    IPR_WrtStruct *wrtStruct;

#ifdef TRCDPC
    TrcPrint("do_full_action, action %d, bufptr %x, DeviceExtension %x\n",
       bD->fullaction,bufptr,DeviceExtension);
#endif

#ifdef DBGDPC
    DbgPrint("Entering do_full_action, action %d, bufptr %x, DeviceExtension %x\n",
       bD->fullaction,bufptr,DeviceExtension);
//    DbgBreakPoint();
#endif
    switch(bD -> fullaction)
    {
    case IPR_FULL_WAKE:
    case IPR_FULL_AST:
#ifdef TRCE
        TrcPrint("Error: IPR_FULL_C_WAKE and IPR_FULL_C_AST, %x, full actions not implemented in NT, use IPR_FULL_C_WRITE, sample buffers not written\n",
           bD -> fullaction);
#endif
        break;
    case IPR_FULL_NONE:
#ifdef TRCW
        TrcPrint("Warning: Write action, %x, NONE given, sample buffers won't be written\n",
           bD -> fullaction);
#endif
        break;
    case IPR_FULL_WRITE:
        wrtStruct = DeviceExtension->wrtStruct;
#ifdef DBGDPC
        DbgPrint("Adding buffer %x to wrtq, entry %x, wrtStruct %x\n",
           bufptr,bufptr->wrtQEntry,&wrtStruct);
//        DbgBreakPoint();
#endif
        prevTail = ExInterlockedInsertTailList(&wrtStruct->wrtQ,
           (LIST_ENTRY *)bufptr->wrtQEntry,&wrtStruct->wrtQ_SpinLock);
#ifdef TRCDPC
        TrcPrint("Buffer %x successfully added to write queue in entry %x, prev tail %x\n",
           bufptr,bufptr->wrtQEntry,prevTail);
#endif
        // Signal writer thread to wake up to write the buffer - not waiting and no priority increment
        wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,FALSE);
        break;
    default:

#ifdef TRCE
        TrcPrint("Error: Invalid write action, %x, sample buffers won't be written\n",
           bD -> fullaction);
#endif

        break;
    }
}

/*
int THIS_PID(void)
{
   int *PIDptr;
//   int *TIDptr;

   PIDptr = (int *)IoGetCurrentProcess();
//   TIDptr = (int *)PsGetCurrentThread();
   if (PIDptr)
   {
      PIDptr = (int *)(((char *)PIDptr) + 0x8c);
#ifdef DBGPID
      tmpcnt++;
      if ((tmpcnt % 1000) == 0)
//      if ((*PIDptr == 0x24) || (*PIDptr == 0x7c))
      {
         DbgPrint("Proc block %x, PIDptr %x, PID %x, tmpcnt %d\n",
           (int)PIDptr-0x8c,PIDptr,*PIDptr,tmpcnt);
         DbgBreakPoint();
      }
#endif
      return *PIDptr;
   }
   else
      return 0;

}

int THIS_PRIORITY(void)
{
   int *Priorptr;
   int Prior;

   Priorptr = (int *)IoGetCurrentProcess();
   if (Priorptr)
   {
      Priorptr = (int *)(((char *)Priorptr) + 0x54);
      Prior = (*Priorptr) >> 24;
#ifdef DBGDPC
      tmpcnt++;
      if ((tmpcnt % 1000) == 0)
      {
         DbgPrint("Proc block %x, Priorptr %x, Prior %x, tmpcnt %d\n",
           (int)Priorptr-0x54,Priorptr,Prior,tmpcnt);
//         DbgBreakPoint();
      }
#endif
      return Prior;
   }
   else
      return 0;

}
*/

void full_action_ast (IPR_BUFDSC *BDptr)
{
}

uint64 THIS_TIME(void)
{
   uint64 quadtime;

   KeQuerySystemTime((LARGE_INTEGER *)&quadtime);
   return quadtime;

}

void * GETBDPTR(IN IPR_CPUNUM cpu)
{
   IPR_STATUS status;
   IPR_BUFDSC *bufptr;

   status = get_bufdsc_address(cpu,&bufptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("GETBDPTR Error: bad status %x on return from GETBDPTR\n",
         status);
#endif
      return NULL;
   }
   return bufptr;
}

#endif  /*  defined (__NT) || defined (NT) */
#endif  /* ifndef __IPR_ISR_NT_DEF_LOADED */
