/*++ 

Copyright (c) 1993  Digital Equipment Corporation

Module Name:

    nta21064.h

Abstract:

    This include file defines the interfaces to low-level routines that
    directly use the NT I/O APIs to control the performance counter driver
    for the Alpha AXP 21064.

Author:

    Joe Notarangelo  25-Jun-1993

Revision History:

    Joe Notarangelo  05-Feb-94
        Add new interfaces for LCA external counter selection.

    Joe Notarangelo  02-Apr-1994
        Add interface for MP open, allows old open interface to remain
        for backward compatibility.


--*/

#ifndef _NTA21064_
#define _NTA21064_

//
// Include the driver interface include file.
//

#include "dda21064.h"

//
// Define the open/close interfaces.
//

NTSTATUS
OpenAxp21064Counter(
    AXP21064_PCCOUNTER Counter,
    HANDLE *FileHandle
    );

NTSTATUS
OpenMpAxp21064Counter(
    ULONG ProcessorNumber,
    AXP21064_PCCOUNTER Counter,
    HANDLE *FileHandle
    ); 

NTSTATUS
CloseAxp21064Counter(
    HANDLE FileHandle
    );

//
// Define the read interfaces.
//

NTSTATUS
ReadAxp21064Counter(
    HANDLE FileHandle,
    AXPCOUNTER *CounterValue
    );

NTSTATUS
ReadAxp21064PalCounters(
    HANDLE FileHandle,
    AXP21064_RDPALCOUNTERS *PalCounters
    );

//
// Define the set/reset interfaces.
//

NTSTATUS
SetAxp21064CounterValue(
    HANDLE FileHandle,
    AXPCOUNTER CounterValue
    );

NTSTATUS
ResetAxp21064CounterValue(
    HANDLE FileHandle
    );

//
// Define the start/stop counter interfaces.
//

NTSTATUS
StartAxp21064Counter(
    HANDLE FileHandle
    );

NTSTATUS
StopAxp21064Counter(
    HANDLE FileHandle
    );
 
//
// Define the set mux control interface.
// 

NTSTATUS
MuxControlAxp21064Counter(
    HANDLE FileHandle,
    AXP21064_PCMUXCONTROL MuxControl
    );

//
// Define the set event count interface.
//

NTSTATUS
EventCountAxp21064Counter(
    HANDLE FileHandle,
    AXP21064_PCEVENTCOUNT EventCount
    );

//
// Define the select external counter interface.
// 

NTSTATUS
SelectAxp21066ExternalCounter(
    HANDLE FileHandle,
    AXP21066_EXTERNALSELECT ExternalSelect
    );

#endif // _NTA21064_
