//
// Program REP for Windows NT
// Module REP_H
// Global defines, typedefs and prototypes
//
// Copyright:
//  1994 by Digital Equipment Corporation
//
//             Patent Pending, Digital Equipment Corporation
//
//
// Created:
//  Michael Dempsey
//  July 6, 1994
//
// Modification history:
//
//
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
//
// Memory leak detection routines - debug only
//
#ifdef _DEBUG
#   define MAXPTRS	8192
	typedef struct _ptr_entry
	{
		void				*ptr;
		unsigned long int	size;
	} PtrEntry;

	void *ckmalloc(int size);
	void ckfree(void *p);
	void HeapCleanup();
	void ReportHeap(void);
#else
#	define ckmalloc malloc
#	define ckfree free
#endif					// End if memory leak detection definitions

#define PATHNAMESIZE 256
#define ENVIRON_STRINGSIZE 8192
#define UNKNOWNROUTINE "(UnknownRoutine)"
#define USAGE "Usage: rep [-kuhlfn] [-p pid] [-i input[,input]] [-o output] [command verb]\n"
#define ReadProcMem(h, mva, buf, size, rcnt) ReadProcessMemory(h, (LPSTR) mva, buf, size, rcnt)

//
// List element used for lists of file names in the command table
// 
typedef struct _filelist_elem
{
	char	FileName[PATHNAMESIZE];
	void	*next;
} FILELIST, *PFILELIST;

//
// Command table -
// Definition of global structure containing flags and file names
// set by command line processing to control REP actions
//
typedef struct _cmd_struct
{
	PFILELIST
			InputFileNames;
    char    OutputFileName[PATHNAMESIZE];
    char    ArgStr[PATHNAMESIZE];
    ULONG   ProcId;
	BOOL	noresolve;
	BOOL	mode_select;
	BOOL	fresolve;
    BOOL    kmode;
    BOOL    umode;
    BOOL    kumode;
    BOOL    logging;
    BOOL    quiet;
    BOOL    help;
    BOOL    valid;
} CMD_TABLE, *PCMD_TABLE;

typedef struct _ep
{
    ULONG   addr;
    char    name[120];
} Entry, *PEntry;

extern
HANDLE          hProc, hO;      // global handles
extern
PCMD_TABLE      cmds;           // global command flags
extern
LPSTR           SysDbgFilesDir; // path names for separately located symbol table files

// Function prototypes
extern PCMD_TABLE
GetArgsAndSwitches(int argc, char** argv);

extern void
SendHelp(void);

extern LPSTR
GetSysDebugDir(LPSTR EnvStr);

extern BOOL
OpenRepFile(LPSTR FileName);

extern BOOL
AppendRepFile(LPSTR FileName);

extern void
CloseRepFile();

extern BOOL
WriteRep(LPSTR RepStr, LPSTR ImgStr, ULONG ba, ULONG ea);

extern BOOL
WriteNotFound(ULONG ba, ULONG ea);

extern void
resolve_addr(char *, char *);

extern void
GetSysReps(void);

extern BOOL
ReadProcMemImage(HANDLE hProc, LPSTR ImageFileName, LPSTR ShortName);

extern BOOL
GetCVSyms(LPSTR FileName, LPSTR ShortName, PIMAGE_DEBUG_DIRECTORY DbgInfo, ULONG ImageBase, ULONG ImageSize);

extern BOOL
GetCoffSyms(LPSTR FileName, LPSTR ShortName, PIMAGE_DEBUG_DIRECTORY DbgInfo, ULONG ImageBase, ULONG ImageSize, PIMAGE_RUNTIME_FUNCTION_ENTRY pFE, ULONG uFEcnt);

extern BOOL
GetDbgFileSyms(LPSTR FileName, ULONG IBase, ULONG ISize, ULONG EndOfCode, PIMAGE_RUNTIME_FUNCTION_ENTRY pFE, ULONG uFEcnt);

extern HANDLE
StartProc(LPSTR, DWORD *, DWORD);
