/**************************************************************************\
* Module name:
*
*    TraceDrv.h version 2
*
* Abstract:
*
*    Include file to support kernel mode and driver tracing.
*
* Author:
*
*    Adrie Sweep
*    Digital equipment bv
*    Utrecht
*    The Netherlands
*
* Environment:
*
*    Kernel mode
*
* Revision History:
*
*/

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#define TRCDEVICENAME     "\\Device\\Trc%d"
#define TRCVERSIONLVL     2
#define TRCNPPOOLQUOT     8192

        PDEVICE_OBJECT    TrcDeviceObject;

/**************************************************************************\
* Device Extension definitions.
*/

typedef struct _TRACEDRV_DEVICE_EXTENSION {
        LIST_ENTRY        Busy_Queue;
        KSPIN_LOCK        Q_SpinLock;
        ULONG             VersionLvl;
        ULONG             Pool_Quota;
        LONG              Msg_Missed;
        UNICODE_STRING    TraceSName;
} TRACEDRV_DEVICE_EXTENSION, *PTRACEDRV_DEVICE_EXTENSION;

NTSTATUS
TrcSetup(
    VOID
    )
/**************************************************************************\
* Routine Description:
*
*    Initialize the connection to the Trace device.
*
* Arguments:
*
*    VOID
*
* Return Value:
*
*    NTSTATUS
*
*/

{
    PFILE_OBJECT               TrcFileObject;
    UNICODE_STRING             TrcDeviceName;
    STRING                     TrcDeviceNameString;
    CCHAR                      TrcDeviceNameBuffer[256];

    sprintf(TrcDeviceNameBuffer,TRCDEVICENAME,0);

    RtlInitAnsiString(&TrcDeviceNameString,TrcDeviceNameBuffer);

    RtlAnsiStringToUnicodeString(&TrcDeviceName,&TrcDeviceNameString,TRUE);

    return (IoGetDeviceObjectPointer(
                &TrcDeviceName,
                FILE_READ_DATA,
                &TrcFileObject,
                &TrcDeviceObject));
}

NTSTATUS
TrcPrint(
    PCCHAR TrcMFormat,
    ...
    )
/**************************************************************************\
* Routine Description:
*
*    Transmit data to the Trace device.
*
* Arguments:
*
*    PCCHAR
*
* Return Value:
*
*    NTSTATUS
*
*/

{
    NTSTATUS                   TrcStatus = STATUS_UNSUCCESSFUL;
    PTRACEDRV_DEVICE_EXTENSION TrcDeviceExtens;
    PLIST_ENTRY                TrcMessageBlock;
    PCCHAR                     TrcMessageBuffr;
    CCHAR                      TrcMessage[256];
    va_list                    TrcMArgumt;

    if (TrcDeviceObject)                                       {
        TrcDeviceExtens = TrcDeviceObject->DeviceExtension;

    /**********************************************************************\
    * Compare the version level from the driver with the one in the .h file.
    */
        if (TrcDeviceExtens->VersionLvl == TRCVERSIONLVL)      {

    /**********************************************************************\
    * Check and update the NonPagedPool quota value. This is to avoid a dri-
    * ver filling up the NonPagedPool with trace messages.
    */
            if (TrcDeviceExtens->Pool_Quota < TRCNPPOOLQUOT)   {

                va_start(TrcMArgumt,TrcMFormat);
                vsprintf(TrcMessage,TrcMFormat,TrcMArgumt);

                TrcMessageBlock = ExAllocatePool(
                                      NonPagedPool,
                                      strlen(TrcMessage) + sizeof(LIST_ENTRY) + 1);

                if (TrcMessageBlock)                           {
                    TrcMessageBuffr = (PCCHAR) TrcMessageBlock + sizeof(LIST_ENTRY);

                    ExInterlockedAddUlong(
                        &TrcDeviceExtens->Pool_Quota,
                        strlen(TrcMessage),
                        &TrcDeviceExtens->Q_SpinLock);

    /**********************************************************************\
    * Copy the data into a message buffer and insert it in the Busy_Queue of
    * the TraceDrv.
    */
                    RtlMoveMemory(
                        TrcMessageBuffr,
                        TrcMessage,
                        strlen(TrcMessage) + 1);

                    ExInterlockedInsertTailList(
                        &TrcDeviceExtens->Busy_Queue,
                        TrcMessageBlock,
                        &TrcDeviceExtens->Q_SpinLock);

                    TrcStatus = STATUS_SUCCESS;
                }
            }
    /**********************************************************************\
    * The message  buffer was  not created.  Either  Pool_Quota exceeded, or
    * there was insufficient NonPaged Pool. Increment the Msg_Missed counter
    * to reflect this.
    */
            if (!NT_SUCCESS(TrcStatus))                        {
                if (TrcDeviceExtens->Msg_Missed < 256)         {
                    ExInterlockedIncrementLong(
                        &TrcDeviceExtens->Msg_Missed,
                        &TrcDeviceExtens->Q_SpinLock);
                }
            }
        }
    }
    return (TrcStatus);
}
