#if defined(__DECC) || defined(__DECCXX)
# pragma module KERNEL_GENERAL_NT "X-5"
#endif

/* KERNEL_GENERAL_NT.C */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the Windows NT specific general
 *              kernel routines based on the IPROBE API kernel
 *              rationalization (reorganization).
 *
 * Author:              Bradley M. Waters
 * Creation Date:       18-Jul-1994
 *
 * Modified by:
 *
 * X-5  BMW             Bradley M. Waters       10-Oct-1994
 *      Added queue entry delete flag to open, close and write operations
 *      so write thread can delete queue entries when done processing
 *      these commands.
 *
 * X-4  BMW             Bradley M. Waters       10-Oct-1994
 *      Code cleanup
 *
 * X-3  BMW             Bradley M. Waters        9-Sep-1994
 *      Added sample file open, close and write functions.
 *
 * X-2  BMW             Bradley M. Waters       17-Aug-1994
 *      Updated based on code review input.
 *
 * X-1  BMW             Bradley M. Waters       18-Jul-1994
 *      Original Version
 *
*/

#ifdef __NT
#  include "ntddk.h"
#endif

#include "ipr_kernel_int_defs.h"
#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#include "TraceDrv.h"
#endif

// This array is initialized in KIprInitSysInfo in OS specific kernel_general
IPR_BUFDSC *pCPUDataArray[IPR_MAX_CPUS];

// The following are globals that are used to contain Windows NT
// specific counter info.

IPR_SYSTEM_DATA sysdata;            // Contains affinity and allocation gran. info.

ZONE_HEADER zhdr[IPR_NUM_ZONES];        // Header for the nonpaged pool zones
IPR_ZONE_INFO zinfo[IPR_NUM_ZONES] = {
{2048,64,NULL},
{8192,512,NULL},
{17000,2048,NULL},
{58000,8192,NULL}
};

// globals for the sample file

HANDLE sampHandle = {0};
UNICODE_STRING *sampFname = NULL;
UNICODE_STRING *sampDFname = NULL;

extern IPR_WrtStruct *wrtStruct;
extern PKTHREAD wrtThread;      // write thread
extern IPR_STATUS openStatus;   // Open status set by write thread         a=8192
extern IPR_STATUS closeStatus;  // Close status set by write thread

extern IPR_STATUS KIprGetChipInfo(OUT IPR_CHIPINFO *retptr)
{

#ifdef TRC0
   TrcPrint("KIprGetChipInfo enter, retptr = %x\n",
       retptr);
#endif
   if (!retptr)
   {
#ifdef TRCE
      TrcPrint("KIprGetChipInfo Error: retptr is null\n");
#endif
      return IPR__NULLPTR;
   }

   return IPR__SUCCESS;
}

extern IPR_STATUS KIprUpdatePerfmon(IN IPR_CPUNUM cpu,
                        IN IPR_COUNT numctrsupd,
                        OUT IPR_COUNTER_BLOCK *counterblock)
{

//   IPR_COUNTER_BLOCK *ctrptr;
//   IPR_STATUS Status;
//   IPR_COUNT cnt;

// The for compatibility and to use Joe's event counter code,
// this routine should not be called.  Instead, the calls that
// Joe defined are called.  This means that the IPROBE counter
// driver is a true superset of the PERFEV4 counter driver.

#ifdef TRC0
   TrcPrint("KIprUpdatePerfmon enter, cpu= %d, numctrsupd= %d\n",
       cpu,numctrsupd);
#endif
#ifdef DBGPM
   DbgPrint("KIprUpdatePerfmon enter, cpu= %d, numctrsupd= %d\n",
       cpu,numctrsupd);
   DbgBreakPoint();
#endif

   if (!counterblock)
   {
#ifdef TRCE
      TrcPrint("KIprUpdatePerfmon Error: counterblock ptr is null\n");
#endif
      return IPR__NULLPTR;
   }

   if(cpu >= IPR_MAX_CPUS ){
      return IPR__INVCPU;
   }

   return IPR__SUCCESS;

}

extern IPR_STATUS KIprGetCurrentIpid(OUT unsigned int *Ipid)
{

#ifdef TRC0
   TrcPrint("KIprGetCurrentIpid enter\n",
       Ipid);
#endif
   if (!Ipid)
   {
#ifdef TRCE
      TrcPrint("KIprGetCurrentIpid Error: Ipid ptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   return IPR__SUCCESS;

}

extern IPR_STATUS KIprSetAffinity(IN IPR_CPUNUM cpu, IN IPR_BOOLEAN setting)
{

#ifdef TRC0
   TrcPrint("KIprSetAffinity enter, cpu= %x, setting=%x\n",
       cpu,setting);
#endif
   // For NT we don't need to set affinity.  NT uses handles for
   // each CPU and counter combination and the lower level driver
   // routines provide the capability to set affinity indirectly
   // based on the handle passed.  For now, this routine simply
   // returns a happy status.

   return IPR__SUCCESS;
}

extern IPR_STATUS KIprCurrentCPU (OUT IPR_CPUNUM *cpu)
{
   extern void *GetProcessorPCR(void);

   if (!cpu)
      return IPR__NULLPTR;
   *cpu = ((KPCR *)GetProcessorPCR())->Number;
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprSetPme(IN unsigned int pid, IN int state, IN IPR_BOOLEAN children, OUT IPR_BOOLEAN *prev_state)
{

#ifdef TRC0
   TrcPrint("KIprSetPme enter, pid= %x, state=%x, prev_state=%x\n",
       pid,state,prev_state);
#endif
   // Tried though we might, NT doesn't have a PME bit at the moment.
   // If a PME bit or some backdoor PME like functionality is added to
   // NT this is where it would be accessed.  For now, we merely return
   // a happy status.

   return IPR__SUCCESS;

}

extern IPR_STATUS KIprResetIpl(void)
{

#ifdef TRC0
   TrcPrint("KIprResetIpl enter\n");
#endif
   // NT doesn't have a PME bit and doesn't need this function. For now,
   // merely return a happy status.
   return IPR__SUCCESS;

}

extern IPR_STATUS KIprInitSysInfo(IN IPR_SYSTEM_DATA *newsysdata)

// This routine fills in system info. which is either passed to it
// or obtained.  Info. passed in is easily obtainable in user mode
// while information more difficult to access is obtained by this
// routine in kernel mode.  This routine fills in appropriate
// global variables such as affinity_cpu_mask and noaffinity_cpu_mask.
// NOTE: This is one of the few routines that has a different number of
// agruments in the kernel form of the routine than the user form of
// of the routine (additional info. is gathered in user mode and passed
// into the kernel).
//
// Currently, all info. is obtained in user mode.

{
        int cnt;

#ifdef TRC0
        TrcPrint("KIprInitSysInfo enter\n");
#endif
        // At this point in time I assume that all existing
        // processors are active.

        // Initialize the CPUDataArray - pointers to CPU buffer descriptors.
        for (cnt=0; cnt < IPR_MAX_CPUS; cnt++)
        {
           pCPUDataArray[cnt] = (IPR_BUFDSC *) NULL;
        }

        return IPR__SUCCESS;
}

extern IPR_STATUS KIprDoneSysInfo(void)
{

#ifdef TRC0
        TrcPrint("KIprDoneSysInfo enter...\n");
#endif
        return IPR__SUCCESS;
}

extern IPR_STATUS KIprOpenSampFile(IN UNICODE_STRING *fname, IN UNICODE_STRING *defname)
{

   IPR_STATUS status;
   IPR_WrtList *wrtLEntry;
   LIST_ENTRY *prevTail;
   KWAIT_REASON wrtReason = {Executive};
   KPROCESSOR_MODE wrtMode = {KernelMode};
//#ifdef TRCG
#if defined(DBGCRSH) || defined(TRCG) || defined(TRCWT)
   ANSI_STRING filenameANSI;
#endif

#ifdef TRC0
   TrcPrint("KIprOpenSampFile enter...\n");
#endif
#ifdef DBGCRSH
   DbgPrint("KIprOpenSampFile: fname %x, defname %x\n",
      fname,defname);
#endif
   if (!fname)
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error: fname ptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if (!defname)
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error: defname ptr is null\n");
#endif
      return IPR__NULLPTR;
   }

   status = allocate_memory (sizeof(UNICODE_STRING),NULL,&sampFname);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error return from allocate_memory for unicode string, status=%x\n",
         status);
#endif
      if (sampFname)
      {
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      return IPR__OPENFAIL;
   }

   status = allocate_memory (fname->MaximumLength,NULL,&sampFname->Buffer);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error return from allocate_memory for unicode string, status=%x\n",
         status);
#endif
      if (sampFname)
      {
         if (sampFname->Buffer)
         {
            deallocate_memory (sampFname->Buffer);
            sampFname->Buffer = NULL;
         }
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      return IPR__OPENFAIL;
   }
   RtlMoveMemory(sampFname->Buffer,fname->Buffer,fname->MaximumLength);
   sampFname->Length = fname->Length;
   sampFname->MaximumLength = fname->MaximumLength;

   status = allocate_memory (sizeof(UNICODE_STRING),NULL,&sampDFname);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error return from allocate_memory for unicode string, status=%x\n",
         status);
#endif
      if (sampFname)
      {
         if (sampFname->Buffer)
         {
            deallocate_memory (sampFname->Buffer);
            sampFname->Buffer = NULL;
         }
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      return IPR__OPENFAIL;
   }
   status = allocate_memory (defname->MaximumLength,NULL,&sampDFname->Buffer);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error return from allocate_memory for unicode string, status=%x\n",
         status);
#endif
      if (sampFname)
      {
         if (sampFname->Buffer)
         {
            deallocate_memory (sampFname->Buffer);
            sampFname->Buffer = NULL;
         }
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      if (sampDFname)
      {
         deallocate_memory (sampDFname);
         sampDFname = NULL;
      }
      return IPR__OPENFAIL;
   }
   RtlMoveMemory(sampDFname->Buffer,defname->Buffer,defname->MaximumLength);
   sampDFname->Length = defname->Length;
   sampDFname->MaximumLength = defname->MaximumLength;

#ifdef DBGCRSH
   DbgPrint("KIprOpenSampFile queuing up open request to write thread, fnptr %x, dfnptr %x\n",
      fname,defname);
   DbgPrint("sampfnptr %x, sampdfnptr %x\n",
      sampFname,sampDFname);
   status = RtlUnicodeStringToAnsiString(&filenameANSI,fname,TRUE);
   if (status == STATUS_SUCCESS)
   {
      DbgPrint("File to open %s\n",
         filenameANSI.Buffer);
      RtlFreeAnsiString(&filenameANSI);
   }
   else
      DbgPrint("Conversion of file name from UNICODE to ANSI failed, status %x\n",
         status);
   status = RtlUnicodeStringToAnsiString(&filenameANSI,defname,TRUE);
   if (status == STATUS_SUCCESS)
   {
      DbgPrint("File to open %s\n",
         filenameANSI.Buffer);
      RtlFreeAnsiString(&filenameANSI);
   }
   else
      DbgPrint("Conversion of deffile name from UNICODE to ANSI failed, status %x\n",
         status);
#endif
//#ifdef TRCG
#ifdef TRCWT
   TrcPrint("KIprOpenSampFile queuing up open request to write thread, fnptr %x, dfnptr %x\n",
      fname,defname);
   TrcPrint("sampfnptr %x, sampdfnptr %x\n",
      sampFname,sampDFname);
   status = RtlUnicodeStringToAnsiString(&filenameANSI,fname,TRUE);
   if (status == STATUS_SUCCESS)
   {
      TrcPrint("File to open %s\n",
         filenameANSI.Buffer);
      RtlFreeAnsiString(&filenameANSI);
   }
   else
      TrcPrint("Conversion of file name from UNICODE to ANSI failed, status %x\n",
         status);
   status = RtlUnicodeStringToAnsiString(&filenameANSI,defname,TRUE);
   if (status == STATUS_SUCCESS)
   {
      TrcPrint("File to open %s\n",
         filenameANSI.Buffer);
      RtlFreeAnsiString(&filenameANSI);
   }
   else
      TrcPrint("Conversion of deffile name from UNICODE to ANSI failed, status %x\n",
         status);
#endif

   status = allocate_memory (sizeof(IPR_WrtList),NULL,&wrtLEntry);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error return from allocate_memory for write thread open entry, status=%x\n",
         status);
#endif
      if (sampFname)
      {
         if (sampFname->Buffer)
         {
            deallocate_memory (sampFname->Buffer);
            sampFname->Buffer = NULL;
         }
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      if (sampDFname)
      {
         if (sampDFname->Buffer)
         {
            deallocate_memory (sampDFname->Buffer);
            sampDFname->Buffer = NULL;
         }
         deallocate_memory (sampDFname);
         sampDFname = NULL;
      }
      return IPR__OPENFAIL;
   }
   wrtLEntry->bufptr = (IPR_SBUFFER *)IPR_OPENWRITE;
   wrtLEntry->delEntry = TRUE;

   // Insert open action at head of queue
   prevTail = ExInterlockedInsertHeadList(&wrtStruct->wrtQ,
      (LIST_ENTRY *)wrtLEntry,&wrtStruct->wrtQ_SpinLock);
#ifdef DBGCRSH
   DbgPrint("Open operation successfully added to write queue in entry %x, prev tail %x\n",
           wrtLEntry,prevTail);
#endif
#ifdef TRCG
   TrcPrint("Open operation successfully added to write queue in entry %x, prev tail %x\n",
           wrtLEntry,prevTail);
#endif
   // Clear event to wait on for open completion
   wrtStruct->wrtoState = KeResetEvent(&wrtStruct->wrtoEvent);
   // Signal writer thread to wake up to write the buffer - waiting for open operation to complete
   wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,TRUE);
   // Wait for open to complete
   status = KeWaitForSingleObject(&wrtStruct->wrtoEvent,
      wrtReason,wrtMode,TRUE,NULL);

   if (!OK_STATUS(openStatus))
   {
#ifdef DBGCRSH
      DbgPrint("KIprOpenSampFile Error on file open, status=%x\n",
         openStatus);
#endif
#ifdef TRCE
      TrcPrint("KIprOpenSampFile Error on file open, status=%x\n",
         openStatus);
#endif
      if (sampFname)
      {
         if (sampFname->Buffer)
         {
            deallocate_memory (sampFname->Buffer);
            sampFname->Buffer = NULL;
         }
         deallocate_memory (sampFname);
         sampFname = NULL;
      }
      if (sampDFname)
      {
         if (sampDFname->Buffer)
         {
            deallocate_memory (sampDFname->Buffer);
            sampDFname->Buffer = NULL;
         }
         deallocate_memory (sampDFname);
         sampDFname = NULL;
      }
      return IPR__OPENFAIL;
   }

#ifdef TRCG
   TrcPrint("File successfully opened, handle %x\n",
      sampHandle);
#endif
#ifdef DBGCRSH
   DbgPrint("File successfully opened, handle %x\n",
      sampHandle);
#endif

   return IPR__SUCCESS;

}

extern IPR_STATUS KIprCloseSampFile(OUT int *buffwrites, OUT int *partwrites)
{
   IPR_STATUS status;
   IPR_WrtList *wrtLEntry;
   LIST_ENTRY *prevTail;

#ifdef TRC0
   TrcPrint("KIprCloseSampFile enter...\n");
#endif

   if (sampFname)
   {
      if (sampFname->Buffer)
      {
         deallocate_memory (sampFname->Buffer);
         sampFname->Buffer = NULL;
      }
      deallocate_memory (sampFname);
      sampFname = NULL;
   }
   if (sampDFname)
   {
      if (sampDFname->Buffer)
      {
         deallocate_memory (sampDFname->Buffer);
         sampDFname->Buffer = NULL;
      }
      deallocate_memory (sampDFname);
      sampDFname = NULL;
   }

   if ((!buffwrites) || (!partwrites))
   {
#ifdef TRCW
      TrcPrint("KIprCloseSampFile Warning buffwrites %x or partwrites %x NULL, NULL counts not updated\n",
         buffwrites,partwrites);
#endif
   }

   // Queue up close operation for write thread
   status = allocate_memory (sizeof(IPR_WrtList),NULL,&wrtLEntry);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprCloseSampFile Error return from allocate_memory for write thread close entry, status=%x\n",
         status);
#endif
      return IPR__CLOSEFAIL;
   }
   wrtLEntry->bufptr = (IPR_SBUFFER *)IPR_CLOSEWRITE;
   wrtLEntry->delEntry = TRUE;

   // Insert close action at tail of queue
   prevTail = ExInterlockedInsertTailList(&wrtStruct->wrtQ,
      (LIST_ENTRY *)wrtLEntry,&wrtStruct->wrtQ_SpinLock);
#ifdef TRCG
   TrcPrint("Close operation successfully added to write queue in entry %x, prev tail %x\n",
           wrtLEntry,prevTail);
#endif
   wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,FALSE);

   if (!OK_STATUS(closeStatus))
   {
#ifdef TRCE
      TrcPrint("KIprCloseSampFile Error return from ZwCloseFile, handle %x, status=%x\n",
         sampHandle,closeStatus);
#endif
      return IPR__CLOSEFAIL;
   }

   if (buffwrites)
      *buffwrites = wrtStruct->buffwrites;
   if (partwrites)
      *partwrites = wrtStruct->partwrites;

#ifdef TRCG
   TrcPrint("File successfully closed, buffer writes %d, partial buffer writes %d\n",
      wrtStruct->buffwrites,wrtStruct->partwrites);
#endif
#ifdef DBGW2
   DbgPrint("close request queued\n");
#endif
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprWriteFile(IN void *bptr, IN IPR_SIZE bsize)
{
   IPR_STATUS status;
   IPR_WrtList *wrtLEntry;
   LIST_ENTRY *prevTail;
   void *tmpBuff = {NULL};

#ifdef TRC0
   TrcPrint("KIprWriteFile enter...\n");
#endif
   if (!bptr)
   {
#ifdef TRCE
      TrcPrint("KIprWriteFile Error: bptr is null\n");
#endif
      return IPR__NULLPTR;
   }

   status = allocate_memory (sizeof(IPR_WrtList),NULL,&wrtLEntry);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprWriteFile Error return from allocate_memory for write thread close entry, status=%x\n",
         status);
#endif
      return IPR__WRITEFAIL;
   }
   // If process context address, copy to system context
   if (((uint)bptr & 0x80000000) == 0)
   {
#ifdef DBGW
      DbgPrint("creating sys. write buffer for proc. context buffer, bptr %x, bsize %d \n",
         bptr,bsize);
#endif
      status = allocate_memory (bsize,NULL,&tmpBuff);
      if (!IPR_SUCCESS(status))
      {
#ifdef TRCE
         TrcPrint("KIprWriteFile Error return from allocate_memory for temporary write buffer, status=%x\n",
            status);
#endif
         if (wrtLEntry)
            deallocate_memory(wrtLEntry);
         return IPR__WRITEFAIL;
      }
      RtlMoveMemory(tmpBuff,bptr,bsize);
      bptr = tmpBuff;
      wrtLEntry->delBuff = TRUE;
   }
   else
      wrtLEntry->delBuff = FALSE;
#ifdef TRCG2
   TrcPrint("KIprWriteFile queueing write request, bptr %x, bsize %d \n",
      bptr,bsize);
#endif
#ifdef DBGG2
   DbgPrint("KIprWriteFile queueing write request, bptr %x, bsize %d \n",
      bptr,bsize);
#endif
   wrtLEntry->bufptr = bptr;
   wrtLEntry->bsize = bsize;
   wrtLEntry->sampBuff = FALSE;
   wrtLEntry->delEntry = TRUE;

   // Insert write action at tail of queue
   prevTail = ExInterlockedInsertTailList(&wrtStruct->wrtQ,
      (LIST_ENTRY *)wrtLEntry,&wrtStruct->wrtQ_SpinLock);
#ifdef TRCG
   TrcPrint("Write operation successfully added to write queue in entry %x, prev tail %x\n",
           wrtLEntry,prevTail);
#endif

   // Signal writer thread to wake up to write the buffer - not waiting and no priority increment
   wrtStruct->wrtState = KeSetEvent(&wrtStruct->wrtEvent,0,FALSE);
   // Write thread will deallocate temporary buffer when its done

   return IPR__SUCCESS;

}
