/* POOL_NT.C
 *
 * Version:     X-6
 */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c)  1994 BY                                                  */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the NT-specific performance monitoring
 *              routines.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       15-Jul-1994
 *
 * Modified by:
 *
 * X-6  BMW             Bradley M. Waters       25-Oct-1994
 *      Updated some of the trace messages and conditionals.
 *
 * X-5  BMW             Bradley M. Waters       10-Oct-1994
 *      Code cleanup.
 *
 * X-4  BMW             Bradley M. Waters       17-Aug-1994
 *      Updated based on code review input.
 *
 * X-3  BMW             Bradley M. Waters       21-Jul-1994
 *      Changed name to POOL_NT and integrated for kernel rationalization
 *
 * X-2  BMW             Bradley M. Waters       18-Jul-1994
 *      Modified based on kernel rationalization.
 *
 * X-1  BMW             Bradley M. Waters       15-Jul-1994
 *      Original version.
 *
 */

#ifdef __NT
#  include "ntddk.h"
#endif

#include "ipr_kernel_int_defs.h"
#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#include "TraceDrv.h"
#endif

extern IPR_STATUS ALONONPAGED(IN int bytecnt,
                        IN IPR_BOOLEAN allocZone,
                        OUT int *retsize,
                        OUT IPR_BUFHDR **areaptr)
{
   IPR_BUFHDR *alloc_area;
   POOL_TYPE npp_type = NonPagedPool;
   int cnt;
   IPR_STATUS Status;

#ifdef TRC0
   TrcPrint("ALONONPAGED enter, bytecnt = %d, allocZ %d\n",
      bytecnt,allocZone);
#endif
   if (!areaptr)
   {
#ifdef TRCE
      TrcPrint("ALONONPAGED Error: areaptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   *areaptr = NULL;
   if (!retsize)
   {
#ifdef TRCE
      TrcPrint("ALONONPAGED Error: retsize is null\n");
#endif
      return IPR__NULLPTR;
   }
   *retsize = 0;
#ifdef TRCP
   if (allocZone)
      TrcPrint("Attempting to allocate from a zone, size = %d\n",
         bytecnt);
   else
      TrcPrint("Allocating from pool, size = %d\n",
         bytecnt);
#endif
#ifdef DBGP
   if (allocZone)
      DbgPrint("Attempting to allocate from a zone, size = %d\n",
         bytecnt);
   else
      DbgPrint("Allocating from pool, size = %d\n",
         bytecnt);
#endif

   if (allocZone)
   {
      for (cnt=0; cnt < IPR_NUM_ZONES; cnt++)
      {
         if (zinfo[cnt].granularity >= bytecnt)
         {
            if (!ExIsFullZone(&zhdr[cnt]))
            {
               alloc_area= ExAllocateFromZone(&zhdr[cnt]);
#ifdef TRCP
               TrcPrint("return from ExAllocateFromZone, zone= %d, alloc_area = %x\n",
                  cnt,alloc_area);
#endif
#ifdef DBGP
               DbgPrint("return from ExAllocateFromZone, zone= %d, alloc_area = %x\n",
                  cnt,alloc_area);
#endif
               if (alloc_area != NULL)
               {
                  *retsize = zinfo[cnt].granularity;
                  *areaptr = alloc_area;
#ifdef TRCP
                  TrcPrint("%d bytes of Nonpaged pool allocated at %x from zone %d\n",
                     *retsize,*areaptr,cnt);
                  TrcPrint("zone %d begins at %x and is %d bytes\n",
                     cnt,zinfo[cnt].start,zinfo[cnt].size);
#endif
#ifdef DBGP
                  DbgPrint("%d bytes of Nonpaged pool allocated at %x from zone %d\n",
                     *retsize,*areaptr,cnt);
                  DbgPrint("zone %d begins at %x and is %d bytes\n",
                     cnt,zinfo[cnt].start,zinfo[cnt].size);
#endif
                  return IPR__SUCCESS;
               }
               else
               {
#ifdef TRCW
                  TrcPrint("ALONONPAGED Warning: Couldn't allocate from zone %d - allocating from NPP, increase size for this zone\n",
                     cnt);
                  DbgPrint("ALONONPAGED Warning: Couldn't allocate from zone %d - allocating from NPP, increase size for this zone\n",
                     cnt);
#endif
                  break;
               }
            }
            else
            {
#ifdef TRCW
               TrcPrint("ALONONPAGED Warning: zone %d is full - allocating from NPP, increase size for this zone\n",
                  cnt);
               DbgPrint("ALONONPAGED Warning: zone %d is full - allocating from NPP, increase size for this zone\n",
                  cnt);
#endif
            }
         }
      }  // end - for
   }  // end - if allocate from zone
   if (allocZone)
   {
#ifdef DBGP
//      TrcPrint("ALONONPAGED Warning: no zone big enough for %d byte - allocating from NPP, may want to create larger zone\n",
//               bytecnt);
      DbgPrint("ALONONPAGED Warning: no zone big enough for %d byte - allocating from NPP, may want to create larger zone\n",
               bytecnt);
#endif
   }
#ifdef DBGP
//      TrcPrint("ALONONPAGED Warning: no zone big enough for %d byte - allocating from NPP, may want to create larger zone\n",
//               bytecnt);
   DbgPrint("ALONONPAGED Allocating %d bytes, type %x - from NPP\n",
               bytecnt,npp_type);
#endif
   alloc_area = ExAllocatePool(npp_type, bytecnt);
   if (alloc_area == NULL)
   {
//#ifdef TRCE
#ifdef DBGP
//      TrcPrint("ALONONPAGED Error Pool allocation of %d bytes failed\n",bytecnt);
      DbgPrint("ALONONPAGED Error Pool allocation of %d bytes failed\n",bytecnt);
#endif
      return IPR__ALLOCFAIL;
   }
#ifdef DBGP
   DbgPrint("ALONONPAGED Allocated %d bytes, at %x as NPP packet\n",
               bytecnt,alloc_area);
#endif

   // Update return values
   *areaptr = alloc_area;
   *retsize = bytecnt;

   // Update NPP list to insure that all allocated pool is returned
   Status = AddToPoolList(alloc_area);
   if (!IPR_SUCCESS(Status))
   {
#ifdef TRC0
       TrcPrint("NPP list update failed, Status=%x\n",
          Status);
#endif
       return Status;
   }
#ifdef TRCP
   TrcPrint("Pool allocation of %d bytes successful at %x\n",
      *retsize,*areaptr);
#endif
#ifdef DBGP
   DbgPrint("Pool allocation of %d bytes successful at %x\n",
      *retsize,*areaptr);
#endif
   return IPR__SUCCESS;
}

extern IPR_STATUS DEANONPAGED(IN IPR_BUFHDR *areaptr)
{
   int cnt;

#ifdef TRC0
   TrcPrint("DEANONPAGED enter...\n");
#endif
   if (!areaptr)
   {
#ifdef TRCE
      TrcPrint("DEANONPAGED Error: areaptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if (((uint)areaptr & 0x80000000) == 0)
   {
#ifdef TRCE
      TrcPrint("DEANONPAGED Error: invalid areaptr, %x\n",
         areaptr);
#endif
#ifdef DBGE
      DbgPrint("DEANONPAGED Error: invalid areaptr, %x\n",
         areaptr);
      DbgBreakPoint();
#endif
      return IPR__FAILURE;
   }
   // Check to see if the block is allocated as a single NPP block - if not,
   // it is part of a zone.
   for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
   {
      if (npp_table.start[cnt] == areaptr)
      {
         // found block, deallocate directly from NPP
#ifdef TRCP
         TrcPrint("Deleting pool (list location %d), loc=%x\n",
            cnt,npp_table.start[cnt]);
#endif
#ifdef DBGP
         DbgPrint("Deleting pool (list location %d), loc=%x\n",
            cnt,npp_table.start[cnt]);
#endif
         ExFreePool(npp_table.start[cnt]);
         npp_table.start[cnt] = NULL;
         return IPR__SUCCESS;
      }
   }

#ifdef DBGP
   DbgPrint("Looking for block %x in a zone\n",
      areaptr);
#endif
   for (cnt=0; cnt < IPR_NUM_ZONES; cnt++)
   {
      if (zinfo[cnt].granularity >= areaptr->size)
      {
         ExFreeToZone(&zhdr[cnt],areaptr);
#ifdef TRCP
         TrcPrint("Returned block %x of size %d to zone %d which begins at %x and is %d bytes\n",
            areaptr,zinfo[cnt].granularity,cnt,zinfo[cnt].start,zinfo[cnt].size);
#endif
#ifdef DBGP
         DbgPrint("Returned block %x of size %d to zone %d which begins at %x and is %d bytes\n",
            areaptr,zinfo[cnt].granularity,cnt,zinfo[cnt].start,zinfo[cnt].size);
#endif
         return IPR__SUCCESS;
      }
   }
#ifdef TRCE
   TrcPrint("DEANONPAGED ERROR: Block %x bigger than largest zone and not independent NPP block.\n",
      areaptr);
   DbgPrint("DEANONPAGED ERROR: Block %x bigger than largest zone and not independent NPP block.\n",
      areaptr);
#endif
   return IPR__FAILURE;
}
