#if defined(__DECC) || defined(__DECCXX)
# pragma module SAMPLE_BUFFER_NT "X-2
#endif

/* SAMPLE_BUFFER_NT.C */

/****************************************************************************/
/*                                                                          */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the sample
 *              kernel routines based on the IPROBE API
 *              kernel rationalization (reorganization)
 *              that are specific to Windows NT.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       10-Oct-1994
 *
 * Modified by:
 *
 * X-2  BMW             Bradley M. Waters       25-Oct-1994
 *      Added delEntry flag initialization (to FALSE) for sample buffer
 *      write queue entries.  One queue entry exists per sample buffer -
 *      these queue entries exist for the life of a sample buffer.
 *
 * X-1  BMW             Bradley M. Waters       10-Oct-1994
 *      Original Version.
 *
*/

#ifdef __NT
#  include "ntddk.h"
#endif
#include "ipr_kernel_int_defs.h"
#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#include "TraceDrv.h"
#endif

// global variables

KEVENT *writeWait = NULL;      // Event used to wait for sample file to close

extern HANDLE sampHandle;


extern IPR_STATUS allocate_wrtQEntry(IN IPR_SBUFFER *Sptr, OUT IPR_WrtList **list_ptr)
{

   IPR_STATUS Status;
   IPR_WrtList *wrtQList;

   // Allocate memory for the write queue entry used for NT
   Status = allocate_memory(sizeof(IPR_WrtList),NULL,list_ptr);
   if (!IPR_SUCCESS(Status))
   {
#ifdef TRCE
      TrcPrint("allocate_wrtQEntry Error: Failure allocating memory for the write list entry: buffer %x, status = %x\n",
        Sptr,Status );
#endif
      return Status;
   }
   wrtQList = *list_ptr;
   // Init. buffer ptr field to this buffer ptr.
   wrtQList-> bufptr = Sptr;
   // Init. buffer size field to size of buffer (including header)
   wrtQList-> bsize = Sptr -> header.size ;
   wrtQList-> sampBuff = TRUE;
   wrtQList-> delBuff = FALSE;
   wrtQList->delEntry = FALSE;

   return IPR__SUCCESS;
}

extern IPR_STATUS SynchClose(void)
{
   int64 waitTime;
   LONG prevState;
   IPR_STATUS Status;
   int cnt;
   int buffwrites,partwrites;

   // Close the sample file in case it hasn't already been closed
   Status = KIprCloseSampFile(&buffwrites,&partwrites);
   if (!IPR_SUCCESS(Status))
   {
#ifdef TRCE
      TrcPrint("SynchClose ERROR: error closing the sample file, status=%x\n",
         Status);
#endif
      return Status;
   }

   // Make sure all file operations are complete and
   // that the file is closed before allowing sample buffers
   // to be deleted.  When sample file is closed, sampHandle
   // is 0.

   // If writeWait is null, create an event for waiting for write and close
   // operations on the file to complete.
   if (!writeWait)
   {
      Status = allocate_memory(sizeof(KEVENT),NULL,&writeWait);
      if (!IPR_SUCCESS(Status))
      {
#ifdef TRCE
         TrcPrint("KIprSbDelete ERROR: allocating memory for write wait event, status=%x\n",
            Status);
#endif
         return IPR__MEMORY_FAIL;
      }
      KeInitializeEvent(writeWait,SynchronizationEvent,FALSE);
   }
   waitTime = -IPR_TimeUnitSecs / IPR_LoopsPerSec;
   cnt = 0;
   while (sampHandle != 0)
   {
      cnt++;
      // wait with a timeout value then recheck
      // the file close info.
      prevState = KeResetEvent(writeWait);
      Status = KeWaitForSingleObject(writeWait,Executive,KernelMode,TRUE,(PLARGE_INTEGER)&waitTime);
      if (Status != STATUS_TIMEOUT)
      {
#ifdef TRCW
         TrcPrint("KIprSbDelete Warning: KeWaitForSingleObject status %x not timeout status as expected.\n",
            Status);
#endif
      }
      if ((cnt > IPR_TimeLoopLim) && (sampHandle != 0))
      {
#ifdef TRCE
         TrcPrint("KIprSbDelete Error: timed out after waiting %d seconds for sample buffer IO to complete, waitTime %Lx, event %x\n",
            cnt/IPR_LoopsPerSec,waitTime,writeWait);
#endif
         return IPR__SBDELTIMEOUT;
      }
   }
   return IPR__SUCCESS;
}
