/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Program: FreeBSD Iprobe driver.
 *
 * Abstract: This contains all of the freebsd module specific stuff.
 *
 * Author: Andrew Gallatin
 *
 * based heavily on the linux driver:
 * Author: Phillip Ezolt
 * Creation Date:  10-Aug-1998
 *
 * Modification history:
 *
 */


#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>         /* SYSINIT stuff */
#include <sys/conf.h>           /* cdevsw stuff */
#include <sys/malloc.h>         /* malloc region definitions */
#include <machine/clock.h>      /* DELAY() */
#include <sys/proc.h>
#include <sys/ioccom.h>
#include <sys/module.h>
#include <sys/signalvar.h>
#include "stdtypes.h"

#include "ipr_kernel_int_defs.h"
#include "ipr_driver_freebsd.h"


#define NON_DATA_VALUE 0xffffffffffffffffL


extern unsigned int ipr_cp_mask, ipr_is_cp_mask; /*active CPU's mask */

#define DEFAULT_MAJOR_NUMBER 99

#define DEVICE_NAME "ipr"

/* Forward Definitions... Should be in a header file.  PHIL */

void ipr_interrupt(unsigned long vector, struct trapframe *framep); 

void do_full_action (IPR_SBUFFER *pSB, IPR_BUFDSC *bD, IPR_CPUNUM cdb);


boolean InUse=FALSE;

void (*temp_perf_irq)(unsigned long, struct trapframe *)=NULL;
extern void (*perf_irq)(unsigned long, struct trapframe *);

/* What is the size of this int? */

int MajorNumber=DEFAULT_MAJOR_NUMBER; 

int ipr_use_count=0;
#define MOD_INC_USE_COUNT ipr_use_count++
#define MOD_DEC_USE_COUNT ipr_use_count--

/* This is a pointer to the original caller of the iprobe driver. */
extern struct proc *caller;

/*int ipr_open(struct inode * inode, struct file * filp)*/
int 
ipr_open(dev_t dev, int oflags, int devtype, struct proc *p)
{

  /* We tried to open a device that is already open. */
  if (InUse)
    {
      printf("Error: Iprobe device cannot be opened twice.\n");
      return -EBUSY; /* Correct? */

    }
  
  
  MOD_INC_USE_COUNT;
  InUse = TRUE;

  return 0;
} 


/* Below is a list of the IOCTL call defined for the IPR_DRIVER 
   The following device controls are recognized for the IPROBE API part of
    the performance counter driver:

    IOCTL_IPR_GetChipInfo - Return chip info
    IOCTL_IPR_UpdatePERFMON - Change PERFMON settings
    IOCTL_IPR_GetCurrentIpid - Return current internal PID
    IOCTL_IPR_SetAffinity - Update processor affinity
    IOCTL_IPR_SetPme - Update PME bit

    IOCTL_IPR_BdCreate - Buffer descriptor create
    IOCTL_IPR_BdDelete - Buffer descriptor delete
    IOCTL_IPR_BdSetNumCounters - Set number of perf. counters
    IOCTL_IPR_BdSetAction - Setup action to occur at particular time
    IOCTL_IPR_BdSetActiveCounters - Set active counter mask
    IOCTL_IPR_BdAddPcRange - Add a PC range
    IOCTL_IPR_BdSetSkipCount - Set number of interrupts to skip between samples

    IOCTL_IPR_SbCreate - Sample buffer create
    IOCTL_IPR_SbDelete - Sample buffer delete
    IOCTL_IPR_SbSetFull - Set full bit for sample buffer
    IOCTL_IPR_SbGetInfo - Returns info. about the sample buffer
    IOCTL_IPR_SbFind - Returns pointer to sample buffer

    IOCTL_IPR_HbElements - Returns number of elements required for given 
                           histogram parameters
    IOCTL_IPR_HbIndex - Returns array index for the specified value and 
                        counter for given histogram parameters
    IOCTL_IPR_HbValue - Return value at given histogram index
    IOCTL_IPR_HbMaxElements - Return maximum number of histogram elements
    IOCTL_IPR_HbCreate - Create histogram buffer
    IOCTL_IPR_HbDelete - Delete histogram buffer
    IOCTL_IPR_HbFind - Return next histogram buffer that matches given criteria
    IOCTL_IPR_HbRead -  Read given elements from histogram buffer
    IOCTL_IPR_HbClear - Clear given elements in histogram buffer
   
   */



/* The following is a clever way of changing the generic ioarg pointer into
   a somewhat readable state.  
   */

#define DChipInfo             	((IPR_ARGChipInfo *)ioarg)
#define DUpdatePERFMON        	((IPR_ARGUPerfmon *)ioarg)
#define DIpid 		      	((IPR_ARGIpid *)ioarg)
#define DAffinity		((IPR_ARGAffinity *)ioarg)
#define DPme			((IPR_ARGPme *)ioarg)
#define DBDCreate		((IPR_ARGBDCreate *)ioarg)
#define DBDelete		((IPR_ARGBDelete *)ioarg)
#define DBDNumctrs		((IPR_ARGBDNumctrs *)ioarg)
#define DBDAction		((IPR_ARGBDAction *)ioarg)
#define DBDActctrs		((IPR_ARGBDActctrs *)ioarg)
#define DBDPcrng		((IPR_ARGBDPcrng *)ioarg)
#define DBDSkip			((IPR_ARGBDSkip *)ioarg)
#define DSBCreate		((IPR_ARGSBCreate *)ioarg)
#define DSBDelete		((IPR_ARGSBDelete *)ioarg)
#define DSBSetFull		((IPR_ARGSBSetFull *)ioarg)
#define DSBGetInfo		((IPR_ARGSBGetInfo *)ioarg)
#define DSBFind			((IPR_ARGSBFind *)ioarg)
#define DHBElements		((IPR_ARGHBElements *)ioarg)
#define DHBMaxElem		((IPR_ARGHBMaxElem *)ioarg)
#define DHBIndex		((IPR_ARGHBIndex *)ioarg)
#define DHBValue		((IPR_ARGHBValue *)ioarg)
#define DHBCreate		((IPR_ARGHBCreate *)ioarg)
#define DHBDelete		((IPR_ARGHBDelete *)ioarg)
#define DHBFind			((IPR_ARGHBFind *)ioarg)
#define DHBRead			((IPR_ARGHBRead *)ioarg)
#define DHBClear		((IPR_ARGHBClear *)ioarg)
#define DGetBuffer		((IPR_ARGGetBuffer *)ioarg)
#define DCpuConf		((IPR_ARGCpuConf *) ioarg)
#define DAllocate		((IPR_ARGAllocate *)ioarg)
#define DDeallocate		((IPR_ARGDeallocate *)ioarg)

/*#define DPRINTF(a)      printf a;*/
#define DPRINTF(a)

int 
ipr_ioctl(dev_t dev, u_long cmd, caddr_t ioarg, int fflag, struct proc *p)
{
   if(ioarg == NULL) {
	   printf("ipr_ioctl: null ioarg\n");
	   return IPR__FAILURE;
   }
   /* Process commands */
   switch(cmd){
	   
     case IOCTL_IPR_GetChipInfo:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DChipInfo->status = KIprGetChipInfo(&(DChipInfo->chip_info));
      	   break;

       }
     case IOCTL_IPR_UpdatePERFMON:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));	 
	   DUpdatePERFMON->status=KIprUpdatePerfmon(DUpdatePERFMON->cpu,
						    DUpdatePERFMON->arg1,
						    DUpdatePERFMON->arg2,
						    &(DUpdatePERFMON->retval));
	   break;
       }
     case IOCTL_IPR_GetCurrentIpid:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
       	   DIpid->status = KIprGetCurrentIpid(&(DIpid->Ipid));
	   break;
       }
     case IOCTL_IPR_SetAffinity:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DAffinity->status = KIprSetAffinity(DAffinity->cpu,
					       DAffinity->setting);
	   break;
       }
     case IOCTL_IPR_SetPme:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DPme->status = KIprSetPme(DPme->pid,DPme->state, DPme->child,
				     &(DPme->prev_state));
	   break;
       }
     case IOCTL_IPR_BdCreate:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDCreate->status = KIprBdCreate(DBDCreate->cpu,
					    &(DBDCreate->retptr));
	   break;
       }
     case IOCTL_IPR_BdDelete:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDelete->status = KIprBdDelete(DBDelete->cpu);
	   break;
       }
     case IOCTL_IPR_BdSetNumCounters:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDNumctrs->status = KIprBdSetNumCounters(DBDNumctrs->cpu,
					     DBDNumctrs->numctrs);
	   break;
       }
     case IOCTL_IPR_BdSetAction:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDAction->status = KIprBdSetAction(DBDAction->cpu,
					  DBDAction->action_type,
					  DBDAction->action,
					  DBDAction->targetroutine,
					  DBDAction->targetpid);
	   break;
       }
     case IOCTL_IPR_BdSetActiveCounters:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDActctrs->status = KIprBdSetActiveCounters(DBDActctrs->cpu,
							DBDActctrs->ctrmask);
	   break;
       }
     case IOCTL_IPR_BdAddPcRange:
       {     
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDPcrng->status = KIprBdAddPcRange(DBDPcrng->cpu,
					       &(DBDPcrng->pcrange));
	   break;
       }
     case IOCTL_IPR_BdSetSkipCount:
       {	
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DBDSkip->status = KIprBdSetSkipCount(DBDSkip->cpu,
						DBDSkip->ctr,
						DBDSkip->skipvalue);
	   break;
       }

/* Sample buffer shells */
     case IOCTL_IPR_SbCreate:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DSBCreate->status = KIprSbCreate(DSBCreate->cpu,
				       DSBCreate->size,
				       DSBCreate->numbuffs,
				       &(DSBCreate->descrip),
				       &(DSBCreate->retptr));
	   break;
       }
     case IOCTL_IPR_SbDelete:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DSBDelete->status = KIprSbDelete(DSBDelete->cpu,DSBDelete->bufptr);
	   break;
       }
     case IOCTL_IPR_SbSetFull:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DSBSetFull->status = KIprSbSetFull(DSBSetFull->bufptr,
					      DSBSetFull->full_val);
	   break;
       }
     case IOCTL_IPR_SbGetInfo:	
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DSBGetInfo->status = KIprSbGetInfo(DSBGetInfo->bufptr,
						 &(DSBGetInfo->descrip));
	   break;
       }
     case IOCTL_IPR_SbFind:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DSBFind->status = KIprSbFind(DSBFind->cpu,
				     &(DSBFind->descrip),
				     DSBFind->previous_buffer,  
				     &(DSBFind->retptr));
	   break;
       }	
/* Histogram Buffer Shells */
     case IOCTL_IPR_HbMaxElements:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBMaxElem->status = KIprHbMaxElements(&(DHBMaxElem->elem));
	   break;
       }
     case IOCTL_IPR_HbCreate:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBCreate->status = KIprHbCreate(DHBCreate->cpu,
				       &(DHBCreate->descrip),
				       &(DHBCreate->retptr));
	   break;
       }
     case IOCTL_IPR_HbDelete:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBDelete->status = KIprHbDelete(DHBDelete->cpu, DHBDelete->bufptr);
	   break;
       }
     case IOCTL_IPR_HbFind:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBFind->status =  KIprHbFind(DHBFind->cpu,
					 DHBFind->previous_buffer,
					 &(DHBFind->descrip),
					 &(DHBFind->retptr));
	   break;
       }
     case IOCTL_IPR_HbRead:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBRead->status = KIprHbRead(DHBRead->cpu,
					DHBRead->bufptr,
					DHBRead->start,
					DHBRead->cnt,
					DHBRead->retptr);
	   break;
       }
     case IOCTL_IPR_HbClear:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DHBClear->status = KIprHbClear(DHBClear->cpu,
					  DHBClear->bufptr,
					  DHBClear->start,
					  DHBClear->cnt);
	   break;
       }
     case IOCTL_IPR_GetBuffer:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
           DGetBuffer->status = KIprGetBuffer(DGetBuffer->srcbuffer,
					      DGetBuffer->usrbuffer);
           break;
       }
     case IOCTL_IPR_CpuConf:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DCpuConf->status = KIprGetSysInfo(&(DCpuConf->Usysstrc));
	   break;
       }
     case IOCTL_IPR_Allocate:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DAllocate->status = KIprAllocate();
	   DAllocate->ipr_is_cp_mask = ipr_is_cp_mask;
	   DAllocate->ipr_cp_mask = ipr_is_cp_mask;
	   break;
       }
     case IOCTL_IPR_Deallocate:
       {
           DPRINTF(("ipr_ioctl: line %d\n", __LINE__));
	   DDeallocate->status =KIprDeallocate();
	   break;
       }
     default:
       {
           DPRINTF(("ipr_ioctl: unknown cmd = 0x%lx\n",cmd));         

	   return IPR__FAILURE;
       }
   }
   return 0;

}



int 
ipr_close(dev_t dev, int fflag, int devtype, struct proc *p)

{
  if (!InUse)
    {
      printf("Error: Tried to close non-open iprobe device.\n");
      return; 
    }

  MOD_DEC_USE_COUNT;
  InUse=FALSE;

} 

#define CDEV_MAJOR DEFAULT_MAJOR_NUMBER
static struct cdevsw ipr_cdevsw = {
        /* open */      ipr_open,
        /* close */     ipr_close,
        /* read */      noread,
        /* write */     nowrite,        
        /* ioctl */     ipr_ioctl,
        /* stop */      nostop,
        /* reset */     noreset,
        /* devtotty */  nodevtotty, 
        /* poll */    	seltrue,
        /* mmap */      nommap,
        /* strategy */	nostrategy,
        /* name */      "ipr",
        /* parms */     noparms,
	/* maj */       /*NULL*/ CDEV_MAJOR,
        /* dump */      nodump,
        /* psize */     nopsize,
        /* flags */     0,
        /* maxio */     0,
        /* bmaj */      -1
};

extern struct cdevsw *cdevsw[];

static int 
ipr_modevent(mod, cmd, arg)
    module_t    mod;
    modeventtype_t cmd;
    void * arg;
{
    int i, err = 0;
    dev_t dev = makedev(CDEV_MAJOR, 0);

    switch (cmd) {
    case MOD_LOAD:
	    /* Store a pointer to the old irq handler */
	    temp_perf_irq = perf_irq;
	    /* Install the new handler. */
	    perf_irq = ipr_interrupt;
/*	    cdevsw_add(&dev, &ipr_cdevsw, NULL);*/
	    if(!cdevsw[CDEV_MAJOR])
		    cdevsw_add(&ipr_cdevsw);
	    printf( "Iprobe Module Loaded.\n");
	    break;
    case MOD_UNLOAD:
	    alpha_pal_wrperfmon(0, 0); 
	    perf_irq=temp_perf_irq;
/*	    cdevsw_add(&dev, NULL, NULL);*/
	    cdevsw[CDEV_MAJOR] = 0;
	    printf( "Iprobe Module Removed.\n");
	    break;
    }
    
    return err;
} 

static moduledata_t iprmod = {
    "ipr",
    ipr_modevent,
    0
};

DECLARE_MODULE(ipr, iprmod, SI_SUB_PSEUDO, SI_ORDER_ANY);


#define GET_BD_PTR(cnum) (pCPUDataArray [cnum])

unsigned int skipHist;

IPR_BUFDSC *pCPUDataArray[IPR_MAX_CPUS];


/* LINUX: This points to the current counter number. */

#define THIS_CTR() (framep->tf_regs[FRAME_TRAPARG_A2])

/*LINUX: These SHOULD be right. */

#define THIS_PC() (framep->tf_regs[FRAME_PC])
#define THIS_PS() (framep->tf_regs[FRAME_PS])
#define THIS_CPU() (0)
#define THIS_PID() (curproc ? curproc->p_pid : 0)

/* LINUX: These MAY be right. */
#define THIS_PRIORITY() (curproc->p_priority)
#define THIS_IPL() (alpha_pal_rdps() & ALPHA_PSL_IPL_MASK)

#define BARRIER() alpha_mb()

/*LINUX: This SHOULD be replaced with gettimeofday */

static struct timeval xtime;

#define THIS_TIME() (*(uint64 *) &xtime)

/* LINUX: These are all wrong. */
#define THIS_MODE() (mode_number(THIS_PS(), THIS_PID()))

/************************************************************************************/
/* Name: mode_number                                                                */
/*                                                                                  */
/* Purpose:  This returns the number of the mode we are currently in.               */
/*                                                                                  */
/* Arguments: cdb:                                                                  */
/*            input_ps:                                                             */
/*            pid:                                                                  */
/*                                                                                  */
/* Notes:  This may not apply to linux.                                             */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: The number of the mode we are currently in.                      */
/*                                                                                  */
/************************************************************************************/                       


/* In OSF we will look for three states for starters with a possible fourth 
   later 0) Kernel Mode 1) User Mode 2) Idle State and 4) interupting an 
   interupt this would be nice if we have time.
*/


/* LINUX: We need an alternate way of doing this. */

inline int mode_number (uint64 ps, uint64 pid)
{

  /* How do we check if we are in the idle 'process'? */


   /* If we are in User mode, return 1 */
   /* Otherwise return 0, kernel space. */

   if ((ps & ALPHA_PSL_USERMODE) != 0)
	   return 1;
   return 0;
 }

/************************************************************************************/
/* Name: PCinRange                                                                  */
/*                                                                                  */
/* Purpose: This functions tells us whether or not we are skipping the current      */
/*          interrupt.                                                              */
/*                                                                                  */
/* Arguments: PCrangeBuf: This is a pointer to all of the given PC ranges.          */
/*            pc: This is the current pc.                                           */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: This returns the number of the first range in which the PC fits. */
/*                 or -1 if it fits in none of the ranges.                          */
/*                                                                                  */
/************************************************************************************/                       

IPR_BOOLEAN PCinRange(IPR_PBUFFER *PCrangeBuf,IN uint64 pc)
{
   register IPR_PC_RANGE *pRange;
   int first, middle, last;
   
   // If no ranges exist the exit
   if(!PCrangeBuf)
     return -1;

   pRange = (IPR_PC_RANGE *)(PCrangeBuf + 1);

   // For now we cannot assume that the PC range list is well behaved,
   // so return the first range into which the candidate PC fits.

   for (first= 0; first < PCrangeBuf->rangecnt; first++)
      if (pc >= pRange[first].StartAddress && 
	  pc <= pRange[first].EndAddress)
      {
	  return first;
      }
   // None found

   return -1;

}

/************************************************************************************/
/* Name: SkipInt                                                                    */
/*                                                                                  */
/* Purpose: This functions tells us whether or not we are skipping the current      */
/*          interrupt.                                                              */
/*                                                                                  */
/* Arguments: SkipBufPtr: This contains information about the previously skipped    */
/*                        interrupts.                                               */
/*            Counter: This is the number of the counter that interrupted.          */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: If we are supposed to skip this interrupt, return TRUE,          */
/*                 otherwise, return FALSE.                                         */
/*                                                                                  */
/************************************************************************************/                       

inline 
IPR_BOOLEAN SkipInt(IPR_IBUFFER *SkipBufPtr, int Counter)
{
   register IPR_ISKIP *pSkip;

   // NOTE: Following code reorged.
   if (!SkipBufPtr)
     {
       return FALSE;
     }

   if(!(SkipBufPtr->skipmask & (1 << Counter)))
     {
       return FALSE;
     }

   /* THIS IS REALLY UGLY CODE.  IT SHOULD BE REMOVED... Adding one to a pointer? */

   pSkip = (IPR_ISKIP *) (SkipBufPtr + 1);
   pSkip += Counter;

   // Log a sample only if the skip counter has reached threshold

   if (pSkip->intspersample &&
       pSkip[Counter].intssincelast < pSkip[Counter].intspersample)
   {
      pSkip[Counter].intssincelast++;
      return TRUE;
   }
   else
   {
      pSkip[Counter].intssincelast = 0;
      return FALSE;
   }
}


int ipr_do_signal(void *sarg1, void *sarg2)
{
      if(caller)
	      psignal(caller, SIGUSR1);
      return 0;
}


/************************************************************************************/
/* Name: add_to_histogram                                                           */
/*                                                                                  */
/* Purpose:                                                                         */
/*                                                                                  */
/* Arguments: pHB:                                                                  */
/*            ctr:                                                                  */
/*            numcounters:                                                          */
/*            v:                                                                    */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: None.                                                            */
/*                                                                                  */
/************************************************************************************/                       

void add_to_histogram (IPR_HBUFFER *pHB, IPR_CTRNUMBER ctr,
                              IPR_COUNT numcounters, IPR_HISTVALUE v)
{
   int hidx;
   IPR_HISTELEM *pHE;
   IPR_STATUS status;

   pHE = (IPR_HISTELEM *) (pHB + 1);
   if (v != NON_DATA_VALUE)
   {
    status = IprHbIndex (v, ctr, numcounters, pHB -> minval, pHB -> pcgran,  &hidx);

      if (IPR_SUCCESS (status) && status != IPR__NOTINHIST && hidx < pHB -> elemcnt)
	{
	  pHE [hidx]++;
	}
      else
	{
         skipHist++;
	}
   }
}


/************************************************************************************/
/* Name: ipr_interrupt                                                              */
/*                                                                                  */
/* Purpose:  This is the performance interrupt handler.                             */
/*                                                                                  */
/* Arguments: irq: this is the number of the irq, irrelavent for performance        */
/*            dev_id: an unimportant id.                                            */
/*            regs: a pointer to all of the registers.                              */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None.                                                                  */
/*                                                                                  */
/* Function Value: It is bad to do all of this work in the interrupt.        */
/*                 However, if we don't, all things will start to skip.  Ie. */
/*		   We need all of this work to be atomic.                    */
/*                                                                           */
/************************************************************************************/                       



void ipr_interrupt(unsigned long vector, struct trapframe *framep)  
{

   IPR_HBUFFER *pHB;
   IPR_BUFDSC *pBD;
   IPR_HISTVALUE datum;
   register int cpu_no;
   int hIndex, ridx;
   uint32 *Data32;
   uint64 *Data64, PCval;
   register unsigned int dummy;
   IPR_BOOLEAN collect_sample, collect_histograms;
   IPR_PC_RANGE *pRange;

   getmicrotime(&xtime);

   cpu_no=THIS_CPU();
   pBD = GET_BD_PTR(cpu_no);
   if(!pBD)
   {
     printf("BD Pointer is NULL.\n");
     return;
   }

   if(!(BIprIsSet(pBD->validcounters, THIS_CTR())))
   {	 
     //LINUX
     //     printk(KERN_DEBUG "validcounters:%d THIS_Contorl:%d\n",pBD->validcounters, THIS_CTR());

     return;
   }
   // Update the skip count and skip this interupt if necessary

   if(SkipInt (pBD->pIBuf, THIS_CTR()))
     {
	     /* printf("We are supposed to skip this one.\n"); */
       return;
     }


   // determine if we are to log data for this PC value

   PCval = THIS_PC();
   collect_sample = pBD->pPBuf ? pBD->pPBuf->sample_always : TRUE;
   collect_histograms = collect_sample;
   pRange = pBD->pPBuf ? ((IPR_PC_RANGE *) (pBD->pPBuf + 1)) : NULL;



   // Do the address histograms first.  Stop searching the range list
   // when the PC is less than the start address, since the range list
   // is sorted by start address.

   for (ridx = PCinRange (pBD->pPBuf, PCval);
       (ridx>=0 && ridx<pBD->pPBuf->rangecnt
           && PCval>=pRange[ridx].StartAddress);
        ridx++)
   {

      // For now, since we have no guarantees about the end address order,
      // check the end address before proceeding with saving the data.


      if (PCval <= pRange[ridx].EndAddress)
      {

         // Add to the address histogram if it exists

         pHB = pRange[ridx].pHBuf;
         if (pHB)
            add_to_histogram (pHB, THIS_CTR(), pBD -> numcounters, PCval);

         // Update the general collection flags

         collect_histograms = collect_histograms || pRange[ridx].histogram;
         collect_sample = collect_sample || pRange[ridx].sample;
      }
   }



   if (collect_histograms && pBD -> pHBuf)
   {
     //
     // CREATING HISTOGRAMS
     //

     //
     // NOTE: The code below assumes that the buffers are linear, i.e.
     //       that the last one points to null.
     //
     //       Also note that the data array is organized with all the elements
     //       corresponding to a single value are located contiguously.
     //       For example, an element sequence on a chip with three counters
     //       might be val 1 ctr 0, val 1 ctr 1, val 1 ctr 2, val 2 ctr 0,
     //       val 2 ctr 1, val 2 ctr 2, etc.
     //
     //       All subscripting is done manually.
     //

        // Histogram buffers to fill in?

      for (pHB = pBD -> pHBuf;
           pHB && pHB -> item.ipr_hmask_l_bits != IPR_HMASK_M_ADDRESS;
           pHB = pHB -> pNext)
      {
         switch (pHB -> item.ipr_hmask_l_bits)
         {
            case IPR_HMASK_M_TOTAL :
               datum = 0;
               break;
            case IPR_HMASK_M_IPL :
               datum = THIS_IPL();
               break;
            case IPR_HMASK_M_MODE :
               datum = THIS_MODE();
               break;
            case IPR_HMASK_M_ADDRESS :
               datum = THIS_PC();
               break;
            case IPR_HMASK_M_PRIORITY :
               datum = THIS_PRIORITY();
               break;
            default :
               datum = NON_DATA_VALUE;
               break;
         }

         add_to_histogram (pHB, THIS_CTR(), pBD -> numcounters, datum);
      }
   }      

   // Sample buffers to fill in?

   if (collect_sample && pBD -> pSBuf)
   {
      ////////////////////////
      // COLLECTING SAMPLES //
      ////////////////////////

      IPR_SBUFFER *BufPtr;
      IPR_SBUFFER *marked_buffer;
      IPR_SBUFFER *wptr;
      IPR_BOOLEAN dropSamp;
      unsigned int
         mask32,
         mask64,
         bmask;

      dropSamp = FALSE;
      marked_buffer = NULL;
      BufPtr = pBD -> pSBuf;
      if (BufPtr -> areaoffset >= BufPtr -> areasize)
      {
         //
         // The location is out of range, which means we have finished with
         // this buffer.  Mark it and try to move on to the next buffer.
         //
         if (BufPtr -> areaoffset == BufPtr -> areasize)
         {
	   BARRIER ();
            BufPtr -> full = IPR_BUFFER_FULL;
            BufPtr -> areaoffset++;
            marked_buffer = BufPtr;
         }
         //
         // The current buffer is full.  Check if the next buffer can
         // be used.
         //
         if (BufPtr -> pNext -> full == IPR_BUFFER_FULL)
         {
            // The next buffer is full, so count a missed sample

            BufPtr->missedcnt++;

            if (marked_buffer)
               do_full_action (marked_buffer, pBD, cpu_no);

            return;
         }

         // The next buffer is empty, so switch buffers

         BufPtr = BufPtr -> pNext;
         BufPtr -> full = IPR_BUFFER_EMPTY;
         BufPtr -> begintime = THIS_TIME();
         BufPtr -> missedcnt = 0;
         BufPtr -> areaoffset = 0;
         pBD -> pSBuf = BufPtr;
      }
      //
      // Save the requested information.  Start with the quadword data
      // to allow alignment easily.
      //
      Data64 = (uint64 *) ((char *) ((BufPtr) + 1) + BufPtr->areaoffset);

      // Convert the masks to non-overlapping form

      mask32 = BufPtr->data.ipr_dmask_l_bits;
      mask64 = mask32 & BufPtr->dataquad.ipr_dmask_l_bits;
      mask32 &= ~mask64;

      // Walk the list of data bits in the quadword mask

      /* THIS SHOULD BE REWRITTEN..... Something LIKE this... Phil */
      /*
	


	if (bmask & IPR_DMASK_M_TIME)  
        *Data64++ = THIS_TIME();     

	if (bmask & IPR_DMASK_M_PS)  
        *Data64++ = THIS_PS();     
	
	if (bmask & IPR_DMASK_M_PC)
            *Data64++ = THIS_PC();

	if (bmask & IPR_DMASK_M_COUNTER)
            *Data64++ = THIS_CTR();

        if (bmask & IPR_DMASK_M_PID)
            *Data64++ = THIS_PID();

        if (bmask & IPR_DMASK_M_PRIORITY)
	*Data64++ = THIS_PRIORITY();
       
	if (bmask & IPR_DMASK_M_TIME & IPR_DMASK_M_PS & IPR_DMASK_M_PC & 
	    IPR_DMASK_M_COUNTER & IPR_DMASK_M_PID & IPR_DMASK_M_PRIORITY)
            *Data64++ = 0;

	  */


      for (bmask = BIprLowest (mask64); bmask;
           mask64 = BIprClr (mask64, bmask), bmask = BIprLowest (mask64))
      {
         if (bmask == IPR_DMASK_M_TIME)
            *Data64++ = THIS_TIME();
         else if (bmask == IPR_DMASK_M_PS)
            *Data64++ = THIS_PS();
         else if (bmask == IPR_DMASK_M_PC)
            *Data64++ = THIS_PC();
         else if (bmask == IPR_DMASK_M_COUNTER)
            *Data64++ = THIS_CTR();
         else if (bmask == IPR_DMASK_M_PID)
            *Data64++ = THIS_PID();
         else if (bmask == IPR_DMASK_M_PRIORITY)
            *Data64++ = THIS_PRIORITY();
         else
            *Data64++ = 0;
      }

      // Walk the list of data bits in the longword mask


      Data32 = (uint32 *) Data64;
      for (bmask = BIprLowest (mask32); bmask;
           mask32 = BIprClr (mask32, bmask), bmask = BIprLowest (mask32))
      {
         if (bmask == IPR_DMASK_M_TIME)
            *Data32++ = THIS_TIME();
         else if (bmask == IPR_DMASK_M_PS)
            *Data32++ = THIS_PS();
         else if (bmask == IPR_DMASK_M_PC)
            *Data32++ = THIS_PC();
         else if (bmask == IPR_DMASK_M_COUNTER)
            *Data32++ = THIS_CTR();
         else if (bmask == IPR_DMASK_M_PID)
            *Data32++ = THIS_PID();
         else if (bmask == IPR_DMASK_M_PRIORITY)
            *Data32++ = THIS_PRIORITY();
         else
            *Data32++ = 0;
      }
      dummy = ((char *) Data32 + BufPtr->pad - (char *) (BufPtr + 1));
      BufPtr -> areaoffset = dummy;

      // Notify the process if necessary

       if (marked_buffer)
	 {
	   do_full_action (marked_buffer, pBD, cpu_no);
	 }
   }  // end - sample code
   
   return;
}

/*****************************************************************************/
/* Name: do_full_action                                                      */
/*                                                                           */
/* Purpose: This sends a signal back to "iprobe" that we need flush the      */
/*          buffers.                                                         */
/*                                                                           */
/* Arguments: pSB: Sample Buffer (Contains a chunk of samples. )             */
/*            bD:Buffer Descriptor                                           */
/*            cdb: Which CPU this is full for.                               */
/*                                                                           */
/* Notes:  None.                                                             */
/*                                                                           */
/* Outputs:  None                                                            */
/*                                                                           */
/* Function Value: None.                                                     */
/*                                                                           */
/*****************************************************************************/

void do_full_action (IPR_SBUFFER *pSB, IPR_BUFDSC *bD, IPR_CPUNUM cdb)
{	
    switch(bD -> fullaction)
    {
    case IPR_FULL_WAKE:
    case IPR_FULL_AST:
	{
	  (void) ipr_do_signal((void *)SIGUSR1, (void *)1);
	  break;
	}
    case IPR_FULL_NONE:
	break;
    }

}



