/*
 * Module:      EV4_ROUTINES_NT.C
 *
 * Version:     X-2
 */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains EV4-specific NT-specific performance
 *              counter support routines.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       05-Jul-1994
 *
 * Modified by:
 *
 * X-2  BMW1001         Bradley M. Waters       18-Jul-1994
 *      Modified based on kernel rationalization.
 *
 * X-1  BMW             Bradley M. Waters       05-Jul-1994
 *      Original version.
 */


#include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT

#include "ev4_routines.h"

#include "api_routines_nt.h"
#include "ev4_masks.h"

#include <assert.h>
#include <stdio.h>
//#include <windows.h>

//extern IPR_CPUMASK AFFINITY_CPU_MASK;

extern HANDLE hPC0[];
extern HANDLE hPC1[];

extern EV4SHARED        // Global pointer to shared area defined in ev4_routines.c
   *Ev4Sh;

typedef enum
{
   EV4_PERFMON_DISABLE,               /* Disable the interrupts           */
   EV4_PERFMON_ENABLE,                /* Enable the interrupts            */
   EV4_PERFMON_MUX_SELECT,            /* Select desired function          */
   EV4_PERFMON_OPTIONS,               /* Specify option desired           */
   EV4_PERFMON_PAL_CONTROL,           /* Turn PAL counters on/off         */
   EV4_PERFMON_PAL_ASSIGN             /* Assign PAL counter array         */
} EV4_PERFMON;

typedef enum
{
   EV4_OPTIONS_NONE,
   EV4_OPTIONS_ALL_PROC,
   EV4_OPTIONS_SEL_PROC
} EV4_OPTIONS;

extern uint64 ev4_get_mux (CTRCTXDATA *cctx)
{
   EV4_MUXCTL
      muxctl;

   muxctl . ev4_muxctl_q_bits = 0;
   muxctl . ev4_muxctl_v_pcmux0 = cctx [0] . event_code;
   muxctl . ev4_muxctl_v_pcmux1 = cctx [1] . event_code;
   muxctl . ev4_muxctl_v_pc0 = cctx [0] . freq_code;
   muxctl . ev4_muxctl_v_pc1 = cctx [1] . freq_code;
   return muxctl . ev4_muxctl_q_bits;
}

/*
extern uint64 Ev4FormatMUX(IN IPR_COUNTER_BLOCK cblk[])
{
   // For NT, this function isn't required yet
   return 0;
}
*/

/*
extern uint64 ev4_get_mux (CTRCTXDATA *cctx)
{
   return 0;
}
*/
extern IPR_STATUS Ev4SetModes (int ctx, IPR_MMASK modes)
{
   IPR_COUNTER_BLOCK
      cblock [1];

   if (modes . ipr_mmask_l_bits
       && (modes . ipr_mmask_l_bits & Ev4Sh -> modes . ipr_mmask_l_bits)
          != modes . ipr_mmask_l_bits)
      return IPR__INVALID_MODES;
   Ev4Sh -> context [ctx] . modes = modes;
   cblock [0] . action = IPR_SETMODE;
   cblock [0] . value = modes . ipr_mmask_l_bits;
   return KIprUpdatePerfmon (ctx, 1, cblock);
}

extern void code_to_global (CTRCTXDATA *cctx)
{
   cctx [0] . event = Ev4Sh -> events_0 [cctx [0] . event_code];
   cctx [1] . event = Ev4Sh -> events_1 [cctx [1] . event_code];
   cctx [0] . freq = cctx [0] . freq_code ? Ev4Sh -> ctr_freq_hi [0]
                                        : Ev4Sh -> ctr_freq_lo [0];
   cctx [1] . freq = cctx [1] . freq_code ? Ev4Sh -> ctr_freq_hi [1]
                                        : Ev4Sh -> ctr_freq_lo [1];
}

extern IPR_STATUS Ev4SetEvents (int ctx, uint64 *MUXvalue)
{
   IPR_COUNTER_BLOCK
      cblock [4];
   IPR_COUNT
      blk,
      cblk,
      ctr;
   IPR_STATUS
      status;
   CTRCTXDATA
      *cctx;
//   EV4_MUXCTL
//      mux;

   cctx = Ev4Sh -> context [ctx] . counter;
   /* If a MUX value was passed, send it on */

   if (MUXvalue)
   {

      /* Fill in the counter block array as needed */

      for (ctr = blk = 0; ctr < EV4_COUNTER_COUNT; ctr++, blk++)
      {
         cblock [blk] . ctr = ctr;
         cblock [blk] . action = IPR_SETFROMCODE;
         cblock [blk] . value = *MUXvalue;
      }

      /* Set the value */

      status = KIprUpdatePerfmon (ctx, blk, cblock);
      if (IPR_SUCCESS (status))
      {
         //
         // Go back through the counter blocks and retrieve the resultant
         // code settings.
         //
//         for (blk--; blk >= 0; blk--)  - due to unsigned int this statement doesn't work
         for (cblk=0; cblk < blk; cblk++)
         {
            ctr = cblock [cblk] . ctr;
            cctx [ctr] . event_code = cblock [cblk] . event;
            cctx [ctr] . freq_code = cblock [cblk] . freq;
         }

         /* Translate the codes to global events and frequencies */

         code_to_global (cctx);
      }
      return status;
   }

   /* No MUX value, so transfer the saved data to counter blocks */

   for (blk = ctr = 0; ctr < EV4_COUNTER_COUNT; ctr++)
   {
      cblock [blk] . ctr = ctr;
      cblock [blk] . action = IPR_SETEVENT;
      cblock [blk] . event = cctx [ctr] . event_code;
      blk++;

      cblock [blk] . ctr = ctr;
      cblock [blk] . action = IPR_SETFREQ;
      cblock [blk] . freq = cctx [ctr] . freq_code;
      blk++;
   }
   status = KIprUpdatePerfmon (ctx, blk, cblock);
   return status;
}

static IPR_STATUS Ev4StartStop (int ctx, IPR_CTRMASK ctrs, IPR_ACTION action)
{
   IPR_COUNTER_BLOCK
      cblock [2];
   IPR_CTRNUMBER
      ctr;
   IPR_COUNT
      blk;
   IPR_STATUS
      status;

   if (ctrs && (ctrs & EV4_COUNTER_MASK) != EV4_COUNTER_MASK)
      return IPR__INCOMPLETE_ERROR;
   for (blk = ctr = 0; ctr < EV4_COUNTER_COUNT; ctr++)
   {
      cblock [blk] . ctr = ctr;
      cblock [blk] . action = action;
      blk++;
   }
   status = KIprUpdatePerfmon (ctx, blk, cblock);
   return status;
}


extern IPR_STATUS Ev4Enable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4StartStop (ctx, ctrs, IPR_ENABLE);
}

extern IPR_STATUS Ev4Disable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4StartStop (ctx, ctrs, IPR_DISABLE);
}

extern void Ev4OSInit (void)
{
//   0;
//   Ev4Sh -> modes . ipr_mmask_v_selected = 0;
}


