/*++

Copyright 1993, 1994, 1995 Digital Equipment Corporation

Module Name:

    ev4ctr.c

Abstract:

    This module maintains the counter states in shared global structures.

Author:

    Santa Wiryaman
    CSG Performance Group

Revision History:

    Brad Waters        25-Jan-1995
        Modified scaling for perf. counters

    Brad Waters        20-Dec-1994
        Updated for use with the IPROBE API

    Brad Waters         7-Jun-1994
        Added the ability to adjust the scaling for the counters.

    Brad Waters        20-May-1994
        Cleaned up the code.

    Brad Waters        18-May-1994
        Modified to return instance data for each CPU.

    Brad Waters         6-May-1994
        Modified to support MP.  Added ProcessorNum array subscript to
        counterInfo

    Santa Wiryaman     29-Nov-1993
        Version 1.0


--*/

#include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
//#include <windows.h>
#include <winperf.h>

// Kludge.  The following two typedefs should be in one of the SDK header
// files.  But apparently I couldn't find it.
typedef unsigned __int64 ULONGLONG;
typedef long NTSTATUS;

#include "ctr_int_defs.h"
#include "ipr_api_int_defs.h"

#include "iprctrnm.h"
#include "ev4ctr.h"
#include "nta21064.h"

//
//  Constant structure initialization for the performance data definition
//  of the EV4 performance counters
//

EV4CTR_DATA_AND_DEFINITION SharedEV4CtrData = {
    {{ // Object type definition
        sizeof(EV4CTR_DATA_AND_DEFINITION),     //TotalByteLength
        sizeof(EV4CTR_DEFINITION),              //DefinitionLength
        sizeof(PERF_OBJECT_TYPE),               //HeaderLength
        EV4_CTR_OBJ,                            //ObjectNameTitleIndex
        0,                                      //ObjectNameTitle
        EV4_CTR_OBJ,                            //ObjectHelpTitleIndex
        0,                                      //ObjectHelpTitle
        PERF_DETAIL_NOVICE,                     //DetailLevel
        (NUM_CTR0_EVENTS+NUM_CTR1_EVENTS),      //NumCounters
        4,                                      //DefaultCounter - branch is 1st alphabetically
        NUM_PROCESSORS,                         //NumInstances
        0,                                      //CodePage - instances in unicode
        0,                                      //PerfTime
        0                                       //PerfFreq
    },
    { // Issues counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ISSUES,                                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ISSUES,                                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ISSUES_OFFSET                       //CounterOffset
    },
    { // PipeDry counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PIPEDRY,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PIPEDRY,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PIPEDRY_OFFSET                      //CounterOffset
    },
    { // Load counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        LOAD,                                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        LOAD,                                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_LOAD_OFFSET                         //CounterOffset
    },
    { // PipeFroze counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PIPEFROZE,                              //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PIPEFROZE,                              //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PIPEFROZE_OFFSET                    //CounterOffset
    },
    { // Branch counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        BRANCH,                                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        BRANCH,                                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_BRANCH_OFFSET                       //CounterOffset
    },
    { // PalCalls counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PAL,                                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PAL,                                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PAL_OFFSET                          //CounterOffset
    },
    { // Cycles counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        CYCLES,                                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        CYCLES,                                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -7,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_CYCLES_OFFSET                       //CounterOffset
    },
    { // NonIssues counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        NONISSUES,                              //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        NONISSUES,                              //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -7,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_NONISSUES_OFFSET                    //CounterOffset
    },
    { // Ext0 counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        EXT0,                                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        EXT0,                                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_EXT0_OFFSET                         //CounterOffset
    },
    { // DCache Miss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DCACHEMISS,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DCACHEMISS,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -4,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DCACHEMISS_OFFSET                   //CounterOffset
    },
    { // ICache Miss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ICACHEMISS,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ICACHEMISS,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ICACHEMISS_OFFSET                   //CounterOffset
    },
    { // Dual Issues counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DUALISSUES,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DUALISSUES,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DUALISSUES_OFFSET                   //CounterOffset
    },
    { // Branch Mispredicts counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        BRANCH_MISPREDICTS,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        BRANCH_MISPREDICTS,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_BRANCH_MISPREDICTS_OFFSET           //CounterOffset
    },
    { // Floating Point Instruction counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        FP_INSTRUCTIONS,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        FP_INSTRUCTIONS,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_FP_INSTRUCTIONS_OFFSET              //CounterOffset
    },
    { // Integer Instruction counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        INT_INSTRUCTIONS,                       //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        INT_INSTRUCTIONS,                       //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_INT_INSTRUCTIONS_OFFSET             //CounterOffset
    },
    { // Store Instruction counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        STORES,                                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        STORES,                                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -5,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_STORES_OFFSET                       //CounterOffset
    },
    { // External Event 1 counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        EXT1,                                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        EXT1,                                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -6,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_EXT1_OFFSET                         //CounterOffset
    }},
    {
    {{{ // Instance 1 - cpu 0 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"0"
    }},
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
    ,
    {{{ // Instance 2 - cpu 1 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"1"
    }},
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{{ // Instance 3 - cpu 2 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"2"
    }},
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{{ // Instance 4 - cpu 3 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"3"
    }},
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
/*
    ,
    {{ // Instance 5 - cpu 4 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"4"
    },
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 6 - cpu 5 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"5"
    },
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 7 - cpu 6 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"6"
    },
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 8 - cpu 7 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"7"
    },
    { // counter data area length for this instance
        sizeof(CTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
    }
*/
    }
};


//
//  Constant structure initialization for the performance data definition
//  of the EV4 PAL routine counters
//

EV4PALCTR_DATA_AND_DEFINITION SharedEV4PALCtrData = {
    {{ // Object type definition
        sizeof(EV4PALCTR_DATA_AND_DEFINITION),  //TotalByteLength
        sizeof(EV4PALCTR_DEFINITION),           //DefinitionLength
        sizeof(PERF_OBJECT_TYPE),               //HeaderLength
        EV4_PALCTR_OBJ,                         //ObjectNameTitleIndex
        0,                                      //ObjectNameTitle
        EV4_PALCTR_OBJ,                         //ObjectHelpTitleIndex
        0,                                      //ObjectHelpTitle
        PERF_DETAIL_NOVICE,                     //DetailLevel
        NUM_PAL_COUNTERS,                       //NumCounters
        1,                                      //DefaultCounter - arithmetic exception is 1st alphabetically
        NUM_PROCESSORS,                         //NumInstances
        0,                                      //CodePage
        0,                                      //PerfTime
        0                                       //PerfFreq
    },
    { // MachineCheck counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        MACHINECHECK,                           //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        MACHINECHECK,                           //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_MACHINECHECK_OFFSET                 //CounterOffset
    },
    { // ArithmeticException counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ARITHMETICEXCEPTION,                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ARITHMETICEXCEPTION,                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ARITHMETICEXCEPTION_OFFSET          //CounterOffset
    },
    { // Interrupt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        INTERRUPT,                              //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        INTERRUPT,                              //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -3,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_INTERRUPT_OFFSET                    //CounterOffset
    },
    { // ItbMiss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ITBMISS,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ITBMISS,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -3,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ITBMISS_OFFSET                      //CounterOffset
    },
    { // NativeDtbMiss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        NATIVEDTBMISS,                          //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        NATIVEDTBMISS,                          //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -3,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_NATIVEDTBMISS_OFFSET                //CounterOffset
    },
    { // PalDtbMiss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PALDTBMISS,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PALDTBMISS,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -2,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PALDTBMISS_OFFSET                   //CounterOffset
    },
    { // ItbAccessViolation counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ITBACCESSVIOLATION,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ITBACCESSVIOLATION,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ITBACCESSVIOLATION_OFFSET           //CounterOffset
    },
    { // DtbAccessViolation counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DTBACCESSVIOLATION,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DTBACCESSVIOLATION,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DTBACCESSVIOLATION_OFFSET           //CounterOffset
    },
    { // UnalignedAccess counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        UNALIGNEDACCESS,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        UNALIGNEDACCESS,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_UNALIGNEDACCESS_OFFSET              //CounterOffset
    },
    { // IllegalInstruction counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ILLEGALINSTRUCTION,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ILLEGALINSTRUCTION,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ILLEGALINSTRUCTION_OFFSET           //CounterOffset
    },
    { // FloatingEnableException counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        FLOATINGENABLEEXCEPTION,                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        FLOATINGENABLEEXCEPTION,                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_FLOATINGENABLEEXCEPTION_OFFSET      //CounterOffset
    },
    { // ItbTranslationNotValid counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ITBTRANSLATIONNOTVALID,                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ITBTRANSLATIONNOTVALID,                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ITBTRANSLATIONNOTVALID_OFFSET       //CounterOffset
    },
    { // DtbTranslationNotValid counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DTBTRANSLATIONNOTVALID,                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DTBTRANSLATIONNOTVALID,                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DTBTRANSLATIONNOTVALID_OFFSET       //CounterOffset
    },
    { // PteMiss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PTEMISS,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PTEMISS,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -2,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PTEMISS_OFFSET                      //CounterOffset
    },
    { // PalKernelStackMiss counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PALKERNELSTACKMISS,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PALKERNELSTACKMISS,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -1,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PALKERNELSTACKMISS_OFFSET           //CounterOffset
    },
    { // PdeTranslationNotValid counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        PDETRANSLATIONNOTVALID,                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        PDETRANSLATIONNOTVALID,                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_PDETRANSLATIONNOTVALID_OFFSET       //CounterOffset
    },
    { // Halt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        HALT,                                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        HALT,                                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_HALT_OFFSET                         //CounterOffset
    },
    { // Restart counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        RESTART,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        RESTART,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_RESTART_OFFSET                      //CounterOffset
    },
    { // DrainAbort counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DRAINABORT,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DRAINABORT,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DRAINABORT_OFFSET                   //CounterOffset
    },
    { // InitPal counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        INITPAL,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        INITPAL,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_INITPAL_OFFSET                      //CounterOffset
    },
    { // Wrentry counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        WRENTRY,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        WRENTRY,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_WRENTRY_OFFSET                      //CounterOffset
    },
    { // SwapIrql counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SWAPIRQL,                               //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SWAPIRQL,                               //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -3,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SWAPIRQL_OFFSET                     //CounterOffset
    },
    { // ReadIrql counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READIRQL,                               //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READIRQL,                               //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -1,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READIRQL_OFFSET                     //CounterOffset
    },
    { // DisableInterrupt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DISABLEINTERRUPT,                       //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DISABLEINTERRUPT,                       //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -2,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DISABLEINTERRUPT_OFFSET             //CounterOffset
    },
    { // EnableInterrupt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        ENABLEINTERRUPT,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        ENABLEINTERRUPT,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -2,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_ENABLEINTERRUPT_OFFSET              //CounterOffset
    },
    { // SwapPalcode counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SWAPPALCODE,                            //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SWAPPALCODE,                            //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SWAPPALCODE_OFFSET                  //CounterOffset
    },
    { // SetSoftwareInterrupt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SETSOFTWAREINTERRUPT,                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SETSOFTWAREINTERRUPT,                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        -1,                                     //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SETSOFTWAREINTERRUPT_OFFSET         //CounterOffset
    },
    { // ClearSoftwareInterrupt counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        CLEARSOFTWAREINTERRUPT,                 //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        CLEARSOFTWAREINTERRUPT,                 //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_CLEARSOFTWAREINTERRUPT_OFFSET       //CounterOffset
    }

#ifndef LODCTRBUG

    ,{ // ReturnFromException counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        RETURNFROMEXCEPTION,                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        RETURNFROMEXCEPTION,                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_RETURNFROMEXCEPTION_OFFSET          //CounterOffset
    },
    { // ReturnFromSystemCall counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        RETURNFROMSYSTEMCALL,                   //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        RETURNFROMSYSTEMCALL,                   //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_RETURNFROMSYSTEMCALL_OFFSET         //CounterOffset
    },
    { // SwapContext counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SWAPCONTEXT,                            //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SWAPCONTEXT,                            //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SWAPCONTEXT_OFFSET                  //CounterOffset
    },
    { // SwapProcess counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SWAPPROCESS,                            //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SWAPPROCESS,                            //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SWAPPROCESS_OFFSET                  //CounterOffset
    },
    { // ReadMces counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READMCES,                               //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READMCES,                               //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READMCES_OFFSET                     //CounterOffset
    },
    { // WriteMces counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        WRITEMCES,                              //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        WRITEMCES,                              //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_WRITEMCES_OFFSET                    //CounterOffset
    },
    { // TbInvalidateAll counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        TBINVALIDATEALL,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        TBINVALIDATEALL,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_TBINVALIDATEALL_OFFSET              //CounterOffset
    },
    { // TbInvalidateSingle counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        TBINVALIDATESINGLE,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        TBINVALIDATESINGLE,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_TBINVALIDATESINGLE_OFFSET           //CounterOffset
    },
    { // DtbInvalidateSingle counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        DTBINVALIDATESINGLE,                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        DTBINVALIDATESINGLE,                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_DTBINVALIDATESINGLE_OFFSET          //CounterOffset
    },
    { // ReadKernelStack counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READKERNELSTACK,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READKERNELSTACK,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READKERNELSTACK_OFFSET              //CounterOffset
    },
    { // SwapKernelStack counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SWAPKERNELSTACK,                        //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SWAPKERNELSTACK,                        //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SWAPKERNELSTACK_OFFSET              //CounterOffset
    },
    { // ReadProcessorStatus counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READPROCESSORSTATUS,                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READPROCESSORSTATUS,                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READPROCESSORSTATUS_OFFSET          //CounterOffset
    },
    { // ReadPcr counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READPCR,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READPCR,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READPCR_OFFSET                      //CounterOffset
    },
    { // ReadThread counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READTHREAD,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READTHREAD,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READTHREAD_OFFSET                   //CounterOffset
    },
    { // ReadCounters counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READCOUNTERS,                           //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READCOUNTERS,                           //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READCOUNTERS_OFFSET                 //CounterOffset
    },
    { // ReadState counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READSTATE,                              //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READSTATE,                              //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READSTATE_OFFSET                    //CounterOffset
    },
    { // WritePerfmon counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        WRITEPERFMON,                           //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        WRITEPERFMON,                           //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_WRITEPERFMON_OFFSET                 //CounterOffset
    },
    { // InitPcr counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        INITPCR,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        INITPCR,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_INITPCR_OFFSET                      //CounterOffset
    },
    { // UserBreakpoint counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        USERBREAKPOINT,                         //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        USERBREAKPOINT,                         //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_USERBREAKPOINT_OFFSET               //CounterOffset
    },
    { // SystemCall counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        SYSTEMCALL,                             //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        SYSTEMCALL,                             //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_SYSTEMCALL_OFFSET                   //CounterOffset
    },
    { // Imb counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        IMB,                                    //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        IMB,                                    //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_IMB_OFFSET                          //CounterOffset
    },
    { // GenerateTrap counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        GENERATETRAP,                           //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        GENERATETRAP,                           //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_GENERATETRAP_OFFSET                 //CounterOffset
    },
    { // ReadTeb counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        READTEB,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        READTEB,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_READTEB_OFFSET                      //CounterOffset
    },
    { // KernelBreakpoint counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        KERNELBREAKPOINT,                       //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        KERNELBREAKPOINT,                       //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_KERNELBREAKPOINT_OFFSET             //CounterOffset
    },
    { // CallKernelDebugger counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        CALLKERNELDEBUGGER,                     //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        CALLKERNELDEBUGGER,                     //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_CALLKERNELDEBUGGER_OFFSET           //CounterOffset
    },
    { // TbisAsn counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        TBISASN,                                //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        TBISASN,                                //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_TBISASN_OFFSET                      //CounterOffset
    },
    { // Misc1 counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        MISC1,                                  //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        MISC1,                                  //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_MISC1_OFFSET                        //CounterOffset
    },
    { // Misc2 counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        MISC2,                                  //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        MISC2,                                  //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_MISC2_OFFSET                        //CounterOffset
    },
    { // Misc3 counter definition
        sizeof(PERF_COUNTER_DEFINITION),        //ByteLength
        MISC3,                                  //CounterNameTitleIndex
        0,                                      //CounterNameTitle
        MISC3,                                  //CounterHelpTitleIndex
        0,                                      //CounterHelpTitle
        0,                                      //Default Scale
        PERF_DETAIL_NOVICE,                     //DetailLevel
        PERF_COUNTER_BULK_COUNT,                //CounterType
        sizeof(AXPCOUNTER),                     //CounterSize
        NUM_MISC3_OFFSET                        //CounterOffset
    }

#endif
    },
    {
    {{{ // Instance 1 - cpu 0 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"0"
    }},
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
    ,
    {{{ // Instance 2 - cpu 1 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),
                                                //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"1"
    }},
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{{ // Instance 3 - cpu 2 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"2"
    }},
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{{ // Instance 4 - cpu 3 definition
        (sizeof(EV4_PERF_INSTANCE_DEF)),        //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"3"
    }},
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
/*
    ,
    {{ // Instance 5 - cpu 4 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"4"
    },
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 6 - cpu 5 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"5"
    },
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 7 - cpu 6 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"6"
    },
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }},
    {{ // Instance 8 - cpu 7 definition
        sizeof(EV4_PERF_INSTANCE_DEF),          //ByteLength - including name
        0,                                      //ParentObjectTitleIndex
        0,                                      //ParentObjectInstance
        PERF_NO_UNIQUE_ID,                      //UniqueID
        sizeof(PERF_INSTANCE_DEFINITION),       //NameOffset
        sizeof(USTRING)                         //NameLength
    },
    { // unicode string for this instance
        L"7"
    },
    { // counter data area length for this instance
        sizeof(PALCTR_DATA)
    },
    { // counter data area for this instance
        0
    }}
    }
*/
    }
};


//
// Counter Type Table
//
const DWORD CounterType[NUM_CTR_TYPES] = {
//      PERF_COUNTER_TIMER,
        PERF_COUNTER_BULK_COUNT,
        PERF_COUNTER_RAWCOUNT
};


//
// Counter Info table containing event types, counter type index, default
// scales, and frequencies.  The "row" is processor number and the "column"
// is counter number.
//  Initialize counterInfo for 1st processor here and clone for other
//  processors in InitCntInf, below.

COUNTER_INFO counterInfo[IPR_MAX_COUNTERS] = {
        //counter 0
        {
/*        // event
          {TotalIssues,
           PipelineDry,
           LoadInstruction,
           PipelineFrozen,
           BranchInstructions,
           PalMode,
           TotalCycles,
           TotalNonIssues,
           ExternalCounter0
          },
*/
          //iCtrType
          {0,
           0,
           0,
           0,
           0,
           0,
           0,
           0,
           0
          }
/*
          ,
          //freq
          { CountEvents2xx16, CountEvents2xx12 }
*/
        },
        //counter 1
        {
/*        //event
          {DcacheMiss,
           IcacheMiss,
           DualIssues,
           BranchMispredicts,
           FPInstructions,
           IntegerOperate,
           StoreInstructions,
           ExternalCounter1,
           0                    // extra unused field
          },
*/
          //iCtrType
          {0,
           0,
           0,
           0,
           0,
           0,
           0,
           0,
           0xffffffff           // extra unused field
          }
/*
          ,
          //freq
          { CountEvents2xx12, CountEvents2xx8 }
*/
        }
};



//COUNTER_SCALE counterScale;  // This will be set to dfltCounterScale
                             // before its used

/*
const COUNTER_SCALE dfltCounterScale = {
{ // Scaling for counter 0
           -6,
           -6,
           -5,
           -6,
           -6,
           -6,
           -7,
           -7,
           -6,
// Scaling for counter 1
           -4,
           -5,
           -5,
           -5,
           -5,
           -5,
           -5,
           -6,
           0                    // extra unused field
}
};
*/
/*
PAL_SCALE palScale = {
// Scales for the PAL counters
        0,                                      //Default Scale
        0,                                      //Default Scale
        -3,                                     //Default Scale
        -3,                                     //Default Scale
        -3,                                     //Default Scale
        -2,                                     //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        -2,                                     //Default Scale
        -1,                                     //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        -3,                                     //Default Scale
        -1,                                     //Default Scale
        -2,                                     //Default Scale
        -2,                                     //Default Scale
        0,                                      //Default Scale
        -1,                                     //Default Scale
        0                                       //Default Scale
#ifndef LODCTRBUG
        ,
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0,                                      //Default Scale
        0                                       //Default Scale
#endif
};
*/
/*
PROC_ACTIVE proc_active_list[NUM_PROCESSORS] = {
1
,0
,0
,0};
*/
// 1ST processor active, other processors inactive;

//ULONG   num_active_processors = 1;

/*
DWORD   InitDone = {0};         // Indicates whether global data was initialized

VOID
InitCntInf()
*/
/*++

Routine Description:

    This routine initializes the counterInfo structure for the 2nd through
    NUM_PROCESSORS-1 (currently 31) processors);  This routine needs to be
    called to handle multiprocessor environments.

Arguments:
    none

Return Value:
    none

--*/
/*
{

   ULONG ProcessorNum,CounterNum;

// Clone processor 0 for other processors.

   for (ProcessorNum = first_cpu (IPR_ALL_CPUS, IPR_NOAFFINITY); ProcessorNum < IPR_MAX_CPUS;
     ProcessorNum = next_cpu (IPR_ALL_CPUS, ProcessorNum, IPR_NOAFFINITY))

      for (CounterNum=0; CounterNum < NUM_COUNTERS; CounterNum++)

         counterInfo[CounterNum+(ProcessorNum*NUM_COUNTERS)] =
          counterInfo[CounterNum];


}
*/
