/*++

Copyright 1993,1994,1995 Digital Equipment Corporation

Patent Pending, Digital Equipment Corporation

Module Name:

    ev4ctr.h

Abstract:

    This header file defines the structures and #defines for the extensible
    counter objects for the EV4 performance counters and PAL routine counters.
    It also defines the counter state structures.

Author:

    Santa Wiryaman                      29-Nov-1993
    CSG Performance Group

Revision History:

    Brad Waters        20-Dec-1994
        Updated for the IPROBE API

    Brad Waters         7-Jun-1994
        Added the ability to adjust the scaling for the counters.

    Brad Waters        20-May-1994
        Cleaned up definitions.

    Brad Waters        18-May-1994
        Modified to return multiple CPU data as instances to PERFMON.  One
        instance per CPU.

    Brad Waters         6-May-1994
        Modified to support MP systems.  Added a definition for NUM_PROCESSORS.


---*/

#pragma pack (4)

//
//  Extensible Object definitions
//

//  Update the following sort of define when adding an object type - an object
//  type here refers to a chunck of data passed to PERFMON.  Currently there are
//  2 object types, 1 for the EV4 counters and 1 for the PAL counters.

#define EV4CTR_NUM_PERF_OBJECT_TYPES 2

#define NUM_COUNTERS 2
#define NUM_PROCESSORS 4        // max. number of processors for now

//----------------------------------------------------------------------------

//
//  These are used in the counter definitions to describe the relative
//  position of each counter in the returned data.
//

#include "dda21064.h"
#include "iprctrnm.h"

typedef WCHAR USTRING[6];

// EV4 Performance Counters definition offsets

#define NUM_ISSUES_OFFSET           sizeof(DWORD)
#define NUM_PIPEDRY_OFFSET          NUM_ISSUES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_LOAD_OFFSET             NUM_PIPEDRY_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PIPEFROZE_OFFSET        NUM_LOAD_OFFSET + sizeof(AXPCOUNTER)
#define NUM_BRANCH_OFFSET           NUM_PIPEFROZE_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PAL_OFFSET              NUM_BRANCH_OFFSET + sizeof(AXPCOUNTER)
#define NUM_CYCLES_OFFSET           NUM_PAL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_NONISSUES_OFFSET        NUM_CYCLES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_EXT0_OFFSET             NUM_NONISSUES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DCACHEMISS_OFFSET       NUM_EXT0_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ICACHEMISS_OFFSET       NUM_DCACHEMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DUALISSUES_OFFSET       NUM_ICACHEMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_BRANCH_MISPREDICTS_OFFSET \
                                    NUM_DUALISSUES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_FP_INSTRUCTIONS_OFFSET  NUM_BRANCH_MISPREDICTS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_INT_INSTRUCTIONS_OFFSET NUM_FP_INSTRUCTIONS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_STORES_OFFSET           NUM_INT_INSTRUCTIONS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_EXT1_OFFSET             NUM_STORES_OFFSET + sizeof(AXPCOUNTER)
#define SIZE_OF_EV4CTR_PERFORMANCE_DATA \
                                    NUM_EXT1_OFFSET + sizeof(AXPCOUNTER)


// EV4 PAL Routine Counters definition offsets

#define NUM_MACHINECHECK_OFFSET                 sizeof(DWORD)
#define NUM_ARITHMETICEXCEPTION_OFFSET          NUM_MACHINECHECK_OFFSET + sizeof(AXPCOUNTER)
#define NUM_INTERRUPT_OFFSET                    NUM_ARITHMETICEXCEPTION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ITBMISS_OFFSET                      NUM_INTERRUPT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_NATIVEDTBMISS_OFFSET                NUM_ITBMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PALDTBMISS_OFFSET                   NUM_NATIVEDTBMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ITBACCESSVIOLATION_OFFSET           NUM_PALDTBMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DTBACCESSVIOLATION_OFFSET           NUM_ITBACCESSVIOLATION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_UNALIGNEDACCESS_OFFSET              NUM_DTBACCESSVIOLATION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ILLEGALINSTRUCTION_OFFSET           NUM_UNALIGNEDACCESS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_FLOATINGENABLEEXCEPTION_OFFSET      NUM_ILLEGALINSTRUCTION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ITBTRANSLATIONNOTVALID_OFFSET       NUM_FLOATINGENABLEEXCEPTION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DTBTRANSLATIONNOTVALID_OFFSET       NUM_ITBTRANSLATIONNOTVALID_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PTEMISS_OFFSET                      NUM_DTBTRANSLATIONNOTVALID_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PALKERNELSTACKMISS_OFFSET           NUM_PTEMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_PDETRANSLATIONNOTVALID_OFFSET       NUM_PALKERNELSTACKMISS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_HALT_OFFSET                         NUM_PDETRANSLATIONNOTVALID_OFFSET + sizeof(AXPCOUNTER)
#define NUM_RESTART_OFFSET                      NUM_HALT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DRAINABORT_OFFSET                   NUM_RESTART_OFFSET + sizeof(AXPCOUNTER)
#define NUM_INITPAL_OFFSET                      NUM_DRAINABORT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_WRENTRY_OFFSET                      NUM_INITPAL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SWAPIRQL_OFFSET                     NUM_WRENTRY_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READIRQL_OFFSET                     NUM_SWAPIRQL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DISABLEINTERRUPT_OFFSET             NUM_READIRQL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_ENABLEINTERRUPT_OFFSET              NUM_DISABLEINTERRUPT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SWAPPALCODE_OFFSET                  NUM_ENABLEINTERRUPT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SETSOFTWAREINTERRUPT_OFFSET         NUM_SWAPPALCODE_OFFSET + sizeof(AXPCOUNTER)
#define NUM_CLEARSOFTWAREINTERRUPT_OFFSET       NUM_SETSOFTWAREINTERRUPT_OFFSET + sizeof(AXPCOUNTER)

#ifdef LODCTRBUG

#define SIZE_OF_EV4PALCTR_PERFORMANCE_DATA      (NUM_CLEARSOFTWAREINTERRUPT_OFFSET + sizeof(AXPCOUNTER))

#endif

#ifndef LODCTRBUG

#define NUM_RETURNFROMEXCEPTION_OFFSET          NUM_CLEARSOFTWAREINTERRUPT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_RETURNFROMSYSTEMCALL_OFFSET         NUM_RETURNFROMEXCEPTION_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SWAPCONTEXT_OFFSET                  NUM_RETURNFROMSYSTEMCALL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SWAPPROCESS_OFFSET                  NUM_SWAPCONTEXT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READMCES_OFFSET                     NUM_SWAPPROCESS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_WRITEMCES_OFFSET                    NUM_READMCES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_TBINVALIDATEALL_OFFSET              NUM_WRITEMCES_OFFSET + sizeof(AXPCOUNTER)
#define NUM_TBINVALIDATESINGLE_OFFSET           NUM_TBINVALIDATEALL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_DTBINVALIDATESINGLE_OFFSET          NUM_TBINVALIDATESINGLE_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READKERNELSTACK_OFFSET              NUM_DTBINVALIDATESINGLE_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SWAPKERNELSTACK_OFFSET              NUM_READKERNELSTACK_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READPROCESSORSTATUS_OFFSET          NUM_SWAPKERNELSTACK_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READPCR_OFFSET                      NUM_READPROCESSORSTATUS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READTHREAD_OFFSET                   NUM_READPCR_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READCOUNTERS_OFFSET                 NUM_READTHREAD_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READSTATE_OFFSET                    NUM_READCOUNTERS_OFFSET + sizeof(AXPCOUNTER)
#define NUM_WRITEPERFMON_OFFSET                 NUM_READSTATE_OFFSET + sizeof(AXPCOUNTER)
#define NUM_INITPCR_OFFSET                      NUM_WRITEPERFMON_OFFSET + sizeof(AXPCOUNTER)
#define NUM_USERBREAKPOINT_OFFSET               NUM_INITPCR_OFFSET + sizeof(AXPCOUNTER)
#define NUM_SYSTEMCALL_OFFSET                   NUM_USERBREAKPOINT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_IMB_OFFSET                          NUM_SYSTEMCALL_OFFSET + sizeof(AXPCOUNTER)
#define NUM_GENERATETRAP_OFFSET                 NUM_IMB_OFFSET + sizeof(AXPCOUNTER)
#define NUM_READTEB_OFFSET                      NUM_GENERATETRAP_OFFSET + sizeof(AXPCOUNTER)
#define NUM_KERNELBREAKPOINT_OFFSET             NUM_READTEB_OFFSET + sizeof(AXPCOUNTER)
#define NUM_CALLKERNELDEBUGGER_OFFSET           NUM_KERNELBREAKPOINT_OFFSET + sizeof(AXPCOUNTER)
#define NUM_TBISASN_OFFSET                      NUM_CALLKERNELDEBUGGER_OFFSET + sizeof(AXPCOUNTER)
#define NUM_MISC1_OFFSET                        NUM_TBISASN_OFFSET + sizeof(AXPCOUNTER)
#define NUM_MISC2_OFFSET                        NUM_MISC1_OFFSET + sizeof(AXPCOUNTER)
#define NUM_MISC3_OFFSET                        NUM_MISC2_OFFSET + sizeof(AXPCOUNTER)

#define SIZE_OF_EV4PALCTR_PERFORMANCE_DATA      (NUM_MISC3_OFFSET + sizeof(AXPCOUNTER))
#endif

#define NUM_ACTR_EVENTS         ((SIZE_OF_EV4CTR_PERFORMANCE_DATA-sizeof(DWORD)) / sizeof(AXPCOUNTER))
#define NUM_PAL_COUNTERS        ((SIZE_OF_EV4PALCTR_PERFORMANCE_DATA-sizeof(DWORD)) / sizeof(AXPCOUNTER))

typedef AXPCOUNTER CTR_DATA[NUM_ACTR_EVENTS];
typedef AXPCOUNTER PALCTR_DATA[NUM_PAL_COUNTERS];

//
// EV4 Performance Counters data definition
//
typedef struct _EV4_PERF_INSTANCE_DEF {
    PERF_INSTANCE_DEFINITION    cpu_inst;
    USTRING                     us;
} EV4_PERF_INSTANCE_DEF;

typedef struct _EV4CTR_INSTANCE {
    EV4_PERF_INSTANCE_DEF       idef;
    PERF_COUNTER_BLOCK          datalen;
    CTR_DATA                    data;

} EV4CTR_INSTANCE;

typedef struct _EV4CTR_DEFINITION {
    PERF_OBJECT_TYPE            EV4CtrObjectType;
    PERF_COUNTER_DEFINITION     Issues;
    PERF_COUNTER_DEFINITION     PipeDry;
    PERF_COUNTER_DEFINITION     Load;
    PERF_COUNTER_DEFINITION     PipeFroze;
    PERF_COUNTER_DEFINITION     Branch;
    PERF_COUNTER_DEFINITION     PALCalls;
    PERF_COUNTER_DEFINITION     Cycles;
    PERF_COUNTER_DEFINITION     NonIssues;
    PERF_COUNTER_DEFINITION     Ext0;
    PERF_COUNTER_DEFINITION     DCacheMiss;
    PERF_COUNTER_DEFINITION     ICacheMiss;
    PERF_COUNTER_DEFINITION     DualIssues;
    PERF_COUNTER_DEFINITION     BranchMispredicts;
    PERF_COUNTER_DEFINITION     FPInstructions;
    PERF_COUNTER_DEFINITION     IntInstructions;
    PERF_COUNTER_DEFINITION     Store;
    PERF_COUNTER_DEFINITION     Ext1;
} EV4CTR_DEFINITION;

typedef struct _EV4CTR_DATA_AND_DEFINITION {
    EV4CTR_DEFINITION           defs;
    EV4CTR_INSTANCE             cpu[NUM_PROCESSORS];
} EV4CTR_DATA_AND_DEFINITION;

//
// EV4 PAL Routine Counters data definition
//

typedef struct _EV4PALCTR_INSTANCE {
    EV4_PERF_INSTANCE_DEF       idef;
    PERF_COUNTER_BLOCK          datalen;
    PALCTR_DATA                 data;

} EV4PALCTR_INSTANCE;

typedef struct _EV4PALCTR_DEFINITION {
    PERF_OBJECT_TYPE            EV4PALCtrObjectType;
    PERF_COUNTER_DEFINITION     MachineCheck;
    PERF_COUNTER_DEFINITION     ArithmeticException;
    PERF_COUNTER_DEFINITION     Interrupt;
    PERF_COUNTER_DEFINITION     ItbMiss;
    PERF_COUNTER_DEFINITION     NativeDtbMiss;
    PERF_COUNTER_DEFINITION     PalDtbMiss;
    PERF_COUNTER_DEFINITION     ItbAccessViolation;
    PERF_COUNTER_DEFINITION     DtbAccessViolation;
    PERF_COUNTER_DEFINITION     UnalignedAccess;
    PERF_COUNTER_DEFINITION     IllegalInstruction;
//10
    PERF_COUNTER_DEFINITION     FloatingEnableException;
    PERF_COUNTER_DEFINITION     ItbTranslationNotValid;
    PERF_COUNTER_DEFINITION     DtbTranslationNotValid;
    PERF_COUNTER_DEFINITION     PteMiss;
    PERF_COUNTER_DEFINITION     PalKernelStackMiss;
    PERF_COUNTER_DEFINITION     PdeTranslationNotValid;
    PERF_COUNTER_DEFINITION     Halt;
    PERF_COUNTER_DEFINITION     Restart;
    PERF_COUNTER_DEFINITION     DrainAbort;
    PERF_COUNTER_DEFINITION     InitPal;
//20
    PERF_COUNTER_DEFINITION     Wrentry;
    PERF_COUNTER_DEFINITION     SwapIrql;
    PERF_COUNTER_DEFINITION     ReadIrql;
    PERF_COUNTER_DEFINITION     DisableInterrupt;
    PERF_COUNTER_DEFINITION     EnableInterrupt;
    PERF_COUNTER_DEFINITION     SwapPalcode;
    PERF_COUNTER_DEFINITION     SetSoftwareInterrupt;
    PERF_COUNTER_DEFINITION     ClearSoftwareInterrupt;
//28
#ifndef LODCTRBUG
    PERF_COUNTER_DEFINITION     ReturnFromException;
    PERF_COUNTER_DEFINITION     ReturnFromSystemCall;
//30
    PERF_COUNTER_DEFINITION     SwapContext;
    PERF_COUNTER_DEFINITION     SwapProcess;
    PERF_COUNTER_DEFINITION     ReadMces;
    PERF_COUNTER_DEFINITION     WriteMces;
    PERF_COUNTER_DEFINITION     TbInvalidateAll;
    PERF_COUNTER_DEFINITION     TbInvalidateSingle;
    PERF_COUNTER_DEFINITION     DtbInvalidateSingle;
    PERF_COUNTER_DEFINITION     ReadKernelStack;
    PERF_COUNTER_DEFINITION     SwapKernelStack;
    PERF_COUNTER_DEFINITION     ReadProcessorStatus;
//40
    PERF_COUNTER_DEFINITION     ReadPcr;
    PERF_COUNTER_DEFINITION     ReadThread;
    PERF_COUNTER_DEFINITION     ReadCounters;
    PERF_COUNTER_DEFINITION     ReadState;
    PERF_COUNTER_DEFINITION     WritePerfmon;
    PERF_COUNTER_DEFINITION     InitPcr;
    PERF_COUNTER_DEFINITION     UserBreakpoint;
    PERF_COUNTER_DEFINITION     SystemCall;
    PERF_COUNTER_DEFINITION     Imb;
    PERF_COUNTER_DEFINITION     GenerateTrap;
//50
    PERF_COUNTER_DEFINITION     ReadTeb;
    PERF_COUNTER_DEFINITION     KernelBreakpoint;
    PERF_COUNTER_DEFINITION     CallKernelDebugger;
    PERF_COUNTER_DEFINITION     TbisAsn;
    PERF_COUNTER_DEFINITION     Misc1;
    PERF_COUNTER_DEFINITION     Misc2;
    PERF_COUNTER_DEFINITION     Misc3;
//57
#endif

} EV4PALCTR_DEFINITION;

typedef struct _EV4PALCTR_DATA_AND_DEFINITION {
    EV4PALCTR_DEFINITION        defs;
    EV4PALCTR_INSTANCE          cpu[NUM_PROCESSORS];
} EV4PALCTR_DATA_AND_DEFINITION;

#pragma pack ()




#define NUM_CTR0_EVENTS         ((EXT0 - ISSUES)/2 + 1)
#define NUM_CTR1_EVENTS         ((EXT1 - DCACHEMISS)/2 + 1)

#if NUM_CTR0_EVENTS > NUM_CTR1_EVENTS
#define NUM_CTR_EVENTS          NUM_CTR0_EVENTS
#else
#define NUM_CTR_EVENTS          NUM_CTR1_EVENTS
#endif


typedef struct _COUNTER_INFO {
//        AXP21064_PCMUXCONTROL event[NUM_CTR_EVENTS];
        DWORD                 iCtrType[NUM_CTR_EVENTS];
//        DWORD                 iCtrScale[NUM_CTR_EVENTS];
//        LONG                  scale[NUM_CTR_EVENTS];
//        AXP21064_PCEVENTCOUNT freq[2];
} COUNTER_INFO, *PCOUNTER_INFO;


typedef struct _COUNTER_SCALE {
        LONG                  scale[NUM_CTR_EVENTS*2];
} COUNTER_SCALE;

typedef struct _PAL_SCALE {
        LONG                  scale[NUM_PAL_COUNTERS];
} PAL_SCALE;

typedef DWORD PROC_ACTIVE;

#define NUM_CTR_TYPES           2
