#ifndef __IPR_KERNEL_INT_DEFS_LOADED
#define __IPR_KERNEL_INT_DEFS_LOADED 1
/*
 * Module:	IPR_KERNEL_INT_DEFS.H
 *
 * Version:	X-2
 */

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1994 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
 * Abstract:	This header file contains definitions for the IPROBE API
 *		kernel.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	10-Jul-1994
 *
 * Modified by:
 *
 * X-3	BMW1002		Bradley M. Waters	22-Sep-1994
 *	Added identifiers for write structure (WBUFFER) and
 *      deferred sampling (DBUFFER) - these structures
 *	are currently used only be NT but it seems best to maintain
 *	all buffer identifiers in one place.  The actual write strucuture
 *      is currently defined in ipr_driver_nt.h.
 *
 * X-2	BMW1001		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-1	BFW1000		Brian F. Wibecan	08-Jul-1994
 *	Original version.
 *
*/
#include "ipr_kernel_ext_defs.h"
#include "ipr_kernel_util.h"

//
// Note: all buffer structures below must be ordered int64s, pointers, ints,
// to allow natural alignment to work correctly.  All buffers must begin with
// a struct _ipr_bufhdr header field.
//

// Defined elsewhere:
//    IPR_BUFHDR
//    IPR_SBUFFER
//    "kind" constants
//    "full" constants
// This allows exporting of the sample buffer structure, for the benefit
// of programs that need to read sample buffers.

struct _ipr_hbuffer {
    IPR_BUFHDR header;
    IPR_HISTVALUE minval;               // Minimum value to keep
    struct _ipr_hbuffer *pNext; 	// Next buffer in list
    int elemcnt;                        // Number of elements in data area
    IPR_HMASK item;                     // What is being histogrammed
    IPR_CPUNUM cpuid;                   // Current CPU ID
    IPR_HISTVALUE numvals;              // Number of values in histogram
    unsigned int pcgran;                // PC histogram granularity in bits
    unsigned int index;
    };

typedef struct _ipr_pbuffer {
    IPR_BUFHDR header;
    IPR_COUNT rangecnt;              // Number of PC ranges
    IPR_COUNT curmaxrangecnt;        // Current maximum number of PC ranges
    IPR_BOOLEAN sample_always;
} IPR_PBUFFER;

// IPR_ISKIP has as many instances as counters (BDptr->numcounters) and
// is appended to the end of the IPR_IBUFFER
typedef struct _ipr_iskip {
    int intspersample;             // Interrupts between samples
    int intssincelast;             // Interrupts seen since the last sample
    } IPR_ISKIP;

typedef struct _ipr_ibuffer {
    IPR_BUFHDR header;
    IPR_COUNT numcounters;
    IPR_CTRMASK skipmask;
} IPR_IBUFFER;

struct _ipr_bufdsc {
    IPR_BUFHDR header;
    IPR_SBUFFER *pSBuf;      // Pointer to sample buffer in use
    IPR_HBUFFER *pHBuf;      // Pointer to histogram buffer in use
    int (*pRoutine)();               // Routine to be called when sample buffer becomes full
    IPR_PBUFFER *pPBuf;      // Pointer to PC ranges
    IPR_IBUFFER *pIBuf;      // Pointer to interrupt skip info
    int sampleaction;                // What to do when collecting a sample
    IPR_COUNT numcounters;           // Number of counters
    int fullaction;                  // What to do when buffer is full
    int targetpid;                   // Notify this process
    IPR_CTRMASK  validcounters;      // Indicates which counters are valid
    };

// 
// bookkeeping structure that contains list of nonpaged pool blocks
// currently allocated.

#define IPR_MAX_NPP_BLOCKS (1000)

typedef struct _ipr_npp_list {
   void *start[IPR_MAX_NPP_BLOCKS];	// Pointer to start of this block of nonpaged pool
} IPR_NPP_LIST;

#if defined(__VMS)
#pragma extern_model save
#pragma extern_model strict_refdef "MEMCTRS"
#endif // #if defined(__VMS)

extern uint64
	all_bd_mem_cnt,		// buffer descriptors
	deall_bd_mem_cnt,
	all_hb_mem_cnt,		// histogram buffers
	deall_hb_mem_cnt,
	all_sb_mem_cnt,     	// sample buffers
	deall_sb_mem_cnt,
	all_sk_mem_cnt,     	// skip count structures
	deall_sk_mem_cnt,
	all_pb_mem_cnt,     	// PC range buffer
	deall_pb_mem_cnt;

extern IPR_NPP_LIST npp_table;	// Struct with ptrs to all npp curr alloc

#if defined(__VMS)
#pragma extern_model restore
#endif // #if defined(__VMS)

extern IPR_STATUS AddToPoolList(IN PVOID alloc_area);
extern IPR_STATUS DelFromPoolList(IN PVOID alloc_area);
extern IPR_STATUS allocate_memory (IN int bytecnt, IN IPR_BUFHDR *hdr,
                                   OUT void *retptr);
extern void deallocate_memory (IN void *area);
extern IPR_STATUS get_bufdsc_address (IN IPR_CPUNUM cpu,
                                      OUT IPR_BUFDSC **retptr);

extern IPR_BUFDSC *find_bufdsc_ptr (IN IPR_CPUNUM cpu);
extern IPR_STATUS ALONONPAGED(IN int bytecnt,
			IN IPR_BOOLEAN allocZone,
			OUT int *retsize,
			OUT IPR_BUFHDR **areaptr);
extern IPR_STATUS DEANONPAGED(OUT IPR_BUFHDR *areaptr);

extern int round_value_up (int xval, int gran);

extern IPR_STATUS KIprHbValidRange(IN IPR_HDESC *descrip);

extern IPR_BUFHDR
   histogram_template,
   sample_template;

#ifdef __NT
#  include "ipr_kernel_int_nt.h"
#endif
#ifdef __osf__
#  include "ipr_kernel_int_osf.h"
#endif
#ifdef __VMS
#  include "ipr_kernel_int_vms.h"
#endif

#endif // ifndef __IPR_KERNEL_INT_DEFS_LOADED
