/* IPR_ROUTINES_NT.C
 *
 * Version:     X-4
 */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c)  1994 BY                                                  */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the NT-specific performance monitoring
 *              routines.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       05-Jul-1994
 *
 * Modified by:
 *
 * X-4  BMW             Bradley M. Waters       23-Aug-1994
 *      Changed names from Pfm to Ipr.
 *
 * X-3  BMW             Bradley M. Waters       18-Jul-1994
 *      Modified based on kernel rationalization.
 *
 * X-2  BMW             Bradley M. Waters       15-Jul-1994
 *      Added api_routines_nt.h include to define api routines to ctr...
 *
 * X-1  BMW             Bradley M. Waters       05-Jul-1994
 *      Original version.
 *
 */

#include <malloc.h>
#include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#include "api_routines_nt.h"

//#include "ipr_kernel_int_defs.h"
#include "ctr_int_defs.h"
#include "ipr_api_int_defs.h"

//  AFFINITY_CPU_MASK,NOAFFINIT_CPU_MASK, ALLOCATION_GRANULARITY globals are
//  initialized in ipr_routines_nt.c in the InitSysInfo routine.

// should really be a ULONG but IPR_CPUMASK is an INT which will suffice
// without causing other type issues
/*
IPR_CPUMASK
   AFFINITY_CPU_MASK,
   NOAFFINITY_CPU_MASK;
*/
DWORD   ALLOCATION_GRANULARITY = {0};

//IPR_SYSTEM_DATA newsysdata = {0};
HANDLE IprKrnlHand = {0};       // Init. to make it shared

// @@@ Temporary definitions

#define BUFSIZE 1024

static IPR_CPUMASK
   affinity_value = 0;

IPR_CHIPINFO chipinfo = {0};
IPR_SHAREDATA GblShareArea = {0};
static uint64 GblCtrShare [1024] = {0};

extern IPR_SHAREDATA
   *ShareArea = NULL;

static int setaff (int cpu, int val)
{
        DWORD   cpuMask = {1};
        HANDLE  ThreadHand;
        IPR_STATUS status;

        // get the handle of the current thread - this is just a
        // pseudohandle that doesn't need to be deallocated.

        ThreadHand = GetCurrentThread();

        if (val)
        {
                // Set affinity to a given CPU
                // create cpuMask from the cpu
                cpuMask = cpuMask << cpu;
#ifdef DBGI
                printf("setaff: Setting affinity for cpu %d, cpumask %x\n",cpu,cpuMask);
#endif
                status = SetThreadAffinityMask(ThreadHand,cpuMask);
                if (OK_STATUS(status))
                   return IPR__SUCCESS;
                else
                   return IPR__AFFINITY_FAIL;
        }
        else
                // Restore affinity to all processors
#ifdef DBGI
                printf("setaff: Restoring affinity, cpumask %x\n",cpuMask);
#endif
                status = SetThreadAffinityMask(ThreadHand,AFFINITY_CPU_MASK);
                if (OK_STATUS(status))
                   return IPR__SUCCESS;
                else
                   return IPR__AFFINITY_FAIL;

}

// In Windows NT handles are used for counter access so it isn't
// necessary to set and clear affinity.  This is done at a lower
// level in the driver based on the handle.  I've implemented the
// setaff routine but it isn't needed at this point so I'm short
// circuiting set_affinity and clear_affinity to simply return a
// happy status.

extern IPR_STATUS set_affinity (IN IPR_CPUNUM cpu)
{

   affinity_value = 1 << cpu;
   return setaff (cpu, 1);
//   return IPR__SUCCESS;

}

extern IPR_STATUS clear_affinity (void)
{
   unsigned int cpu;
   IPR_CPUMASK temp_affinity;

   temp_affinity = affinity_value;
   if (temp_affinity)
   {
      cpu = 1;
      while ((temp_affinity & 1) == 0)
      {
         cpu++;
         temp_affinity = temp_affinity >> 1;
      }
      temp_affinity = 0;
      return setaff (cpu, 0);
   }
   return IPR__SUCCESS;
}



extern IPR_STATUS E_IprWriteToSampleFile (IN void *bptr, IN IPR_SIZE bsize)
{
        IPR_STATUS  error;

        error = KIprWriteFile(bptr,bsize);

        return error;
}

extern IPR_STATUS E_IprWriteOneSampleBuffer (IN IPR_SBUFFER *bptr, IN IPR_SIZE bsize)
{
        return E_IprWriteToSampleFile(bptr, bsize);
}

extern IPR_STATUS OpenCounters(void)
{
   IPR_STATUS status;
   IPR_COUNTER_BLOCK counterblock[2];
   IPR_CPUNUM current_cpu;

   for (current_cpu = first_cpu (AFFINITY_CPU_MASK, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (AFFINITY_CPU_MASK, current_cpu, IPR_AFFINITY))
   {
      // Open handles to the counters for each processor and counter
      // (start with just processor 0 for now)
      counterblock[0].ctr = 0;
      counterblock[0].action = IPR_OPEN;
      counterblock[0].value = 0;
      counterblock[1].ctr = 1;
      counterblock[1].action = IPR_OPEN;
      counterblock[1].value = 0;

      status = KIprUpdatePerfmon(current_cpu,2,counterblock);
      if (!IPR_SUCCESS(status))
        return status;
   }
}

extern IPR_STATUS InitSysInfo(void)
{
   IPR_STATUS status;
   IPR_BOOLEAN InitShare = {TRUE};
//   IPR_COUNTER_BLOCK counterblock[2];
//   IPR_CPUNUM current_cpu;

// This routine obtains system information and fills in appropriate
// global variables such as AFFINITY_CPU_MASK and NOAFFINITY_CPU_MASK
   status = KIprInitSysInfo(&sysdata);
   AFFINITY_CPU_MASK = sysdata.affinity_cpu_mask;
   NOAFFINITY_CPU_MASK = sysdata.noaffinity_cpu_mask;
//   num_cpus = sysdata.num_cpus;
   if (!IPR_SUCCESS(status))
      return status;
   ShareArea = &GblShareArea;
   ShareArea->counterowner = 0;
   ShareArea->countersrunning = 0;
   status = KIprGetChipInfo(&chipinfo);
   if (!IPR_SUCCESS(status))
      return status;
   status = CtrInitialize (GblCtrShare,InitShare);
   if (!IPR_SUCCESS(status))
      return status;

   return OpenCounters();
}

extern IPR_STATUS DoneSysInfo(void)
{
   IPR_STATUS status;
   IPR_COUNTER_BLOCK counterblock[2];
   IPR_CPUNUM current_cpu;

   for (current_cpu = first_cpu (AFFINITY_CPU_MASK, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (AFFINITY_CPU_MASK, current_cpu, IPR_AFFINITY))
   {
   // Close handles to the counters for each processor and counter
      counterblock[0].ctr = 0;
      counterblock[0].action = IPR_CLOSE;
      counterblock[0].value = 0;
      counterblock[1].ctr = 1;
      counterblock[1].action = IPR_CLOSE;
      counterblock[1].value = 0;

      status = KIprUpdatePerfmon(current_cpu,2,counterblock);
      if (!IPR_SUCCESS(status))
         return status;
   }
   status = KIprDoneSysInfo();
   ShareArea = NULL;
   return status;
}


extern IPR_STATUS allocate_iprdevice(void)
{
//   IPR_STATUS status;

   // NOTE: The attach to the driver is done implicitly when the
   // dll is attached to (in LibMain)
   return IPR__SUCCESS;

}

extern IPR_STATUS deallocate_iprdevice(void)
{
//   IPR_STATUS status;

   // NOTE: The detach to the driver is done implicitly when the
   // dll is disconnected from (in LibMain)
   return IPR__SUCCESS;
}

extern IPR_STATUS IprSetAffinity (IPR_CPUMASK cpus, IPR_BOOLEAN setflag, IPR_CPUMASK *prevcpus)
{
   DWORD   cpuMask, oldCpuMask;
   HANDLE  ThreadHand;
   IPR_STATUS status;
   IPR_CPUNUM curcpu;

   if (!setflag)
   {
      setflag = TRUE;
      cpus = IPR_ALL_CPUS;
   }
   if (cpus == IPR_CURRENT_CPU)
   {
      status = KIprCurrentCPU (&curcpu);
      cpuMask = 1 << curcpu;
   }
   else if (cpus == IPR_ALL_CPUS)
      cpuMask = AFFINITY_CPU_MASK;
   else
      cpuMask = (DWORD) cpus;

   // get the handle of the current thread - this is just a
   // pseudohandle that doesn't need to be deallocated.

   ThreadHand = GetCurrentThread();

#ifdef DBGI
   printf("setaff: Setting affinity for cpumask %x\n",cpuMask);
#endif
   oldCpuMask = SetThreadAffinityMask(ThreadHand,cpuMask);
   if (!oldCpuMask)
      return IPR__AFFINITY_FAIL;
   if (prevcpus)
      *prevcpus = (IPR_CPUMASK) oldCpuMask;
   return IPR__SUCCESS;
}
