#if defined(__DECC) || defined(__DECCXX)
# pragma module IPR_USER_ROUTINES_NT "X-5
#endif

/* IPR_USER_ROUTINES_NT.C */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994, 1995 BY                                             */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the user-mode Windows NT-specific performance
 *              monitoring routines.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       2-Jun-1994
 *
 * Modified by:
 *
 * X-5 BMW              Bradley M. Waters       19-Dec-1994
 *      Added PERFMON support similar to that provided by PERFEV4.
 *
 * X-4 BMW              Bradley M. Waters       23-Aug-1994
 *      Changed names from Pfm to Ipr.
 *
 * X-3  BMW             Bradley M. Waters       18-Jul-1994
 *      Modified based on kernel rationalization.
 *
 * X-2  BMW             Bradley M. Waters       20-Jun-1994
 *      Added OpenIprPerformanceData, CollectIprPerformanceData,
 *      CloseIprPerformanceData routines to allow PERFMON counters
 *      to monitor IPROBE user mode routines.
 *
 * X-1  BMW             Bradley M. Waters        2-Jun-1994
 *      Original version.
 *
 */

// Kludge.  The following two typedefs should be in one of the SDK header
// files.  But apparently I couldn't find it.
typedef unsigned __int64 ULONGLONG;
typedef long NTSTATUS;
#define STATUS_INVALID_PARAMETER        0xC000000DL

#include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#include <string.h>
#include <winperf.h>
#include <stdio.h>
#include <stdlib.h>

#include "ntevx.h"
#include "iprctrnm.h"
#include "ev4ctr.h"
#include "ipr_msg_nt.h"
#include "iprmsg_nt.h"

//#include "ctr_int_defs.h"
#include "ipr_api_int_defs.h"

#define STATUS_INVALID_PARAMETER        0xC000000DL

#define MUTEX_NAME              L"IPRMutex"

#define STATUS_SUCCESS(status)  ((status) == NO_ERROR)

//
// Local functions
//

static WCHAR GLOBAL_STRING[] = L"Global";
static WCHAR FOREIGN_STRING[] = L"Foreign";
static WCHAR COSTLY_STRING[] = L"Costly";

#define QUERY_INVALID           0
#define QUERY_GLOBAL            1
#define QUERY_ITEMS             2
#define QUERY_FOREIGN           3
#define QUERY_COSTLY            4

//
// Global shared variables.  Each variable has to be initialized, and access
// has to be protected by a mutex.
// NOTE: global shared variables should not contain any initialized pointers,
// as they are not accessible by every processes (e.g., WCHAR str[] =
// {L"abc", L"def"} do not work).
//

DWORD MuxControl[IPR_MAX_COUNTERS] = {6,0};     // Item selected to count for
                                                // each counter-init. in InitGblData
DWORD numPrevEvt[IPR_MAX_COUNTERS] =
{
 0,
 NUM_CTR0_EVENTS,
 NUM_CTR0_EVENTS+NUM_CTR1_EVENTS
};

DWORD sourcePid = 0xffffffff;                   // the first process' ID
#define MAX_PROCIDS 10
typedef struct _other_info {
   DWORD Pid;
   HANDLE apiHandle;
   HANDLE ctrHand[IPR_MAX_CPUS][IPR_MAX_COUNTERS];
} OTHER_INFO;
OTHER_INFO otherSource[MAX_PROCIDS] = {0};      // Array hold other sourcePids

DWORD dwOpenCount = {0};                        // counts how many times the
                                                // registry opens this library.
BOOLEAN bInitOK = {FALSE};                      // Indicator to perf. routines whether library
                                                // has been successfully opened
DWORD dwAttach = {0};                           // Count of number of processes attached to lib.
HANDLE FileHandle = {0};

IPR_SYSTEM_DATA sysdata = {0};

int gui_event[IPR_MAX_COUNTERS][MAX_EVT_CODE] =
{
// Counter 0
{0,             // 0
 0xffffffff,
 1,             // 2
 0xffffffff,
 2,             // 4
 0xffffffff,
 3,             // 6
 0xffffffff,
 4,             // 8
 0xffffffff,
 6,             // 10
 5,
 7,             // 12
 0xffffffff,
 8              // 14
},
// Counter 1
{0,             // 0
 1,
 2,             // 2
 3,
 4,             // 4
 5,
 6,             // 6
 7,
 0xffffffff,    // 8
 0xffffffff,
 0xffffffff,    // 10
 0xffffffff,
 0xffffffff,    // 12
 0xffffffff,
 0xffffffff     // 14
}
};

extern EV4CTR_DATA_AND_DEFINITION SharedEV4CtrData;
                                                // template for the EV4 perf.
                                                // counter object
extern EV4PALCTR_DATA_AND_DEFINITION SharedEV4PALCtrData;
                                                // template for the EV4 PAL
                                                // counter object
extern COUNTER_INFO counterInfo[];                // counter event and freq.
                                                // array.
extern DWORD CounterType[];                     // counter types (see winperf.h
                                                // for descriptions)
extern HANDLE gIprApiHandle;                    // Shared (owner) Api handle
//
// Global non-shared variables.  Each variable should not be initialized.
// Each process will have a local copy of the variable
//

extern HANDLE IprApiHandle;                     // Api handle specific to this process
HANDLE hMutex;                                  // shared variable mutex

EV4CTR_DATA_AND_DEFINITION EV4CtrData;          // non-shared copy, to be filled
EV4PALCTR_DATA_AND_DEFINITION EV4PALCtrData;    // in by OpenEV4PerformanceData.

//
// Prototypes of functions that will be called by the registry
//

PM_OPEN_PROC OpenIprPerformanceData;
PM_COLLECT_PROC CollectIprPerformanceData;
PM_CLOSE_PROC CloseIprPerformanceData;

static int
   exit_handler_block [6],
   exit_status;

static DWORD
GetQueryType(IN LPWSTR lpStr)
/*++

Routine Description:

    This routine reads the input string argument and returns:
        if string is empty or contains L"Global"
            returns QUERY_GLOBAL
        if string contains L"Foreign"
            returns QUERY_FOREIGN
        if string contains L"Costly"
            returns QUERY_COSTLY
        if string contains a list of numbers separated by a whitespace
            returns QUERY_ITEMS
        else
            returns QUERY_INVALID

Arguments:

        IN LPWSTR lpStr         - supplies the string to be read

Return Value:

        QUERY_{GLOBAL, FOREIGN, COSTLY, ITEMS, INVALID}

--*/
{
        if( (lpStr == NULL) || (*lpStr == 0) ||
                (wcsncmp(lpStr, GLOBAL_STRING, (sizeof(GLOBAL_STRING)-1)/sizeof(WCHAR)) == 0))
                return QUERY_GLOBAL;
        if(wcsncmp(lpStr, FOREIGN_STRING, (sizeof(FOREIGN_STRING)-1)/sizeof(WCHAR)) == 0)
                return QUERY_FOREIGN;
        if(wcsncmp(lpStr, COSTLY_STRING, (sizeof(COSTLY_STRING)-1)/sizeof(WCHAR)) == 0)
                return QUERY_COSTLY;
        if(iswdigit(*lpStr))
                return QUERY_ITEMS;

        return QUERY_INVALID;
}

extern IPR_STATUS IprCloseSampleFile (OUT int *buffwrites, OUT int *partwrites)
{
   IPR_STATUS
      status;

   status = KIprCloseSampFile(buffwrites,partwrites);

   return status;
}


extern IPR_STATUS IprOpenSampleFile (IN char *fname, IN char *defname)
// NOTE: For NT, the file handle is stored internally and not
// passed out to the caller so hndl will always be 0.
{
   IPR_STATUS
      status;
   UNICODE_STRING unamebuff,udefname;
#define bufflen (MAX_PATH + 255)
   WCHAR ufbuff[bufflen];
   WCHAR udfbuff[bufflen];
   char namebuff[bufflen];
   char namebuff2[bufflen];
   char namebuff3[bufflen];
   char *filepos;
   DWORD fpathLen;
   char pathPrefix[] = {"\\DosDevices\\"};
   char pathSuffix[] = {0};
   int cnt;

   strcpy(namebuff2,pathPrefix);
   strcpy(namebuff3,pathPrefix);

   fpathLen = GetFullPathNameA(fname,bufflen,namebuff,&filepos);
   strcat(namebuff2,namebuff);
   strcat(namebuff2,pathSuffix);
   namebuff[filepos-namebuff-1] = (char)0;
   strcat(namebuff3,namebuff);
   strcat(namebuff3,pathSuffix);

   unamebuff.MaximumLength = (bufflen*2);
   unamebuff.Buffer = ufbuff;
   udefname.MaximumLength = (bufflen*2);
   udefname.Buffer = udfbuff;
   for (cnt = 0; cnt < bufflen; cnt++)
   {
      ufbuff[cnt] = 0;
      udfbuff[cnt] = 0;
   }
   RtlCreateUnicodeStringFromAsciiz( &unamebuff, &namebuff2 );
   strcat(namebuff,pathSuffix);
   RtlCreateUnicodeStringFromAsciiz( &udefname, &namebuff3 );

   status = KIprOpenSampFile(&unamebuff,&udefname);
//   *hndl = 0;

   return status;
}

//
// Exported Functions
//

INT
LibMain(
        HANDLE hInst,
        ULONG ulReason,
        LPVOID  lpReserved
        )
/*++

Routine Description:

    This is the library entry point, called by the loader or LoadLibrary()
    whenever a process or thread attaches or detaches the this library.
    It does the necessary library initialization at process attach:
        - open the EventLog.
        - create/open the library mutex object
        - if it is the first time opened, create the device files for the
          performance counters, else duplicate the performance counter device
          handles

    and cleanup at process detach:
        - close the performance counter handles
        - close the EventLog

Arguments:

    IN HANDLE hInst             - unused
    IN ULONG  ulReason          - reason was called
    IN LPVOID lpReserved        - unused

Returns:

    1 - Success
    0 - Failure

--*/

{
   DWORD waitStatus;
   DWORD retVal = 1;
   DWORD error;

   DWORD suspCnt;
   DWORD currProcId,cnt,ctrcnt,ccpu;

   if (!hEventLog)
     hEventLog = MonOpenEventLog();

   if((hMutex = CreateMutex(NULL, FALSE, MUTEX_NAME)) == NULL)
   {
      error = GetLastError();
      REPORT_ERROR_DATA(IPR_MUTEX_CREATE_FAILURE, LOG_USER,
       &error, sizeof(error));
      return 0;
   }
   switch(ulReason)
   {
      case DLL_PROCESS_ATTACH:
        dwAttach++;
        waitStatus = WaitForSingleObject(hMutex, INFINITE);
        if(waitStatus == WAIT_FAILED)
        {
           retVal = 0;
           error = GetLastError();
           REPORT_ERROR_DATA(IPR_MUTEX_WAIT_FAILURE,
            LOG_USER,
            &error, sizeof(error));
        }
        else
        {
           if(sourcePid == 0xffffffff)
           {
              if (dwAttach == 1)
              {
                 sourcePid = GetCurrentProcessId();
                 // Open handle to IprKrnl driver
                 error = OpenIprApiKernel(&FileHandle);
                 if (!IPR_SUCCESS(error))
                 {
                    REPORT_ERROR_DATA(IPR_OPEN_API_FAILURE,
                      LOG_USER,
                      &error,
                      sizeof(error));
                    retVal = 0;
                 }
                 // NOTE: InitSysInfo sets AFFINITY_CPU_MASK so it is important not
                 // to use AFFINITY_CPU_MASK before InitSysInfo is called.
                 error = InitSysInfo();
                 if (!IPR_SUCCESS(error))
                 {
                    REPORT_ERROR_DATA(IPR_INIT_SYS_FAILURE,
                     LOG_USER,
                     &error,
                     sizeof(error));
                    retVal = 0;
                 }
              }
              else
              {
                 // Owner of the counters went away and another
                 // process exists with duplicate handles.  Try to
                 // find another owner.

                 for (cnt = 0; cnt < MAX_PROCIDS; cnt++)
                    if (otherSource[cnt].Pid != 0)
                    {
                       sourcePid = otherSource[cnt].Pid;
                       for (ccpu = 0; ccpu < IPR_MAX_CPUS; ccpu++)
                          for (ctrcnt = 0; ctrcnt < IPR_MAX_COUNTERS; ctrcnt++)

                             gctrHand[ccpu][ctrcnt] = otherSource[cnt].ctrHand[ccpu][ctrcnt];

                       gIprApiHandle = otherSource[cnt].apiHandle;
                    }
                 if (sourcePid == 0xffffffff)
                 {
                    error = IPR__FAILURE;
                    REPORT_ERROR_DATA(IPR_NO_OWNER_AND_PROC_EXISTS,
                     LOG_USER,
                     &error,
                     sizeof(error));
                    retVal = 0;
                 }
                 else
                 {
                    // Duplicate source handles of the source process
                    HANDLE hSourceProc;

                    if((hSourceProc = OpenProcess(PROCESS_DUP_HANDLE,
                     FALSE,sourcePid)) == NULL)
                    {
                       error = GetLastError();
                       REPORT_ERROR_DATA(IPR_OPEN_PROC_FAILURE,
                        LOG_USER,
                        &error,
                        sizeof(error));
                       retVal = 0;
                    }
                    // Duplicate handles for this process
                    error = DuplHand(hSourceProc,GetCurrentProcess(),IprNumCounters());
                    if (!IPR_SUCCESS(error))
                    {
                       REPORT_ERROR_DATA(IPR_DUPL_HANDLE_FAILURE,
                        LOG_USER,
                        &error,
                        sizeof(error));
                       retVal = 0;
                    }
                    if (dwAttach <= MAX_PROCIDS)
                    {
                       otherSource[dwAttach-1].Pid = GetCurrentProcessId();
                       for (ccpu = 0; ccpu < IPR_MAX_CPUS; ccpu++)
                          for (ctrcnt = 0; ctrcnt < IPR_MAX_COUNTERS; ctrcnt++)
                             otherSource[dwAttach-1].ctrHand[ccpu][ctrcnt] = ctrHand[ccpu][ctrcnt];
                       otherSource[dwAttach-1].apiHandle = IprApiHandle;
                    }
                 }
              }
           }
           else
           {
              HANDLE hSourceProc;
              // Duplicate the source handles of the owner process
              if((hSourceProc = OpenProcess(PROCESS_DUP_HANDLE,
                  FALSE,sourcePid)) == NULL)
              {
                 error = GetLastError();
                 REPORT_ERROR_DATA(IPR_OPEN_PROC_FAILURE,
                  LOG_USER,
                  &error,
                  sizeof(error));
                 retVal = 0;
              }
              // Duplicate handles for this process
              error = DuplHand(hSourceProc,GetCurrentProcess(),IprNumCounters());
              if (!IPR_SUCCESS(error))
              {
                 REPORT_ERROR_DATA(IPR_DUPL_HANDLE_FAILURE,
                  LOG_USER,
                  &error,
                  sizeof(error));
                 retVal = 0;
              }
              if (dwAttach <= MAX_PROCIDS)
              {
                 otherSource[dwAttach-1].Pid = GetCurrentProcessId();
                 for (ccpu = 0; ccpu < IPR_MAX_CPUS; ccpu++)
                    for (ctrcnt = 0; ctrcnt < IPR_MAX_COUNTERS; ctrcnt++)
                       otherSource[dwAttach-1].ctrHand[ccpu][ctrcnt] = ctrHand[ccpu][ctrcnt];
                 otherSource[dwAttach-1].apiHandle = IprApiHandle;
              }
           }
           if(!ReleaseMutex(hMutex))
           {
              error = GetLastError();
              REPORT_ERROR_DATA(IPR_MUTEX_RELEASE_FAILURE,
               LOG_USER,
               &error,
               sizeof(error));
              retVal = 0;
           }
        }
        return retVal;

     case DLL_PROCESS_DETACH:
        dwAttach--;
        waitStatus = WaitForSingleObject(hMutex, INFINITE);
        if(waitStatus == WAIT_FAILED)
        {
           error = GetLastError();
           REPORT_ERROR_DATA(IPR_MUTEX_WAIT_FAILURE,
            LOG_USER,
            &error, sizeof(error));
           retVal = 0;
        }  // end - if waitStatus
        else
        {
           // Make sure that a deallocate is done - this will stop the
           // counters and relinquish ownership if in fact the calling process
           // is the owner of the counters
           error = IprDeallocate();
           // If this process isn't the owner of the counters IprDeallocate
           // will return IPR__NOTOWNER and do nothing.
           if (dwAttach == 0)
           {
              error = DoneSysInfo();
              if (!IPR_SUCCESS(error))
              {
                 REPORT_ERROR_DATA(IPR_DONE_SYS_FAILURE,
                  LOG_USER,
                  &error,
                  sizeof(error));
                 retVal = 0;
              }  // end if
              error = CloseIprApiKernel(FileHandle);
              if (!IPR_SUCCESS(error))
              {
                 REPORT_ERROR_DATA(IPR_CLOSE_API_FAILURE,
                  LOG_USER,
                  &error,
                  sizeof(error));
                 retVal = 0;
              }  // end if
           }  // end - if owner of counters (did IprAllocate)
           currProcId = GetCurrentProcessId();
           for (cnt = 0; cnt < MAX_PROCIDS; cnt++)
              if (otherSource[cnt].Pid == currProcId)
                 otherSource[cnt].Pid = 0;
           if (sourcePid == currProcId)
              sourcePid = 0xffffffff;
           // Do detach action here if this is not the last
           // user of the current users.
           if(!ReleaseMutex(hMutex))
           {
              error = GetLastError();
              REPORT_ERROR_DATA(IPR_MUTEX_RELEASE_FAILURE,
               LOG_USER,
               &error,
               sizeof(error));
              retVal = 0;
           }  // end - if ReleaseMutex
           else
           {
              CloseHandle(hMutex);
           }  // end - else if ReleaseMutex
        }  // end - else Mutex ok
        MonCloseEventLog();
        return retVal;
  }  // end - Switch
  UNREFERENCED_PARAMETER(hInst);
  UNREFERENCED_PARAMETER(lpReserved);
}

//
// Extensible counter object interface routines
//

// NT specific routines to support CTRCTL

extern IPR_STATUS ResetCounter(IN DWORD counter)

/*++

Routine Description:

    This routine resets both performance counter values.

Arguments:

    IN DWORD counter            - which counter to reset
                                  (0 <= counter <= NUM_COUNTERS)

Return value:

    STATUS_INVALID_PARAMETER    - counter is not within the above range

    or status returned by KIprUpdatePerfmon

--*/
{
        DWORD status;
        ULONG ProcessorNum;
        IPR_STATUS retstatus;
        IPR_COUNTER_BLOCK counterblock[2];

        if(counter < 0 || counter > IPR_MAX_COUNTERS-1)
           return IPR__INVPARAM;

        retstatus = IPR__SUCCESS;
        // Iterate opening counters for each valid CPU
        for (ProcessorNum = first_cpu (AFFINITY_CPU_MASK, IPR_NOAFFINITY); ProcessorNum < IPR_MAX_CPUS;
         ProcessorNum = next_cpu (AFFINITY_CPU_MASK, ProcessorNum, IPR_NOAFFINITY))
        {
           // Update return status here to return intermediate error messages

           // Read counters for this processor
           counterblock[0].ctr = counter;
           counterblock[0].action = IPR_RESETCOUNT;
           counterblock[0].value = 0;
           status = KIprUpdatePerfmon(ProcessorNum,1,counterblock);
           if (!IPR_SUCCESS(status))
              retstatus = status;
        }
        return retstatus;
}

extern IPR_STATUS
SetCounterType(
        IN DWORD counter,
        IN DWORD iType
        )
/*++

Routine Description:

    This routine sets the counter type index of performance counter
    to iType

Arguments:

    IN DWORD counter            - which counter
                                  (0 <= counter < NUM_COUNTERS)
    IN DWORD iType              - CounterType[iType] is the type for this
                                  counter.   (0 <= iType < NUM_CTR_TYPES)

Return value:

    IPR__INVPARAM - counter or iType is not within range
    IPR__FAILURE - if WaitForSingleObject() call fails
    IPR__SUCCESS - successful

--*/
{
        DWORD status;

        if(iType < 0 || iType >= NUM_CTR_TYPES ||
           counter < 0 || counter >= IPR_MAX_COUNTERS)
                return IPR__INVPARAM;

        status = WaitForSingleObject(hMutex, INFINITE);
        if(status == WAIT_FAILED) {
                status = GetLastError();
                REPORT_ERROR_DATA(IPR_MUTEX_WAIT_FAILURE,
                                  LOG_USER,
                                  &status, sizeof(status));
                return IPR__FAILURE;
        }

        counterInfo[counter].iCtrType[MuxControl[counter]] = iType;

        ReleaseMutex(hMutex);

        return IPR__SUCCESS;
}

// Routines called by PERFMON

extern DWORD
OpenIprPerformanceData(
        LPWSTR lpDeviceNames
        )
{
        HKEY    hKeyDriverPerf;
        WCHAR   szSubKey[256];
        LONG    status;
        DWORD   size;
        DWORD   type;
        DWORD   dwFirstCounter;
        DWORD   dwFirstHelp;

        PROCESS_INFORMATION procInfo;
        LPCTSTR detachImage = L"tmpdtch.exe";
        if(!dwOpenCount)
        {
                wcscpy(szSubKey,
                        L"SYSTEM\\CurrentControlSet\\Services\\IprKrnl\\Performance");
                status = RegOpenKeyEx(
                                HKEY_LOCAL_MACHINE,
                                szSubKey,
                                0L,
                                KEY_ALL_ACCESS,
                                &hKeyDriverPerf);
                if(status != ERROR_SUCCESS) {
                        REPORT_ERROR_DATA(IPR_UNABLE_OPEN_DRIVER_KEY,
                                          LOG_USER,
                                          &status,
                                          sizeof(status));
                        goto OpenExitPoint;
                }
                size = sizeof(DWORD);
                status = RegQueryValueEx(
                                hKeyDriverPerf,
                                L"First Counter",
                                0L,
                                &type,
                                (LPBYTE)&dwFirstCounter,
                                &size);
                if(status != ERROR_SUCCESS) {
                        REPORT_ERROR_DATA(IPR_UNABLE_READ_FIRST_COUNTER,
                                          LOG_USER,
                                          &status,
                                          sizeof(status));
                        goto OpenExitPoint;
                }
                size = sizeof(DWORD);
                status = RegQueryValueEx(
                                hKeyDriverPerf,
                                L"First Help",
                                0L,
                                &type,
                                (LPBYTE)&dwFirstHelp,
                                &size);
                if(status != ERROR_SUCCESS)
                {
                        REPORT_ERROR_DATA(IPR_UNABLE_READ_FIRST_HELP,
                                          LOG_USER,
                                          &status,
                                          sizeof(status));
                        goto OpenExitPoint;
                }
                // calculate the sizes and update the number of instances
                // in the data structure passed to PERFMON
                // NOTE:  This code assumes that there are no gaps in the number
                // processors - so if num_active_processors is 3 than processors
                // 0,1, and 2 are present.

                SharedEV4CtrData.defs.EV4CtrObjectType.TotalByteLength=
                 (sizeof(EV4CTR_DEFINITION)+
                 (sizeof(EV4CTR_INSTANCE)*sysdata.num_cpus));
                SharedEV4CtrData.defs.EV4CtrObjectType.NumInstances = sysdata.num_cpus;

                SharedEV4PALCtrData.defs.EV4PALCtrObjectType.TotalByteLength=
                 (sizeof(EV4PALCTR_DEFINITION)+
                 (sizeof(EV4PALCTR_INSTANCE)*sysdata.num_cpus));
                SharedEV4PALCtrData.defs.EV4PALCtrObjectType.NumInstances = sysdata.num_cpus;

                memmove(&EV4CtrData,
                        &SharedEV4CtrData,
                        SharedEV4CtrData.defs.EV4CtrObjectType.TotalByteLength
                        );

                memmove(&EV4PALCtrData,
                        &SharedEV4PALCtrData,
                        SharedEV4PALCtrData.defs.EV4PALCtrObjectType.TotalByteLength
                        );

                EV4CtrData.defs.EV4CtrObjectType.ObjectNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.EV4CtrObjectType.ObjectHelpTitleIndex += dwFirstHelp;

                EV4CtrData.defs.Issues.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Issues.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.PipeDry.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.PipeDry.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Load.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Load.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.PipeFroze.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.PipeFroze.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Branch.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Branch.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.PALCalls.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.PALCalls.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Cycles.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Cycles.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.NonIssues.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.NonIssues.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Ext0.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Ext0.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.DCacheMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.DCacheMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.ICacheMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.ICacheMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.DualIssues.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.DualIssues.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.BranchMispredicts.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.BranchMispredicts.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.FPInstructions.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.FPInstructions.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.IntInstructions.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.IntInstructions.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Store.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Store.CounterHelpTitleIndex += dwFirstHelp;
                EV4CtrData.defs.Ext1.CounterNameTitleIndex += dwFirstCounter;
                EV4CtrData.defs.Ext1.CounterHelpTitleIndex += dwFirstHelp;


                EV4PALCtrData.defs.EV4PALCtrObjectType.ObjectNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.EV4PALCtrObjectType.ObjectHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.MachineCheck.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.MachineCheck.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ArithmeticException.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ArithmeticException.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Interrupt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Interrupt.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ItbMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ItbMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.NativeDtbMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.NativeDtbMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.PalDtbMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.PalDtbMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ItbAccessViolation.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ItbAccessViolation.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.DtbAccessViolation.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.DtbAccessViolation.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.UnalignedAccess.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.UnalignedAccess.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.IllegalInstruction.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.IllegalInstruction.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.FloatingEnableException.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.FloatingEnableException.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ItbTranslationNotValid.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ItbTranslationNotValid.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.DtbTranslationNotValid.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.DtbTranslationNotValid.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.PteMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.PteMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.PalKernelStackMiss.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.PalKernelStackMiss.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.PdeTranslationNotValid.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.PdeTranslationNotValid.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Halt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Halt.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Restart.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Restart.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.DrainAbort.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.DrainAbort.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.InitPal.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.InitPal.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Wrentry.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Wrentry.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SwapIrql.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SwapIrql.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadIrql.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadIrql.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.DisableInterrupt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.DisableInterrupt.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.EnableInterrupt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.EnableInterrupt.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SwapPalcode.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SwapPalcode.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SetSoftwareInterrupt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SetSoftwareInterrupt.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ClearSoftwareInterrupt.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ClearSoftwareInterrupt.CounterHelpTitleIndex += dwFirstHelp;

#ifndef LODCTRBUG
                EV4PALCtrData.defs.ReturnFromException.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReturnFromException.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReturnFromSystemCall.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReturnFromSystemCall.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SwapContext.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SwapContext.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SwapProcess.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SwapProcess.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadMces.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadMces.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.WriteMces.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.WriteMces.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.TbInvalidateAll.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.TbInvalidateAll.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.TbInvalidateSingle.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.TbInvalidateSingle.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.DtbInvalidateSingle.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.DtbInvalidateSingle.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadKernelStack.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadKernelStack.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SwapKernelStack.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SwapKernelStack.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadProcessorStatus.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadProcessorStatus.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadPcr.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadPcr.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadThread.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadThread.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadCounters.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadCounters.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadState.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadState.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.WritePerfmon.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.WritePerfmon.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.InitPcr.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.InitPcr.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.UserBreakpoint.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.UserBreakpoint.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.SystemCall.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.SystemCall.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Imb.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Imb.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.GenerateTrap.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.GenerateTrap.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.ReadTeb.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.ReadTeb.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.KernelBreakpoint.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.KernelBreakpoint.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.CallKernelDebugger.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.CallKernelDebugger.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.TbisAsn.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.TbisAsn.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Misc1.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Misc1.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Misc2.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Misc2.CounterHelpTitleIndex += dwFirstHelp;
                EV4PALCtrData.defs.Misc3.CounterNameTitleIndex += dwFirstCounter;
                EV4PALCtrData.defs.Misc3.CounterHelpTitleIndex += dwFirstHelp;

#endif

                RegCloseKey(hKeyDriverPerf);
                bInitOK = TRUE;
        }

        dwOpenCount++;

        status = ERROR_SUCCESS;

OpenExitPoint:

        return status;

}


extern DWORD
CollectIprPerformanceData(
        LPWSTR lpValueName,
        LPVOID *lppData,
        LPDWORD lpcbTotalBytes,
        LPDWORD lpNumObjectTypes
        )
{
    ULONG SpaceNeeded = 0;
    ULONGLONG *pdwCounter;
    PERF_COUNTER_BLOCK *pPerfCounterBlock;
    EV4CTR_DATA_AND_DEFINITION *pEV4CtrData;
    EV4PALCTR_DATA_AND_DEFINITION *pEV4PALCtrData;
    PPERF_COUNTER_DEFINITION pCtrDef;

    BOOLEAN bWantEventCounters = FALSE,
            bWantPALCounters = FALSE;
    DWORD   dwQueryType;
    DWORD   status;
    INT     i;
    ULONG   ProcessorNum;
    IPR_COUNTER_BLOCK counterblock[IPR_MAX_COUNTERS];
    unsigned int cnt,counterCnt;

//    IPR_EVENT evt_arr[IPR_MAX_COUNTERS];

   if (!bInitOK)
   {
       // unable to continue because open failed.
           *lpcbTotalBytes = (DWORD) 0;
           *lpNumObjectTypes = (DWORD) 0;
       return ERROR_SUCCESS; // yes, this is a successful exit
   }

   switch(dwQueryType = GetQueryType(lpValueName))
   {

     case QUERY_GLOBAL:
        bWantEventCounters = TRUE;
        bWantPALCounters = TRUE;
        // All instances
        SpaceNeeded = (
                SharedEV4CtrData.defs.EV4CtrObjectType.TotalByteLength+
                SharedEV4PALCtrData.defs.EV4PALCtrObjectType.TotalByteLength);

        break;
     case QUERY_ITEMS:
     {
        PWCHAR str, endptr;
        ULONG ulCounterIndex;

        endptr = str = lpValueName;
        for( ; *endptr != (WCHAR)NULL; str = endptr)
        {
                ulCounterIndex = wcstoul(str, &endptr, 10);
                if(endptr == str)
                        break;

                if(ulCounterIndex ==
                   EV4CtrData.defs.EV4CtrObjectType.ObjectNameTitleIndex)
                {
                        bWantEventCounters = TRUE;
                        SpaceNeeded +=
                                SharedEV4CtrData.defs.EV4CtrObjectType.TotalByteLength;

                }
                else if(ulCounterIndex ==
                   EV4PALCtrData.defs.EV4PALCtrObjectType.ObjectNameTitleIndex)
                {
                        bWantPALCounters = TRUE;
                        SpaceNeeded +=
                                SharedEV4PALCtrData.defs.EV4PALCtrObjectType.TotalByteLength;
                }
        }  // end - for

        if(!bWantEventCounters && !bWantPALCounters)
        {
                *lpcbTotalBytes = 0;
                *lpNumObjectTypes = 0;
                return ERROR_SUCCESS;
        }

        break;
     }  // end - case QUERY_ITEMS

     case QUERY_COSTLY:
     case QUERY_FOREIGN:
     case QUERY_INVALID:
     default:
        *lpcbTotalBytes = 0;
        *lpNumObjectTypes = 0;
        return ERROR_SUCCESS;
   } // end - switch

   if ( *lpcbTotalBytes < SpaceNeeded )
   {
       *lpcbTotalBytes = (DWORD) 0;
       *lpNumObjectTypes = (DWORD) 0;
       return ERROR_MORE_DATA;
   }

   if(bWantEventCounters)
   {
        AXPCOUNTER PCValue[IPR_MAX_COUNTERS];

        pEV4CtrData = (EV4CTR_DATA_AND_DEFINITION *) *lppData;
        pCtrDef = &(pEV4CtrData->defs.Issues);

        // Copy entire data block including instance info. for active processors
        memmove(pEV4CtrData,
                &EV4CtrData,
//              sizeof(EV4CTR_DATA_AND_DEFINITION)
                SharedEV4CtrData.defs.EV4CtrObjectType.TotalByteLength
                );

        pPerfCounterBlock =  &(pEV4CtrData->cpu[0].datalen );

        for (ProcessorNum = first_cpu (AFFINITY_CPU_MASK, IPR_NOAFFINITY); ProcessorNum < IPR_MAX_CPUS;
          ProcessorNum = next_cpu (AFFINITY_CPU_MASK, ProcessorNum, IPR_NOAFFINITY))
        {
           // Init. counter data area to 0s for this processor
           pPerfCounterBlock->ByteLength = SIZE_OF_EV4CTR_PERFORMANCE_DATA;

           pdwCounter = (AXPCOUNTER *) (&pPerfCounterBlock[1]);

           counterCnt = IprNumCounters();
           for (cnt = 0; cnt < counterCnt; cnt++)
           {
              // Read counters for this processor
              counterblock[cnt].ctr = cnt;
              counterblock[cnt].action = IPR_READCOUNT;
              counterblock[cnt].value = 0;
           }
           status = KIprUpdatePerfmon(ProcessorNum,counterCnt,counterblock);
           if (!IPR_SUCCESS(status))
           {
              REPORT_ERROR_DATA(IPR_READ_COUNTER_FAILURE,
                 LOG_USER,
                 &status,
                 sizeof(status));
              *lpcbTotalBytes = 0;
              *lpNumObjectTypes = 0;
              return ERROR_SUCCESS;
           }
           for (cnt = 0; cnt < counterCnt; cnt++)
              PCValue[cnt] = counterblock[cnt].value;

           status = WaitForSingleObject(hMutex, INFINITE);
           if(status == WAIT_FAILED)
           {
              REPORT_ERROR_DATA(IPR_MUTEX_WAIT_FAILURE,
                 LOG_USER,
                 &status, sizeof(status));
              *lpNumObjectTypes = 0;
              *lpcbTotalBytes = 0;
           }

           for (i = 0; i < NUM_CTR0_EVENTS; i++)
           {
              pCtrDef[i].CounterType =
               CounterType[counterInfo[0].iCtrType[i]];
           }

           for (i = 0; i < NUM_CTR1_EVENTS; i++)
           {
              pCtrDef[NUM_CTR0_EVENTS+i].CounterType =
               CounterType[counterInfo[1].iCtrType[i]];
           }

           for (cnt = 0; cnt < counterCnt; cnt++)
           {
              memmove(&pdwCounter[numPrevEvt[cnt]+MuxControl[cnt]],
               &PCValue[cnt], sizeof(PCValue[cnt]));
           }
           ReleaseMutex(hMutex);
           pdwCounter = (ULONGLONG *)((PBYTE)pdwCounter + SIZE_OF_EV4CTR_PERFORMANCE_DATA +
            sizeof(EV4_PERF_INSTANCE_DEF) - sizeof(DWORD));
           pPerfCounterBlock = (PERF_COUNTER_BLOCK *) pdwCounter;
        }
        pdwCounter = (ULONGLONG *)((PBYTE)pdwCounter - sizeof(EV4_PERF_INSTANCE_DEF));
        *lppData = (PVOID) pdwCounter;
        *lpNumObjectTypes = 1;
        *lpcbTotalBytes = ((PBYTE)pdwCounter - (PBYTE) pEV4CtrData);
   }

   if(bWantPALCounters)
   {

        AXP21064_RDPALCOUNTERS PALCounters;

        pEV4PALCtrData = (EV4PALCTR_DATA_AND_DEFINITION *) *lppData;

        // Copy entire data block including instance info...
        memmove(pEV4PALCtrData,
                &EV4PALCtrData,
//              sizeof(EV4PALCTR_DATA_AND_DEFINITION)
                SharedEV4PALCtrData.defs.EV4PALCtrObjectType.TotalByteLength
                );

        pPerfCounterBlock =  &(pEV4PALCtrData->cpu[0].datalen );

        //  Iterate through all processors
        for (ProcessorNum = first_cpu (AFFINITY_CPU_MASK, IPR_NOAFFINITY); ProcessorNum < IPR_MAX_CPUS;
          ProcessorNum = next_cpu (AFFINITY_CPU_MASK, ProcessorNum, IPR_NOAFFINITY))
        {
           pPerfCounterBlock->ByteLength = SIZE_OF_EV4PALCTR_PERFORMANCE_DATA;

           pdwCounter = (AXPCOUNTER *) (&pPerfCounterBlock[1]);

           // Read PAL counters for this processor
           counterblock[0].ctr = 0;
           counterblock[0].action = IPR_READPAL;
           counterblock[0].value = (int64) (&PALCounters);

           status = KIprUpdatePerfmon(ProcessorNum,1,counterblock);
           if (!IPR_SUCCESS(status))
           {
              REPORT_ERROR_DATA(IPR_READ_PALCOUNTER_FAILURE,
                 LOG_USER,
                 &status,
                 sizeof(status));
              return ERROR_SUCCESS;
           }

           memmove(pdwCounter, &PALCounters, SIZE_OF_EV4PALCTR_PERFORMANCE_DATA-sizeof(DWORD));
//           pdwCounter = (ULONGLONG *)((PBYTE)pdwCounter + SIZE_OF_EV4PALCTR_PERFORMANCE_DATA);

           pdwCounter = (ULONGLONG *)((PBYTE)pdwCounter + SIZE_OF_EV4PALCTR_PERFORMANCE_DATA +
            sizeof(EV4_PERF_INSTANCE_DEF) - sizeof(DWORD));
           pPerfCounterBlock = (PERF_COUNTER_BLOCK *) pdwCounter;

        }  // end - for
        pdwCounter = (ULONGLONG *)((PBYTE)pdwCounter - sizeof(EV4_PERF_INSTANCE_DEF));
        *lppData = (PVOID)pdwCounter;
        (*lpNumObjectTypes)++;
        *lpcbTotalBytes += ((PBYTE)pdwCounter - (PBYTE) pEV4PALCtrData);
   }

   return ERROR_SUCCESS;
}


extern DWORD
CloseIprPerformanceData()
{
   dwOpenCount--;
   return ERROR_SUCCESS;
}
