#if defined(__DECC) || defined(__DECCXX)
# pragma module SHELL_NT "X-1"
#endif

/* SHELL_NT.C */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the Windows NT IPROBE API driver shells for
 *              the kernel routines.
 *
 * Author:              Bradley M. Waters
 * Creation Date:       26-Jul-1994
 *
 * Modified by:
 *
 * X-1  BMW             Bradley M. Waters       26-Jul-1994
 *      Original Version.
 *
*/

#include "stdarg.h"
#include "stdio.h"
#include "ntddk.h"
#include "ipr_kernel_int_defs.h"
#include "ipr_driver_nt.h"
#include <winerror.h>

#include "ev4_masks.h"

extern MuxControl[];

HANDLE gIprApiHandle = {0};   // Shared Handle for the IPROBE API - initialized at startup
HANDLE IprApiHandle;          // Nonshared Handle for the IPROBE API - initialized at startup

HANDLE gctrHand[IPR_MAX_CPUS][IPR_MAX_COUNTERS] = {0};  // shared counter handles
HANDLE ctrHand[IPR_MAX_CPUS][IPR_MAX_COUNTERS];         // nonshared counter handles

static IPR_BOOLEAN HandleExists = {FALSE};

// Prototypes for external functions that were not define

//
// Routines to create the device handle
//
// NOTE WELL!!!:  Use the output parameter for returned data in combination
// with the input buffer.  These are buffered I/O operations and
// the system will trample what is pointed to by the output
// parameter supplying its own output buffer if no input buffer is
// supplied.  If data needs to be updated and preserved, make sure it is passed
// in the input parameter and returned in the output parameter.
//

extern IPR_STATUS OpenIprApiKernel(OUT HANDLE *fHandle)
{

    UCHAR DeviceNameBuffer[80];
    IO_STATUS_BLOCK IoStatus;
    OBJECT_ATTRIBUTES ObjectAttributes;
    NTSTATUS Status;
    UNICODE_STRING UnicodeDeviceName;

    if (!HandleExists)
    {
       sprintf( DeviceNameBuffer, "%s",
                DD_IPROBE_API_DEVICE_NAME);

       RtlCreateUnicodeStringFromAsciiz( &UnicodeDeviceName, DeviceNameBuffer );

       InitializeObjectAttributes( &ObjectAttributes,
                                &UnicodeDeviceName,
                                0,
                                NULL,
                                NULL );

       Status = NtCreateFile( fHandle,
                           FILE_READ_DATA | FILE_WRITE_DATA,
                           &ObjectAttributes,
                           &IoStatus,
                           (PLARGE_INTEGER)NULL,
                           0L,
                           0L,
                           FILE_OPEN_IF,
                           0L,
                           (PVOID)NULL,
                           0L );

      RtlFreeUnicodeString( &UnicodeDeviceName );

      if( OK_STATUS(Status))
          Status = IoStatus.Status;
      if( OK_STATUS(Status))
          Status = IPR__SUCCESS;
      else
          Status = IPR__FAILURE;

// Update global that holds the IPROBE API handle for future calls.
// The IPROBE API handle is kept in a global because only 1 connection
// to the IPROBE API is allowed and to keep the kernel routine shell
// call interface identical to those that exist in the kernel.

      gIprApiHandle = *fHandle;
      IprApiHandle = *fHandle;
      HandleExists = TRUE;
   }
   else
      Status = IPR__KAPIHANDEXISTS;

   return (IPR_STATUS) Status;
}

extern IPR_STATUS CloseIprApiKernel(IN HANDLE fHandle)
{

    NTSTATUS Status;

    if (HandleExists)
    {
       if (fHandle == IprApiHandle)
       {
          Status = NtClose(fHandle);
          HandleExists = FALSE;
          IprApiHandle = NULL;
          gIprApiHandle = NULL;
          if( OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          return (IPR_STATUS) Status;
       }
       else
          return IPR__KAPIHANDDIFF;
    }
    else
       return IPR__KAPIHANDNOTDEF;

}

//
// General shells
//

extern IPR_STATUS KIprGetChipInfo(OUT IPR_CHIPINFO *retptr)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // Get chip major Ident
   Status = GetChipData(retptr);
   if( !IPR_SUCCESS(Status))
       return Status;

   // Call driver to fill in chip info. structure
#ifdef TRCSH
   printf ("Shell: KIprGetChipInfo, retptr = %x\n",
       retptr);
#endif
   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_GetChipInfo,
                                    retptr,
                                    sizeof(IPR_CHIPINFO),
                                    retptr,
                                    sizeof(IPR_CHIPINFO));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprUpdatePerfmon(IN IPR_CPUNUM cpu,
                        IN IPR_COUNT numctrsupd,
                        OUT IPR_COUNTER_BLOCK *counterblock )
{

   IPR_COUNTER_BLOCK *ctrptr;
   IPR_STATUS Status;
   IPR_STATUS rtnstatus = IPR__SUCCESS;
   IPR_COUNT cnt;

// Grab info. from the counter block and make appropriate call
// to update the counters

// Currently, we'll use the counter interface defined by Joe N.
// rather than rewrite it.

   ctrptr = counterblock;
   for (cnt = 0; cnt < numctrsupd; cnt++)
   {
       // For each counter given in the counter block
       switch (ctrptr->action) {
       case IPR_OPEN:
       {
          Status = OpenMpAxp21064Counter(cpu,
                ctrptr->ctr,
                &gctrHand[cpu][ctrptr->ctr]);
          if (OK_STATUS(Status))
          {
             // Update nonshared copy of ctrHand;
             ctrHand[cpu][ctrptr->ctr] = gctrHand[cpu][ctrptr->ctr];
             Status = IPR__SUCCESS;
          }
          else
             Status = IPR__FAILURE;
          break;
       }
        case IPR_CLOSE:
       {
          Status = CloseAxp21064Counter(ctrHand[cpu][ctrptr->ctr]);
          if (OK_STATUS(Status))
          {
             ctrHand[cpu][ctrptr->ctr] = NULL;
             gctrHand[cpu][ctrptr->ctr] = NULL;
             Status = IPR__SUCCESS;
          }
          else
             Status = IPR__FAILURE;
          break;
       }
        case IPR_ENABLE:
       {
          Status = StartAxp21064Counter(ctrHand[cpu][ctrptr->ctr]);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_DISABLE:
       {
          Status = StopAxp21064Counter(ctrHand[cpu][ctrptr->ctr]);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_SETFREQ:
       {
          IPR_FREQ freq;
          // Adjust counter frequency to NT defined type
          if (ctrptr->ctr == 1)
          {
             freq = ctrptr->freq + 1;
          }
          else
             freq = ctrptr->freq;
          // Convert frequency from 0 (low) or 1 (high) to that used by NT
          switch (freq) {
             case 0:
                freq = CountEvents2xx16;
                break;
             case 1:
                freq = CountEvents2xx12;
                break;
             case 2:
                freq = CountEvents2xx8;
                break;
          }
          Status = EventCountAxp21064Counter(ctrHand[cpu][ctrptr->ctr],freq);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_SETEVENT:
       {
          Status = MuxControlAxp21064Counter(ctrHand[cpu][ctrptr->ctr],ctrptr->event);
          if (OK_STATUS(Status))
          {
             MuxControl[ctrptr->ctr] = gui_event[ctrptr->ctr][ctrptr->event];
             Status = IPR__SUCCESS;
          }
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_READCOUNT:
       {
          // NOTE: value field is the value to return
          Status = ReadAxp21064Counter(ctrHand[cpu][ctrptr->ctr],&ctrptr->value);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_READPAL:
       {
          // NOTE: value field is pointer to PalCounter block
          Status = ReadAxp21064PalCounters(ctrHand[cpu][ctrptr->ctr],
             (AXP21064_RDPALCOUNTERS *)ctrptr->value);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_SETCOUNT:
       {
          // NOTE: value field is the value to set for the counter
          Status = SetAxp21064CounterValue(ctrHand[cpu][ctrptr->ctr],ctrptr->value);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }
       case IPR_RESETCOUNT:
       {
          Status = ResetAxp21064CounterValue(ctrHand[cpu][ctrptr->ctr]);
          if (OK_STATUS(Status))
             Status = IPR__SUCCESS;
          else
             Status = IPR__FAILURE;
          break;
       }

       case IPR_SETMODE:
       {
          Status = IPR__SUCCESS;
          break;
       }

       case IPR_SETFROMCODE:
       {
          IPR_FREQ freq;
//          IPR_EVENT event;
          EV4_MUXCTL muxctl;

          muxctl.ev4_muxctl_q_bits = ctrptr->value;
          if (ctrptr->ctr == 0)
          {
             ctrptr->event = muxctl.ev4_muxctl_v_pcmux0;
             ctrptr->freq = muxctl.ev4_muxctl_v_pc0;
          }
          else
          {
             ctrptr->event = muxctl.ev4_muxctl_v_pcmux1;
             ctrptr->freq = muxctl.ev4_muxctl_v_pc1;
          }
          Status = MuxControlAxp21064Counter(ctrHand[cpu][ctrptr->ctr],ctrptr->event);
          if (OK_STATUS(Status))
          {
             MuxControl[ctrptr->ctr] = gui_event[ctrptr->ctr][ctrptr->event];
             // Adjust counter frequency to NT defined type
             if (ctrptr->ctr == 1)
             {
                freq = ctrptr->freq + 1;
             }
             else
                freq = ctrptr->freq;
             // Convert frequency from 0 (low) or 1 (high) to that used by NT
             switch (freq)
             {
                case 0:
                   freq = CountEvents2xx16;
                   break;
                case 1:
                   freq = CountEvents2xx12;
                   break;
                case 2:
                   freq = CountEvents2xx8;
                   break;
             }
             Status = EventCountAxp21064Counter(ctrHand[cpu][ctrptr->ctr],freq);
             if (OK_STATUS(Status))
                Status = IPR__SUCCESS;
             else
                Status = IPR__FAILURE;
          }
          else
             Status = IPR__FAILURE;

          break;
       }

       default:
          Status = IPR__INVACTION;
       }
       ctrptr->status = Status;         // Return status for each counter
       if (!IPR_SUCCESS(Status))
          rtnstatus = IPR__FAILURE;

       // update ctrptr to point to the next item in the block
       ctrptr++;
   }
   return (IPR_STATUS) rtnstatus;
/*
// Currently, don't simply pass on the UpdatePerfmon call to
// the kernel.  Instead, use the interface that Joe defined.
// Pass counter block onto the kernel to make appropriate calls

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGUPerfmon alist;

   // Call driver to fill update PERFMON register
   alist.cpu = cpu;
   alist.numctrsupd = numctrsupd;
   alist.counterblock = counterblock;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_UpdatePerfmon,
                                    &alist,
                                    sizeof(IPR_ARGUPerfmon),
                                    &alist,
                                    sizeof(IPR_ARGUPerfmon));

   if( OK_STATUS(Status) )
       Status = IoStatus.Status;

   return (IPR_STATUS) Status;
*/

}

extern IPR_STATUS KIprGetCurrentIpid(OUT unsigned int *Ipid)
{

   IPR_STATUS Status;
//   IO_STATUS_BLOCK IoStatus;
//   NTSTATUS Status;

   // Currently, this case will be handled here through a system service
   // call rather than going into the kernel.
   Status = GetProcId(Ipid);
#ifdef TRCSH
   printf("Shell_nt: KIprGetCurrentIpid Ipid = %x, pointer to Ipid = %x \n",
      *Ipid,Ipid);
#endif
/*
   // Call driver to get ipid

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_GetCurrentIpid,
                                    Ipid,
                                    sizeof(int),
                                    Ipid,
                                    sizeof(int) );

   if( OK_STATUS(Status) )
       Status = IoStatus.Status;
*/
   return Status;
}

extern IPR_STATUS KIprSetAffinity(IN IPR_CPUNUM cpu, IN IPR_BOOLEAN setting)
{

//   IPR_STATUS status;

   return SetAffin(cpu,setting);

// The kernel doesn't provide a documented interface to set affinity
// but the WIN32 API does.  This call will be handled, therefore, by
// the WIN32 API without going into kernel mode.

/*
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGAffinity alist;


   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.setting = setting;

   // Call driver to fill in chip info. structure

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SetAffinity,
                                    &alist,
                                    sizeof(IPR_ARGAffinity),
                                    &alist,
                                    sizeof(IPR_ARGAffinity));

   if( OK_STATUS(Status) )
       Status = IoStatus.Status;

   return (IPR_STATUS) Status;
*/
}

extern IPR_STATUS KIprSetPme(IN unsigned int pid, IN IPR_BOOLEAN state, OUT IPR_BOOLEAN *prev_state)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGPme alist;

#ifdef TRCSH
   printf("Shell KIprSetPme pid = %x, state = %x, prev_state = %x\n",
      pid,state,prev_state);
#endif
   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.pid = pid;
   alist.state = state;
   alist.prev_state = prev_state;

   // Call driver to fill in chip info. structure

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SetPme,
                                    &alist,
                                    sizeof(IPR_ARGPme),
                                    &alist,
                                    sizeof(IPR_ARGPme));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprResetIpl(void)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // Call driver to fill in chip info. structure

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_ResetIpl,
                                    NULL,
                                    0,
                                    NULL,
                                    0 );

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprInitSysInfo(IN IPR_SYSTEM_DATA *newsysdata)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   GetNewSystemData(newsysdata);

   // Call driver to fill in chip info. structure

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_InitSysInfo,
                                    newsysdata,
                                    sizeof(IPR_SYSTEM_DATA),
                                    newsysdata,
                                    sizeof(IPR_SYSTEM_DATA));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprDoneSysInfo(void)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // Deallocate system structures

#ifdef TRCSH
   printf ("Shell_nt: KIprDoneSysInfo enter\n");
#endif
   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_DoneSysInfo,
                                    NULL,
                                    0,
                                    NULL,
                                    0);

#ifdef TRCSH
   printf ("Shell_nt: return from driver,Status=%x, IoStatus.Status=%x\n",
      Status,IoStatus.Status);
#endif
   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprOpenSampFile(IN UNICODE_STRING *fname, IN UNICODE_STRING *defname)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGOpenSamp alist;

   // Call driver to fill in chip info. structure

   alist.fname = fname;
   alist.defname = defname;
   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_OpenSampFile,
                                    &alist,
                                    sizeof(IPR_ARGOpenSamp),
                                    &alist,
                                    sizeof(IPR_ARGOpenSamp));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprCloseSampFile(OUT int *buffwrites, OUT int *partwrites)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGCloseSamp alist;

   // Call driver to close sample file
   alist.buffwrites = buffwrites;
   alist.partwrites = partwrites;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_CloseSampFile,
                                    &alist,
                                    sizeof(IPR_ARGCloseSamp),
                                    &alist,
                                    sizeof(IPR_ARGCloseSamp));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprWriteFile(IN void *bptr, IN IPR_SIZE bsize)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGWriteSamp alist;

   // Call driver to fill in chip info. structure

   alist.bptr = bptr;
   alist.bsize = bsize;
   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_WriteFile,
                                    &alist,
                                    sizeof(IPR_ARGWriteSamp),
                                    &alist,
                                    sizeof(IPR_ARGWriteSamp));

   return (IPR_STATUS) Status;
}

//
// Buffer Descriptor shells
//

extern IPR_STATUS KIprBdCreate (IN IPR_CPUNUM cpu, OUT IPR_BUFDSC **retptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDCreate alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdCreate,
                                    &alist,
                                    sizeof(IPR_ARGBDCreate),
                                    &alist,
                                    sizeof(IPR_ARGBDCreate));
//                                    NULL,
//                                    0 );
   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdDelete (IN IPR_CPUNUM cpu)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // InputBuffer and Length is cpu

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdDelete,
                                    &cpu,
                                    sizeof(int),
                                    &cpu,
                                    sizeof(int));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdSetNumCounters (IN IPR_CPUNUM cpu, IN IPR_COUNT numctrs)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDNumctrs alist;

#ifdef TRCSH
   printf ("Shell: KIprBdSetNumCounters cpu = %d, numctrs = %d\n",
      cpu,numctrs);
#endif
   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.numctrs = numctrs;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdSetNumCounters,
                                    &alist,
                                    sizeof(IPR_ARGBDNumctrs),
                                    &alist,
                                    sizeof(IPR_ARGBDNumctrs));
//                                    NULL,
//                                    0 );

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdSetAction (IPR_CPUNUM cpu,
                        int action_type,        // ACTION_SAMPLE or ACTION_FULL - need to define
                        int action,
                        int (*targetroutine) (),
                        unsigned int targetpid)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDAction alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.action_type = action_type;
   alist.action = action;
   alist.targetroutine = targetroutine;
   alist.targetpid = targetpid;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdSetAction,
                                    &alist,
                                    sizeof(IPR_ARGBDAction),
                                    &alist,
                                    sizeof(IPR_ARGBDAction));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdSetActiveCounters (IN IPR_CPUNUM cpu, IN IPR_CTRMASK ctrmask)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDActctrs alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.ctrmask = ctrmask;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdSetActiveCounters,
                                    &alist,
                                    sizeof(IPR_ARGBDActctrs),
                                    &alist,
                                    sizeof(IPR_ARGBDActctrs));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdAddPcRange (IN IPR_CPUNUM cpu, IN IPR_PC_RANGE *pcrange)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDPcrng alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.pcrange = pcrange;
   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdAddPcRange,
                                    &alist,
                                    sizeof(IPR_ARGBDPcrng),
                                    &alist,
                                    sizeof(IPR_ARGBDPcrng));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprBdSetSkipCount (IN IPR_CPUNUM cpu, IN IPR_CTRNUMBER ctr, IN int skipvalue)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGBDSkip alist;

#ifdef TRCSH
   printf("Shell: KIprBdSetSkipcount, cpu= %d, ctr = %d, skipvalue = %d\n",
      cpu,ctr,skipvalue);
#endif
   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.ctr = ctr;
   alist.skipvalue = skipvalue;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_BdSetSkipCount,
                                    &alist,
                                    sizeof(IPR_ARGBDSkip),
                                    &alist,
                                    sizeof(IPR_ARGBDSkip));

#ifdef TRCSH
   printf("Shell: return from NtDeviceIoControlFile, Status= %x, IoStatus.Status = %x\n",
      Status,IoStatus.Status);
#endif

   return (IPR_STATUS) Status;

}

//
// Sample Buffer Shells
//

extern IPR_STATUS KIprSbCreate (IN IPR_CPUNUM cpu,
                        IN IPR_SIZE size,
                        IN unsigned int numbuffs,
                        OUT IPR_SDESC *descrip,
                        OUT IPR_SBUFFER **retptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGSBCreate alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.size = size;
   alist.numbuffs = numbuffs;
   alist.descrip = descrip;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SbCreate,
                                    &alist,
                                    sizeof(IPR_ARGSBCreate),
                                    &alist,
                                    sizeof(IPR_ARGSBCreate));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprSbDelete (IN IPR_CPUNUM cpu, IN IPR_SBUFFER *bufptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGSBDelete alist;

   // InputBufferLength is argument list length
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.bufptr = bufptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SbDelete,
                                    &alist,
                                    sizeof(IPR_ARGSBDelete),
                                    &alist,
                                    sizeof(IPR_ARGSBDelete));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprSbSetFull(IN IPR_SBUFFER *bufptr, IN IPR_FULL_STATE full_val)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGSBSetFull alist;

   // InputBufferLength is argument list length
   // InputBuffer is the argument list

   alist.bufptr = bufptr;
   alist.full_val = full_val;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SbSetFull,
                                    &alist,
                                    sizeof(IPR_ARGSBSetFull),
                                    &alist,
                                    sizeof(IPR_ARGSBSetFull));

   return (IPR_STATUS) Status;


}

extern IPR_STATUS KIprSbGetInfo(IN IPR_SBUFFER *bufptr, OUT IPR_SDESC *descrip)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGSBGetInfo alist;

   // InputBufferLength is argument list length
   // InputBuffer is the argument list

   alist.bufptr = bufptr;
   alist.descrip = descrip;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SbGetInfo,
                                    &alist,
                                    sizeof(IPR_ARGSBGetInfo),
                                    &alist,
                                    sizeof(IPR_ARGSBGetInfo));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprSbFind(IN IPR_CPUNUM cpu,
                        OUT IPR_SDESC *descrip,
                        IN IPR_SBUFFER *previous_buffer,
                        OUT IPR_SBUFFER **retptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGSBFind alist;

   // InputBufferLength is argument list length
   // InputBuffer is the argument list
   alist.cpu = cpu;
   alist.descrip = descrip;
   alist.previous_buffer = previous_buffer;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_SbFind,
                                    &alist,
                                    sizeof(IPR_ARGSBFind),
                                    &alist,
                                    sizeof(IPR_ARGSBFind));

   return (IPR_STATUS) Status;

}


//
// Histogram Buffer Shells
//

extern IPR_STATUS KIprHbElements (IN IPR_COUNT numctrs,
                IN IPR_COUNT numval,
                IN int gran,
                OUT int *numelem)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBElements alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.numctrs = numctrs;
   alist.numval = numval;
   alist.gran = gran;
   alist.numelem = numelem;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbElements,
                                    &alist,
                                    sizeof(IPR_ARGHBElements),
                                    &alist,
                                    sizeof(IPR_ARGHBElements));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbIndex (IN IPR_HISTVALUE value,
                IN IPR_CTRNUMBER ctr,
                IN IPR_COUNT numctrs,
                IN IPR_HISTVALUE minval,
                IN int gran,
                OUT int *index)
{

   IPR_ARGHBIndex alist;
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.value = value;
   alist.ctr = ctr;
   alist.numctrs = numctrs;
   alist.minval = minval;
   alist.gran = gran;
   alist.index = index;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbIndex,
                                    &alist,
                                    sizeof(IPR_ARGHBIndex),
                                    &alist,
                                    sizeof(IPR_ARGHBIndex));
   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbValue (IN int index,
                IN IPR_COUNT numctrs,
                IN IPR_HISTVALUE minval,
                IN int gran,
                OUT IPR_HISTELEM *value)
{

   IPR_ARGHBValue alist;
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // InputBuffer and Length is argument list

   alist.index = index;
   alist.numctrs = numctrs;
   alist.minval = minval;
   alist.gran = gran;
   alist.value = value;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbValue,
                                    &alist,
                                    sizeof(IPR_ARGHBValue),
                                    &alist,
                                    sizeof(IPR_ARGHBValue));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbCreate (IN IPR_CPUNUM cpu, IN IPR_HDESC *descrip, OUT IPR_HBUFFER **retptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBCreate alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list
   alist.cpu = cpu;
   alist.descrip = descrip;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbCreate,
                                    &alist,
                                    sizeof(IPR_ARGHBCreate),
                                    &alist,
                                    sizeof(IPR_ARGHBCreate));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbDelete (IN IPR_CPUNUM cpu, IN IPR_HBUFFER *bufptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBDelete alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list
   alist.cpu = cpu;
   alist.bufptr = bufptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbDelete,
                                    &alist,
                                    sizeof(IPR_ARGHBDelete),
                                    &alist,
                                    sizeof(IPR_ARGHBDelete));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbFind (IN IPR_CPUNUM cpu,
                        IN IPR_HBUFFER *previous_buffer,
                        IN IPR_HDESC *descrip,
                        OUT IPR_HBUFFER **retptr)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBFind alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.previous_buffer = previous_buffer;
   alist.descrip = descrip;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbFind,
                                    &alist,
                                    sizeof(IPR_ARGHBFind),
                                    &alist,
                                    sizeof(IPR_ARGHBFind));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprHbRead(IN IPR_CPUNUM cpu,
                      IN IPR_HBUFFER *bufptr,
                      IN int start,
                      IN IPR_COUNT cnt,
                      OUT IPR_HISTELEM *retptr)
{

   IPR_ARGHBRead alist;
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;

   // InputBuffer and Length is argument list

   alist.cpu = cpu;
   alist.bufptr = bufptr;
   alist.start = start;
   alist.cnt = cnt;
   alist.retptr = retptr;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbRead,
                                    &alist,
                                    sizeof(IPR_ARGHBRead),
                                    &alist,
                                    sizeof(IPR_ARGHBRead));

   return (IPR_STATUS) Status;

}

extern IPR_STATUS KIprHbClear (IN IPR_CPUNUM cpu,
                        IN IPR_HBUFFER *bufptr,
                        IN int start,
                        IN IPR_COUNT cnt)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBClear alist;

   // InputBufferLength is length of the argument list
   // InputBuffer is the argument list

   alist.cpu = cpu;
   alist.bufptr = bufptr;
   alist.start = start;
   alist.cnt = cnt;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbClear,
                                    &alist,
                                    sizeof(IPR_ARGHBClear),
                                    &alist,
                                    sizeof(IPR_ARGHBClear));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprHbMaxElements(OUT IPR_COUNT *elem)
{

   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGHBMaxElem alist;

   // OutputBuffer and Length is elem

   alist.elem = elem;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                    (HANDLE)NULL,
                                    (PIO_APC_ROUTINE)NULL,
                                    (PVOID)NULL,
                                    &IoStatus,
                                    IOCTL_IPR_HbMaxElements,
                                    &alist,
                                    sizeof(IPR_ARGHBMaxElem),
                                    &alist,
                                    sizeof(IPR_ARGHBMaxElem));

   return (IPR_STATUS) Status;
}

extern IPR_STATUS KIprCurrentCPU (OUT IPR_CPUNUM *cpu)
{
   IO_STATUS_BLOCK IoStatus;
   NTSTATUS Status;
   IPR_ARGCurrentCPU alist;

   alist.cpu = cpu;

   Status = NtDeviceIoControlFile( IprApiHandle,
                                   (HANDLE) NULL,
                                   (PIO_APC_ROUTINE) NULL,
                                   (PVOID) NULL,
                                   &IoStatus,
                                   IOCTL_IPR_CurrentCPU,
                                   &alist,
                                   sizeof (IPR_ARGCurrentCPU),
                                   &alist,
                                   sizeof (IPR_ARGCurrentCPU));
   return (IPR_STATUS) Status;
}
