#if defined(__DECC) || defined(__DECCXX)
# pragma module SHELL_WIN_NT "X-1"
#endif

/* SHELL_WIN_NT.C */

/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1994 BY                                                   */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.                                                            */
/*                                                                          */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.                                                            */
/*                                                                          */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 */
/*                                                                          */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:    This file contains the Windows NT IPROBE API driver shell
 *              support routines that need to use the windows.h include
 *              file (rather than the ntddk.h include file that is used
 *              by shell_nt.c).
 *
 * Author:              Bradley M. Waters
 * Creation Date:       04-Aug-1994
 *
 * Modified by:
 *
 * X-1  BMW             Bradley M. Waters       04-Aug-1994
 *      Original Version.
 *
*/

#include "windows.h"
#include "stdarg.h"
#include "stdio.h"

#include "ctr_int_defs.h"
#include "ipr_api_int_defs.h"

//extern HANDLE ctrHand[][];
//extern HANDLE gctrHand[][];
extern HANDLE gIprApiHandle,IprApiHandle;

extern IPR_STATUS DuplHand(IN HANDLE hSourceProc, IN HANDLE hTargetProc,
                           IN DWORD num_ctrs)
{

//   NTSTATUS Status;
   IPR_CPUNUM current_cpu;
   DWORD current_ctr;

//    if (!HandleExists)
//    {

   if(!DuplicateHandle(hSourceProc,
       gIprApiHandle,
       hTargetProc,
       &IprApiHandle,
       0,
       TRUE,
       DUPLICATE_SAME_ACCESS))
   {
      return GetLastError();
   }
   for (current_cpu = first_cpu (AFFINITY_CPU_MASK, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (AFFINITY_CPU_MASK, current_cpu, IPR_AFFINITY))
   {
      for (current_ctr = 0; current_ctr < num_ctrs; current_ctr++)
      {
         if(!DuplicateHandle(hSourceProc,
              gctrHand[current_cpu][current_ctr],
              hTargetProc,
              &ctrHand[current_cpu][current_ctr],
              0,
              TRUE,
              DUPLICATE_SAME_ACCESS))
         {

            return GetLastError();
         }
      }
   }
//    }
//    else
//       Status = IPR__KAPIHANDEXISTS;

   return IPR__SUCCESS;
}

void GetNewSystemData(OUT IPR_SYSTEM_DATA *newsysdata)
{
   SYSTEM_INFO siSysInfo;

#ifdef TRCSH
   printf ("Shell_win: GetNewSystemData, newsysdata = %x\n",
        newsysdata);
#endif
   GetSystemInfo(&siSysInfo);

#ifdef TRCSH
   printf ("affin = %x,alloc gran. = %x \n",
        siSysInfo.dwActiveProcessorMask,siSysInfo.dwAllocationGranularity);
#endif
    newsysdata->affinity_cpu_mask = siSysInfo.dwActiveProcessorMask;
    newsysdata->noaffinity_cpu_mask = siSysInfo.dwActiveProcessorMask;
// The allocation granularity returned by this call isn't the value
// I'm looking for.  I'll return 64 instead which is a more appropriate
// value for nonpaged pool allocation granularity.
    newsysdata->allocation_granularity = 64;
    // siSysInfo.dwAllocationGranularity;
    newsysdata->num_cpus = siSysInfo.dwNumberOfProcessors;
}

extern IPR_STATUS GetProcId(OUT unsigned int *Ipid)
{

   *Ipid = GetCurrentProcessId();
//#ifdef TRCSH
//   printf("Shell_win_nt: Ipid = %x, pointer to Ipid = %x \n",*Ipid,Ipid);
//#endif
   return IPR__SUCCESS;
}

extern IPR_STATUS GetChipData(void *chip)
{

#define lenChip 5

   SYSTEM_INFO siSysInfo;
   IPR_CHIPINFO *chipinfo;

   chipinfo = chip;
#ifdef TRCSH
   printf ("Shell_win: GetChipData, chipinfo = %x\n",
        chipinfo);
#endif
   GetSystemInfo(&siSysInfo);
   switch (siSysInfo.dwProcessorType) {
   case PROCESSOR_INTEL_386:
      chipinfo->majorIdent = 386;
      break;
   case PROCESSOR_INTEL_486:
      chipinfo->majorIdent = 486;
      break;
   case PROCESSOR_INTEL_PENTIUM:
      chipinfo->majorIdent = 586;
      break;
//   case PROCESSOR_MIPS_R2000:
//      chipinfo->majorIdent = 2000;
//      break;
//   case PROCESSOR_MIPS_R3000:
//      chipinfo->majorIdent = 3000;
//      break;
   case PROCESSOR_MIPS_R4000:
      chipinfo->majorIdent = 4000;
      break;
   case PROCESSOR_ALPHA_21064:
      chipinfo->majorIdent = 21064;
      break;
   }

#ifdef TRCSH
   printf ("majorIdent = %x\n",
        chipinfo->majorIdent);
#endif
   return IPR__SUCCESS;
}

extern IPR_STATUS SetAffin(IN IPR_CPUNUM cpu, IN IPR_BOOLEAN setting)
{
   DWORD   cpuMask = {1};
   HANDLE  ThreadHand;
   IPR_STATUS status;

   ThreadHand = GetCurrentThread();
   if (setting)
   {
       cpuMask = cpuMask << cpu;
       status = SetThreadAffinityMask(ThreadHand,cpuMask);
       if (OK_STATUS(status))
          return IPR__SUCCESS;
       else
          return IPR__AFFINITY_FAIL;
   }
   else
   {
       status = SetThreadAffinityMask(ThreadHand,sysdata.affinity_cpu_mask);
       if (OK_STATUS(status))
          return IPR__SUCCESS;
       else
          return IPR__AFFINITY_FAIL;
   }

}

