#ifdef __NT
#include "ntddk.h"
#endif
#include "hb_library.h"

extern IPR_STATUS IprHbIndex (IN IPR_HISTVALUE value,
		IN IPR_CTRNUMBER ctr,
		IN IPR_COUNT numctrs,
		IN IPR_HISTVALUE minval,
		IN int gran,
		OUT int *index)
{
   IPR_HISTVALUE
      tmpidx;

   if (!index)
      return IPR__NULLPTR;
   if (value < minval)
      return IPR__NOTINHIST;
   tmpidx = ((((value - minval) >> gran) * numctrs) + ctr);
   *index = (int) tmpidx;
   if (*index < 0 || *index != tmpidx)
      return IPR__NOTINHIST;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprHbValue (IN int index,
		IN IPR_COUNT numctrs,
		IN IPR_HISTVALUE minval,
		IN int gran,
		OUT IPR_HISTVALUE *value)
{
   IPR_HISTVALUE
      tval,
      tval2;

   if (!value)
      return IPR__NULLPTR;

   // Get the first value into a 64-bit form so as to avoid truncation
   // later.

   tval = index / numctrs;
   tval2 = tval << gran;

   // Check for overflow on the shift.

   if (tval2 >> gran != tval)
      return IPR__HISTOVFLW;

   // Check for overflow on the addition.

   tval = tval2 + minval;
   if (tval < minval)
      return IPR__HISTOVFLW;

   // All done, return result.

   *value = tval;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprHbElements (IN IPR_COUNT numctrs,
		IN IPR_HISTVALUE numval,
		IN int gran,
		OUT IPR_COUNT *numelem)
{
   IPR_HISTVALUE
      tmpelems;

   if (!numelem)
      return IPR__NULLPTR;
   tmpelems = (((numval - 1) >> gran) + 1) * numctrs;
   *numelem = (IPR_COUNT) tmpelems;
   if (*numelem != tmpelems)
      return IPR__HISTOVFLW;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprHbMaxValue (IN int index,
		IN IPR_COUNT numctrs,
		IN IPR_HISTVALUE minval,
		IN IPR_HISTVALUE numvals,
		IN int gran,
		OUT IPR_HISTVALUE *hival)
{
   IPR_HISTVALUE tval;
   IPR_HISTVALUE maxidx;

   if (!hival)
      return IPR__NULLPTR;

   // Verify that index is within histogram bounds

   if ((numvals >> gran) < (index / numctrs))
      return IPR__HISTOVFLW;

   // Check for badly constructed histogram

   if (minval + numvals - 1 < minval)
      return IPR__HISTOVFLW;

   // First do the division and get into 64-bit format

   tval = (index / numctrs) + 1;

   // Next do the shift and addition

   tval = minval + (tval << gran) - 1;

   // Save the result

   *hival = tval;

   // Bound the result

   if (*hival > (minval + numvals - 1) || *hival < minval)
      *hival = minval + numvals - 1;

   // All done

   return IPR__SUCCESS;
}
