#ifndef __CTR_INT_DEFS_LOADED
#define __CTR_INT_DEFS_LOADED

/*
 *
 * Module:	CTR_INT_DEFS.H
 *
 * Version:	X-10
 */

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1993, 1994 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//

/*
 * Abstract:	This header file contains some specific definitions used
 * 		by performance counter support code in C.  These definitions
 *		are pertinent only to the counter support layer, not to the
 *		the performance monitor layer.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-10	BMW		Bradley M. Waters		18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-9	BFW1099		Brian F. Wibecan		18-May-1994
 *	Added support for xxxSelectEventsFromArray.
 *
 * X-8	BFW1091		Brian F. Wibecan		13-May-1994
 *	Fix version numbers.
 *
 * X-7	BFW1091		Brian F. Wibecan		13-May-1994
 *	Add ifdefs around pragmas so the OSF/1 compiler doesn't complain.
 *
 * X-6	BFW1040		Brian F. Wibecan		11-Mar-1994
 *	Add FreqCounterMap.
 *
 * X-5	BFW1028		Brian F. Wibecan		16-Feb-1994
 *	Added zero_freq_OK argument to fill_event_from_set.
 *
 * X-4	No change
 *
 * X-3	BFW1001		Brian F. Wibecan		17-Dec-1993
 *	Moved the definitions of the function pointers into a separately
 *	named program section.
 *
 * X-2	???		Brian F. Wibecan		???
 *	Added header.
 *
 * X-1	BFW1000		Brian F. Wibecan		16-Dec-1993
 *	Original version.
 *
*/

#include "ctr_ext_defs.h"
#include "ipr_events.h"
#include "ipr_kernel_util.h"

#define EVENT_MASK_COUNT ((IPR_EVT_MAXIMUM + 63) >> 6)
#define CONTEXT_AREA_COUNT IPR_MAX_CPUS

struct _ipr_eventset
   {
      uint64 mask [EVENT_MASK_COUNT];
   };

typedef struct _ctrctxdata
   {
      IPR_EVENT event;
      IPR_FREQ freq;
      unsigned int event_code;
      unsigned int freq_code;
      int specified;
   } CTRCTXDATA;

extern IPR_FREQ
   highest_freq (IPR_FREQ),
   lowest_freq (IPR_FREQ),
   int_to_ext_freq (const IPR_EVENT,
                    IPR_FREQ,
                    const IPR_EVENTSET *,
                    const int *,
                    IPR_SIZE),
   ext_to_int_freq (const IPR_EVENT,
                    IPR_FREQ,
                    const IPR_EVENTSET *,
                    const int *,
                    IPR_SIZE),
   available_freqs (const IPR_EVENT,
                    IPR_FREQ *,
                    const IPR_EVENTSET *,
                    const IPR_FREQ *,
                    IPR_COUNT,
                    const IPR_EVENTSET *,
                    const int *,
                    IPR_SIZE);
extern IPR_BOOLEAN
   valid_event_set (const IPR_EVENTSET *,
                    const IPR_EVENTSET *,
                    IPR_COUNT);
extern IPR_STATUS
   available_events (const IPR_EVENTSET *,
                     IPR_EVENTSET *,
                     const IPR_EVENTSET *,
                     IPR_COUNT),
   current_events (IPR_EVENTSET *,
                   const CTRCTXDATA *,
                   IPR_COUNT),
   get_counter_number (const IPR_EVENT,
                       IPR_CTRNUMBER *,
                       const CTRCTXDATA *,
                       IPR_COUNT),
   event_counter_map (IPR_EVENT *,
                      IPR_SIZE,
                      const CTRCTXDATA *,
                      IPR_COUNT),
   freq_counter_map (IPR_FREQ *,
                     IPR_SIZE,
                     const CTRCTXDATA *,
                     IPR_COUNT,
                     const IPR_EVENTSET *,
                     const int *,
                     IPR_SIZE),
   clear_counter_data (CTRCTXDATA *,
                       IPR_COUNT),
   get_current_freq (const IPR_EVENT,
                     IPR_FREQ *,
                     const CTRCTXDATA *,
                     IPR_COUNT,
                     const IPR_EVENTSET *,
                     const int *,
                     IPR_SIZE),
   fill_event_from_set (const IPR_EVENTSET *,
                        IPR_FREQ,
                        const IPR_EVENTSET *,
                        const IPR_FREQ *,
                        IPR_COUNT,
                        CTRCTXDATA *,
                        const IPR_EVENTSET *,
                        const int *,
                        IPR_SIZE,
                        IPR_BOOLEAN),
   fill_event_from_array (const IPR_EVENT *,
                          const IPR_FREQ *,
                          IPR_SIZE,
                          const IPR_EVENTSET *,
                          const IPR_FREQ *,
                          IPR_COUNT,
                          const CTRCTXDATA *,
                          CTRCTXDATA *,
                          const IPR_EVENTSET *,
                          const int *,
                          IPR_SIZE,
                          IPR_BOOLEAN),
   counter_events (IPR_CTRNUMBER, const IPR_EVENTSET *, IPR_COUNT,
                   IPR_EVENTSET *),
   available_counters (IPR_EVENT,
                       const IPR_EVENTSET *,
                       IPR_COUNT,
                       IPR_CTRMASK *);

typedef IPR_STATUS (*STATUSFUNCPTR) ();
typedef IPR_COUNT (*COUNTFUNCPTR) ();

extern IPR_STATUS
   NullUnavailableRoutine (void);
extern IPR_COUNT
   NullZeroRoutine (void);

#if defined(__VMS)
#pragma extern_model save
#pragma extern_model strict_refdef "FUNCPTRS"
#endif // #if defined(__VMS)

extern STATUSFUNCPTR
   PtrAvailableEvents,
   PtrAvailableFreqs,
   PtrAvailableModes,
   PtrAvailableFuncs,
   PtrCounterNumber,
   PtrEventCounterMap,
   PtrFreqCounterMap,
   PtrCurrentEvents,
   PtrCurrentFreqs,
   PtrCurrentModes,
   PtrCurrentState,
   PtrClearContext,
   PtrSelectEvents,
   PtrSelectEventsFromArray,
   PtrSetEvents,
   PtrSetModes,
   PtrEnable,
   PtrDisable,
   PtrPause,
   PtrResume,
   PtrClear,
   PtrWrite,
   PtrRead,
   PtrReadAll,
   PtrAvailableCounters,
   PtrCounterEvents;
extern COUNTFUNCPTR
   PtrNumCounters;

#if defined(__VMS)
#pragma extern_model restore
#endif // #if defined(__VMS)

#endif // __CTR_INT_DEFS_LOADED
