#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ipr_events.h"
#include "ipr_events.def"
#include "ipr_base_defs.h"

typedef unsigned long int boolean;
boolean IsPca56(void);

extern const char *IprEventNumberToName (IPR_EVENT evt)
  {
	if (IsPca56() && evt >= IPR_EVT_SCACHE_ACCESS)
	  {
		evt += 14;
	  }
	return event_names[evt];
//	return (evt > 0 && evt < IPR_EVT_MAXIMUM) ? event_names [evt] : "<Unknown>";
  }

extern IPR_COUNT IprNumberOfEvents (void)
  {
	return IsPca56() ? IPR_EVT_MAXIMUM - 1 : IPR_EVT_MAXIMUM - 2;
  }

extern IPR_STATUS IprEventNameToNumber(char *ename,
  IPR_EVENT *evtnum,
  const char **matches)
  {
	IPR_EVENT foundnum;
	int i, len, found, exact;
	char *sname;
	static char *found_list = NULL;

	if (found_list != NULL)
	  {
		free(found_list);
		found_list = NULL;
	  }

	if (ename == NULL || evtnum == NULL)
	  {
		return IPR__NULLPTR;
	  }

	*evtnum = 0;

	if ((len = strlen(ename)) == 0)
	  {
		return IPR__INVEVENTNAME;
	  }

	sname = (char *)malloc(len + 1);
	for (i = 0; i < len + 1; i++)
	  {
		sname[i] = tolower(ename[i]);
	  }
	exact = 0;
	for (i = 1, found = 0; i < (sizeof(event_names) / sizeof(char *)) && !exact; i++)
	  {
		if (strncmp (event_names [i], sname, len) == 0)
		  {
			// The beginning of the event name matches the requested name,
			// so the requested name is at least a truncation.

			foundnum = i;
			found++;
			if (len == (int)strlen (event_names [i]))
			  {
				// An exact match
	
				exact = i;
				found = 1;
			  }
			else
		         {

            // Non-exact match.  Add this name to the found list.

            if (found_list)
            {
               found_list = realloc (found_list, strlen (event_names [i]) +
                                     strlen (found_list) + 2);
               (void) strcat (found_list, " ");
               (void) strcat (found_list, event_names [i]);
            }
            else
            {
               found_list = malloc (strlen (event_names [i]) + 1);
               (void) strcpy (found_list, event_names [i]);
            }
         }
      }
   }
   // Get rid of temporary storage

   free (sname);

   // Return the string(s) if requested

   if (matches)
      *matches = (found == 1) ? event_names [foundnum] : found_list;
   switch (found)
   {
      case 0:

         // No match found

         return IPR__INVEVENTNAME;
      case 1:

         // One match found.  Do not need the found list, so get rid of it.
         // In this case, return the event number; all other cases leave it
         // at zero.

         if (found_list)
            free (found_list);
         found_list = NULL;

	*evtnum = foundnum;

	if (IsPca56())
	  {
		if (foundnum == IPR_EVT_BCACHE_VICTIM)
		  {
			foundnum = IPR_EVT_PCA56_BCACHE_VICTIM;
		  }

		if (foundnum >= IPR_EVT_SCACHE_ACCESS && foundnum <= IPR_EVT_SYS_READ_REQ)
		  {
			return IPR__INVEVENTNAME;
		  }

		if (foundnum >= IPR_EVT_PCA56_BCACHE_READ)
		  {
			*evtnum -= 14;
		  }
	  }
	else
	  {
		if (foundnum > 49) // this is a hack
		  {
			return IPR__INVEVENTNAME;
		  }
	  }


         return IPR__SUCCESS;
   }

   // More than one match found

   return IPR__AMBIGUOUS;
}
