#include "ipr_messages_t.h"
#include "ipr_base_defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define PRINT_MESSAGE_SEP ": "
#define PRINT_MESSAGE_UNKNOWN "Unknown error message "
#define PRINT_MESSAGE_U_SIZE 8
#define PRINT_MESSAGE_IV_P1 "Message "
#define PRINT_MESSAGE_IV_P2 " has invalid code "
#define PRINT_MESSAGE_TEXT ", text "

static void defcallback (char *s)
{
   (void) fputs (s, stderr);
}

extern IPR_STATUS IprPrintMessage (char *tag, IPR_STATUS errcode,
                                   void (*callback) (char *))
{
   int
      idx,
      slen;
   IPR_STATUS
      retcode;
   char
      *smsg;

   if (!callback)
      callback = &defcallback;
   slen = 1;
   smsg = malloc (1);
   smsg [0] = '\0';
   if (tag)
   {
      slen += strlen (tag) + strlen (PRINT_MESSAGE_SEP);
      smsg = realloc (smsg, slen);
      (void) strcat (smsg, tag);
      (void) strcat (smsg, PRINT_MESSAGE_SEP);
   }
   if ((errcode & IPR__M_FACILITY) != IPR__V_FACILITY)
   {
      slen += strlen (PRINT_MESSAGE_UNKNOWN) + PRINT_MESSAGE_U_SIZE + 2;
      smsg = realloc (smsg, slen);
      (void) sprintf (smsg + strlen (smsg), PRINT_MESSAGE_UNKNOWN "%0*X\n\n",
                      PRINT_MESSAGE_U_SIZE, errcode);
      retcode = IPR__UNKNOWNERR;
   }
   else
   {
      idx = ((errcode & IPR__M_CODE) / IPR__V_CODE_DIVIDE) - 1;
      if (sizeof (message_strings [0]) * idx >= sizeof (message_strings))
      {
         slen += strlen (PRINT_MESSAGE_IV_P1) + PRINT_MESSAGE_U_SIZE +
                 strlen (PRINT_MESSAGE_IV_P2) + 11;
         smsg = realloc (smsg, slen);
         (void) sprintf (smsg + strlen (smsg),
                         PRINT_MESSAGE_IV_P1 "%0*X"
                         PRINT_MESSAGE_IV_P2 "%d\n\n",
                         PRINT_MESSAGE_U_SIZE, errcode, idx);
         retcode = IPR__INVALID;
      }
      else
      {
         slen += strlen (message_strings [idx] . ename) +
                 strlen (PRINT_MESSAGE_TEXT) +
                 strlen (message_strings [idx] . etext) + 4;
         smsg = realloc (smsg, slen);
         (void) sprintf (smsg + strlen (smsg),
                         "%s" PRINT_MESSAGE_TEXT "\"%s\"\n\n",
                         message_strings [idx] . ename,
                         message_strings [idx] . etext);
         retcode = IPR__SUCCESS;
      }
   }
   (*callback) (smsg);
   free (smsg);
   return retcode;
}
