/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1999 BY                                                   */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the EV6-specific performance counter
 *		support routines. This file is a modified copy of
 *              ev4_routines.c.
 *
 * Author:		Jim Goddard
 * Creation Date:	15-Jun-1999
 *
 * Modified by:
 *                      Jim Goddard       16-Jun-1999
 *	Original version.
 *
 */

#ifdef __NT
#  include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#endif

#include "ev6_routines.h"

#ifdef __NT
#  include "api_routines_nt.h"
#endif

#include <assert.h>

#ifdef __NT

#include <stdio.h>
#include <windows.h>

extern HANDLE hPC0[];
extern HANDLE hPC1[];

#endif





EV6SHARED *Ev6Sh = NULL;

/* Auxiliary routines */

static void ev6_init_events (void)
{
   int
      i;

   (void) CtrClearEventSet (&Ev6Sh -> ctr_events [0]);
   (void) CtrClearEventSet (&Ev6Sh -> ctr_events [1]);
   for (i = 0; i < EV6_MUX0_TOTAL; i++)
      Ev6Sh -> events_0 [i] = 0;
   for (i = 0; i < EV6_MUX1_TOTAL; i++)
      Ev6Sh -> events_1 [i] = 0;

   Ev6Sh -> events_0 [EV6_MUX0_CYCLESA] = IPR_EVT_CYCLESA;
   Ev6Sh -> events_0 [EV6_MUX0_RET_INSTRUCTIONS] = IPR_EVT_RET_INSTRUCTIONS;

   Ev6Sh -> events_1 [EV6_MUX1_CYCLES] = IPR_EVT_CYCLES;
   Ev6Sh -> events_1 [EV6_MUX1_RET_COND_BRANCHES] = IPR_EVT_RET_COND_BRANCHES;
   Ev6Sh -> events_1 [EV6_MUX1_RET_BRANCH_MISP] = IPR_EVT_RET_BRANCH_MISP;
   Ev6Sh -> events_1 [EV6_MUX1_RET_DTB_SGL_MISSESx2] = IPR_EVT_RET_DTB_SGL_MISSESx2;
   Ev6Sh -> events_1 [EV6_MUX1_RET_DTB_DBL_MISSES] = IPR_EVT_RET_DTB_DBL_MISSES;
   Ev6Sh -> events_1 [EV6_MUX1_RET_ITB_MISSES] = IPR_EVT_RET_ITB_MISSES;
   Ev6Sh -> events_1 [EV6_MUX1_RET_UNALGN_TRAPS] = IPR_EVT_RET_UNALGN_TRAPS;
   Ev6Sh -> events_1 [EV6_MUX1_REPLAY_TRAPS] = IPR_EVT_REPLAY_TRAP;

   for (i = 0; i < EV6_MUX0_TOTAL; i++)
      if (Ev6Sh -> events_0 [i])
         (void) CtrInsertEvent (&Ev6Sh -> ctr_events [0],
                                Ev6Sh -> events_0 [i]);
   for (i = 0; i < EV6_MUX1_TOTAL; i++)
      if (Ev6Sh -> events_1 [i])
         (void) CtrInsertEvent (&Ev6Sh -> ctr_events [1],
                                Ev6Sh -> events_1 [i]);
}


/*****************************************************************************/
/* EV6-specific routines                                                     */
/*                                                                           */
/* Many of these routines are simply calls to generic routines with          */
/* appropriate parameters for EV6.                                           */
/*****************************************************************************/

// static IPR_BOOLEAN Ev4ValidEventSet (const IPR_EVENTSET *candidate)
// {
//    return valid_event_set (candidate, Ev4Sh -> ctr_events, EV4_COUNTER_COUNT);
// }

static IPR_STATUS Ev6AvailableEvents (const IPR_EVENTSET *initial_events,
                                    IPR_EVENTSET *more_events)
{
   return available_events (initial_events, more_events, Ev6Sh -> ctr_events,
                            EV6_COUNTER_COUNT);
}

static IPR_STATUS Ev6AvailableFreqs (const IPR_EVENT event, IPR_FREQ *freqs)
{
   return available_freqs (event, freqs, Ev6Sh -> ctr_events,
                           Ev6Sh -> ctr_freqs, EV6_COUNTER_COUNT,
                           Ev6Sh -> adjusted_events, Ev6Sh -> adjusted_shifts,
                           EV6_ADJUSTED_COUNT);
}

static IPR_STATUS Ev6AvailableModes (IPR_MMASK *modes)
{
   *modes = Ev6Sh -> modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev6AvailableFuncs (IPR_FMASK *funcs)
{
   *funcs = Ev6Sh -> funcs;
   return IPR__SUCCESS;
}

static IPR_COUNT Ev6NumCounters (void)
{
   return EV6_COUNTER_COUNT;
}

static IPR_STATUS Ev6CurrentEvents (int ctx, IPR_EVENTSET *events)
{
   return current_events (events, Ev6Sh -> context [ctx] . counter,
                          EV6_COUNTER_COUNT);
}

static IPR_STATUS Ev6CounterNumber (int ctx, const IPR_EVENT event,
                               IPR_CTRNUMBER *counter_number)
{
   return get_counter_number (event, counter_number,
                              Ev6Sh -> context [ctx] . counter,
                              EV6_COUNTER_COUNT);
}

static IPR_STATUS Ev6EventCounterMap (int ctx, IPR_EVENT *event_array,
                                    IPR_SIZE asize)
{
   return event_counter_map (event_array, asize,
                             Ev6Sh -> context [ctx] . counter,
                             EV6_COUNTER_COUNT);
}

static IPR_STATUS Ev6FreqCounterMap (int ctx, IPR_FREQ *freq_array,
                                     IPR_SIZE asize)
{
   return freq_counter_map (freq_array, asize,
                            Ev6Sh -> context [ctx] . counter,
                            EV6_COUNTER_COUNT, Ev6Sh -> adjusted_events,
                            Ev6Sh -> adjusted_shifts, EV6_ADJUSTED_COUNT);
}

static IPR_STATUS Ev6CurrentFreqs (int ctx, const IPR_EVENT event,
                              IPR_FREQ *freq)
{
   return get_current_freq (event, freq, Ev6Sh -> context [ctx] . counter,
                            EV6_COUNTER_COUNT, Ev6Sh -> adjusted_events,
                            Ev6Sh -> adjusted_shifts, EV6_ADJUSTED_COUNT);
}

static IPR_STATUS Ev6CurrentModes (int ctx, IPR_MMASK *modes)
{
   *modes = Ev6Sh -> context [ctx] . modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev6CurrentState (int ctx, const IPR_EVENT event)
{
   IPR_CTRNUMBER
      ctr;
   IPR_STATUS
      status;

   status = Ev6CounterNumber (ctx, event, &ctr);
   if (!IPR_SUCCESS (status))
      return status;

   return IPR__INTERRUPTING;
}

static IPR_STATUS Ev6ClearContext (int ctx)
{
   int
      low,
      high,
      i;

   if (ctx == IPR_ALL_CONTEXT_AREAS)
   {
      low = 0;
      high = CONTEXT_AREA_COUNT;
   }
   else
   {
      high = ctx;
      low = high++;
   }

   for (i = low; i < high; i++)
   {
      Ev6Sh -> context [i] . all_specified = FALSE;
      Ev6Sh -> context [i] . modes . ipr_mmask_l_bits = 0;
      (void) clear_counter_data (Ev6Sh -> context [i] . counter,
                                 EV6_COUNTER_COUNT);
   }
   return IPR__SUCCESS;
}

static int global_to_code (IPR_EVENT *pE, IPR_COUNT ecnt, IPR_EVENT cand)
{
   IPR_COUNT
      i;

   if (cand != 0)
   {
      for (i = 0; i < ecnt; i++)
      {
         if (cand == pE[i])
	 {
           return i;
	 }
      }
   }
   return 0;
}

static void ev6_globals_to_codes (CTRCTXDATA *cctx)
{

   /* Translate the events */

   cctx [0] . event_code =
      global_to_code (Ev6Sh -> events_0, EV6_MUX0_TOTAL, cctx [0] . event);
   cctx [1] . event_code =
      global_to_code (Ev6Sh -> events_1, EV6_MUX1_TOTAL, cctx [1] . event);

   /* Translate the frequencies */

   cctx [0] . freq_code = (cctx [0] . freq == Ev6Sh -> ctr_freq_hi [0]);
   cctx [1] . freq_code = (cctx [1] . freq == Ev6Sh -> ctr_freq_hi [1]);
}

// static void ev4_codes_to_globals (CTRCTXDATA *cctx)
// {
//    cctx [0] . event = Ev4Sh -> events_0 [cctx [0] . event_code];
//    cctx [1] . event = Ev4Sh -> events_1 [cctx [1] . event_code];
//    cctx [0] . freq = cctx [0] . freq_code ? Ev4Sh -> ctr_freq_hi [0]
//                                         : Ev4Sh -> ctr_freq_lo [0];
//    cctx [1] . freq = cctx [1] . freq_code ? Ev4Sh -> ctr_freq_hi [1]
//                                         : Ev4Sh -> ctr_freq_lo [1];
// }

static IPR_STATUS Ev6SelectEvents (int ctx, const IPR_EVENTSET *events,
                                   IPR_FREQ freq, uint64 *MUXvalue)
{
   static ev6_context_block
      new_cb;
   ev6_context_block
      *pECB;
   int
      i;
   IPR_STATUS
      status;

   for (i = 0; i < EV6_COUNTER_COUNT; i++)
      new_cb . counter [i] . specified = FALSE;
   status = fill_event_from_set (events, freq,
                                 Ev6Sh -> ctr_events,
                                 Ev6Sh -> ctr_freqs,
                                 EV6_COUNTER_COUNT,
                                 new_cb . counter,
                                 Ev6Sh -> adjusted_events,
                                 Ev6Sh -> adjusted_shifts,
                                 EV6_ADJUSTED_COUNT,
                                 FALSE);
   if (!IPR_SUCCESS (status))
      return status;

   /* Save everything */

   pECB = &Ev6Sh -> context [ctx];
   pECB -> all_specified = TRUE;
   for (i = 0; i < EV6_COUNTER_COUNT; i++)
   {
      if (new_cb . counter [i] . specified)
         pECB -> counter [i] = new_cb . counter [i];
      if (!pECB -> counter [i] . specified)
         pECB -> all_specified = FALSE;
   }

   /* Fill in the code values */

   ev6_globals_to_codes (pECB -> counter);

   /* Return appropriate condition if something is missing */

   if (!pECB -> all_specified)
      return IPR__INCOMPLETE;

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
     {
      *MUXvalue = ev6_get_mux (pECB -> counter);
     }
   return IPR__SUCCESS;
}

static IPR_STATUS Ev6SelectEventsFromArray (int ctx, const IPR_EVENT *events,
                                            const IPR_FREQ *freqs,
                                            IPR_SIZE asize,
                                            uint64 *MUXvalue)
{
   static ev6_context_block
      new_cb;
   ev6_context_block
      *pECB;
   int
      i;
   IPR_STATUS
      status;

   for (i = 0; i < EV6_COUNTER_COUNT; i++)
      new_cb . counter [i] . specified = FALSE;
   pECB = &Ev6Sh -> context [ctx];
   status = fill_event_from_array (events, freqs, asize,
                                  Ev6Sh -> ctr_events,
                                  Ev6Sh -> ctr_freqs,
                                  EV6_COUNTER_COUNT,
                                  pECB -> counter,
                                  new_cb . counter,
                                  Ev6Sh -> adjusted_events,
                                  Ev6Sh -> adjusted_shifts,
                                  EV6_ADJUSTED_COUNT,
                                  FALSE);
   if (!IPR_SUCCESS (status))
      return status;

   /* Save everything */

   pECB -> all_specified = TRUE;
   for (i = 0; i < EV6_COUNTER_COUNT; i++)
   {
      if (new_cb . counter [i] . specified)
         pECB -> counter [i] = new_cb . counter [i];
      if (!pECB -> counter [i] . specified)
         pECB -> all_specified = FALSE;
   }

   /* Fill in the code values */

   ev6_globals_to_codes (pECB -> counter);
   pECB -> muxvalue = ev6_get_mux (pECB -> counter);

   /* Return appropriate condition if something is missing */

   if (!pECB -> all_specified)
      return IPR__INCOMPLETE;

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
   {
      *MUXvalue = ev6_get_mux (pECB -> counter);
   }
   return IPR__SUCCESS;
}

static IPR_STATUS Ev6Pause (int ctx, IPR_CTRMASK ctrs)
{
   return Ev6Disable (ctx, ctrs);
}

static IPR_STATUS Ev6Resume (int ctx, IPR_CTRMASK ctrs)
{
   return Ev6Enable (ctx, ctrs);
}

static IPR_STATUS Ev6CounterEvents (IPR_CTRNUMBER ctr, IPR_EVENTSET *e)
{
   return counter_events (ctr, Ev6Sh -> ctr_events, EV6_COUNTER_COUNT, e);
}

static IPR_STATUS Ev6AvailableCounters (IPR_EVENT evt, IPR_CTRMASK *c)
{
   return available_counters (evt, Ev6Sh -> ctr_events, EV6_COUNTER_COUNT, c);
}

extern void Ev6Initialize (void *Areaptr, IPR_BOOLEAN init_shared)
{
   Ev6Sh = Areaptr;
   assert (sizeof (EV6SHARED) < (8192 - 16));
   if (init_shared)
   {
      (void) Ev6ClearContext (IPR_ALL_CONTEXT_AREAS);
      ev6_init_events ();

      (void) CtrClearEventSet (&Ev6Sh -> adjusted_events [0]);

      Ev6Sh -> ctr_freq_lo [0] = 1 << EV6_FREQ0_LOW;
      Ev6Sh -> ctr_freq_hi [0] = 1 << EV6_FREQ0_HIGH;
      Ev6Sh -> ctr_freq_lo [1] = 1 << EV6_FREQ1_LOW;
      Ev6Sh -> ctr_freq_hi [1] = 1 << EV6_FREQ1_HIGH;
      Ev6Sh -> ctr_freqs [0] =         
	Ev6Sh -> ctr_freq_lo [0] | Ev6Sh -> ctr_freq_hi [0];
      Ev6Sh -> ctr_freqs [1] =
         Ev6Sh -> ctr_freq_lo [1] | Ev6Sh -> ctr_freq_hi [1];

      Ev6Sh -> funcs . ipr_fmask_l_bits = 0;
      Ev6Sh -> funcs . ipr_fmask_v_enable = 1;
      Ev6Sh -> funcs . ipr_fmask_v_disable = 1;
      Ev6Sh -> funcs . ipr_fmask_v_pause = 1;
      Ev6Sh -> funcs . ipr_fmask_v_resume = 1;
      Ev6Sh -> funcs . ipr_fmask_v_interrupt = 1;

      Ev6Sh -> modes . ipr_mmask_l_bits = 0;  // Valid bits set in Ev6OSInit
   }
   PtrAvailableEvents = &Ev6AvailableEvents;
   PtrAvailableFreqs = &Ev6AvailableFreqs;
   PtrAvailableModes = &Ev6AvailableModes;
   PtrAvailableFuncs = &Ev6AvailableFuncs;
   PtrNumCounters = &Ev6NumCounters;
   PtrCounterNumber = &Ev6CounterNumber;
   PtrEventCounterMap = &Ev6EventCounterMap;
   PtrFreqCounterMap = &Ev6FreqCounterMap;
   PtrCurrentEvents = &Ev6CurrentEvents;
   PtrCurrentFreqs = &Ev6CurrentFreqs;
   PtrCurrentModes = &Ev6CurrentModes;
   PtrCurrentState = &Ev6CurrentState;
   PtrClearContext = &Ev6ClearContext;
   PtrSelectEvents = &Ev6SelectEvents;
   PtrSelectEventsFromArray = &Ev6SelectEventsFromArray;
   PtrSetEvents = &Ev6SetEvents;
   PtrSetModes = &Ev6SetModes;
   PtrEnable = &Ev6Enable;
   PtrDisable = &Ev6Disable;
   PtrPause = &Ev6Pause;
   PtrResume = &Ev6Resume;
   PtrCounterEvents = &Ev6CounterEvents;
   PtrAvailableCounters = &Ev6AvailableCounters;
   Ev6OSInit ();
}
