#if defined(__DECC) || defined(__DECCXX)
# pragma module EV6_ROUTINES_VMS_OSF "X-1"
#endif

// EV6_ROUTINES_VMS_OSF.C

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1999 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/

//++
// Abstract:	This file contains the EV6-specific performance counter
//		support routines.
//
// Author:		Jim Goddard
// Creation Date:	17-Jun-1999
//
// Modified by:
//
//                      Jim Goddard 	17-Jun-1999
//	Original version.
//--

#include "ev6_routines.h"
#include "ev6_masks.h"

extern EV6SHARED
   *Ev6Sh;

typedef enum
{
   EV6_PERFMON_DISABLE,                                   // disable counting
   EV6_PERFMON_ENABLE,                                    // enable counting
   EV6_PERFMON_MUX_SELECT,                                // select event
   EV6_PERFMON_OPTIONS,                                   // options
   EV6_PERFMON_READ_COUNTERS = 5,                         // read counters
   EV6_PERFMON_WRITE_COUNTERS,                            // write counter values
   EV6_PERFMON_FREQ_SELECT = EV6_PERFMON_WRITE_COUNTERS,  // select counter freq.
   EV6_PERFMON_ENABLE_RESET                               // enable and write counters
} EV6_PERFMON;

typedef enum
{
   EV6_OPTIONS_ALL_PROC,
   EV6_OPTIONS_SEL_PROC
} EV6_OPTIONS;

extern IPR_STATUS Ev6SetModes (int ctx, IPR_MMASK modes)
{
      return KIprUpdatePerfmon (ctx, EV6_PERFMON_OPTIONS, 0, NULL);
}

extern uint64 ev6_get_mux (CTRCTXDATA *cctx)
{
   EV6_MUXCTL
      muxctl;
   unsigned long int temp;

   muxctl . ev6_muxctl_q_bits = 0;
   muxctl . ev6_muxctl_q_bits |= cctx [0] . event_code << EV6_MUXCTL_K_PCMUX0;
   muxctl . ev6_muxctl_q_bits |= cctx [1] . event_code << EV6_MUXCTL_K_PCMUX1;

   temp = (((unsigned long int ) EV6_MAX_COUNTER_VALUE) - (unsigned long int) (1 << (cctx [1] . freq_code ? EV6_FREQ1_HIGH : EV6_FREQ1_LOW))) << EV6_MUXCTL_K_PC1;
   muxctl . ev6_muxctl_q_bits |= temp;
   temp = (((unsigned long int ) EV6_MAX_COUNTER_VALUE) - (unsigned long int) (1 << (cctx [0] . freq_code ? EV6_FREQ0_HIGH : EV6_FREQ0_LOW))) << EV6_MUXCTL_K_PC0;
   muxctl . ev6_muxctl_q_bits |= temp;
   return muxctl . ev6_muxctl_q_bits;
}

#define ev6_mode_to_option(m) \
   ((m) . ipr_mmask_v_selected ? EV6_OPTIONS_SEL_PROC : EV6_OPTIONS_ALL_PROC)

static IPR_EVENT code_to_global (IPR_CTRNUMBER ctr, unsigned int code)
{

   switch (ctr)
   {
      case 0 :
         return (code < EV6_MUX0_TOTAL) ? Ev6Sh -> events_0 [code] : 0;
      case 1 :
         return (code < EV6_MUX1_TOTAL) ? Ev6Sh -> events_1 [code] : 0;
   }
   return 0;
}

extern IPR_STATUS Ev6SetEvents (int ctx, uint64 *MUXvalue)
{
   CTRCTXDATA
      *c0,
      *c1;
   EV6_MUXCTL
      mux;

   /*
    * If a MUX value was passed, save it and convert the information
    * to the normal form in the counter context area
    */
   if (MUXvalue)
      mux . ev6_muxctl_q_bits = *MUXvalue;
   if (MUXvalue && mux . ev6_muxctl_q_bits !=
                   Ev6Sh -> context [ctx] . muxvalue)
   {
      Ev6Sh -> context [ctx] . muxvalue = mux . ev6_muxctl_q_bits = *MUXvalue;

      /* Save temp values to make code below easier to read */

      c0 = &Ev6Sh -> context [ctx] . counter [0];
      c1 = &Ev6Sh -> context [ctx] . counter [1];

      /* Convert the MUX events back to event numbers */

      c0 -> event_code = mux . ev6_muxctl_v_pcmux0;
      c1 -> event_code = mux . ev6_muxctl_v_pcmux1;
      c0 -> freq_code = EV6_MAX_COUNTER_VALUE - mux . ev6_muxctl_v_pc0;
      c1 -> freq_code = EV6_MAX_COUNTER_VALUE - mux . ev6_muxctl_v_pc1;

      c0 -> event = code_to_global (0, c0 -> event_code);
      c1 -> event = code_to_global (1, c1 -> event_code);

   }
   else
     {
      mux . ev6_muxctl_q_bits = Ev6Sh -> context [ctx] . muxvalue;
     }

   /* Set the MUX selection as requested */

      KIprUpdatePerfmon (ctx, EV6_PERFMON_MUX_SELECT,
                             mux . ev6_muxctl_q_bits & (EV6_MUXCTL_M_PCMUX0 | EV6_MUXCTL_M_PCMUX1), NULL);
      return KIprUpdatePerfmon (ctx,  EV6_PERFMON_FREQ_SELECT,
                             (mux . ev6_muxctl_q_bits & (EV6_MUXCTL_M_PC0 | EV6_MUXCTL_M_PC1)) | EV6_COUNTER_MASK, NULL);
}

static IPR_STATUS Ev6StartStop (int ctx, IPR_CTRMASK ctrs, uint64 option)
{
   IPR_STATUS
      status;

   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs &= EV6_COUNTER_MASK;

   if (ctrs != EV6_COUNTER_MASK)
      return IPR__INCOMPLETE_ERROR;

   status = KIprUpdatePerfmon (ctx, option, ctrs, NULL);
   // RESET_IPL ();
   return status;
}

extern IPR_STATUS Ev6Enable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev6StartStop (ctx, ctrs, EV6_PERFMON_ENABLE);
}

extern IPR_STATUS Ev6Disable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev6StartStop (ctx, ctrs, EV6_PERFMON_DISABLE);
}

extern IPR_STATUS Ev6GetStopCode (uint64 *arg1, uint64 *arg2)
{
   *arg1 = EV6_PERFMON_DISABLE;
   *arg2 = 0;
   return IPR__SUCCESS;
}

extern void Ev6OSInit (void)
{
   PtrGetStopCode = &Ev6GetStopCode;
   Ev6Sh -> modes . ipr_mmask_v_selected = 1;
}
