#ifndef __EV6_ROUTINES_LOADED
#define __EV6_ROUTINES_LOADED 1

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1999 BY			                            */
/*  Digital Equiptment/Compaq Computer Corporation.       		    */
/*  ALL RIGHTS RESERVED.						    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*									    */
/****************************************************************************/
/*
 * Abstract:	This header file contains EV6-specific definitions used
 * 		by the EV6 performance counter support code. This file
 *              is a modified copy of ev4_routines.h.
 *
 * Author:		Jim Goddard
 * Creation Date:	15-Jun-1999
 *
 * Modified by:
 *
 *                      Jim Goddard	15-Jun-1999
 *	Original version.
 *
*/

#include "ctr_int_defs.h"

typedef enum
{
   //
   // MUX selections for counter 0
   //
   EV6_MUX0_CYCLESA,             // cylces
   EV6_MUX0_RET_INSTRUCTIONS,    // retired instructions
   EV6_MUX0_TOTAL
} EV6_MUX0;

typedef enum
{
   //
   // MUX selections for counter 1
   //
   EV6_MUX1_CYCLES,                // cycles
   EV6_MUX1_RET_COND_BRANCHES,     // retired conditional branches
   EV6_MUX1_RET_BRANCH_MISP,       //    "    branch mispredictions
   EV6_MUX1_RET_DTB_SGL_MISSESx2,  //    "    DTB single misses times 2
   EV6_MUX1_RET_DTB_DBL_MISSES,    //    "     "  double misses
   EV6_MUX1_RET_ITB_MISSES,        //    "    ITB misses
   EV6_MUX1_RET_UNALGN_TRAPS,      //    "    unaligned traps
   EV6_MUX1_REPLAY_TRAPS,          // replay traps
   EV6_MUX1_TOTAL
} EV6_MUX1;

//
// Counter 0 frequency selections
//
//#define EV6_FREQ0_LOW 16
#define EV6_FREQ0_LOW 20
#define EV6_FREQ0_HIGH 12

//
// Counter 1 frequency selections
//
//#define EV6_FREQ1_LOW 12
#define EV6_FREQ1_LOW 20
#define EV6_FREQ1_HIGH 8

#define EV6_COUNTER_COUNT 2
#define EV6_COUNTER_MASK 3
#define EV6_ADJUSTED_COUNT 1

typedef struct _ev6_context_block
   {
      IPR_MMASK modes;
      int all_specified;
      uint64 muxvalue;
      CTRCTXDATA counter [EV6_COUNTER_COUNT];
   } ev6_context_block;

typedef struct {
   ev6_context_block
      context [CONTEXT_AREA_COUNT];
   IPR_EVENTSET
      ctr_events [EV6_COUNTER_COUNT],
      adjusted_events [EV6_ADJUSTED_COUNT];
   IPR_FMASK
      funcs;
   IPR_MMASK
      modes;
   int
      adjusted_shifts [EV6_ADJUSTED_COUNT];
   IPR_FREQ
      ctr_freqs [EV6_COUNTER_COUNT],
      ctr_freq_hi [EV6_COUNTER_COUNT],
      ctr_freq_lo [EV6_COUNTER_COUNT];
   IPR_EVENT
      events_0 [EV6_MUX0_TOTAL],
      events_1 [EV6_MUX1_TOTAL];
} EV6SHARED;

// OS specific routines
extern uint64
   ev6_get_mux (CTRCTXDATA *cctx);
extern IPR_STATUS
   Ev6SetModes (int ctx, IPR_MMASK modes),
   Ev6SetEvents (int ctx, uint64 *MUXvalue),
   Ev6Enable (int ctx, IPR_CTRMASK ctrs),
   Ev6Disable (int ctx, IPR_CTRMASK ctrs);
extern void
   Ev6OSInit (void);

#ifdef __VMS
#  include "ev4_routines_vms.h"
#endif
#ifdef __osf__
#  include "ev4_routines_osf1.h"
#endif
#ifdef __LINUX__
#  include "ev6_routines_linux.h"
#endif
#ifdef __FreeBSD__
#  include "ev6_routines_freebsd.h"
#endif
#ifdef __NT
#  include "ev4_routines_nt.h"
#endif

#endif /* ifndef __EV6_ROUTINES_LOADED */
